/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2018 BayLibre, SAS
 * Author: Neil Armstrong <narmstrong@baylibre.com>
 */

#ifndef __MESON_OVERLAY_H
#define __MESON_OVERLAY_H

#include "meson_drv.h"

int meson_overlay_create(struct meson_drm *priv);

#endif /* __MESON_OVERLAY_H */
