/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2020 frank@allwinnertech.com
 * Copyright (C) 2021 Samuel Holland <samuel@sholland.org>
 */

#ifndef _CCU_SUN20I_D1_H_
#define _CCU_SUN20I_D1_H_

#include <dt-bindings/clock/sun20i-d1-ccu.h>
#include <dt-bindings/reset/sun20i-d1-ccu.h>

#define CLK_NUMBER		(CLK_FANOUT2 + 1)

#endif /* _CCU_SUN20I_D1_H_ */
