// SPDX-License-Identifier: MIT
/*
 * Copyright (C) 2021 Advanced Micro Devices, Inc.
 *
 * Authors: AMD
 */

#ifndef _DMUB_DCN303_H_
#define _DMUB_DCN303_H_

#include "dmub_dcn20.h"

/* Registers. */

extern const struct dmub_srv_common_regs dmub_srv_dcn303_regs;

/* Hardware functions. */

#endif /* _DMUB_DCN303_H_ */
