/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * linux/drivers/media/platform/samsung/s5p-mfc/s5p_mfc_cmd_v6.h
 *
 * Copyright (C) 2011 Samsung Electronics Co., Ltd.
 *		http://www.samsung.com/
 */

#ifndef S5P_MFC_CMD_V6_H_
#define S5P_MFC_CMD_V6_H_

#include "s5p_mfc_common.h"

struct s5p_mfc_hw_cmds *s5p_mfc_init_hw_cmds_v6(void);

#endif /* S5P_MFC_CMD_H_ */
