/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
	Mantis PCI bridge driver

	Copyright (C) Manu Abraham (abraham.manu@gmail.com)

*/

#ifndef __MANTIS_HIF_H
#define __MANTIS_HIF_H

#define MANTIS_HIF_MEMRD		1
#define MANTIS_HIF_MEMWR		2
#define MANTIS_HIF_IOMRD		3
#define MANTIS_HIF_IOMWR		4

#endif /* __MANTIS_HIF_H */
