// SPDX-License-Identifier: MIT
/*
 * Copyright (C) 2021 Advanced Micro Devices, Inc.
 *
 * Authors: AMD
 */

#ifndef __DAL_IRQ_SERVICE_DCN303_H__
#define __DAL_IRQ_SERVICE_DCN303_H__

#include "../irq_service.h"

struct irq_service *dal_irq_service_dcn303_create(struct irq_service_init_data *init_data);

#endif /* __DAL_IRQ_SERVICE_DCN303_H__ */
