/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (c) 1996, 2003 VIA Networking Technologies, Inc.
 * All rights reserved.
 *
 * Purpose: Country Regulation Rules header file
 *
 * Author: Lucas Lin
 *
 * Date: Dec 23, 2004
 *
 */

#ifndef _CHANNEL_H_
#define _CHANNEL_H_

#include "device.h"

void vnt_init_bands(struct vnt_private *priv);

#endif  /* _CHANNEL_H_ */
