// SPDX-License-Identifier: GPL-2.0+
/*
 * Copyright (C) 2017 Free Electrons
 * Maxime Ripard <maxime.ripard@free-electrons.com>
 */

#ifndef _SUN4I_LVDS_H_
#define _SUN4I_LVDS_H_

int sun4i_lvds_init(struct drm_device *drm, struct sun4i_tcon *tcon);

#endif /* _SUN4I_LVDS_H_ */
