/*
 * Copyright (C) 2019  Advanced Micro Devices, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef _gc_10_3_0_DEFAULT_HEADER
#define _gc_10_3_0_DEFAULT_HEADER


// addressBlock: gc_sdma0_sdma0dec
#define mmSDMA0_DEC_START_DEFAULT                                                0x00000000
#define mmSDMA0_GLOBAL_TIMESTAMP_LO_DEFAULT                                      0x00000000
#define mmSDMA0_GLOBAL_TIMESTAMP_HI_DEFAULT                                      0x00000000
#define mmSDMA0_PG_CNTL_DEFAULT                                                  0x00000000
#define mmSDMA0_PG_CTX_LO_DEFAULT                                                0x00000000
#define mmSDMA0_PG_CTX_HI_DEFAULT                                                0x00000000
#define mmSDMA0_PG_CTX_CNTL_DEFAULT                                              0x00000000
#define mmSDMA0_POWER_CNTL_DEFAULT                                               0x40000050
#define mmSDMA0_CLK_CTRL_DEFAULT                                                 0x00000100
#define mmSDMA0_CNTL_DEFAULT                                                     0x000000c2
#define mmSDMA0_CHICKEN_BITS_DEFAULT                                             0x03ef0107
#define mmSDMA0_GB_ADDR_CONFIG_DEFAULT                                           0x00000444
#define mmSDMA0_GB_ADDR_CONFIG_READ_DEFAULT                                      0x00000444
#define mmSDMA0_RB_RPTR_FETCH_HI_DEFAULT                                         0x00000000
#define mmSDMA0_SEM_WAIT_FAIL_TIMER_CNTL_DEFAULT                                 0x00000000
#define mmSDMA0_RB_RPTR_FETCH_DEFAULT                                            0x00000000
#define mmSDMA0_IB_OFFSET_FETCH_DEFAULT                                          0x00000000
#define mmSDMA0_PROGRAM_DEFAULT                                                  0x00000000
#define mmSDMA0_STATUS_REG_DEFAULT                                               0x46dee557
#define mmSDMA0_STATUS1_REG_DEFAULT                                              0x000003ff
#define mmSDMA0_RD_BURST_CNTL_DEFAULT                                            0x00000002
#define mmSDMA0_HBM_PAGE_CONFIG_DEFAULT                                          0x00000000
#define mmSDMA0_UCODE_CHECKSUM_DEFAULT                                           0x00000000
#define mmSDMA0_F32_CNTL_DEFAULT                                                 0x00000001
#define mmSDMA0_FREEZE_DEFAULT                                                   0x00000000
#define mmSDMA0_PHASE0_QUANTUM_DEFAULT                                           0x00010002
#define mmSDMA0_PHASE1_QUANTUM_DEFAULT                                           0x00010002
#define mmSDMA0_EDC_CONFIG_DEFAULT                                               0x00000002
#define mmSDMA0_BA_THRESHOLD_DEFAULT                                             0x03ff03ff
#define mmSDMA0_ID_DEFAULT                                                       0x00000001
#define mmSDMA0_VERSION_DEFAULT                                                  0x00000500
#define mmSDMA0_EDC_COUNTER_DEFAULT                                              0x00000000
#define mmSDMA0_EDC_COUNTER_CLEAR_DEFAULT                                        0x00000000
#define mmSDMA0_STATUS2_REG_DEFAULT                                              0x00000000
#define mmSDMA0_ATOMIC_CNTL_DEFAULT                                              0x00000200
#define mmSDMA0_ATOMIC_PREOP_LO_DEFAULT                                          0x00000000
#define mmSDMA0_ATOMIC_PREOP_HI_DEFAULT                                          0x00000000
#define mmSDMA0_UTCL1_CNTL_DEFAULT                                               0xd0000191
#define mmSDMA0_UTCL1_WATERMK_DEFAULT                                            0xfffbd9fb
#define mmSDMA0_UTCL1_RD_STATUS_DEFAULT                                          0x01011555
#define mmSDMA0_UTCL1_WR_STATUS_DEFAULT                                          0x51011555
#define mmSDMA0_UTCL1_INV0_DEFAULT                                               0x00000800
#define mmSDMA0_UTCL1_INV1_DEFAULT                                               0x00000000
#define mmSDMA0_UTCL1_INV2_DEFAULT                                               0x00000000
#define mmSDMA0_UTCL1_RD_XNACK0_DEFAULT                                          0x00000000
#define mmSDMA0_UTCL1_RD_XNACK1_DEFAULT                                          0x00000000
#define mmSDMA0_UTCL1_WR_XNACK0_DEFAULT                                          0x00000000
#define mmSDMA0_UTCL1_WR_XNACK1_DEFAULT                                          0x00000000
#define mmSDMA0_UTCL1_TIMEOUT_DEFAULT                                            0x00000000
#define mmSDMA0_UTCL1_PAGE_DEFAULT                                               0x010cec00
#define mmSDMA0_RELAX_ORDERING_LUT_DEFAULT                                       0xc0000006
#define mmSDMA0_CHICKEN_BITS_2_DEFAULT                                           0x00100007
#define mmSDMA0_STATUS3_REG_DEFAULT                                              0x03f00000
#define mmSDMA0_PHYSICAL_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA0_PHYSICAL_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA0_PHASE2_QUANTUM_DEFAULT                                           0x00010002
#define mmSDMA0_ERROR_LOG_DEFAULT                                                0x0000000f
#define mmSDMA0_PUB_DUMMY_REG0_DEFAULT                                           0x00000000
#define mmSDMA0_PUB_DUMMY_REG1_DEFAULT                                           0x00000000
#define mmSDMA0_PUB_DUMMY_REG2_DEFAULT                                           0x00000000
#define mmSDMA0_PUB_DUMMY_REG3_DEFAULT                                           0x00000000
#define mmSDMA0_F32_COUNTER_DEFAULT                                              0x00000000
#define mmSDMA0_CRD_CNTL_DEFAULT                                                 0x1850c640
#define mmSDMA0_AQL_STATUS_DEFAULT                                               0x00000003
#define mmSDMA0_EA_DBIT_ADDR_DATA_DEFAULT                                        0x00000000
#define mmSDMA0_EA_DBIT_ADDR_INDEX_DEFAULT                                       0x00000000
#define mmSDMA0_TLBI_GCR_CNTL_DEFAULT                                            0x40180454
#define mmSDMA0_TILING_CONFIG_DEFAULT                                            0x00000000
#define mmSDMA0_INT_STATUS_DEFAULT                                               0x00000000
#define mmSDMA0_HOLE_ADDR_LO_DEFAULT                                             0x00000000
#define mmSDMA0_HOLE_ADDR_HI_DEFAULT                                             0x00000000
#define mmSDMA0_CLOCK_GATING_REG_DEFAULT                                         0x00000000
#define mmSDMA0_STATUS4_REG_DEFAULT                                              0x00000001
#define mmSDMA0_SCRATCH_RAM_DATA_DEFAULT                                         0x00000000
#define mmSDMA0_SCRATCH_RAM_ADDR_DEFAULT                                         0x00000000
#define mmSDMA0_TIMESTAMP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA0_STATUS5_REG_DEFAULT                                              0x00000000
#define mmSDMA0_QUEUE_RESET_REQ_DEFAULT                                          0x00000000
#define mmSDMA0_GFX_RB_CNTL_DEFAULT                                              0x80840000
#define mmSDMA0_GFX_RB_BASE_DEFAULT                                              0x00000000
#define mmSDMA0_GFX_RB_BASE_HI_DEFAULT                                           0x00000000
#define mmSDMA0_GFX_RB_RPTR_DEFAULT                                              0x00000000
#define mmSDMA0_GFX_RB_RPTR_HI_DEFAULT                                           0x00000000
#define mmSDMA0_GFX_RB_WPTR_DEFAULT                                              0x00000000
#define mmSDMA0_GFX_RB_WPTR_HI_DEFAULT                                           0x00000000
#define mmSDMA0_GFX_RB_WPTR_POLL_CNTL_DEFAULT                                    0x00403000
#define mmSDMA0_GFX_RB_RPTR_ADDR_HI_DEFAULT                                      0x00000000
#define mmSDMA0_GFX_RB_RPTR_ADDR_LO_DEFAULT                                      0x00000000
#define mmSDMA0_GFX_IB_CNTL_DEFAULT                                              0x00000100
#define mmSDMA0_GFX_IB_RPTR_DEFAULT                                              0x00000000
#define mmSDMA0_GFX_IB_OFFSET_DEFAULT                                            0x00000000
#define mmSDMA0_GFX_IB_BASE_LO_DEFAULT                                           0x00000000
#define mmSDMA0_GFX_IB_BASE_HI_DEFAULT                                           0x00000000
#define mmSDMA0_GFX_IB_SIZE_DEFAULT                                              0x00000000
#define mmSDMA0_GFX_SKIP_CNTL_DEFAULT                                            0x00000000
#define mmSDMA0_GFX_CONTEXT_STATUS_DEFAULT                                       0x00000005
#define mmSDMA0_GFX_DOORBELL_DEFAULT                                             0x00000000
#define mmSDMA0_GFX_CONTEXT_CNTL_DEFAULT                                         0x00000000
#define mmSDMA0_GFX_STATUS_DEFAULT                                               0x00000000
#define mmSDMA0_GFX_DOORBELL_LOG_DEFAULT                                         0x00000000
#define mmSDMA0_GFX_WATERMARK_DEFAULT                                            0x00000000
#define mmSDMA0_GFX_DOORBELL_OFFSET_DEFAULT                                      0x00000000
#define mmSDMA0_GFX_CSA_ADDR_LO_DEFAULT                                          0x00000000
#define mmSDMA0_GFX_CSA_ADDR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_GFX_IB_SUB_REMAIN_DEFAULT                                        0x00000000
#define mmSDMA0_GFX_PREEMPT_DEFAULT                                              0x00000000
#define mmSDMA0_GFX_DUMMY_REG_DEFAULT                                            0x0000000f
#define mmSDMA0_GFX_RB_WPTR_POLL_ADDR_HI_DEFAULT                                 0x00000000
#define mmSDMA0_GFX_RB_WPTR_POLL_ADDR_LO_DEFAULT                                 0x00000000
#define mmSDMA0_GFX_RB_AQL_CNTL_DEFAULT                                          0x00004000
#define mmSDMA0_GFX_MINOR_PTR_UPDATE_DEFAULT                                     0x00000000
#define mmSDMA0_GFX_MIDCMD_DATA0_DEFAULT                                         0x00000000
#define mmSDMA0_GFX_MIDCMD_DATA1_DEFAULT                                         0x00000000
#define mmSDMA0_GFX_MIDCMD_DATA2_DEFAULT                                         0x00000000
#define mmSDMA0_GFX_MIDCMD_DATA3_DEFAULT                                         0x00000000
#define mmSDMA0_GFX_MIDCMD_DATA4_DEFAULT                                         0x00000000
#define mmSDMA0_GFX_MIDCMD_DATA5_DEFAULT                                         0x00000000
#define mmSDMA0_GFX_MIDCMD_DATA6_DEFAULT                                         0x00000000
#define mmSDMA0_GFX_MIDCMD_DATA7_DEFAULT                                         0x00000000
#define mmSDMA0_GFX_MIDCMD_DATA8_DEFAULT                                         0x00000000
#define mmSDMA0_GFX_MIDCMD_DATA9_DEFAULT                                         0x00000000
#define mmSDMA0_GFX_MIDCMD_DATA10_DEFAULT                                        0x00000000
#define mmSDMA0_GFX_MIDCMD_CNTL_DEFAULT                                          0x00000000
#define mmSDMA0_PAGE_RB_CNTL_DEFAULT                                             0x80840000
#define mmSDMA0_PAGE_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA0_PAGE_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_PAGE_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_PAGE_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_PAGE_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA0_PAGE_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_PAGE_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA0_PAGE_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA0_PAGE_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA0_PAGE_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA0_PAGE_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_PAGE_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA0_PAGE_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA0_PAGE_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_PAGE_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA0_PAGE_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA0_PAGE_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA0_PAGE_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA0_PAGE_STATUS_DEFAULT                                              0x00000000
#define mmSDMA0_PAGE_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA0_PAGE_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA0_PAGE_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA0_PAGE_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA0_PAGE_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA0_PAGE_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA0_PAGE_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA0_PAGE_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA0_PAGE_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA0_PAGE_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA0_PAGE_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA0_PAGE_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA0_PAGE_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA0_PAGE_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA0_PAGE_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA0_PAGE_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA0_PAGE_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA0_PAGE_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA0_PAGE_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA0_PAGE_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA0_PAGE_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA0_PAGE_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA0_PAGE_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA0_PAGE_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA0_RLC0_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA0_RLC0_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC0_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC0_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC0_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC0_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC0_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC0_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA0_RLC0_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA0_RLC0_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA0_RLC0_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA0_RLC0_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC0_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA0_RLC0_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA0_RLC0_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC0_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC0_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA0_RLC0_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA0_RLC0_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA0_RLC0_STATUS_DEFAULT                                              0x00000000
#define mmSDMA0_RLC0_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA0_RLC0_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA0_RLC0_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA0_RLC0_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA0_RLC0_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA0_RLC0_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA0_RLC0_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA0_RLC0_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA0_RLC0_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA0_RLC0_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA0_RLC0_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA0_RLC0_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA0_RLC0_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA0_RLC0_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA0_RLC0_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA0_RLC0_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA0_RLC0_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA0_RLC0_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA0_RLC0_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA0_RLC0_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA0_RLC0_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA0_RLC0_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA0_RLC0_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA0_RLC0_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA0_RLC1_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA0_RLC1_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC1_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC1_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC1_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC1_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC1_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC1_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA0_RLC1_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA0_RLC1_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA0_RLC1_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA0_RLC1_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC1_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA0_RLC1_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA0_RLC1_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC1_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC1_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA0_RLC1_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA0_RLC1_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA0_RLC1_STATUS_DEFAULT                                              0x00000000
#define mmSDMA0_RLC1_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA0_RLC1_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA0_RLC1_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA0_RLC1_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA0_RLC1_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA0_RLC1_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA0_RLC1_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA0_RLC1_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA0_RLC1_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA0_RLC1_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA0_RLC1_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA0_RLC1_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA0_RLC1_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA0_RLC1_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA0_RLC1_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA0_RLC1_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA0_RLC1_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA0_RLC1_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA0_RLC1_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA0_RLC1_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA0_RLC1_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA0_RLC1_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA0_RLC1_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA0_RLC1_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA0_RLC2_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA0_RLC2_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC2_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC2_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC2_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC2_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC2_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC2_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA0_RLC2_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA0_RLC2_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA0_RLC2_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA0_RLC2_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC2_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA0_RLC2_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA0_RLC2_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC2_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC2_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA0_RLC2_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA0_RLC2_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA0_RLC2_STATUS_DEFAULT                                              0x00000000
#define mmSDMA0_RLC2_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA0_RLC2_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA0_RLC2_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA0_RLC2_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA0_RLC2_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA0_RLC2_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA0_RLC2_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA0_RLC2_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA0_RLC2_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA0_RLC2_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA0_RLC2_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA0_RLC2_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA0_RLC2_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA0_RLC2_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA0_RLC2_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA0_RLC2_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA0_RLC2_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA0_RLC2_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA0_RLC2_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA0_RLC2_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA0_RLC2_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA0_RLC2_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA0_RLC2_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA0_RLC2_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA0_RLC3_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA0_RLC3_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC3_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC3_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC3_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC3_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC3_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC3_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA0_RLC3_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA0_RLC3_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA0_RLC3_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA0_RLC3_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC3_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA0_RLC3_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA0_RLC3_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC3_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC3_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA0_RLC3_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA0_RLC3_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA0_RLC3_STATUS_DEFAULT                                              0x00000000
#define mmSDMA0_RLC3_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA0_RLC3_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA0_RLC3_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA0_RLC3_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA0_RLC3_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA0_RLC3_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA0_RLC3_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA0_RLC3_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA0_RLC3_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA0_RLC3_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA0_RLC3_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA0_RLC3_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA0_RLC3_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA0_RLC3_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA0_RLC3_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA0_RLC3_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA0_RLC3_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA0_RLC3_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA0_RLC3_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA0_RLC3_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA0_RLC3_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA0_RLC3_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA0_RLC3_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA0_RLC3_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA0_RLC4_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA0_RLC4_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC4_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC4_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC4_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC4_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC4_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC4_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA0_RLC4_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA0_RLC4_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA0_RLC4_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA0_RLC4_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC4_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA0_RLC4_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA0_RLC4_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC4_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC4_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA0_RLC4_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA0_RLC4_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA0_RLC4_STATUS_DEFAULT                                              0x00000000
#define mmSDMA0_RLC4_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA0_RLC4_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA0_RLC4_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA0_RLC4_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA0_RLC4_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA0_RLC4_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA0_RLC4_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA0_RLC4_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA0_RLC4_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA0_RLC4_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA0_RLC4_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA0_RLC4_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA0_RLC4_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA0_RLC4_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA0_RLC4_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA0_RLC4_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA0_RLC4_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA0_RLC4_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA0_RLC4_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA0_RLC4_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA0_RLC4_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA0_RLC4_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA0_RLC4_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA0_RLC4_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA0_RLC5_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA0_RLC5_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC5_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC5_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC5_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC5_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC5_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC5_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA0_RLC5_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA0_RLC5_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA0_RLC5_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA0_RLC5_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC5_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA0_RLC5_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA0_RLC5_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC5_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC5_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA0_RLC5_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA0_RLC5_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA0_RLC5_STATUS_DEFAULT                                              0x00000000
#define mmSDMA0_RLC5_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA0_RLC5_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA0_RLC5_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA0_RLC5_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA0_RLC5_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA0_RLC5_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA0_RLC5_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA0_RLC5_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA0_RLC5_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA0_RLC5_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA0_RLC5_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA0_RLC5_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA0_RLC5_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA0_RLC5_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA0_RLC5_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA0_RLC5_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA0_RLC5_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA0_RLC5_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA0_RLC5_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA0_RLC5_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA0_RLC5_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA0_RLC5_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA0_RLC5_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA0_RLC5_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA0_RLC6_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA0_RLC6_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC6_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC6_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC6_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC6_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC6_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC6_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA0_RLC6_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA0_RLC6_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA0_RLC6_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA0_RLC6_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC6_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA0_RLC6_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA0_RLC6_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC6_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC6_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA0_RLC6_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA0_RLC6_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA0_RLC6_STATUS_DEFAULT                                              0x00000000
#define mmSDMA0_RLC6_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA0_RLC6_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA0_RLC6_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA0_RLC6_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA0_RLC6_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA0_RLC6_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA0_RLC6_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA0_RLC6_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA0_RLC6_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA0_RLC6_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA0_RLC6_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA0_RLC6_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA0_RLC6_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA0_RLC6_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA0_RLC6_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA0_RLC6_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA0_RLC6_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA0_RLC6_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA0_RLC6_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA0_RLC6_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA0_RLC6_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA0_RLC6_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA0_RLC6_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA0_RLC6_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA0_RLC7_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA0_RLC7_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC7_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC7_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC7_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC7_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC7_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC7_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA0_RLC7_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA0_RLC7_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA0_RLC7_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA0_RLC7_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA0_RLC7_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA0_RLC7_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA0_RLC7_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA0_RLC7_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA0_RLC7_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA0_RLC7_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA0_RLC7_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA0_RLC7_STATUS_DEFAULT                                              0x00000000
#define mmSDMA0_RLC7_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA0_RLC7_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA0_RLC7_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA0_RLC7_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA0_RLC7_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA0_RLC7_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA0_RLC7_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA0_RLC7_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA0_RLC7_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA0_RLC7_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA0_RLC7_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA0_RLC7_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA0_RLC7_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA0_RLC7_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA0_RLC7_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA0_RLC7_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA0_RLC7_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA0_RLC7_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA0_RLC7_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA0_RLC7_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA0_RLC7_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA0_RLC7_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA0_RLC7_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA0_RLC7_MIDCMD_CNTL_DEFAULT                                         0x00000000


// addressBlock: gc_sdma1_sdma1dec
#define mmSDMA1_DEC_START_DEFAULT                                                0x00000000
#define mmSDMA1_GLOBAL_TIMESTAMP_LO_DEFAULT                                      0x00000000
#define mmSDMA1_GLOBAL_TIMESTAMP_HI_DEFAULT                                      0x00000000
#define mmSDMA1_PG_CNTL_DEFAULT                                                  0x00000000
#define mmSDMA1_PG_CTX_LO_DEFAULT                                                0x00000000
#define mmSDMA1_PG_CTX_HI_DEFAULT                                                0x00000000
#define mmSDMA1_PG_CTX_CNTL_DEFAULT                                              0x00000000
#define mmSDMA1_POWER_CNTL_DEFAULT                                               0x40000050
#define mmSDMA1_CLK_CTRL_DEFAULT                                                 0x00000100
#define mmSDMA1_CNTL_DEFAULT                                                     0x000000c2
#define mmSDMA1_CHICKEN_BITS_DEFAULT                                             0x03ef0107
#define mmSDMA1_GB_ADDR_CONFIG_DEFAULT                                           0x00000444
#define mmSDMA1_GB_ADDR_CONFIG_READ_DEFAULT                                      0x00000444
#define mmSDMA1_RB_RPTR_FETCH_HI_DEFAULT                                         0x00000000
#define mmSDMA1_SEM_WAIT_FAIL_TIMER_CNTL_DEFAULT                                 0x00000000
#define mmSDMA1_RB_RPTR_FETCH_DEFAULT                                            0x00000000
#define mmSDMA1_IB_OFFSET_FETCH_DEFAULT                                          0x00000000
#define mmSDMA1_PROGRAM_DEFAULT                                                  0x00000000
#define mmSDMA1_STATUS_REG_DEFAULT                                               0x46dee557
#define mmSDMA1_STATUS1_REG_DEFAULT                                              0x000003ff
#define mmSDMA1_RD_BURST_CNTL_DEFAULT                                            0x00000002
#define mmSDMA1_HBM_PAGE_CONFIG_DEFAULT                                          0x00000000
#define mmSDMA1_UCODE_CHECKSUM_DEFAULT                                           0x00000000
#define mmSDMA1_F32_CNTL_DEFAULT                                                 0x00000001
#define mmSDMA1_FREEZE_DEFAULT                                                   0x00000000
#define mmSDMA1_PHASE0_QUANTUM_DEFAULT                                           0x00010002
#define mmSDMA1_PHASE1_QUANTUM_DEFAULT                                           0x00010002
#define mmSDMA1_EDC_CONFIG_DEFAULT                                               0x00000002
#define mmSDMA1_BA_THRESHOLD_DEFAULT                                             0x03ff03ff
#define mmSDMA1_ID_DEFAULT                                                       0x00000001
#define mmSDMA1_VERSION_DEFAULT                                                  0x00000500
#define mmSDMA1_EDC_COUNTER_DEFAULT                                              0x00000000
#define mmSDMA1_EDC_COUNTER_CLEAR_DEFAULT                                        0x00000000
#define mmSDMA1_STATUS2_REG_DEFAULT                                              0x00000001
#define mmSDMA1_ATOMIC_CNTL_DEFAULT                                              0x00000200
#define mmSDMA1_ATOMIC_PREOP_LO_DEFAULT                                          0x00000000
#define mmSDMA1_ATOMIC_PREOP_HI_DEFAULT                                          0x00000000
#define mmSDMA1_UTCL1_CNTL_DEFAULT                                               0xd0000191
#define mmSDMA1_UTCL1_WATERMK_DEFAULT                                            0xfffbd9fb
#define mmSDMA1_UTCL1_RD_STATUS_DEFAULT                                          0x01011555
#define mmSDMA1_UTCL1_WR_STATUS_DEFAULT                                          0x51011555
#define mmSDMA1_UTCL1_INV0_DEFAULT                                               0x00000800
#define mmSDMA1_UTCL1_INV1_DEFAULT                                               0x00000000
#define mmSDMA1_UTCL1_INV2_DEFAULT                                               0x00000000
#define mmSDMA1_UTCL1_RD_XNACK0_DEFAULT                                          0x00000000
#define mmSDMA1_UTCL1_RD_XNACK1_DEFAULT                                          0x00000000
#define mmSDMA1_UTCL1_WR_XNACK0_DEFAULT                                          0x00000000
#define mmSDMA1_UTCL1_WR_XNACK1_DEFAULT                                          0x00000000
#define mmSDMA1_UTCL1_TIMEOUT_DEFAULT                                            0x00000000
#define mmSDMA1_UTCL1_PAGE_DEFAULT                                               0x010cec00
#define mmSDMA1_RELAX_ORDERING_LUT_DEFAULT                                       0xc0000006
#define mmSDMA1_CHICKEN_BITS_2_DEFAULT                                           0x00100007
#define mmSDMA1_STATUS3_REG_DEFAULT                                              0x03f00000
#define mmSDMA1_PHYSICAL_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA1_PHYSICAL_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA1_PHASE2_QUANTUM_DEFAULT                                           0x00010002
#define mmSDMA1_ERROR_LOG_DEFAULT                                                0x0000000f
#define mmSDMA1_PUB_DUMMY_REG0_DEFAULT                                           0x00000000
#define mmSDMA1_PUB_DUMMY_REG1_DEFAULT                                           0x00000000
#define mmSDMA1_PUB_DUMMY_REG2_DEFAULT                                           0x00000000
#define mmSDMA1_PUB_DUMMY_REG3_DEFAULT                                           0x00000000
#define mmSDMA1_F32_COUNTER_DEFAULT                                              0x00000000
#define mmSDMA1_CRD_CNTL_DEFAULT                                                 0x1850c640
#define mmSDMA1_AQL_STATUS_DEFAULT                                               0x00000003
#define mmSDMA1_EA_DBIT_ADDR_DATA_DEFAULT                                        0x00000000
#define mmSDMA1_EA_DBIT_ADDR_INDEX_DEFAULT                                       0x00000000
#define mmSDMA1_TLBI_GCR_CNTL_DEFAULT                                            0x40180454
#define mmSDMA1_TILING_CONFIG_DEFAULT                                            0x00000000
#define mmSDMA1_INT_STATUS_DEFAULT                                               0x00000000
#define mmSDMA1_HOLE_ADDR_LO_DEFAULT                                             0x00000000
#define mmSDMA1_HOLE_ADDR_HI_DEFAULT                                             0x00000000
#define mmSDMA1_CLOCK_GATING_REG_DEFAULT                                         0x00000000
#define mmSDMA1_STATUS4_REG_DEFAULT                                              0x00000001
#define mmSDMA1_SCRATCH_RAM_DATA_DEFAULT                                         0x00000000
#define mmSDMA1_SCRATCH_RAM_ADDR_DEFAULT                                         0x00000000
#define mmSDMA1_TIMESTAMP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA1_STATUS5_REG_DEFAULT                                              0x00000000
#define mmSDMA1_QUEUE_RESET_REQ_DEFAULT                                          0x00000000
#define mmSDMA1_GFX_RB_CNTL_DEFAULT                                              0x80840000
#define mmSDMA1_GFX_RB_BASE_DEFAULT                                              0x00000000
#define mmSDMA1_GFX_RB_BASE_HI_DEFAULT                                           0x00000000
#define mmSDMA1_GFX_RB_RPTR_DEFAULT                                              0x00000000
#define mmSDMA1_GFX_RB_RPTR_HI_DEFAULT                                           0x00000000
#define mmSDMA1_GFX_RB_WPTR_DEFAULT                                              0x00000000
#define mmSDMA1_GFX_RB_WPTR_HI_DEFAULT                                           0x00000000
#define mmSDMA1_GFX_RB_WPTR_POLL_CNTL_DEFAULT                                    0x00403000
#define mmSDMA1_GFX_RB_RPTR_ADDR_HI_DEFAULT                                      0x00000000
#define mmSDMA1_GFX_RB_RPTR_ADDR_LO_DEFAULT                                      0x00000000
#define mmSDMA1_GFX_IB_CNTL_DEFAULT                                              0x00000100
#define mmSDMA1_GFX_IB_RPTR_DEFAULT                                              0x00000000
#define mmSDMA1_GFX_IB_OFFSET_DEFAULT                                            0x00000000
#define mmSDMA1_GFX_IB_BASE_LO_DEFAULT                                           0x00000000
#define mmSDMA1_GFX_IB_BASE_HI_DEFAULT                                           0x00000000
#define mmSDMA1_GFX_IB_SIZE_DEFAULT                                              0x00000000
#define mmSDMA1_GFX_SKIP_CNTL_DEFAULT                                            0x00000000
#define mmSDMA1_GFX_CONTEXT_STATUS_DEFAULT                                       0x00000005
#define mmSDMA1_GFX_DOORBELL_DEFAULT                                             0x00000000
#define mmSDMA1_GFX_CONTEXT_CNTL_DEFAULT                                         0x00000000
#define mmSDMA1_GFX_STATUS_DEFAULT                                               0x00000000
#define mmSDMA1_GFX_DOORBELL_LOG_DEFAULT                                         0x00000000
#define mmSDMA1_GFX_WATERMARK_DEFAULT                                            0x00000000
#define mmSDMA1_GFX_DOORBELL_OFFSET_DEFAULT                                      0x00000000
#define mmSDMA1_GFX_CSA_ADDR_LO_DEFAULT                                          0x00000000
#define mmSDMA1_GFX_CSA_ADDR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_GFX_IB_SUB_REMAIN_DEFAULT                                        0x00000000
#define mmSDMA1_GFX_PREEMPT_DEFAULT                                              0x00000000
#define mmSDMA1_GFX_DUMMY_REG_DEFAULT                                            0x0000000f
#define mmSDMA1_GFX_RB_WPTR_POLL_ADDR_HI_DEFAULT                                 0x00000000
#define mmSDMA1_GFX_RB_WPTR_POLL_ADDR_LO_DEFAULT                                 0x00000000
#define mmSDMA1_GFX_RB_AQL_CNTL_DEFAULT                                          0x00004000
#define mmSDMA1_GFX_MINOR_PTR_UPDATE_DEFAULT                                     0x00000000
#define mmSDMA1_GFX_MIDCMD_DATA0_DEFAULT                                         0x00000000
#define mmSDMA1_GFX_MIDCMD_DATA1_DEFAULT                                         0x00000000
#define mmSDMA1_GFX_MIDCMD_DATA2_DEFAULT                                         0x00000000
#define mmSDMA1_GFX_MIDCMD_DATA3_DEFAULT                                         0x00000000
#define mmSDMA1_GFX_MIDCMD_DATA4_DEFAULT                                         0x00000000
#define mmSDMA1_GFX_MIDCMD_DATA5_DEFAULT                                         0x00000000
#define mmSDMA1_GFX_MIDCMD_DATA6_DEFAULT                                         0x00000000
#define mmSDMA1_GFX_MIDCMD_DATA7_DEFAULT                                         0x00000000
#define mmSDMA1_GFX_MIDCMD_DATA8_DEFAULT                                         0x00000000
#define mmSDMA1_GFX_MIDCMD_DATA9_DEFAULT                                         0x00000000
#define mmSDMA1_GFX_MIDCMD_DATA10_DEFAULT                                        0x00000000
#define mmSDMA1_GFX_MIDCMD_CNTL_DEFAULT                                          0x00000000
#define mmSDMA1_PAGE_RB_CNTL_DEFAULT                                             0x80840000
#define mmSDMA1_PAGE_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA1_PAGE_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_PAGE_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_PAGE_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_PAGE_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA1_PAGE_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_PAGE_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA1_PAGE_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA1_PAGE_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA1_PAGE_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA1_PAGE_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_PAGE_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA1_PAGE_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA1_PAGE_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_PAGE_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA1_PAGE_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA1_PAGE_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA1_PAGE_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA1_PAGE_STATUS_DEFAULT                                              0x00000000
#define mmSDMA1_PAGE_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA1_PAGE_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA1_PAGE_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA1_PAGE_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA1_PAGE_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA1_PAGE_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA1_PAGE_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA1_PAGE_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA1_PAGE_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA1_PAGE_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA1_PAGE_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA1_PAGE_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA1_PAGE_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA1_PAGE_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA1_PAGE_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA1_PAGE_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA1_PAGE_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA1_PAGE_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA1_PAGE_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA1_PAGE_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA1_PAGE_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA1_PAGE_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA1_PAGE_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA1_PAGE_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA1_RLC0_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA1_RLC0_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC0_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC0_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC0_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC0_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC0_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC0_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA1_RLC0_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA1_RLC0_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA1_RLC0_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA1_RLC0_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC0_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA1_RLC0_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA1_RLC0_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC0_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC0_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA1_RLC0_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA1_RLC0_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA1_RLC0_STATUS_DEFAULT                                              0x00000000
#define mmSDMA1_RLC0_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA1_RLC0_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA1_RLC0_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA1_RLC0_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA1_RLC0_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA1_RLC0_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA1_RLC0_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA1_RLC0_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA1_RLC0_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA1_RLC0_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA1_RLC0_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA1_RLC0_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA1_RLC0_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA1_RLC0_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA1_RLC0_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA1_RLC0_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA1_RLC0_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA1_RLC0_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA1_RLC0_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA1_RLC0_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA1_RLC0_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA1_RLC0_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA1_RLC0_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA1_RLC0_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA1_RLC1_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA1_RLC1_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC1_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC1_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC1_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC1_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC1_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC1_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA1_RLC1_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA1_RLC1_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA1_RLC1_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA1_RLC1_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC1_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA1_RLC1_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA1_RLC1_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC1_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC1_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA1_RLC1_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA1_RLC1_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA1_RLC1_STATUS_DEFAULT                                              0x00000000
#define mmSDMA1_RLC1_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA1_RLC1_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA1_RLC1_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA1_RLC1_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA1_RLC1_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA1_RLC1_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA1_RLC1_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA1_RLC1_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA1_RLC1_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA1_RLC1_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA1_RLC1_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA1_RLC1_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA1_RLC1_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA1_RLC1_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA1_RLC1_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA1_RLC1_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA1_RLC1_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA1_RLC1_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA1_RLC1_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA1_RLC1_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA1_RLC1_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA1_RLC1_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA1_RLC1_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA1_RLC1_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA1_RLC2_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA1_RLC2_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC2_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC2_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC2_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC2_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC2_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC2_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA1_RLC2_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA1_RLC2_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA1_RLC2_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA1_RLC2_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC2_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA1_RLC2_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA1_RLC2_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC2_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC2_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA1_RLC2_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA1_RLC2_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA1_RLC2_STATUS_DEFAULT                                              0x00000000
#define mmSDMA1_RLC2_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA1_RLC2_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA1_RLC2_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA1_RLC2_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA1_RLC2_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA1_RLC2_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA1_RLC2_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA1_RLC2_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA1_RLC2_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA1_RLC2_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA1_RLC2_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA1_RLC2_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA1_RLC2_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA1_RLC2_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA1_RLC2_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA1_RLC2_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA1_RLC2_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA1_RLC2_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA1_RLC2_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA1_RLC2_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA1_RLC2_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA1_RLC2_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA1_RLC2_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA1_RLC2_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA1_RLC3_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA1_RLC3_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC3_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC3_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC3_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC3_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC3_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC3_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA1_RLC3_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA1_RLC3_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA1_RLC3_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA1_RLC3_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC3_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA1_RLC3_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA1_RLC3_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC3_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC3_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA1_RLC3_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA1_RLC3_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA1_RLC3_STATUS_DEFAULT                                              0x00000000
#define mmSDMA1_RLC3_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA1_RLC3_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA1_RLC3_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA1_RLC3_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA1_RLC3_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA1_RLC3_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA1_RLC3_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA1_RLC3_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA1_RLC3_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA1_RLC3_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA1_RLC3_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA1_RLC3_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA1_RLC3_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA1_RLC3_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA1_RLC3_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA1_RLC3_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA1_RLC3_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA1_RLC3_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA1_RLC3_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA1_RLC3_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA1_RLC3_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA1_RLC3_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA1_RLC3_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA1_RLC3_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA1_RLC4_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA1_RLC4_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC4_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC4_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC4_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC4_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC4_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC4_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA1_RLC4_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA1_RLC4_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA1_RLC4_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA1_RLC4_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC4_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA1_RLC4_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA1_RLC4_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC4_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC4_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA1_RLC4_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA1_RLC4_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA1_RLC4_STATUS_DEFAULT                                              0x00000000
#define mmSDMA1_RLC4_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA1_RLC4_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA1_RLC4_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA1_RLC4_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA1_RLC4_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA1_RLC4_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA1_RLC4_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA1_RLC4_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA1_RLC4_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA1_RLC4_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA1_RLC4_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA1_RLC4_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA1_RLC4_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA1_RLC4_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA1_RLC4_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA1_RLC4_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA1_RLC4_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA1_RLC4_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA1_RLC4_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA1_RLC4_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA1_RLC4_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA1_RLC4_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA1_RLC4_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA1_RLC4_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA1_RLC5_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA1_RLC5_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC5_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC5_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC5_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC5_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC5_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC5_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA1_RLC5_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA1_RLC5_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA1_RLC5_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA1_RLC5_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC5_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA1_RLC5_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA1_RLC5_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC5_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC5_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA1_RLC5_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA1_RLC5_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA1_RLC5_STATUS_DEFAULT                                              0x00000000
#define mmSDMA1_RLC5_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA1_RLC5_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA1_RLC5_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA1_RLC5_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA1_RLC5_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA1_RLC5_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA1_RLC5_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA1_RLC5_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA1_RLC5_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA1_RLC5_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA1_RLC5_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA1_RLC5_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA1_RLC5_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA1_RLC5_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA1_RLC5_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA1_RLC5_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA1_RLC5_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA1_RLC5_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA1_RLC5_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA1_RLC5_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA1_RLC5_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA1_RLC5_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA1_RLC5_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA1_RLC5_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA1_RLC6_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA1_RLC6_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC6_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC6_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC6_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC6_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC6_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC6_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA1_RLC6_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA1_RLC6_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA1_RLC6_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA1_RLC6_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC6_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA1_RLC6_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA1_RLC6_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC6_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC6_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA1_RLC6_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA1_RLC6_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA1_RLC6_STATUS_DEFAULT                                              0x00000000
#define mmSDMA1_RLC6_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA1_RLC6_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA1_RLC6_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA1_RLC6_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA1_RLC6_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA1_RLC6_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA1_RLC6_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA1_RLC6_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA1_RLC6_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA1_RLC6_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA1_RLC6_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA1_RLC6_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA1_RLC6_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA1_RLC6_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA1_RLC6_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA1_RLC6_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA1_RLC6_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA1_RLC6_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA1_RLC6_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA1_RLC6_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA1_RLC6_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA1_RLC6_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA1_RLC6_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA1_RLC6_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA1_RLC7_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA1_RLC7_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC7_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC7_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC7_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC7_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC7_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC7_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA1_RLC7_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA1_RLC7_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA1_RLC7_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA1_RLC7_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA1_RLC7_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA1_RLC7_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA1_RLC7_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA1_RLC7_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA1_RLC7_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA1_RLC7_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA1_RLC7_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA1_RLC7_STATUS_DEFAULT                                              0x00000000
#define mmSDMA1_RLC7_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA1_RLC7_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA1_RLC7_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA1_RLC7_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA1_RLC7_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA1_RLC7_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA1_RLC7_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA1_RLC7_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA1_RLC7_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA1_RLC7_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA1_RLC7_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA1_RLC7_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA1_RLC7_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA1_RLC7_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA1_RLC7_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA1_RLC7_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA1_RLC7_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA1_RLC7_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA1_RLC7_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA1_RLC7_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA1_RLC7_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA1_RLC7_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA1_RLC7_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA1_RLC7_MIDCMD_CNTL_DEFAULT                                         0x00000000


// addressBlock: gc_grbmdec
#define mmGRBM_CNTL_DEFAULT                                                      0x00000018
#define mmGRBM_SKEW_CNTL_DEFAULT                                                 0x00000020
#define mmGRBM_STATUS2_DEFAULT                                                   0x00000000
#define mmGRBM_PWR_CNTL_DEFAULT                                                  0x00000000
#define mmGRBM_STATUS_DEFAULT                                                    0x00000000
#define mmGRBM_STATUS_SE0_DEFAULT                                                0x00000000
#define mmGRBM_STATUS_SE1_DEFAULT                                                0x00000000
#define mmGRBM_STATUS3_DEFAULT                                                   0x00000000
#define mmGRBM_SOFT_RESET_DEFAULT                                                0x00000000
#define mmGRBM_GFX_CLKEN_CNTL_DEFAULT                                            0x00001008
#define mmGRBM_WAIT_IDLE_CLOCKS_DEFAULT                                          0x00000030
#define mmGRBM_STATUS_SE2_DEFAULT                                                0x00000000
#define mmGRBM_STATUS_SE3_DEFAULT                                                0x00000000
#define mmGRBM_READ_ERROR_DEFAULT                                                0x00000000
#define mmGRBM_READ_ERROR2_DEFAULT                                               0x00000000
#define mmGRBM_INT_CNTL_DEFAULT                                                  0x00000000
#define mmGRBM_TRAP_OP_DEFAULT                                                   0x00000000
#define mmGRBM_TRAP_ADDR_DEFAULT                                                 0x00000000
#define mmGRBM_TRAP_ADDR_MSK_DEFAULT                                             0x0003ffff
#define mmGRBM_TRAP_WD_DEFAULT                                                   0x00000000
#define mmGRBM_TRAP_WD_MSK_DEFAULT                                               0xffffffff
#define mmGRBM_DSM_BYPASS_DEFAULT                                                0x00000000
#define mmGRBM_WRITE_ERROR_DEFAULT                                               0x00000000
#define mmGRBM_CHIP_REVISION_DEFAULT                                             0x00000000
#define mmGRBM_GFX_CNTL_DEFAULT                                                  0x00000000
#define mmGRBM_IH_CREDIT_DEFAULT                                                 0x00010000
#define mmGRBM_PWR_CNTL2_DEFAULT                                                 0x00010000
#define mmGRBM_UTCL2_INVAL_RANGE_START_DEFAULT                                   0x0000286d
#define mmGRBM_UTCL2_INVAL_RANGE_END_DEFAULT                                     0x000028c6
#define mmGRBM_FENCE_RANGE0_DEFAULT                                              0x00000000
#define mmGRBM_FENCE_RANGE1_DEFAULT                                              0x00000000
#define mmGRBM_NOWHERE_DEFAULT                                                   0x00000000
#define mmGRBM_SCRATCH_REG0_DEFAULT                                              0x00000000
#define mmGRBM_SCRATCH_REG1_DEFAULT                                              0x00000000
#define mmGRBM_SCRATCH_REG2_DEFAULT                                              0x00000000
#define mmGRBM_SCRATCH_REG3_DEFAULT                                              0x00000000
#define mmGRBM_SCRATCH_REG4_DEFAULT                                              0x00000000
#define mmGRBM_SCRATCH_REG5_DEFAULT                                              0x00000000
#define mmGRBM_SCRATCH_REG6_DEFAULT                                              0x00000000
#define mmGRBM_SCRATCH_REG7_DEFAULT                                              0x00000000
#define mmVIOLATION_DATA_ASYNC_VF_PROG_DEFAULT                                   0x00000000


// addressBlock: gc_cpdec
#define mmCP_CPC_STATUS_DEFAULT                                                  0x00000000
#define mmCP_CPC_BUSY_STAT_DEFAULT                                               0x00000000
#define mmCP_CPC_STALLED_STAT1_DEFAULT                                           0x00000000
#define mmCP_CPF_STATUS_DEFAULT                                                  0x00000000
#define mmCP_CPF_BUSY_STAT_DEFAULT                                               0x00000000
#define mmCP_CPF_STALLED_STAT1_DEFAULT                                           0x00000000
#define mmCP_CPC_BUSY_STAT2_DEFAULT                                              0x00000000
#define mmCP_CPC_GRBM_FREE_COUNT_DEFAULT                                         0x00000008
#define mmCP_CPC_PRIV_VIOLATION_ADDR_DEFAULT                                     0x00000000
#define mmCP_MEC_ME1_HEADER_DUMP_DEFAULT                                         0x00000000
#define mmCP_MEC_ME2_HEADER_DUMP_DEFAULT                                         0x00000000
#define mmCP_CPC_SCRATCH_INDEX_DEFAULT                                           0x00000000
#define mmCP_CPC_SCRATCH_DATA_DEFAULT                                            0x00000000
#define mmCP_CPF_GRBM_FREE_COUNT_DEFAULT                                         0x00000002
#define mmCP_CPF_BUSY_STAT2_DEFAULT                                              0x00000000
#define mmCONFIG_RESERVED_REG0_DEFAULT                                           0x00000000
#define mmCONFIG_RESERVED_REG1_DEFAULT                                           0x00000000
#define mmCP_CPC_HALT_HYST_COUNT_DEFAULT                                         0x00000002
#define mmCP_CE_COMPARE_COUNT_DEFAULT                                            0x00000000
#define mmCP_CE_DE_COUNT_DEFAULT                                                 0x00000000
#define mmCP_DE_CE_COUNT_DEFAULT                                                 0x00000000
#define mmCP_DE_LAST_INVAL_COUNT_DEFAULT                                         0x00000000
#define mmCP_DE_DE_COUNT_DEFAULT                                                 0x00000000
#define mmCP_STALLED_STAT3_DEFAULT                                               0x00000000
#define mmCP_STALLED_STAT1_DEFAULT                                               0x00000000
#define mmCP_STALLED_STAT2_DEFAULT                                               0x00000000
#define mmCP_BUSY_STAT_DEFAULT                                                   0x00000000
#define mmCP_STAT_DEFAULT                                                        0x00000000
#define mmCP_ME_HEADER_DUMP_DEFAULT                                              0x00000000
#define mmCP_PFP_HEADER_DUMP_DEFAULT                                             0x00000000
#define mmCP_GRBM_FREE_COUNT_DEFAULT                                             0x000c0c0c
#define mmCP_CE_HEADER_DUMP_DEFAULT                                              0x00000000
#define mmCP_PFP_INSTR_PNTR_DEFAULT                                              0x00000000
#define mmCP_ME_INSTR_PNTR_DEFAULT                                               0x00000000
#define mmCP_CE_INSTR_PNTR_DEFAULT                                               0x00000000
#define mmCP_MEC1_INSTR_PNTR_DEFAULT                                             0x00000000
#define mmCP_MEC2_INSTR_PNTR_DEFAULT                                             0x00000000
#define mmCP_CSF_STAT_DEFAULT                                                    0x00000000
#define mmCP_MEC_CNTL_DEFAULT                                                    0x50000000
#define mmCP_ME_CNTL_DEFAULT                                                     0x15000000
#define mmCP_CNTX_STAT_DEFAULT                                                   0x00000000
#define mmCP_ME_PREEMPTION_DEFAULT                                               0x00000000
#define mmCP_ROQ_THRESHOLDS_DEFAULT                                              0x00003010
#define mmCP_MEQ_STQ_THRESHOLD_DEFAULT                                           0x00000010
#define mmCP_RB2_RPTR_DEFAULT                                                    0x00000000
#define mmCP_RB1_RPTR_DEFAULT                                                    0x00000000
#define mmCP_RB0_RPTR_DEFAULT                                                    0x00000000
#define mmCP_RB_RPTR_DEFAULT                                                     0x00000000
#define mmCP_RB_WPTR_DELAY_DEFAULT                                               0x00000000
#define mmCP_RB_WPTR_POLL_CNTL_DEFAULT                                           0x00400100
#define mmCP_ROQ1_THRESHOLDS_DEFAULT                                             0x06008010
#define mmCP_ROQ2_THRESHOLDS_DEFAULT                                             0x000380a0
#define mmCP_STQ_THRESHOLDS_DEFAULT                                              0x00804000
#define mmCP_QUEUE_THRESHOLDS_DEFAULT                                            0x00002b16
#define mmCP_MEQ_THRESHOLDS_DEFAULT                                              0x00008040
#define mmCP_ROQ_AVAIL_DEFAULT                                                   0x00000000
#define mmCP_STQ_AVAIL_DEFAULT                                                   0x00000000
#define mmCP_ROQ2_AVAIL_DEFAULT                                                  0x00000000
#define mmCP_MEQ_AVAIL_DEFAULT                                                   0x00000000
#define mmCP_CMD_INDEX_DEFAULT                                                   0x00000000
#define mmCP_CMD_DATA_DEFAULT                                                    0x00000000
#define mmCP_ROQ_RB_STAT_DEFAULT                                                 0x00000000
#define mmCP_ROQ_IB1_STAT_DEFAULT                                                0x00000000
#define mmCP_ROQ_IB2_STAT_DEFAULT                                                0x00000000
#define mmCP_STQ_STAT_DEFAULT                                                    0x00000000
#define mmCP_STQ_WR_STAT_DEFAULT                                                 0x00000000
#define mmCP_MEQ_STAT_DEFAULT                                                    0x00000000
#define mmCP_CEQ1_AVAIL_DEFAULT                                                  0x00000000
#define mmCP_CEQ2_AVAIL_DEFAULT                                                  0x00000000
#define mmCP_CE_ROQ_RB_STAT_DEFAULT                                              0x00000000
#define mmCP_CE_ROQ_IB1_STAT_DEFAULT                                             0x00000000
#define mmCP_CE_ROQ_IB2_STAT_DEFAULT                                             0x00000000
#define mmCP_CE_ROQ_DB_STAT_DEFAULT                                              0x00000000
#define mmCP_ROQ3_THRESHOLDS_DEFAULT                                             0x0004c120
#define mmCP_ROQ_DB_STAT_DEFAULT                                                 0x00000000
#define mmCP_PRIV_VIOLATION_ADDR_DEFAULT                                         0x00000000


// addressBlock: gc_padec
#define mmVGT_CACHE_INVALIDATION_DEFAULT                                         0x09000000
#define mmVGT_ESGS_RING_SIZE_DEFAULT                                             0x00000000
#define mmVGT_GSVS_RING_SIZE_DEFAULT                                             0x00000000
#define mmVGT_TF_RING_SIZE_DEFAULT                                               0x0000c000
#define mmVGT_HS_OFFCHIP_PARAM_DEFAULT                                           0x00000000
#define mmVGT_TF_MEMORY_BASE_DEFAULT                                             0x00000000
#define mmVGT_TF_MEMORY_BASE_HI_DEFAULT                                          0x00000000
#define mmVGT_VTX_VECT_EJECT_REG_DEFAULT                                         0x0000007d
#define mmVGT_DMA_DATA_FIFO_DEPTH_DEFAULT                                        0x00000200
#define mmVGT_DMA_REQ_FIFO_DEPTH_DEFAULT                                         0x00000020
#define mmVGT_DRAW_INIT_FIFO_DEPTH_DEFAULT                                       0x00000020
#define mmVGT_LAST_COPY_STATE_DEFAULT                                            0x00000000
#define mmVGT_FIFO_DEPTHS_DEFAULT                                                0x10180040
#define mmVGT_GS_VERTEX_REUSE_DEFAULT                                            0x00000010
#define mmVGT_MC_LAT_CNTL_DEFAULT                                                0x00000002
#define mmIA_UTCL1_STATUS_2_DEFAULT                                              0x00000000
#define mmWD_CNTL_STATUS_DEFAULT                                                 0x00000000
#define mmCC_GC_PRIM_CONFIG_DEFAULT                                              0x000ffaa0
#define mmGC_USER_PRIM_CONFIG_DEFAULT                                            0x000ffaa0
#define mmWD_QOS_DEFAULT                                                         0x00000000
#define mmWD_UTCL1_CNTL_DEFAULT                                                  0x00000080
#define mmWD_UTCL1_STATUS_DEFAULT                                                0x00000000
#define mmGE_PC_CNTL_DEFAULT                                                     0x00000400
#define mmIA_UTCL1_CNTL_DEFAULT                                                  0x00000080
#define mmIA_UTCL1_STATUS_DEFAULT                                                0x00000000
#define mmCC_GC_SA_UNIT_DISABLE_DEFAULT                                          0x00000000
#define mmGC_USER_SA_UNIT_DISABLE_DEFAULT                                        0x00000000
#define mmVGT_SYS_CONFIG_DEFAULT                                                 0x00000011
#define mmGE_PRIV_CONTROL_DEFAULT                                                0x000000fe
#define mmGE_STATUS_DEFAULT                                                      0x00000000
#define mmVGT_VS_MAX_WAVE_ID_DEFAULT                                             0x000002ff
#define mmVGT_GS_MAX_WAVE_ID_DEFAULT                                             0x000007ff
#define mmCC_GC_SHADER_ARRAY_CONFIG_GEN1_DEFAULT                                 0x00000000
#define mmCC_GC_SHADER_ARRAY_CONFIG_GEN0_DEFAULT                                 0x00000000
#define mmGFX_PIPE_CONTROL_DEFAULT                                               0x00000000
#define mmCC_GC_SHADER_ARRAY_CONFIG_DEFAULT                                      0xffe00000
#define mmGC_USER_SHADER_ARRAY_CONFIG_DEFAULT                                    0x00000000
#define mmVGT_DMA_PRIMITIVE_TYPE_DEFAULT                                         0x00000000
#define mmVGT_DMA_CONTROL_DEFAULT                                                0x00000000
#define mmVGT_DMA_LS_HS_CONFIG_DEFAULT                                           0x00000000
#define mmVGT_STRMOUT_DELAY_DEFAULT                                              0x00092400
#define mmWD_BUF_RESOURCE_1_DEFAULT                                              0x00000000
#define mmWD_BUF_RESOURCE_2_DEFAULT                                              0x00000000
#define mmPA_CL_CNTL_STATUS_DEFAULT                                              0x00000000
#define mmPA_CL_ENHANCE_DEFAULT                                                  0x00200007
#define mmPA_SU_CNTL_STATUS_DEFAULT                                              0x00000000
#define mmPA_SC_FIFO_DEPTH_CNTL_DEFAULT                                          0x00000100
#define mmPA_SC_P3D_TRAP_SCREEN_HV_LOCK_DEFAULT                                  0x00000000
#define mmPA_SC_HP3D_TRAP_SCREEN_HV_LOCK_DEFAULT                                 0x00000000
#define mmPA_SC_TRAP_SCREEN_HV_LOCK_DEFAULT                                      0x00000000
#define mmPA_SC_FORCE_EOV_MAX_CNTS_DEFAULT                                       0x00ffffff
#define mmPA_SC_BINNER_EVENT_CNTL_0_DEFAULT                                      0x842a4c02
#define mmPA_SC_BINNER_EVENT_CNTL_1_DEFAULT                                      0x82000008
#define mmPA_SC_BINNER_EVENT_CNTL_2_DEFAULT                                      0x9118aab8
#define mmPA_SC_BINNER_EVENT_CNTL_3_DEFAULT                                      0xc2400024
#define mmPA_SC_BINNER_TIMEOUT_COUNTER_DEFAULT                                   0x00000000
#define mmPA_SC_BINNER_PERF_CNTL_0_DEFAULT                                       0x00000000
#define mmPA_SC_BINNER_PERF_CNTL_1_DEFAULT                                       0x00000000
#define mmPA_SC_BINNER_PERF_CNTL_2_DEFAULT                                       0x00000000
#define mmPA_SC_BINNER_PERF_CNTL_3_DEFAULT                                       0x00000000
#define mmPA_SC_ENHANCE_2_DEFAULT                                                0x00000020
#define mmPA_SC_ENHANCE_INTERNAL_DEFAULT                                         0x00000000
#define mmPA_SC_BINNER_CNTL_OVERRIDE_DEFAULT                                     0x08000000
#define mmPA_SC_PBB_OVERRIDE_FLAG_DEFAULT                                        0x00000000
#define mmPA_PH_INTERFACE_FIFO_SIZE_DEFAULT                                      0x00000100
#define mmPA_PH_ENHANCE_DEFAULT                                                  0x00001000
#define mmPA_SC_BC_WAVE_BREAK_DEFAULT                                            0x00360040
#define mmPA_SC_ENHANCE_3_DEFAULT                                                0x00000000
#define mmPA_SC_FIFO_SIZE_DEFAULT                                                0x00000000
#define mmPA_SC_IF_FIFO_SIZE_DEFAULT                                             0x00000000
#define mmPA_SC_PKR_WAVE_TABLE_CNTL_DEFAULT                                      0x00000000
#define mmPA_SIDEBAND_REQUEST_DELAYS_DEFAULT                                     0x08000020
#define mmPA_SC_ENHANCE_DEFAULT                                                  0x08000009
#define mmPA_SC_ENHANCE_1_DEFAULT                                                0x040c2000
#define mmPA_SC_DSM_CNTL_DEFAULT                                                 0x00000000
#define mmPA_SC_TILE_STEERING_CREST_OVERRIDE_DEFAULT                             0x00000000


// addressBlock: gc_sqdec
#define mmSQ_CONFIG_DEFAULT                                                      0x00180020
#define mmSQC_CONFIG_DEFAULT                                                     0x000a2000
#define mmLDS_CONFIG_DEFAULT                                                     0x00000000
#define mmSQ_RANDOM_WAVE_PRI_DEFAULT                                             0x0000007f
#define mmSQG_STATUS_DEFAULT                                                     0x00000000
#define mmSQ_FIFO_SIZES_DEFAULT                                                  0x0000d001
#define mmSQ_DSM_CNTL_DEFAULT                                                    0x00000000
#define mmSQ_DSM_CNTL2_DEFAULT                                                   0x00000000
#define mmSQ_RUNTIME_CONFIG_DEFAULT                                              0x00000000
#define mmSH_MEM_BASES_DEFAULT                                                   0x00000000
#define mmSP_CONFIG_DEFAULT                                                      0x00000020
#define mmSQ_ARB_CONFIG_DEFAULT                                                  0x00000030
#define mmSH_MEM_CONFIG_DEFAULT                                                  0x0000c000
#define mmSQ_SHADER_TBA_LO_DEFAULT                                               0x00000000
#define mmSQ_SHADER_TBA_HI_DEFAULT                                               0x00000000
#define mmSQ_SHADER_TMA_LO_DEFAULT                                               0x00000000
#define mmSQ_SHADER_TMA_HI_DEFAULT                                               0x00000000
#define mmSQG_UTCL0_CNTL1_DEFAULT                                                0x00000580
#define mmSQG_UTCL0_CNTL2_DEFAULT                                                0x00000000
#define mmSQG_UTCL0_STATUS_DEFAULT                                               0x00000000
#define mmSQG_CONFIG_DEFAULT                                                     0x00000000
#define mmCC_GC_SHADER_RATE_CONFIG_DEFAULT                                       0x00000000
#define mmGC_USER_SHADER_RATE_CONFIG_DEFAULT                                     0x00000000
#define mmSQ_INTERRUPT_AUTO_MASK_DEFAULT                                         0x00ffffff
#define mmSQ_INTERRUPT_MSG_CTRL_DEFAULT                                          0x00000000
#define mmSQ_WATCH0_ADDR_H_DEFAULT                                               0x00000000
#define mmSQ_WATCH0_ADDR_L_DEFAULT                                               0x00000000
#define mmSQ_WATCH0_CNTL_DEFAULT                                                 0x00000000
#define mmSQ_WATCH1_ADDR_H_DEFAULT                                               0x00000000
#define mmSQ_WATCH1_ADDR_L_DEFAULT                                               0x00000000
#define mmSQ_WATCH1_CNTL_DEFAULT                                                 0x00000000
#define mmSQ_WATCH2_ADDR_H_DEFAULT                                               0x00000000
#define mmSQ_WATCH2_ADDR_L_DEFAULT                                               0x00000000
#define mmSQ_WATCH2_CNTL_DEFAULT                                                 0x00000000
#define mmSQ_WATCH3_ADDR_H_DEFAULT                                               0x00000000
#define mmSQ_WATCH3_ADDR_L_DEFAULT                                               0x00000000
#define mmSQ_WATCH3_CNTL_DEFAULT                                                 0x00000000
#define mmSQ_THREAD_TRACE_BUF0_BASE_DEFAULT                                      0x00000000
#define mmSQ_THREAD_TRACE_BUF0_SIZE_DEFAULT                                      0x00000000
#define mmSQ_THREAD_TRACE_BUF1_BASE_DEFAULT                                      0x00000000
#define mmSQ_THREAD_TRACE_BUF1_SIZE_DEFAULT                                      0x00000000
#define mmSQ_THREAD_TRACE_WPTR_DEFAULT                                           0x00000000
#define mmSQ_THREAD_TRACE_MASK_DEFAULT                                           0x00000000
#define mmSQ_THREAD_TRACE_TOKEN_MASK_DEFAULT                                     0x00000000
#define mmSQ_THREAD_TRACE_CTRL_DEFAULT                                           0x00400000
#define mmSQ_THREAD_TRACE_STATUS_DEFAULT                                         0x00000000
#define mmSQ_THREAD_TRACE_DROPPED_CNTR_DEFAULT                                   0x00000000
#define mmSQ_THREAD_TRACE_GFX_DRAW_CNTR_DEFAULT                                  0x00000000
#define mmSQ_THREAD_TRACE_GFX_MARKER_CNTR_DEFAULT                                0x00000000
#define mmSQ_THREAD_TRACE_HP3D_DRAW_CNTR_DEFAULT                                 0x00000000
#define mmSQ_THREAD_TRACE_HP3D_MARKER_CNTR_DEFAULT                               0x00000000
#define mmSQ_THREAD_TRACE_STATUS2_DEFAULT                                        0x00000000
#define mmSQ_IND_INDEX_DEFAULT                                                   0x00000000
#define mmSQ_IND_DATA_DEFAULT                                                    0x00000000
#define mmSQ_CMD_DEFAULT                                                         0x00000000
#define mmSQ_TIME_HI_DEFAULT                                                     0x00000000
#define mmSQ_TIME_LO_DEFAULT                                                     0x00000000
#define mmSQ_LB_CTR_CTRL_DEFAULT                                                 0x00000000
#define mmSQ_LB_DATA0_DEFAULT                                                    0x00000000
#define mmSQ_LB_DATA1_DEFAULT                                                    0x00000000
#define mmSQ_LB_DATA2_DEFAULT                                                    0x00000000
#define mmSQ_LB_DATA3_DEFAULT                                                    0x00000000
#define mmSQ_LB_CTR_SEL0_DEFAULT                                                 0x00000000
#define mmSQ_LB_CTR_SEL1_DEFAULT                                                 0x00000000
#define mmSQ_EDC_CNT_DEFAULT                                                     0x00000000
#define mmSQ_EDC_FUE_CNTL_DEFAULT                                                0x00000000
#define mmSQ_WREXEC_EXEC_HI_DEFAULT                                              0x00000000
#define mmSQ_WREXEC_EXEC_LO_DEFAULT                                              0x00000000
#define mmSQC_ICACHE_UTCL0_CNTL1_DEFAULT                                         0x00000480
#define mmSQC_ICACHE_UTCL0_CNTL2_DEFAULT                                         0x00000000
#define mmSQC_DCACHE_UTCL0_CNTL1_DEFAULT                                         0x00000500
#define mmSQC_DCACHE_UTCL0_CNTL2_DEFAULT                                         0x00000000
#define mmSQC_ICACHE_UTCL0_STATUS_DEFAULT                                        0x00000000
#define mmSQC_DCACHE_UTCL0_STATUS_DEFAULT                                        0x00000000


// addressBlock: gc_shsdec
#define mmSX_DEBUG_1_DEFAULT                                                     0x00000020
#define mmSPI_PS_MAX_WAVE_ID_DEFAULT                                             0x020000cf
#define mmSPI_START_PHASE_DEFAULT                                                0x000000a0
#define mmSPI_GFX_CNTL_DEFAULT                                                   0x00000000
#define mmSPI_DSM_CNTL_DEFAULT                                                   0x00000000
#define mmSPI_DSM_CNTL2_DEFAULT                                                  0x00000000
#define mmSPI_EDC_CNT_DEFAULT                                                    0x00000000
#define mmSPI_USER_ACCUM_VMID_CNTL_DEFAULT                                       0x00000000
#define mmSPI_CONFIG_CNTL_DEFAULT                                                0xc062c688
#define mmSPI_WAVE_LIMIT_CNTL_DEFAULT                                            0x00000000
#define mmSPI_CONFIG_CNTL_2_DEFAULT                                              0x00000011
#define mmSPI_CONFIG_CNTL_1_DEFAULT                                              0x000c0104
#define mmSPI_CONFIG_PS_CU_EN_DEFAULT                                            0x00000000
#define mmSPI_WF_LIFETIME_CNTL_DEFAULT                                           0x00000000
#define mmSPI_WF_LIFETIME_LIMIT_0_DEFAULT                                        0x00000100
#define mmSPI_WF_LIFETIME_LIMIT_1_DEFAULT                                        0x00000100
#define mmSPI_WF_LIFETIME_LIMIT_2_DEFAULT                                        0x00000100
#define mmSPI_WF_LIFETIME_LIMIT_3_DEFAULT                                        0x00000100
#define mmSPI_WF_LIFETIME_LIMIT_4_DEFAULT                                        0x00000100
#define mmSPI_WF_LIFETIME_LIMIT_5_DEFAULT                                        0x00000100
#define mmSPI_WF_LIFETIME_STATUS_0_DEFAULT                                       0x00000000
#define mmSPI_WF_LIFETIME_STATUS_1_DEFAULT                                       0x00000000
#define mmSPI_WF_LIFETIME_STATUS_2_DEFAULT                                       0x00000000
#define mmSPI_WF_LIFETIME_STATUS_4_DEFAULT                                       0x00000000
#define mmSPI_WF_LIFETIME_STATUS_6_DEFAULT                                       0x00000000
#define mmSPI_WF_LIFETIME_STATUS_7_DEFAULT                                       0x00000000
#define mmSPI_WF_LIFETIME_STATUS_8_DEFAULT                                       0x00000000
#define mmSPI_WF_LIFETIME_STATUS_9_DEFAULT                                       0x00000000
#define mmSPI_WF_LIFETIME_STATUS_11_DEFAULT                                      0x00000000
#define mmSPI_WF_LIFETIME_STATUS_13_DEFAULT                                      0x00000000
#define mmSPI_WF_LIFETIME_STATUS_14_DEFAULT                                      0x00000000
#define mmSPI_WF_LIFETIME_STATUS_15_DEFAULT                                      0x00000000
#define mmSPI_WF_LIFETIME_STATUS_16_DEFAULT                                      0x00000000
#define mmSPI_WF_LIFETIME_STATUS_17_DEFAULT                                      0x00000000
#define mmSPI_WF_LIFETIME_STATUS_18_DEFAULT                                      0x00000000
#define mmSPI_WF_LIFETIME_STATUS_19_DEFAULT                                      0x00000000
#define mmSPI_WF_LIFETIME_STATUS_20_DEFAULT                                      0x00000000
#define mmSPI_WF_LIFETIME_STATUS_21_DEFAULT                                      0x00000000
#define mmSPI_LB_CTR_CTRL_DEFAULT                                                0x00000000
#define mmSPI_LB_WGP_MASK_DEFAULT                                                0x0000ffff
#define mmSPI_LB_DATA_REG_DEFAULT                                                0x00000000
#define mmSPI_PG_ENABLE_STATIC_WGP_MASK_DEFAULT                                  0x0000ffff
#define mmSPI_GDS_CREDITS_DEFAULT                                                0x0000403c
#define mmSPI_SX_EXPORT_BUFFER_SIZES_DEFAULT                                     0x10000800
#define mmSPI_SX_SCOREBOARD_BUFFER_SIZES_DEFAULT                                 0x00800040
#define mmSPI_CSQ_WF_ACTIVE_STATUS_DEFAULT                                       0x00000000
#define mmSPI_CSQ_WF_ACTIVE_COUNT_0_DEFAULT                                      0x00000000
#define mmSPI_CSQ_WF_ACTIVE_COUNT_1_DEFAULT                                      0x00000000
#define mmSPI_CSQ_WF_ACTIVE_COUNT_2_DEFAULT                                      0x00000000
#define mmSPI_CSQ_WF_ACTIVE_COUNT_3_DEFAULT                                      0x00000000
#define mmSPI_LB_DATA_WAVES_DEFAULT                                              0x00000000
#define mmSPI_LB_DATA_PERWGP_WAVE_HSGS_DEFAULT                                   0x00000000
#define mmSPI_LB_DATA_PERWGP_WAVE_VSPS_DEFAULT                                   0x00000000
#define mmSPI_LB_DATA_PERWGP_WAVE_CS_DEFAULT                                     0x00000000
#define mmSPI_P0_TRAP_SCREEN_PSBA_LO_DEFAULT                                     0x00000000
#define mmSPI_P0_TRAP_SCREEN_PSBA_HI_DEFAULT                                     0x00000000
#define mmSPI_P0_TRAP_SCREEN_PSMA_LO_DEFAULT                                     0x00000000
#define mmSPI_P0_TRAP_SCREEN_PSMA_HI_DEFAULT                                     0x00000000
#define mmSPI_P0_TRAP_SCREEN_GPR_MIN_DEFAULT                                     0x00000000
#define mmSPI_P1_TRAP_SCREEN_PSBA_LO_DEFAULT                                     0x00000000
#define mmSPI_P1_TRAP_SCREEN_PSBA_HI_DEFAULT                                     0x00000000
#define mmSPI_P1_TRAP_SCREEN_PSMA_LO_DEFAULT                                     0x00000000
#define mmSPI_P1_TRAP_SCREEN_PSMA_HI_DEFAULT                                     0x00000000
#define mmSPI_P1_TRAP_SCREEN_GPR_MIN_DEFAULT                                     0x00000000


// addressBlock: gc_tpdec
#define mmTD_STATUS_DEFAULT                                                      0x00000000
#define mmTD_DSM_CNTL_DEFAULT                                                    0x00000000
#define mmTD_DSM_CNTL2_DEFAULT                                                   0x00000000
#define mmTD_SCRATCH_DEFAULT                                                     0x00000000
#define mmTA_CNTL_DEFAULT                                                        0xc0040000
#define mmTA_RESERVED_010C_DEFAULT                                               0x00000000
#define mmTA_STATUS_DEFAULT                                                      0x00000000
#define mmTA_SCRATCH_DEFAULT                                                     0x00000000


// addressBlock: gc_gdsdec
#define mmGDS_CONFIG_DEFAULT                                                     0x00000000
#define mmGDS_CNTL_STATUS_DEFAULT                                                0x00000000
#define mmGDS_ENHANCE_DEFAULT                                                    0x00000000
#define mmGDS_PROTECTION_FAULT_DEFAULT                                           0x00000000
#define mmGDS_VM_PROTECTION_FAULT_DEFAULT                                        0x00000000
#define mmGDS_EDC_CNT_DEFAULT                                                    0x00000000
#define mmGDS_EDC_GRBM_CNT_DEFAULT                                               0x00000000
#define mmGDS_EDC_OA_DED_DEFAULT                                                 0x00000000
#define mmGDS_DSM_CNTL_DEFAULT                                                   0x00000000
#define mmGDS_EDC_OA_PHY_CNT_DEFAULT                                             0x00000000
#define mmGDS_EDC_OA_PIPE_CNT_DEFAULT                                            0x00000000
#define mmGDS_DSM_CNTL2_DEFAULT                                                  0x00000000
#define mmGDS_WD_GDS_CSB_DEFAULT                                                 0x00000000


// addressBlock: gc_rbdec
#define mmDB_DEBUG_DEFAULT                                                       0x00000000
#define mmDB_DEBUG2_DEFAULT                                                      0x00000420
#define mmDB_DEBUG3_DEFAULT                                                      0x00000000
#define mmDB_DEBUG4_DEFAULT                                                      0x04000000
#define mmDB_ETILE_STUTTER_CONTROL_DEFAULT                                       0x00000000
#define mmDB_LTILE_STUTTER_CONTROL_DEFAULT                                       0x00000000
#define mmDB_EQUAD_STUTTER_CONTROL_DEFAULT                                       0x00000000
#define mmDB_LQUAD_STUTTER_CONTROL_DEFAULT                                       0x00000000
#define mmDB_CREDIT_LIMIT_DEFAULT                                                0x00000000
#define mmDB_WATERMARKS_DEFAULT                                                  0x0a040a04
#define mmDB_SUBTILE_CONTROL_DEFAULT                                             0x00000000
#define mmDB_FREE_CACHELINES_DEFAULT                                             0x00000000
#define mmDB_FIFO_DEPTH1_DEFAULT                                                 0x00000000
#define mmDB_FIFO_DEPTH2_DEFAULT                                                 0x00000000
#define mmDB_LAST_OF_BURST_CONFIG_DEFAULT                                        0x01c28210
#define mmDB_RING_CONTROL_DEFAULT                                                0x00000001
#define mmDB_MEM_ARB_WATERMARKS_DEFAULT                                          0x04040404
#define mmDB_FIFO_DEPTH3_DEFAULT                                                 0x00000000
#define mmDB_RMI_BC_GL2_CACHE_CONTROL_DEFAULT                                    0x00150055
#define mmDB_EXCEPTION_CONTROL_DEFAULT                                           0x00f80000
#define mmDB_DFSM_CONFIG_DEFAULT                                                 0x00000002
#define mmDB_DEBUG5_DEFAULT                                                      0x00000060
#define mmDB_DFSM_TILES_IN_FLIGHT_DEFAULT                                        0x000003e8
#define mmDB_DFSM_PRIMS_IN_FLIGHT_DEFAULT                                        0x000000c8
#define mmDB_DFSM_WATCHDOG_DEFAULT                                               0x000f4240
#define mmDB_DFSM_FLUSH_ENABLE_DEFAULT                                           0x000007ff
#define mmDB_DFSM_FLUSH_AUX_EVENT_DEFAULT                                        0x00000000
#define mmDB_FGCG_SRAMS_CLK_CTRL_DEFAULT                                         0x00000000
#define mmDB_FGCG_INTERFACES_CLK_CTRL_DEFAULT                                    0x00000000
#define mmCC_RB_REDUNDANCY_DEFAULT                                               0x00000000
#define mmCC_RB_BACKEND_DISABLE_DEFAULT                                          0x00000000
#define mmGB_ADDR_CONFIG_DEFAULT                                                 0x00000444
#define mmGB_BACKEND_MAP_DEFAULT                                                 0x00000000
#define mmGB_GPU_ID_DEFAULT                                                      0x00000000
#define mmCC_RB_DAISY_CHAIN_DEFAULT                                              0x76543210
#define mmGB_ADDR_CONFIG_READ_DEFAULT                                            0x00000444
#define mmCB_HW_CONTROL_4_DEFAULT                                                0x003c0014
#define mmCB_HW_CONTROL_3_DEFAULT                                                0x00000000
#define mmCB_HW_CONTROL_DEFAULT                                                  0x00000100
#define mmCB_HW_CONTROL_1_DEFAULT                                                0x00000000
#define mmCB_HW_CONTROL_2_DEFAULT                                                0x30000000
#define mmCB_DCC_CONFIG_DEFAULT                                                  0x00000000
#define mmCB_HW_MEM_ARBITER_RD_DEFAULT                                           0x00029000
#define mmCB_HW_MEM_ARBITER_WR_DEFAULT                                           0x00029000
#define mmCB_RMI_BC_GL2_CACHE_CONTROL_DEFAULT                                    0x00550055
#define mmCB_STUTTER_CONTROL_CMASK_RDLAT_DEFAULT                                 0x00000000
#define mmCB_STUTTER_CONTROL_FMASK_RDLAT_DEFAULT                                 0x00000000
#define mmCB_STUTTER_CONTROL_COLOR_RDLAT_DEFAULT                                 0x00000000
#define mmCB_CACHE_EVICT_POINTS_DEFAULT                                          0x0e101410
#define mmGC_USER_RB_REDUNDANCY_DEFAULT                                          0x00000000
#define mmGC_USER_RB_BACKEND_DISABLE_DEFAULT                                     0x00000000


// addressBlock: gc_gceadec2
#define mmGCEA_MISC_DEFAULT                                                      0x0de8bff0
#define mmGCEA_LATENCY_SAMPLING_DEFAULT                                          0x00000000
#define mmGCEA_DSM_CNTL_DEFAULT                                                  0x00000000
#define mmGCEA_DSM_CNTLA_DEFAULT                                                 0x00000000
#define mmGCEA_DSM_CNTLB_DEFAULT                                                 0x00000000
#define mmGCEA_DSM_CNTL2_DEFAULT                                                 0x00000000
#define mmGCEA_DSM_CNTL2A_DEFAULT                                                0x00000000
#define mmGCEA_DSM_CNTL2B_DEFAULT                                                0x00000000
#define mmGCEA_GL2C_XBR_CREDITS_DEFAULT                                          0x637f637f
#define mmGCEA_GL2C_XBR_MAXBURST_DEFAULT                                         0x00333333
#define mmGCEA_PROBE_CNTL_DEFAULT                                                0x00000000
#define mmGCEA_PROBE_MAP_DEFAULT                                                 0x0000aaaa
#define mmGCEA_ERR_STATUS_DEFAULT                                                0x00000300
#define mmGCEA_MISC2_DEFAULT                                                     0x00000000


// addressBlock: gc_spipdec2
#define mmSPI_PQEV_CTRL_DEFAULT                                                  0x00ff1008
#define mmSPI_EXP_THROTTLE_CTRL_DEFAULT                                          0x08782e2e


// addressBlock: gc_gceadec3
#define mmGCEA_RRET_MEM_RESERVE_DEFAULT                                          0x00000000


// addressBlock: gc_rmi_rmidec
#define mmRMI_GENERAL_CNTL_DEFAULT                                               0x01e00000
#define mmRMI_GENERAL_CNTL1_DEFAULT                                              0x00003201
#define mmRMI_GENERAL_STATUS_DEFAULT                                             0x00000000
#define mmRMI_SUBBLOCK_STATUS0_DEFAULT                                           0x00000000
#define mmRMI_SUBBLOCK_STATUS1_DEFAULT                                           0x00000000
#define mmRMI_SUBBLOCK_STATUS2_DEFAULT                                           0x00000000
#define mmRMI_SUBBLOCK_STATUS3_DEFAULT                                           0x00000000
#define mmRMI_XBAR_CONFIG_DEFAULT                                                0x00000f00
#define mmRMI_PROBE_POP_LOGIC_CNTL_DEFAULT                                       0x0003fcff
#define mmRMI_UTC_XNACK_N_MISC_CNTL_DEFAULT                                      0x00000564
#define mmRMI_DEMUX_CNTL_DEFAULT                                                 0x02000200
#define mmRMI_UTCL1_CNTL1_DEFAULT                                                0x00020000
#define mmRMI_UTCL1_CNTL2_DEFAULT                                                0x00010000
#define mmRMI_UTC_UNIT_CONFIG_DEFAULT                                            0x00000000
#define mmRMI_TCIW_FORMATTER0_CNTL_DEFAULT                                       0x4404001e
#define mmRMI_TCIW_FORMATTER1_CNTL_DEFAULT                                       0x4404001e
#define mmRMI_SCOREBOARD_CNTL_DEFAULT                                            0x001ffe00
#define mmRMI_SCOREBOARD_STATUS0_DEFAULT                                         0x00000000
#define mmRMI_SCOREBOARD_STATUS1_DEFAULT                                         0x00000000
#define mmRMI_SCOREBOARD_STATUS2_DEFAULT                                         0x00000000
#define mmRMI_XBAR_ARBITER_CONFIG_DEFAULT                                        0x08000800
#define mmRMI_XBAR_ARBITER_CONFIG_1_DEFAULT                                      0x0000ffff
#define mmRMI_CLOCK_CNTRL_DEFAULT                                                0x00008822
#define mmRMI_UTCL1_STATUS_DEFAULT                                               0x00000000
#define mmRMI_RB_GLX_CID_MAP_DEFAULT                                             0xbcaa9987
#define mmRMI_SPARE_DEFAULT                                                      0xffff310d
#define mmRMI_SPARE_1_DEFAULT                                                    0x00000a00
#define mmRMI_SPARE_2_DEFAULT                                                    0x00000000
#define mmCC_RMI_REDUNDANCY_DEFAULT                                              0x00000010
#define mmGC_USER_RMI_REDUNDANCY_DEFAULT                                         0x00000010


// addressBlock: gc_dbgu_gfx_dbgudec


// addressBlock: gc_pmmdec
#define mmGCR_GENERAL_CNTL_DEFAULT                                               0x00000400
#define mmGCR_CMD_STATUS_DEFAULT                                                 0x00000000
#define mmGCR_SPARE_DEFAULT                                                      0x00000000
#define mmPMM_GENERAL_CNTL_DEFAULT                                               0x00000000
#define mmGCR_PIO_CNTL_DEFAULT                                                   0x00000000
#define mmGCR_PIO_DATA_DEFAULT                                                   0x00000000


// addressBlock: gc_utcl1dec
#define mmUTCL1_CTRL_DEFAULT                                                     0x00000000
#define mmUTCL1_ALOG_DEFAULT                                                     0x00186482
#define mmUTCL1_UTCL0_INVREQ_DISABLE_DEFAULT                                     0x00000000
#define mmGCRD_SA_TARGETS_DISABLE_DEFAULT                                        0x00000000
#define mmUTCL1_STATUS_DEFAULT                                                   0x00000000


// addressBlock: gc_gcvml2pfdec
#define mmGCVM_L2_CNTL_DEFAULT                                                   0x00080602
#define mmGCVM_L2_CNTL2_DEFAULT                                                  0x00000000
#define mmGCVM_L2_CNTL3_DEFAULT                                                  0x80100007
#define mmGCVM_L2_STATUS_DEFAULT                                                 0x00000000
#define mmGCVM_DUMMY_PAGE_FAULT_CNTL_DEFAULT                                     0x00000090
#define mmGCVM_DUMMY_PAGE_FAULT_ADDR_LO32_DEFAULT                                0x00000000
#define mmGCVM_DUMMY_PAGE_FAULT_ADDR_HI32_DEFAULT                                0x00000000
#define mmGCVM_INVALIDATE_CNTL_DEFAULT                                           0x0000010f
#define mmGCVM_L2_PROTECTION_FAULT_CNTL_DEFAULT                                  0x3ffffffc
#define mmGCVM_L2_PROTECTION_FAULT_CNTL2_DEFAULT                                 0x000a0000
#define mmGCVM_L2_PROTECTION_FAULT_MM_CNTL3_DEFAULT                              0xffffffff
#define mmGCVM_L2_PROTECTION_FAULT_MM_CNTL4_DEFAULT                              0xffffffff
#define mmGCVM_L2_PROTECTION_FAULT_STATUS_DEFAULT                                0x00000000
#define mmGCVM_L2_PROTECTION_FAULT_ADDR_LO32_DEFAULT                             0x00000000
#define mmGCVM_L2_PROTECTION_FAULT_ADDR_HI32_DEFAULT                             0x00000000
#define mmGCVM_L2_PROTECTION_FAULT_DEFAULT_ADDR_LO32_DEFAULT                     0x00000000
#define mmGCVM_L2_PROTECTION_FAULT_DEFAULT_ADDR_HI32_DEFAULT                     0x00000000
#define mmGCVM_L2_CONTEXT1_IDENTITY_APERTURE_LOW_ADDR_LO32_DEFAULT               0x00000000
#define mmGCVM_L2_CONTEXT1_IDENTITY_APERTURE_LOW_ADDR_HI32_DEFAULT               0x00000000
#define mmGCVM_L2_CONTEXT1_IDENTITY_APERTURE_HIGH_ADDR_LO32_DEFAULT              0x00000000
#define mmGCVM_L2_CONTEXT1_IDENTITY_APERTURE_HIGH_ADDR_HI32_DEFAULT              0x00000000
#define mmGCVM_L2_CONTEXT_IDENTITY_PHYSICAL_OFFSET_LO32_DEFAULT                  0x00000000
#define mmGCVM_L2_CONTEXT_IDENTITY_PHYSICAL_OFFSET_HI32_DEFAULT                  0x00000000
#define mmGCVM_L2_CNTL4_DEFAULT                                                  0x000000c1
#define mmGCVM_L2_MM_GROUP_RT_CLASSES_DEFAULT                                    0x00000000
#define mmGCVM_L2_BANK_SELECT_RESERVED_CID_DEFAULT                               0x00000000
#define mmGCVM_L2_BANK_SELECT_RESERVED_CID2_DEFAULT                              0x00000000
#define mmGCVM_L2_CACHE_PARITY_CNTL_DEFAULT                                      0x00000000
#define mmGCVM_L2_CNTL5_DEFAULT                                                  0x00003fe0
#define mmGCVM_L2_GCR_CNTL_DEFAULT                                               0x00000000
#define mmGCVML2_WALKER_MACRO_THROTTLE_TIME_DEFAULT                              0x00000000
#define mmGCVML2_WALKER_MACRO_THROTTLE_FETCH_LIMIT_DEFAULT                       0x00000000
#define mmGCVML2_WALKER_MICRO_THROTTLE_TIME_DEFAULT                              0x00000000
#define mmGCVML2_WALKER_MICRO_THROTTLE_FETCH_LIMIT_DEFAULT                       0x00000000
#define mmGCVM_L2_PTE_CACHE_DUMP_CNTL_DEFAULT                                    0x00000000
#define mmGCVM_L2_PTE_CACHE_DUMP_READ_DEFAULT                                    0x00000000


// addressBlock: gc_gcvml2vcdec
#define mmGCVM_CONTEXT0_CNTL_DEFAULT                                             0x007ffe80
#define mmGCVM_CONTEXT1_CNTL_DEFAULT                                             0x007ffe80
#define mmGCVM_CONTEXT2_CNTL_DEFAULT                                             0x007ffe80
#define mmGCVM_CONTEXT3_CNTL_DEFAULT                                             0x007ffe80
#define mmGCVM_CONTEXT4_CNTL_DEFAULT                                             0x007ffe80
#define mmGCVM_CONTEXT5_CNTL_DEFAULT                                             0x007ffe80
#define mmGCVM_CONTEXT6_CNTL_DEFAULT                                             0x007ffe80
#define mmGCVM_CONTEXT7_CNTL_DEFAULT                                             0x007ffe80
#define mmGCVM_CONTEXT8_CNTL_DEFAULT                                             0x007ffe80
#define mmGCVM_CONTEXT9_CNTL_DEFAULT                                             0x007ffe80
#define mmGCVM_CONTEXT10_CNTL_DEFAULT                                            0x007ffe80
#define mmGCVM_CONTEXT11_CNTL_DEFAULT                                            0x007ffe80
#define mmGCVM_CONTEXT12_CNTL_DEFAULT                                            0x007ffe80
#define mmGCVM_CONTEXT13_CNTL_DEFAULT                                            0x007ffe80
#define mmGCVM_CONTEXT14_CNTL_DEFAULT                                            0x007ffe80
#define mmGCVM_CONTEXT15_CNTL_DEFAULT                                            0x007ffe80
#define mmGCVM_CONTEXTS_DISABLE_DEFAULT                                          0x00000000
#define mmGCVM_INVALIDATE_ENG0_SEM_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG1_SEM_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG2_SEM_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG3_SEM_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG4_SEM_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG5_SEM_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG6_SEM_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG7_SEM_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG8_SEM_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG9_SEM_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG10_SEM_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG11_SEM_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG12_SEM_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG13_SEM_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG14_SEM_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG15_SEM_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG16_SEM_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG17_SEM_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG0_REQ_DEFAULT                                       0x02f80000
#define mmGCVM_INVALIDATE_ENG1_REQ_DEFAULT                                       0x02f80000
#define mmGCVM_INVALIDATE_ENG2_REQ_DEFAULT                                       0x02f80000
#define mmGCVM_INVALIDATE_ENG3_REQ_DEFAULT                                       0x02f80000
#define mmGCVM_INVALIDATE_ENG4_REQ_DEFAULT                                       0x02f80000
#define mmGCVM_INVALIDATE_ENG5_REQ_DEFAULT                                       0x02f80000
#define mmGCVM_INVALIDATE_ENG6_REQ_DEFAULT                                       0x02f80000
#define mmGCVM_INVALIDATE_ENG7_REQ_DEFAULT                                       0x02f80000
#define mmGCVM_INVALIDATE_ENG8_REQ_DEFAULT                                       0x02f80000
#define mmGCVM_INVALIDATE_ENG9_REQ_DEFAULT                                       0x02f80000
#define mmGCVM_INVALIDATE_ENG10_REQ_DEFAULT                                      0x02f80000
#define mmGCVM_INVALIDATE_ENG11_REQ_DEFAULT                                      0x02f80000
#define mmGCVM_INVALIDATE_ENG12_REQ_DEFAULT                                      0x02f80000
#define mmGCVM_INVALIDATE_ENG13_REQ_DEFAULT                                      0x02f80000
#define mmGCVM_INVALIDATE_ENG14_REQ_DEFAULT                                      0x02f80000
#define mmGCVM_INVALIDATE_ENG15_REQ_DEFAULT                                      0x02f80000
#define mmGCVM_INVALIDATE_ENG16_REQ_DEFAULT                                      0x02f80000
#define mmGCVM_INVALIDATE_ENG17_REQ_DEFAULT                                      0x02f80000
#define mmGCVM_INVALIDATE_ENG0_ACK_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG1_ACK_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG2_ACK_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG3_ACK_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG4_ACK_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG5_ACK_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG6_ACK_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG7_ACK_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG8_ACK_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG9_ACK_DEFAULT                                       0x00000000
#define mmGCVM_INVALIDATE_ENG10_ACK_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG11_ACK_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG12_ACK_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG13_ACK_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG14_ACK_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG15_ACK_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG16_ACK_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG17_ACK_DEFAULT                                      0x00000000
#define mmGCVM_INVALIDATE_ENG0_ADDR_RANGE_LO32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG0_ADDR_RANGE_HI32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG1_ADDR_RANGE_LO32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG1_ADDR_RANGE_HI32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG2_ADDR_RANGE_LO32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG2_ADDR_RANGE_HI32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG3_ADDR_RANGE_LO32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG3_ADDR_RANGE_HI32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG4_ADDR_RANGE_LO32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG4_ADDR_RANGE_HI32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG5_ADDR_RANGE_LO32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG5_ADDR_RANGE_HI32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG6_ADDR_RANGE_LO32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG6_ADDR_RANGE_HI32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG7_ADDR_RANGE_LO32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG7_ADDR_RANGE_HI32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG8_ADDR_RANGE_LO32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG8_ADDR_RANGE_HI32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG9_ADDR_RANGE_LO32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG9_ADDR_RANGE_HI32_DEFAULT                           0x00000000
#define mmGCVM_INVALIDATE_ENG10_ADDR_RANGE_LO32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG10_ADDR_RANGE_HI32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG11_ADDR_RANGE_LO32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG11_ADDR_RANGE_HI32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG12_ADDR_RANGE_LO32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG12_ADDR_RANGE_HI32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG13_ADDR_RANGE_LO32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG13_ADDR_RANGE_HI32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG14_ADDR_RANGE_LO32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG14_ADDR_RANGE_HI32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG15_ADDR_RANGE_LO32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG15_ADDR_RANGE_HI32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG16_ADDR_RANGE_LO32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG16_ADDR_RANGE_HI32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG17_ADDR_RANGE_LO32_DEFAULT                          0x00000000
#define mmGCVM_INVALIDATE_ENG17_ADDR_RANGE_HI32_DEFAULT                          0x00000000
#define mmGCVM_CONTEXT0_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT0_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT1_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT1_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT2_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT2_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT3_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT3_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT4_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT4_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT5_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT5_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT6_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT6_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT7_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT7_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT8_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT8_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT9_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT9_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT10_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT10_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT11_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT11_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT12_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT12_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT13_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT13_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT14_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT14_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT15_PAGE_TABLE_BASE_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT15_PAGE_TABLE_BASE_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT0_PAGE_TABLE_START_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT0_PAGE_TABLE_START_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT1_PAGE_TABLE_START_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT1_PAGE_TABLE_START_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT2_PAGE_TABLE_START_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT2_PAGE_TABLE_START_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT3_PAGE_TABLE_START_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT3_PAGE_TABLE_START_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT4_PAGE_TABLE_START_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT4_PAGE_TABLE_START_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT5_PAGE_TABLE_START_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT5_PAGE_TABLE_START_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT6_PAGE_TABLE_START_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT6_PAGE_TABLE_START_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT7_PAGE_TABLE_START_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT7_PAGE_TABLE_START_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT8_PAGE_TABLE_START_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT8_PAGE_TABLE_START_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT9_PAGE_TABLE_START_ADDR_LO32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT9_PAGE_TABLE_START_ADDR_HI32_DEFAULT                       0x00000000
#define mmGCVM_CONTEXT10_PAGE_TABLE_START_ADDR_LO32_DEFAULT                      0x00000000
#define mmGCVM_CONTEXT10_PAGE_TABLE_START_ADDR_HI32_DEFAULT                      0x00000000
#define mmGCVM_CONTEXT11_PAGE_TABLE_START_ADDR_LO32_DEFAULT                      0x00000000
#define mmGCVM_CONTEXT11_PAGE_TABLE_START_ADDR_HI32_DEFAULT                      0x00000000
#define mmGCVM_CONTEXT12_PAGE_TABLE_START_ADDR_LO32_DEFAULT                      0x00000000
#define mmGCVM_CONTEXT12_PAGE_TABLE_START_ADDR_HI32_DEFAULT                      0x00000000
#define mmGCVM_CONTEXT13_PAGE_TABLE_START_ADDR_LO32_DEFAULT                      0x00000000
#define mmGCVM_CONTEXT13_PAGE_TABLE_START_ADDR_HI32_DEFAULT                      0x00000000
#define mmGCVM_CONTEXT14_PAGE_TABLE_START_ADDR_LO32_DEFAULT                      0x00000000
#define mmGCVM_CONTEXT14_PAGE_TABLE_START_ADDR_HI32_DEFAULT                      0x00000000
#define mmGCVM_CONTEXT15_PAGE_TABLE_START_ADDR_LO32_DEFAULT                      0x00000000
#define mmGCVM_CONTEXT15_PAGE_TABLE_START_ADDR_HI32_DEFAULT                      0x00000000
#define mmGCVM_CONTEXT0_PAGE_TABLE_END_ADDR_LO32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT0_PAGE_TABLE_END_ADDR_HI32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT1_PAGE_TABLE_END_ADDR_LO32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT1_PAGE_TABLE_END_ADDR_HI32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT2_PAGE_TABLE_END_ADDR_LO32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT2_PAGE_TABLE_END_ADDR_HI32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT3_PAGE_TABLE_END_ADDR_LO32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT3_PAGE_TABLE_END_ADDR_HI32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT4_PAGE_TABLE_END_ADDR_LO32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT4_PAGE_TABLE_END_ADDR_HI32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT5_PAGE_TABLE_END_ADDR_LO32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT5_PAGE_TABLE_END_ADDR_HI32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT6_PAGE_TABLE_END_ADDR_LO32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT6_PAGE_TABLE_END_ADDR_HI32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT7_PAGE_TABLE_END_ADDR_LO32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT7_PAGE_TABLE_END_ADDR_HI32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT8_PAGE_TABLE_END_ADDR_LO32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT8_PAGE_TABLE_END_ADDR_HI32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT9_PAGE_TABLE_END_ADDR_LO32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT9_PAGE_TABLE_END_ADDR_HI32_DEFAULT                         0x00000000
#define mmGCVM_CONTEXT10_PAGE_TABLE_END_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT10_PAGE_TABLE_END_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT11_PAGE_TABLE_END_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT11_PAGE_TABLE_END_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT12_PAGE_TABLE_END_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT12_PAGE_TABLE_END_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT13_PAGE_TABLE_END_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT13_PAGE_TABLE_END_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT14_PAGE_TABLE_END_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT14_PAGE_TABLE_END_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT15_PAGE_TABLE_END_ADDR_LO32_DEFAULT                        0x00000000
#define mmGCVM_CONTEXT15_PAGE_TABLE_END_ADDR_HI32_DEFAULT                        0x00000000
#define mmGCVM_L2_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT                      0x00001c00
#define mmGCVM_L2_CONTEXT0_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT             0x00001c00
#define mmGCVM_L2_CONTEXT1_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT             0x00001c00
#define mmGCVM_L2_CONTEXT2_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT             0x00001c00
#define mmGCVM_L2_CONTEXT3_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT             0x00001c00
#define mmGCVM_L2_CONTEXT4_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT             0x00001c00
#define mmGCVM_L2_CONTEXT5_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT             0x00001c00
#define mmGCVM_L2_CONTEXT6_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT             0x00001c00
#define mmGCVM_L2_CONTEXT7_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT             0x00001c00
#define mmGCVM_L2_CONTEXT8_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT             0x00001c00
#define mmGCVM_L2_CONTEXT9_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT             0x00001c00
#define mmGCVM_L2_CONTEXT10_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT            0x00001c00
#define mmGCVM_L2_CONTEXT11_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT            0x00001c00
#define mmGCVM_L2_CONTEXT12_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT            0x00001c00
#define mmGCVM_L2_CONTEXT13_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT            0x00001c00
#define mmGCVM_L2_CONTEXT14_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT            0x00001c00
#define mmGCVM_L2_CONTEXT15_PER_PFVF_PTE_CACHE_FRAGMENT_SIZES_DEFAULT            0x00001c00


// addressBlock: gc_gcvmsharedpfdec
#define mmGCMC_VM_NB_MMIOBASE_DEFAULT                                            0x00000000
#define mmGCMC_VM_NB_MMIOLIMIT_DEFAULT                                           0x00000000
#define mmGCMC_VM_NB_PCI_CTRL_DEFAULT                                            0x00000000
#define mmGCMC_VM_NB_PCI_ARB_DEFAULT                                             0x00000008
#define mmGCMC_VM_NB_TOP_OF_DRAM_SLOT1_DEFAULT                                   0x00000000
#define mmGCMC_VM_NB_LOWER_TOP_OF_DRAM2_DEFAULT                                  0x00000000
#define mmGCMC_VM_NB_UPPER_TOP_OF_DRAM2_DEFAULT                                  0x00000000
#define mmGCMC_VM_FB_OFFSET_DEFAULT                                              0x00000000
#define mmGCMC_VM_SYSTEM_APERTURE_DEFAULT_ADDR_LSB_DEFAULT                       0x00000000
#define mmGCMC_VM_SYSTEM_APERTURE_DEFAULT_ADDR_MSB_DEFAULT                       0x00000000
#define mmGCMC_VM_STEERING_DEFAULT                                               0x00000001
#define mmGCMC_SHARED_VIRT_RESET_REQ_DEFAULT                                     0x00000000
#define mmGCMC_MEM_POWER_LS_DEFAULT                                              0x00000208
#define mmGCMC_VM_CACHEABLE_DRAM_ADDRESS_START_DEFAULT                           0x00000000
#define mmGCMC_VM_CACHEABLE_DRAM_ADDRESS_END_DEFAULT                             0x000fffff
#define mmGCMC_VM_APT_CNTL_DEFAULT                                               0x0000000c
#define mmGCMC_VM_LOCAL_HBM_ADDRESS_LOCK_CNTL_DEFAULT                            0x00000000
#define mmGCMC_VM_LOCAL_HBM_ADDRESS_START_DEFAULT                                0x00000000
#define mmGCMC_VM_LOCAL_HBM_ADDRESS_END_DEFAULT                                  0x000fffff
#define mmGCMC_SHARED_ACTIVE_FCN_ID_DEFAULT                                      0x00000000
#define mmGCMC_SHARED_VIRT_RESET_REQ2_DEFAULT                                    0x00000000
#define mmGCMC_VM_XGMI_LFB_CNTL_DEFAULT                                          0x00000000
#define mmGCMC_VM_XGMI_LFB_SIZE_DEFAULT                                          0x00000000
#define mmGCMC_VM_FB_NOALLOC_CNTL_DEFAULT                                        0x00000000
#define mmGCUTCL2_HARVEST_BYPASS_GROUPS_DEFAULT                                  0x00000000


// addressBlock: gc_gcvmsharedvcdec
#define mmGCMC_VM_FB_LOCATION_BASE_DEFAULT                                       0x00000000
#define mmGCMC_VM_FB_LOCATION_TOP_DEFAULT                                        0x00000000
#define mmGCMC_VM_AGP_TOP_DEFAULT                                                0x00000000
#define mmGCMC_VM_AGP_BOT_DEFAULT                                                0x00000000
#define mmGCMC_VM_AGP_BASE_DEFAULT                                               0x00000000
#define mmGCMC_VM_SYSTEM_APERTURE_LOW_ADDR_DEFAULT                               0x00000000
#define mmGCMC_VM_SYSTEM_APERTURE_HIGH_ADDR_DEFAULT                              0x00000000
#define mmGCMC_VM_MX_L1_TLB_CNTL_DEFAULT                                         0x00000501


// addressBlock: gc_gceadec
#define mmGCEA_DRAM_RD_CLI2GRP_MAP0_DEFAULT                                      0xa9503aaa
#define mmGCEA_DRAM_RD_CLI2GRP_MAP1_DEFAULT                                      0xa9503aaa
#define mmGCEA_DRAM_WR_CLI2GRP_MAP0_DEFAULT                                      0xa9503aaa
#define mmGCEA_DRAM_WR_CLI2GRP_MAP1_DEFAULT                                      0xa9503aaa
#define mmGCEA_DRAM_RD_GRP2VC_MAP_DEFAULT                                        0x00000924
#define mmGCEA_DRAM_WR_GRP2VC_MAP_DEFAULT                                        0x00000324
#define mmGCEA_DRAM_RD_LAZY_DEFAULT                                              0x78000924
#define mmGCEA_DRAM_WR_LAZY_DEFAULT                                              0x78000924
#define mmGCEA_DRAM_RD_CAM_CNTL_DEFAULT                                          0x16db4444
#define mmGCEA_DRAM_WR_CAM_CNTL_DEFAULT                                          0x16db4444
#define mmGCEA_DRAM_PAGE_BURST_DEFAULT                                           0x20082008
#define mmGCEA_DRAM_RD_PRI_AGE_DEFAULT                                           0x00db6249
#define mmGCEA_DRAM_WR_PRI_AGE_DEFAULT                                           0x00db6249
#define mmGCEA_DRAM_RD_PRI_QUEUING_DEFAULT                                       0x00000db6
#define mmGCEA_DRAM_WR_PRI_QUEUING_DEFAULT                                       0x00000db6
#define mmGCEA_DRAM_RD_PRI_FIXED_DEFAULT                                         0x00000924
#define mmGCEA_DRAM_WR_PRI_FIXED_DEFAULT                                         0x00000924
#define mmGCEA_DRAM_RD_PRI_URGENCY_DEFAULT                                       0x0000fdb6
#define mmGCEA_DRAM_WR_PRI_URGENCY_DEFAULT                                       0x0000fdb6
#define mmGCEA_DRAM_RD_PRI_QUANT_PRI1_DEFAULT                                    0x3f3f3f3f
#define mmGCEA_DRAM_RD_PRI_QUANT_PRI2_DEFAULT                                    0x7f7f7f7f
#define mmGCEA_DRAM_RD_PRI_QUANT_PRI3_DEFAULT                                    0xffffffff
#define mmGCEA_DRAM_WR_PRI_QUANT_PRI1_DEFAULT                                    0x3f3f3f3f
#define mmGCEA_DRAM_WR_PRI_QUANT_PRI2_DEFAULT                                    0x7f7f7f7f
#define mmGCEA_DRAM_WR_PRI_QUANT_PRI3_DEFAULT                                    0xffffffff
#define mmGCEA_IO_RD_CLI2GRP_MAP0_DEFAULT                                        0xa9503aaa
#define mmGCEA_IO_RD_CLI2GRP_MAP1_DEFAULT                                        0xa9503aaa
#define mmGCEA_IO_WR_CLI2GRP_MAP0_DEFAULT                                        0xa9503aaa
#define mmGCEA_IO_WR_CLI2GRP_MAP1_DEFAULT                                        0xa9503aaa
#define mmGCEA_IO_RD_COMBINE_FLUSH_DEFAULT                                       0x00007777
#define mmGCEA_IO_WR_COMBINE_FLUSH_DEFAULT                                       0x00007777
#define mmGCEA_IO_GROUP_BURST_DEFAULT                                            0x1f031f03
#define mmGCEA_IO_RD_PRI_AGE_DEFAULT                                             0x00db6249
#define mmGCEA_IO_WR_PRI_AGE_DEFAULT                                             0x00db6249
#define mmGCEA_IO_RD_PRI_QUEUING_DEFAULT                                         0x00000db6
#define mmGCEA_IO_WR_PRI_QUEUING_DEFAULT                                         0x00000db6
#define mmGCEA_IO_RD_PRI_FIXED_DEFAULT                                           0x00000924
#define mmGCEA_IO_WR_PRI_FIXED_DEFAULT                                           0x00000924
#define mmGCEA_IO_RD_PRI_URGENCY_DEFAULT                                         0x00000492
#define mmGCEA_IO_WR_PRI_URGENCY_DEFAULT                                         0x00000492
#define mmGCEA_IO_RD_PRI_URGENCY_MASKING_DEFAULT                                 0xffffffff
#define mmGCEA_IO_WR_PRI_URGENCY_MASKING_DEFAULT                                 0xffffffff
#define mmGCEA_IO_RD_PRI_QUANT_PRI1_DEFAULT                                      0x3f3f3f3f
#define mmGCEA_IO_RD_PRI_QUANT_PRI2_DEFAULT                                      0x7f7f7f7f
#define mmGCEA_IO_RD_PRI_QUANT_PRI3_DEFAULT                                      0xffffffff
#define mmGCEA_IO_WR_PRI_QUANT_PRI1_DEFAULT                                      0x3f3f3f3f
#define mmGCEA_IO_WR_PRI_QUANT_PRI2_DEFAULT                                      0x7f7f7f7f
#define mmGCEA_IO_WR_PRI_QUANT_PRI3_DEFAULT                                      0xffffffff


// addressBlock: gc_tcdec
#define mmTCP_INVALIDATE_DEFAULT                                                 0x00000000
#define mmTCP_STATUS_DEFAULT                                                     0x00000000
#define mmTCP_EDC_CNT_DEFAULT                                                    0x00000000
#define mmTCI_STATUS_DEFAULT                                                     0x00000000
#define mmTCI_CNTL_1_DEFAULT                                                     0x40080022
#define mmTCI_CNTL_2_DEFAULT                                                     0x00000041


// addressBlock: gc_shdec
#define mmSPI_SHADER_PGM_RSRC4_PS_DEFAULT                                        0x0000ffff
#define mmSPI_SHADER_PGM_CHKSUM_PS_DEFAULT                                       0x00000000
#define mmSPI_SHADER_PGM_RSRC3_PS_DEFAULT                                        0x0000ffff
#define mmSPI_SHADER_PGM_LO_PS_DEFAULT                                           0x00000000
#define mmSPI_SHADER_PGM_HI_PS_DEFAULT                                           0x00000000
#define mmSPI_SHADER_PGM_RSRC1_PS_DEFAULT                                        0x00000000
#define mmSPI_SHADER_PGM_RSRC2_PS_DEFAULT                                        0x00000000
#define mmSPI_SHADER_USER_DATA_PS_0_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_PS_1_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_PS_2_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_PS_3_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_PS_4_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_PS_5_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_PS_6_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_PS_7_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_PS_8_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_PS_9_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_PS_10_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_11_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_12_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_13_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_14_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_15_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_16_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_17_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_18_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_19_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_20_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_21_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_22_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_23_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_24_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_25_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_26_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_27_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_28_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_29_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_30_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_PS_31_DEFAULT                                     0x00000000
#define mmSPI_SHADER_REQ_CTRL_PS_DEFAULT                                         0x00000000
#define mmSPI_SHADER_USER_ACCUM_PS_0_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_ACCUM_PS_1_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_ACCUM_PS_2_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_ACCUM_PS_3_DEFAULT                                     0x00000000
#define mmSPI_SHADER_PGM_RSRC4_VS_DEFAULT                                        0x0000ffff
#define mmSPI_SHADER_PGM_CHKSUM_VS_DEFAULT                                       0x00000000
#define mmSPI_SHADER_PGM_RSRC3_VS_DEFAULT                                        0x0000ffff
#define mmSPI_SHADER_LATE_ALLOC_VS_DEFAULT                                       0x00000000
#define mmSPI_SHADER_PGM_LO_VS_DEFAULT                                           0x00000000
#define mmSPI_SHADER_PGM_HI_VS_DEFAULT                                           0x00000000
#define mmSPI_SHADER_PGM_RSRC1_VS_DEFAULT                                        0x00000000
#define mmSPI_SHADER_PGM_RSRC2_VS_DEFAULT                                        0x00000000
#define mmSPI_SHADER_USER_DATA_VS_0_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_VS_1_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_VS_2_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_VS_3_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_VS_4_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_VS_5_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_VS_6_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_VS_7_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_VS_8_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_VS_9_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_VS_10_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_11_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_12_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_13_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_14_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_15_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_16_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_17_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_18_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_19_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_20_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_21_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_22_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_23_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_24_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_25_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_26_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_27_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_28_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_29_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_30_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_VS_31_DEFAULT                                     0x00000000
#define mmSPI_SHADER_REQ_CTRL_VS_DEFAULT                                         0x00000000
#define mmSPI_SHADER_USER_ACCUM_VS_0_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_ACCUM_VS_1_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_ACCUM_VS_2_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_ACCUM_VS_3_DEFAULT                                     0x00000000
#define mmSPI_SHADER_PGM_RSRC2_GS_VS_DEFAULT                                     0x00000000
#define mmSPI_SHADER_PGM_CHKSUM_GS_DEFAULT                                       0x00000000
#define mmSPI_SHADER_PGM_RSRC4_GS_DEFAULT                                        0x0010ffff
#define mmSPI_SHADER_USER_DATA_ADDR_LO_GS_DEFAULT                                0x00000000
#define mmSPI_SHADER_USER_DATA_ADDR_HI_GS_DEFAULT                                0x00000000
#define mmSPI_SHADER_PGM_LO_ES_GS_DEFAULT                                        0x00000000
#define mmSPI_SHADER_PGM_HI_ES_GS_DEFAULT                                        0x00000000
#define mmSPI_SHADER_PGM_RSRC3_GS_DEFAULT                                        0x0000fffe
#define mmSPI_SHADER_PGM_LO_GS_DEFAULT                                           0x00000000
#define mmSPI_SHADER_PGM_HI_GS_DEFAULT                                           0x00000000
#define mmSPI_SHADER_PGM_RSRC1_GS_DEFAULT                                        0x00000000
#define mmSPI_SHADER_PGM_RSRC2_GS_DEFAULT                                        0x00000000
#define mmSPI_SHADER_USER_DATA_GS_0_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_GS_1_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_GS_2_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_GS_3_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_GS_4_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_GS_5_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_GS_6_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_GS_7_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_GS_8_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_GS_9_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_GS_10_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_11_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_12_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_13_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_14_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_15_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_16_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_17_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_18_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_19_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_20_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_21_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_22_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_23_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_24_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_25_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_26_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_27_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_28_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_29_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_30_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_GS_31_DEFAULT                                     0x00000000
#define mmSPI_SHADER_REQ_CTRL_ESGS_DEFAULT                                       0x00000000
#define mmSPI_SHADER_USER_ACCUM_ESGS_0_DEFAULT                                   0x00000000
#define mmSPI_SHADER_USER_ACCUM_ESGS_1_DEFAULT                                   0x00000000
#define mmSPI_SHADER_USER_ACCUM_ESGS_2_DEFAULT                                   0x00000000
#define mmSPI_SHADER_USER_ACCUM_ESGS_3_DEFAULT                                   0x00000000
#define mmSPI_SHADER_PGM_LO_ES_DEFAULT                                           0x00000000
#define mmSPI_SHADER_PGM_HI_ES_DEFAULT                                           0x00000000
#define mmSPI_SHADER_PGM_CHKSUM_HS_DEFAULT                                       0x00000000
#define mmSPI_SHADER_PGM_RSRC4_HS_DEFAULT                                        0x0000ffff
#define mmSPI_SHADER_USER_DATA_ADDR_LO_HS_DEFAULT                                0x00000000
#define mmSPI_SHADER_USER_DATA_ADDR_HI_HS_DEFAULT                                0x00000000
#define mmSPI_SHADER_PGM_LO_LS_HS_DEFAULT                                        0x00000000
#define mmSPI_SHADER_PGM_HI_LS_HS_DEFAULT                                        0x00000000
#define mmSPI_SHADER_PGM_RSRC3_HS_DEFAULT                                        0xffff0000
#define mmSPI_SHADER_PGM_LO_HS_DEFAULT                                           0x00000000
#define mmSPI_SHADER_PGM_HI_HS_DEFAULT                                           0x00000000
#define mmSPI_SHADER_PGM_RSRC1_HS_DEFAULT                                        0x00000000
#define mmSPI_SHADER_PGM_RSRC2_HS_DEFAULT                                        0x00000000
#define mmSPI_SHADER_USER_DATA_HS_0_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_HS_1_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_HS_2_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_HS_3_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_HS_4_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_HS_5_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_HS_6_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_HS_7_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_HS_8_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_HS_9_DEFAULT                                      0x00000000
#define mmSPI_SHADER_USER_DATA_HS_10_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_11_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_12_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_13_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_14_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_15_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_16_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_17_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_18_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_19_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_20_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_21_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_22_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_23_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_24_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_25_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_26_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_27_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_28_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_29_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_30_DEFAULT                                     0x00000000
#define mmSPI_SHADER_USER_DATA_HS_31_DEFAULT                                     0x00000000
#define mmSPI_SHADER_REQ_CTRL_LSHS_DEFAULT                                       0x00000000
#define mmSPI_SHADER_USER_ACCUM_LSHS_0_DEFAULT                                   0x00000000
#define mmSPI_SHADER_USER_ACCUM_LSHS_1_DEFAULT                                   0x00000000
#define mmSPI_SHADER_USER_ACCUM_LSHS_2_DEFAULT                                   0x00000000
#define mmSPI_SHADER_USER_ACCUM_LSHS_3_DEFAULT                                   0x00000000
#define mmSPI_SHADER_PGM_LO_LS_DEFAULT                                           0x00000000
#define mmSPI_SHADER_PGM_HI_LS_DEFAULT                                           0x00000000
#define mmCOMPUTE_DISPATCH_INITIATOR_DEFAULT                                     0x00000000
#define mmCOMPUTE_DIM_X_DEFAULT                                                  0x00000000
#define mmCOMPUTE_DIM_Y_DEFAULT                                                  0x00000000
#define mmCOMPUTE_DIM_Z_DEFAULT                                                  0x00000000
#define mmCOMPUTE_START_X_DEFAULT                                                0x00000000
#define mmCOMPUTE_START_Y_DEFAULT                                                0x00000000
#define mmCOMPUTE_START_Z_DEFAULT                                                0x00000000
#define mmCOMPUTE_NUM_THREAD_X_DEFAULT                                           0x00000000
#define mmCOMPUTE_NUM_THREAD_Y_DEFAULT                                           0x00000000
#define mmCOMPUTE_NUM_THREAD_Z_DEFAULT                                           0x00000000
#define mmCOMPUTE_PIPELINESTAT_ENABLE_DEFAULT                                    0x00000001
#define mmCOMPUTE_PERFCOUNT_ENABLE_DEFAULT                                       0x00000000
#define mmCOMPUTE_PGM_LO_DEFAULT                                                 0x00000000
#define mmCOMPUTE_PGM_HI_DEFAULT                                                 0x00000000
#define mmCOMPUTE_DISPATCH_PKT_ADDR_LO_DEFAULT                                   0x00000000
#define mmCOMPUTE_DISPATCH_PKT_ADDR_HI_DEFAULT                                   0x00000000
#define mmCOMPUTE_DISPATCH_SCRATCH_BASE_LO_DEFAULT                               0x00000000
#define mmCOMPUTE_DISPATCH_SCRATCH_BASE_HI_DEFAULT                               0x00000000
#define mmCOMPUTE_PGM_RSRC1_DEFAULT                                              0x00000000
#define mmCOMPUTE_PGM_RSRC2_DEFAULT                                              0x00000000
#define mmCOMPUTE_VMID_DEFAULT                                                   0x00000000
#define mmCOMPUTE_RESOURCE_LIMITS_DEFAULT                                        0x00000000
#define mmCOMPUTE_DESTINATION_EN_SE0_DEFAULT                                     0xffffffff
#define mmCOMPUTE_STATIC_THREAD_MGMT_SE0_DEFAULT                                 0xffffffff
#define mmCOMPUTE_DESTINATION_EN_SE1_DEFAULT                                     0xffffffff
#define mmCOMPUTE_STATIC_THREAD_MGMT_SE1_DEFAULT                                 0xffffffff
#define mmCOMPUTE_TMPRING_SIZE_DEFAULT                                           0x00000000
#define mmCOMPUTE_DESTINATION_EN_SE2_DEFAULT                                     0xffffffff
#define mmCOMPUTE_STATIC_THREAD_MGMT_SE2_DEFAULT                                 0xffffffff
#define mmCOMPUTE_DESTINATION_EN_SE3_DEFAULT                                     0xffffffff
#define mmCOMPUTE_STATIC_THREAD_MGMT_SE3_DEFAULT                                 0xffffffff
#define mmCOMPUTE_RESTART_X_DEFAULT                                              0x00000000
#define mmCOMPUTE_RESTART_Y_DEFAULT                                              0x00000000
#define mmCOMPUTE_RESTART_Z_DEFAULT                                              0x00000000
#define mmCOMPUTE_THREAD_TRACE_ENABLE_DEFAULT                                    0x00000000
#define mmCOMPUTE_MISC_RESERVED_DEFAULT                                          0x00000003
#define mmCOMPUTE_DISPATCH_ID_DEFAULT                                            0x00000000
#define mmCOMPUTE_THREADGROUP_ID_DEFAULT                                         0x00000000
#define mmCOMPUTE_REQ_CTRL_DEFAULT                                               0x00000000
#define mmCOMPUTE_USER_ACCUM_0_DEFAULT                                           0x00000000
#define mmCOMPUTE_USER_ACCUM_1_DEFAULT                                           0x00000000
#define mmCOMPUTE_USER_ACCUM_2_DEFAULT                                           0x00000000
#define mmCOMPUTE_USER_ACCUM_3_DEFAULT                                           0x00000000
#define mmCOMPUTE_PGM_RSRC3_DEFAULT                                              0x00000000
#define mmCOMPUTE_DDID_INDEX_DEFAULT                                             0x00000000
#define mmCOMPUTE_SHADER_CHKSUM_DEFAULT                                          0x00000000
#define mmCOMPUTE_RELAUNCH_DEFAULT                                               0x00000000
#define mmCOMPUTE_WAVE_RESTORE_ADDR_LO_DEFAULT                                   0x00000000
#define mmCOMPUTE_WAVE_RESTORE_ADDR_HI_DEFAULT                                   0x00000000
#define mmCOMPUTE_RELAUNCH2_DEFAULT                                              0x00000000
#define mmCOMPUTE_USER_DATA_0_DEFAULT                                            0x00000000
#define mmCOMPUTE_USER_DATA_1_DEFAULT                                            0x00000000
#define mmCOMPUTE_USER_DATA_2_DEFAULT                                            0x00000000
#define mmCOMPUTE_USER_DATA_3_DEFAULT                                            0x00000000
#define mmCOMPUTE_USER_DATA_4_DEFAULT                                            0x00000000
#define mmCOMPUTE_USER_DATA_5_DEFAULT                                            0x00000000
#define mmCOMPUTE_USER_DATA_6_DEFAULT                                            0x00000000
#define mmCOMPUTE_USER_DATA_7_DEFAULT                                            0x00000000
#define mmCOMPUTE_USER_DATA_8_DEFAULT                                            0x00000000
#define mmCOMPUTE_USER_DATA_9_DEFAULT                                            0x00000000
#define mmCOMPUTE_USER_DATA_10_DEFAULT                                           0x00000000
#define mmCOMPUTE_USER_DATA_11_DEFAULT                                           0x00000000
#define mmCOMPUTE_USER_DATA_12_DEFAULT                                           0x00000000
#define mmCOMPUTE_USER_DATA_13_DEFAULT                                           0x00000000
#define mmCOMPUTE_USER_DATA_14_DEFAULT                                           0x00000000
#define mmCOMPUTE_USER_DATA_15_DEFAULT                                           0x00000000
#define mmCOMPUTE_DISPATCH_TUNNEL_DEFAULT                                        0x00000000
#define mmCOMPUTE_DISPATCH_END_DEFAULT                                           0x00000000
#define mmCOMPUTE_NOWHERE_DEFAULT                                                0x00000000
#define mmSH_RESERVED_REG0_DEFAULT                                               0x00000000
#define mmSH_RESERVED_REG1_DEFAULT                                               0x00000000


// addressBlock: gc_cppdec
#define mmCP_EOPQ_WAIT_TIME_DEFAULT                                              0x0000052c
#define mmCP_CPC_MGCG_SYNC_CNTL_DEFAULT                                          0x00001020
#define mmCPC_INT_INFO_DEFAULT                                                   0x00000000
#define mmCP_VIRT_STATUS_DEFAULT                                                 0x00000000
#define mmCPC_INT_ADDR_DEFAULT                                                   0x00000000
#define mmCPC_INT_PASID_DEFAULT                                                  0x00000000
#define mmCP_GFX_ERROR_DEFAULT                                                   0x00000000
#define mmCPG_UTCL1_CNTL_DEFAULT                                                 0x00000080
#define mmCPC_UTCL1_CNTL_DEFAULT                                                 0x00000080
#define mmCPF_UTCL1_CNTL_DEFAULT                                                 0x00000080
#define mmCP_AQL_SMM_STATUS_DEFAULT                                              0x00000000
#define mmCP_RB0_BASE_DEFAULT                                                    0x00000000
#define mmCP_RB_BASE_DEFAULT                                                     0x00000000
#define mmCP_RB0_CNTL_DEFAULT                                                    0x00a00000
#define mmCP_RB_CNTL_DEFAULT                                                     0x00a00000
#define mmCP_RB_RPTR_WR_DEFAULT                                                  0x00000000
#define mmCP_RB0_RPTR_ADDR_DEFAULT                                               0x00000000
#define mmCP_RB_RPTR_ADDR_DEFAULT                                                0x00000000
#define mmCP_RB0_RPTR_ADDR_HI_DEFAULT                                            0x00000000
#define mmCP_RB_RPTR_ADDR_HI_DEFAULT                                             0x00000000
#define mmCP_RB0_BUFSZ_MASK_DEFAULT                                              0x00000000
#define mmCP_RB_BUFSZ_MASK_DEFAULT                                               0x00000000
#define mmCP_INT_CNTL_DEFAULT                                                    0x00000000
#define mmCP_INT_STATUS_DEFAULT                                                  0x00000000
#define mmCP_DEVICE_ID_DEFAULT                                                   0x00000000
#define mmCP_ME0_PIPE_PRIORITY_CNTS_DEFAULT                                      0x08081020
#define mmCP_RING_PRIORITY_CNTS_DEFAULT                                          0x08081020
#define mmCP_ME0_PIPE0_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_RING0_PRIORITY_DEFAULT                                              0x00000002
#define mmCP_ME0_PIPE1_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_RING1_PRIORITY_DEFAULT                                              0x00000002
#define mmCP_ME0_PIPE2_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_RING2_PRIORITY_DEFAULT                                              0x00000002
#define mmCP_FATAL_ERROR_DEFAULT                                                 0x00000000
#define mmCP_RB_VMID_DEFAULT                                                     0x00000000
#define mmCP_ME0_PIPE0_VMID_DEFAULT                                              0x00000000
#define mmCP_ME0_PIPE1_VMID_DEFAULT                                              0x00000000
#define mmCP_RB0_WPTR_DEFAULT                                                    0x00000000
#define mmCP_RB_WPTR_DEFAULT                                                     0x00000000
#define mmCP_RB0_WPTR_HI_DEFAULT                                                 0x00000000
#define mmCP_RB_WPTR_HI_DEFAULT                                                  0x00000000
#define mmCP_RB1_WPTR_DEFAULT                                                    0x00000000
#define mmCP_RB1_WPTR_HI_DEFAULT                                                 0x00000000
#define mmCP_RB2_WPTR_DEFAULT                                                    0x00000000
#define mmCP_PROCESS_QUANTUM_DEFAULT                                             0x00000008
#define mmCP_RB_DOORBELL_RANGE_LOWER_DEFAULT                                     0x00000000
#define mmCP_RB_DOORBELL_RANGE_UPPER_DEFAULT                                     0x00000108
#define mmCP_MEC_DOORBELL_RANGE_LOWER_DEFAULT                                    0x00000110
#define mmCP_MEC_DOORBELL_RANGE_UPPER_DEFAULT                                    0x00000ffc
#define mmCPG_UTCL1_ERROR_DEFAULT                                                0x00000000
#define mmCPC_UTCL1_ERROR_DEFAULT                                                0x00000000
#define mmCP_RB1_BASE_DEFAULT                                                    0x00000000
#define mmCP_RB1_CNTL_DEFAULT                                                    0x00a00000
#define mmCP_RB1_RPTR_ADDR_DEFAULT                                               0x00000000
#define mmCP_RB1_RPTR_ADDR_HI_DEFAULT                                            0x00000000
#define mmCP_RB1_BUFSZ_MASK_DEFAULT                                              0x00000000
#define mmCP_RB2_BASE_DEFAULT                                                    0x00000000
#define mmCP_RB2_CNTL_DEFAULT                                                    0x00a00000
#define mmCP_RB2_RPTR_ADDR_DEFAULT                                               0x00000000
#define mmCP_RB2_RPTR_ADDR_HI_DEFAULT                                            0x00000000
#define mmCP_INT_CNTL_RING0_DEFAULT                                              0x00000000
#define mmCP_INT_CNTL_RING1_DEFAULT                                              0x00000000
#define mmCP_INT_CNTL_RING2_DEFAULT                                              0x00000000
#define mmCP_INT_STATUS_RING0_DEFAULT                                            0x00000000
#define mmCP_INT_STATUS_RING1_DEFAULT                                            0x00000000
#define mmCP_INT_STATUS_RING2_DEFAULT                                            0x00000000
#define mmCP_ME_F32_INTERRUPT_DEFAULT                                            0x00000000
#define mmCP_PFP_F32_INTERRUPT_DEFAULT                                           0x00000000
#define mmCP_CE_F32_INTERRUPT_DEFAULT                                            0x00000000
#define mmCP_MEC1_F32_INTERRUPT_DEFAULT                                          0x00000000
#define mmCP_MEC2_F32_INTERRUPT_DEFAULT                                          0x00000000
#define mmCP_PWR_CNTL_DEFAULT                                                    0x00000000
#define mmCP_MEM_SLP_CNTL_DEFAULT                                                0x00020200
#define mmCP_ECC_FIRSTOCCURRENCE_DEFAULT                                         0x00000000
#define mmCP_ECC_FIRSTOCCURRENCE_RING0_DEFAULT                                   0x00000000
#define mmCP_ECC_FIRSTOCCURRENCE_RING1_DEFAULT                                   0x00000000
#define mmCP_ECC_FIRSTOCCURRENCE_RING2_DEFAULT                                   0x00000000
#define mmGB_EDC_MODE_DEFAULT                                                    0x00000000
#define mmCP_PQ_WPTR_POLL_CNTL_DEFAULT                                           0x00000001
#define mmCP_PQ_WPTR_POLL_CNTL1_DEFAULT                                          0x00000000
#define mmCP_ME1_PIPE0_INT_CNTL_DEFAULT                                          0x00000000
#define mmCP_ME1_PIPE1_INT_CNTL_DEFAULT                                          0x00000000
#define mmCP_ME1_PIPE2_INT_CNTL_DEFAULT                                          0x00000000
#define mmCP_ME1_PIPE3_INT_CNTL_DEFAULT                                          0x00000000
#define mmCP_ME2_PIPE0_INT_CNTL_DEFAULT                                          0x00000000
#define mmCP_ME2_PIPE1_INT_CNTL_DEFAULT                                          0x00000000
#define mmCP_ME2_PIPE2_INT_CNTL_DEFAULT                                          0x00000000
#define mmCP_ME2_PIPE3_INT_CNTL_DEFAULT                                          0x00000000
#define mmCP_ME1_PIPE0_INT_STATUS_DEFAULT                                        0x00000000
#define mmCP_ME1_PIPE1_INT_STATUS_DEFAULT                                        0x00000000
#define mmCP_ME1_PIPE2_INT_STATUS_DEFAULT                                        0x00000000
#define mmCP_ME1_PIPE3_INT_STATUS_DEFAULT                                        0x00000000
#define mmCP_ME2_PIPE0_INT_STATUS_DEFAULT                                        0x00000000
#define mmCP_ME2_PIPE1_INT_STATUS_DEFAULT                                        0x00000000
#define mmCP_ME2_PIPE2_INT_STATUS_DEFAULT                                        0x00000000
#define mmCP_ME2_PIPE3_INT_STATUS_DEFAULT                                        0x00000000
#define mmCP_GFX_QUEUE_INDEX_DEFAULT                                             0x00000000
#define mmCC_GC_EDC_CONFIG_DEFAULT                                               0x00000000
#define mmCP_ME1_PIPE_PRIORITY_CNTS_DEFAULT                                      0x08081020
#define mmCP_ME1_PIPE0_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_ME1_PIPE1_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_ME1_PIPE2_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_ME1_PIPE3_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_ME2_PIPE_PRIORITY_CNTS_DEFAULT                                      0x08081020
#define mmCP_ME2_PIPE0_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_ME2_PIPE1_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_ME2_PIPE2_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_ME2_PIPE3_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_CE_PRGRM_CNTR_START_DEFAULT                                         0x00000000
#define mmCP_PFP_PRGRM_CNTR_START_DEFAULT                                        0x00000000
#define mmCP_ME_PRGRM_CNTR_START_DEFAULT                                         0x00000000
#define mmCP_MEC1_PRGRM_CNTR_START_DEFAULT                                       0x00000000
#define mmCP_MEC2_PRGRM_CNTR_START_DEFAULT                                       0x00000000
#define mmCP_CE_INTR_ROUTINE_START_DEFAULT                                       0x00000002
#define mmCP_PFP_INTR_ROUTINE_START_DEFAULT                                      0x00000002
#define mmCP_ME_INTR_ROUTINE_START_DEFAULT                                       0x00000002
#define mmCP_MEC1_INTR_ROUTINE_START_DEFAULT                                     0x00000002
#define mmCP_MEC2_INTR_ROUTINE_START_DEFAULT                                     0x00000002
#define mmCP_CONTEXT_CNTL_DEFAULT                                                0x00750075
#define mmCP_MAX_CONTEXT_DEFAULT                                                 0x00000007
#define mmCP_IQ_WAIT_TIME1_DEFAULT                                               0x40404040
#define mmCP_IQ_WAIT_TIME2_DEFAULT                                               0x40404040
#define mmCP_RB0_BASE_HI_DEFAULT                                                 0x00000000
#define mmCP_RB1_BASE_HI_DEFAULT                                                 0x00000000
#define mmCP_VMID_RESET_DEFAULT                                                  0x00000000
#define mmCPC_INT_CNTL_DEFAULT                                                   0x00000000
#define mmCPC_INT_STATUS_DEFAULT                                                 0x00000000
#define mmCP_VMID_PREEMPT_DEFAULT                                                0x00000000
#define mmCPC_INT_CNTX_ID_DEFAULT                                                0x00000000
#define mmCP_PQ_STATUS_DEFAULT                                                   0x00000000
#define mmCP_MEC1_F32_INT_DIS_DEFAULT                                            0x00000000
#define mmCP_MEC2_F32_INT_DIS_DEFAULT                                            0x00000000
#define mmCP_VMID_STATUS_DEFAULT                                                 0x00000000
#define mmCPC_SUSPEND_CTX_SAVE_BASE_ADDR_LO_DEFAULT                              0x00000000
#define mmCPC_SUSPEND_CTX_SAVE_BASE_ADDR_HI_DEFAULT                              0x00000000
#define mmCPC_SUSPEND_CTX_SAVE_CONTROL_DEFAULT                                   0x00000000
#define mmCPC_SUSPEND_CNTL_STACK_OFFSET_DEFAULT                                  0x00000000
#define mmCPC_SUSPEND_CNTL_STACK_SIZE_DEFAULT                                    0x00000000
#define mmCPC_SUSPEND_WG_STATE_OFFSET_DEFAULT                                    0x00000000
#define mmCPC_SUSPEND_CTX_SAVE_SIZE_DEFAULT                                      0x00000000
#define mmCPC_OS_PIPES_DEFAULT                                                   0x00000000
#define mmCP_SUSPEND_RESUME_REQ_DEFAULT                                          0x00000000
#define mmCP_SUSPEND_CNTL_DEFAULT                                                0x00000002
#define mmCP_IQ_WAIT_TIME3_DEFAULT                                               0x00000040
#define mmCPC_DDID_BASE_ADDR_LO_DEFAULT                                          0x00000000
#define mmCP_DDID_BASE_ADDR_LO_DEFAULT                                           0x00000000
#define mmCPC_DDID_BASE_ADDR_HI_DEFAULT                                          0x00000000
#define mmCP_DDID_BASE_ADDR_HI_DEFAULT                                           0x00000000
#define mmCPC_DDID_CNTL_DEFAULT                                                  0x00000080
#define mmCP_DDID_CNTL_DEFAULT                                                   0x00000080
#define mmCP_GFX_DDID_INFLIGHT_COUNT_DEFAULT                                     0x00000000
#define mmCP_GFX_DDID_WPTR_DEFAULT                                               0x00000000
#define mmCP_GFX_DDID_RPTR_DEFAULT                                               0x00000000
#define mmCP_GFX_DDID_DELTA_RPT_COUNT_DEFAULT                                    0x00000000
#define mmCP_GFX_HPD_STATUS0_DEFAULT                                             0x01000000
#define mmCP_GFX_HPD_CONTROL0_DEFAULT                                            0x00000000
#define mmCP_GFX_HPD_OSPRE_FENCE_ADDR_LO_DEFAULT                                 0x00000000
#define mmCP_GFX_HPD_OSPRE_FENCE_ADDR_HI_DEFAULT                                 0x00000000
#define mmCP_GFX_HPD_OSPRE_FENCE_DATA_LO_DEFAULT                                 0x00000000
#define mmCP_GFX_HPD_OSPRE_FENCE_DATA_HI_DEFAULT                                 0x00000000
#define mmCP_GFX_INDEX_MUTEX_DEFAULT                                             0x00000000
#define mmCP_GFX_MQD_BASE_ADDR_DEFAULT                                           0x00000000
#define mmCP_GFX_MQD_BASE_ADDR_HI_DEFAULT                                        0x00000000
#define mmCP_GFX_HQD_ACTIVE_DEFAULT                                              0x00000000
#define mmCP_GFX_HQD_VMID_DEFAULT                                                0x00000000
#define mmCP_GFX_HQD_QUEUE_PRIORITY_DEFAULT                                      0x00000000
#define mmCP_GFX_HQD_QUANTUM_DEFAULT                                             0x00000a01
#define mmCP_GFX_HQD_BASE_DEFAULT                                                0x00000000
#define mmCP_GFX_HQD_BASE_HI_DEFAULT                                             0x00000000
#define mmCP_GFX_HQD_RPTR_DEFAULT                                                0x00000000
#define mmCP_GFX_HQD_RPTR_ADDR_DEFAULT                                           0x00000000
#define mmCP_GFX_HQD_RPTR_ADDR_HI_DEFAULT                                        0x00000000
#define mmCP_RB_WPTR_POLL_ADDR_LO_DEFAULT                                        0x00000000
#define mmCP_RB_WPTR_POLL_ADDR_HI_DEFAULT                                        0x00000000
#define mmCP_RB_DOORBELL_CONTROL_DEFAULT                                         0x00000000
#define mmCP_GFX_HQD_OFFSET_DEFAULT                                              0x00000000
#define mmCP_GFX_HQD_CNTL_DEFAULT                                                0x00a00000
#define mmCP_GFX_HQD_CSMD_RPTR_DEFAULT                                           0x00000000
#define mmCP_GFX_HQD_WPTR_DEFAULT                                                0x00000000
#define mmCP_GFX_HQD_WPTR_HI_DEFAULT                                             0x00000000
#define mmCP_GFX_HQD_DEQUEUE_REQUEST_DEFAULT                                     0x00000000
#define mmCP_GFX_HQD_MAPPED_DEFAULT                                              0x00000000
#define mmCP_GFX_HQD_QUE_MGR_CONTROL_DEFAULT                                     0x00000000
#define mmCP_GFX_HQD_HQ_STATUS0_DEFAULT                                          0x40000000
#define mmCP_GFX_HQD_HQ_CONTROL0_DEFAULT                                         0x00000000
#define mmCP_GFX_MQD_CONTROL_DEFAULT                                             0x00000100
#define mmCP_HQD_GFX_CONTROL_DEFAULT                                             0x00000000
#define mmCP_HQD_GFX_STATUS_DEFAULT                                              0x00000000
#define mmCP_GFX_HQD_CE_RPTR_WR_DEFAULT                                          0x00000000
#define mmCP_GFX_HQD_CE_BASE_DEFAULT                                             0x00000000
#define mmCP_GFX_HQD_CE_BASE_HI_DEFAULT                                          0x00000000
#define mmCP_GFX_HQD_CE_RPTR_DEFAULT                                             0x00000000
#define mmCP_GFX_HQD_CE_RPTR_ADDR_DEFAULT                                        0x00000000
#define mmCP_GFX_HQD_CE_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmCP_GFX_HQD_CE_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmCP_GFX_HQD_CE_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmCP_GFX_HQD_CE_OFFSET_DEFAULT                                           0x00000000
#define mmCP_GFX_HQD_CE_CNTL_DEFAULT                                             0x08a00000
#define mmCP_GFX_HQD_CE_CSMD_RPTR_DEFAULT                                        0x00000000
#define mmCP_GFX_HQD_CE_WPTR_DEFAULT                                             0x00000000
#define mmCP_GFX_HQD_CE_WPTR_HI_DEFAULT                                          0x00000000
#define mmCP_CE_DOORBELL_CONTROL_DEFAULT                                         0x00000000
#define mmCP_DMA_WATCH0_ADDR_LO_DEFAULT                                          0x00000000
#define mmCP_DMA_WATCH0_ADDR_HI_DEFAULT                                          0x00000000
#define mmCP_DMA_WATCH0_MASK_DEFAULT                                             0x00000000
#define mmCP_DMA_WATCH0_CNTL_DEFAULT                                             0x00000000
#define mmCP_DMA_WATCH1_ADDR_LO_DEFAULT                                          0x00000000
#define mmCP_DMA_WATCH1_ADDR_HI_DEFAULT                                          0x00000000
#define mmCP_DMA_WATCH1_MASK_DEFAULT                                             0x00000000
#define mmCP_DMA_WATCH1_CNTL_DEFAULT                                             0x00000000
#define mmCP_DMA_WATCH2_ADDR_LO_DEFAULT                                          0x00000000
#define mmCP_DMA_WATCH2_ADDR_HI_DEFAULT                                          0x00000000
#define mmCP_DMA_WATCH2_MASK_DEFAULT                                             0x00000000
#define mmCP_DMA_WATCH2_CNTL_DEFAULT                                             0x00000000
#define mmCP_DMA_WATCH3_ADDR_LO_DEFAULT                                          0x00000000
#define mmCP_DMA_WATCH3_ADDR_HI_DEFAULT                                          0x00000000
#define mmCP_DMA_WATCH3_MASK_DEFAULT                                             0x00000000
#define mmCP_DMA_WATCH3_CNTL_DEFAULT                                             0x00000000
#define mmCP_DMA_WATCH_STAT_ADDR_LO_DEFAULT                                      0x00000000
#define mmCP_DMA_WATCH_STAT_ADDR_HI_DEFAULT                                      0x00000000
#define mmCP_DMA_WATCH_STAT_DEFAULT                                              0x00000000
#define mmCP_PFP_JT_STAT_DEFAULT                                                 0x00000000
#define mmCP_CE_JT_STAT_DEFAULT                                                  0x00000000
#define mmCP_MEC_JT_STAT_DEFAULT                                                 0x00000000
#define mmCP_FETCHER_SOURCE_DEFAULT                                              0x00000000
#define mmCP_CE_CS_PARTITION_INDEX_DEFAULT                                       0x00000000
#define mmCP_RB_DOORBELL_CLEAR_DEFAULT                                           0x00000000
#define mmCP_RB0_ACTIVE_DEFAULT                                                  0x00000000
#define mmCP_RB_ACTIVE_DEFAULT                                                   0x00000000
#define mmCP_RB1_ACTIVE_DEFAULT                                                  0x00000000
#define mmCP_RB_STATUS_DEFAULT                                                   0x00000000
#define mmCPG_RCIU_CAM_INDEX_DEFAULT                                             0x00000000
#define mmCPG_RCIU_CAM_DATA_DEFAULT                                              0x00000000
#define mmCPG_RCIU_CAM_DATA_PHASE0_DEFAULT                                       0x00000000
#define mmCPG_RCIU_CAM_DATA_PHASE1_DEFAULT                                       0x00000000
#define mmCPG_RCIU_CAM_DATA_PHASE2_DEFAULT                                       0x00000000
#define mmCP_GPU_TIMESTAMP_OFFSET_LO_DEFAULT                                     0x00000000
#define mmCP_GPU_TIMESTAMP_OFFSET_HI_DEFAULT                                     0x00000000
#define mmCPF_GCR_CNTL_DEFAULT                                                   0x0001c7f0
#define mmCPG_UTCL1_STATUS_DEFAULT                                               0x00000000
#define mmCPC_UTCL1_STATUS_DEFAULT                                               0x00000000
#define mmCPF_UTCL1_STATUS_DEFAULT                                               0x00000000
#define mmCP_SD_CNTL_DEFAULT                                                     0x0000047f
#define mmCP_SOFT_RESET_CNTL_DEFAULT                                             0x00000000
#define mmCP_CPC_GFX_CNTL_DEFAULT                                                0x00000000


// addressBlock: gc_spipdec
#define mmSPI_ARB_PRIORITY_DEFAULT                                               0x00000000
#define mmSPI_ARB_CYCLES_0_DEFAULT                                               0x00000000
#define mmSPI_ARB_CYCLES_1_DEFAULT                                               0x00000000
#define mmSPI_WCL_PIPE_PERCENT_GFX_DEFAULT                                       0x07c1f07f
#define mmSPI_WCL_PIPE_PERCENT_HP3D_DEFAULT                                      0x07c1f07f
#define mmSPI_WCL_PIPE_PERCENT_CS0_DEFAULT                                       0x0000007f
#define mmSPI_WCL_PIPE_PERCENT_CS1_DEFAULT                                       0x0000007f
#define mmSPI_WCL_PIPE_PERCENT_CS2_DEFAULT                                       0x0000007f
#define mmSPI_WCL_PIPE_PERCENT_CS3_DEFAULT                                       0x0000007f
#define mmSPI_COMPUTE_QUEUE_RESET_DEFAULT                                        0x00000000
#define mmSPI_RESOURCE_RESERVE_CU_0_DEFAULT                                      0x00000000
#define mmSPI_RESOURCE_RESERVE_CU_1_DEFAULT                                      0x00000000
#define mmSPI_RESOURCE_RESERVE_CU_2_DEFAULT                                      0x00000000
#define mmSPI_RESOURCE_RESERVE_CU_3_DEFAULT                                      0x00000000
#define mmSPI_RESOURCE_RESERVE_CU_4_DEFAULT                                      0x00000000
#define mmSPI_RESOURCE_RESERVE_CU_5_DEFAULT                                      0x00000000
#define mmSPI_RESOURCE_RESERVE_CU_6_DEFAULT                                      0x00000000
#define mmSPI_RESOURCE_RESERVE_CU_7_DEFAULT                                      0x00000000
#define mmSPI_RESOURCE_RESERVE_CU_8_DEFAULT                                      0x00000000
#define mmSPI_RESOURCE_RESERVE_CU_9_DEFAULT                                      0x00000000
#define mmSPI_RESOURCE_RESERVE_EN_CU_0_DEFAULT                                   0x00000000
#define mmSPI_RESOURCE_RESERVE_EN_CU_1_DEFAULT                                   0x00000000
#define mmSPI_RESOURCE_RESERVE_EN_CU_2_DEFAULT                                   0x00000000
#define mmSPI_RESOURCE_RESERVE_EN_CU_3_DEFAULT                                   0x00000000
#define mmSPI_RESOURCE_RESERVE_EN_CU_4_DEFAULT                                   0x00000000
#define mmSPI_RESOURCE_RESERVE_EN_CU_5_DEFAULT                                   0x00000000
#define mmSPI_RESOURCE_RESERVE_EN_CU_6_DEFAULT                                   0x00000000
#define mmSPI_RESOURCE_RESERVE_EN_CU_7_DEFAULT                                   0x00000000
#define mmSPI_RESOURCE_RESERVE_EN_CU_8_DEFAULT                                   0x00000000
#define mmSPI_RESOURCE_RESERVE_EN_CU_9_DEFAULT                                   0x00000000
#define mmSPI_COMPUTE_WF_CTX_SAVE_DEFAULT                                        0x00000000
#define mmSPI_ARB_CNTL_0_DEFAULT                                                 0x00000000
#define mmSPI_FEATURE_CTRL_DEFAULT                                               0x00000000
#define mmSPI_SHADER_RSRC_LIMIT_CTRL_DEFAULT                                     0x00000000


// addressBlock: gc_cpphqddec
#define mmCP_HPD_MES_ROQ_OFFSETS_DEFAULT                                         0x00400000
#define mmCP_HPD_ROQ_OFFSETS_DEFAULT                                             0x00200604
#define mmCP_HPD_STATUS0_DEFAULT                                                 0x01000000
#define mmCP_HPD_UTCL1_CNTL_DEFAULT                                              0x00000000
#define mmCP_HPD_UTCL1_ERROR_DEFAULT                                             0x00000000
#define mmCP_HPD_UTCL1_ERROR_ADDR_DEFAULT                                        0x00000000
#define mmCP_MQD_BASE_ADDR_DEFAULT                                               0x00000000
#define mmCP_MQD_BASE_ADDR_HI_DEFAULT                                            0x00000000
#define mmCP_HQD_ACTIVE_DEFAULT                                                  0x00000000
#define mmCP_HQD_VMID_DEFAULT                                                    0x00000000
#define mmCP_HQD_PERSISTENT_STATE_DEFAULT                                        0x0be05301
#define mmCP_HQD_PIPE_PRIORITY_DEFAULT                                           0x00000000
#define mmCP_HQD_QUEUE_PRIORITY_DEFAULT                                          0x00000000
#define mmCP_HQD_QUANTUM_DEFAULT                                                 0x00000000
#define mmCP_HQD_PQ_BASE_DEFAULT                                                 0x00000000
#define mmCP_HQD_PQ_BASE_HI_DEFAULT                                              0x00000000
#define mmCP_HQD_PQ_RPTR_DEFAULT                                                 0x00000000
#define mmCP_HQD_PQ_RPTR_REPORT_ADDR_DEFAULT                                     0x00000000
#define mmCP_HQD_PQ_RPTR_REPORT_ADDR_HI_DEFAULT                                  0x00000000
#define mmCP_HQD_PQ_WPTR_POLL_ADDR_DEFAULT                                       0x00000000
#define mmCP_HQD_PQ_WPTR_POLL_ADDR_HI_DEFAULT                                    0x00000000
#define mmCP_HQD_PQ_DOORBELL_CONTROL_DEFAULT                                     0x00000000
#define mmCP_HQD_PQ_CONTROL_DEFAULT                                              0x00308509
#define mmCP_HQD_IB_BASE_ADDR_DEFAULT                                            0x00000000
#define mmCP_HQD_IB_BASE_ADDR_HI_DEFAULT                                         0x00000000
#define mmCP_HQD_IB_RPTR_DEFAULT                                                 0x00000000
#define mmCP_HQD_IB_CONTROL_DEFAULT                                              0x00300000
#define mmCP_HQD_IQ_TIMER_DEFAULT                                                0x00000000
#define mmCP_HQD_IQ_RPTR_DEFAULT                                                 0x00000000
#define mmCP_HQD_DEQUEUE_REQUEST_DEFAULT                                         0x00000000
#define mmCP_HQD_DMA_OFFLOAD_DEFAULT                                             0x00000000
#define mmCP_HQD_OFFLOAD_DEFAULT                                                 0x00000000
#define mmCP_HQD_SEMA_CMD_DEFAULT                                                0x00000000
#define mmCP_HQD_MSG_TYPE_DEFAULT                                                0x00000000
#define mmCP_HQD_ATOMIC0_PREOP_LO_DEFAULT                                        0x00000000
#define mmCP_HQD_ATOMIC0_PREOP_HI_DEFAULT                                        0x00000000
#define mmCP_HQD_ATOMIC1_PREOP_LO_DEFAULT                                        0x00000000
#define mmCP_HQD_ATOMIC1_PREOP_HI_DEFAULT                                        0x00000000
#define mmCP_HQD_HQ_SCHEDULER0_DEFAULT                                           0x00000000
#define mmCP_HQD_HQ_STATUS0_DEFAULT                                              0x40000000
#define mmCP_HQD_HQ_CONTROL0_DEFAULT                                             0x00000000
#define mmCP_HQD_HQ_SCHEDULER1_DEFAULT                                           0x00000000
#define mmCP_MQD_CONTROL_DEFAULT                                                 0x00000100
#define mmCP_HQD_HQ_STATUS1_DEFAULT                                              0x00000000
#define mmCP_HQD_HQ_CONTROL1_DEFAULT                                             0x00000000
#define mmCP_HQD_EOP_BASE_ADDR_DEFAULT                                           0x00000000
#define mmCP_HQD_EOP_BASE_ADDR_HI_DEFAULT                                        0x00000000
#define mmCP_HQD_EOP_CONTROL_DEFAULT                                             0x00000006
#define mmCP_HQD_EOP_RPTR_DEFAULT                                                0x40000000
#define mmCP_HQD_EOP_WPTR_DEFAULT                                                0x007f8000
#define mmCP_HQD_EOP_EVENTS_DEFAULT                                              0x00000000
#define mmCP_HQD_CTX_SAVE_BASE_ADDR_LO_DEFAULT                                   0x00000000
#define mmCP_HQD_CTX_SAVE_BASE_ADDR_HI_DEFAULT                                   0x00000000
#define mmCP_HQD_CTX_SAVE_CONTROL_DEFAULT                                        0x00000000
#define mmCP_HQD_CNTL_STACK_OFFSET_DEFAULT                                       0x00000000
#define mmCP_HQD_CNTL_STACK_SIZE_DEFAULT                                         0x00000000
#define mmCP_HQD_WG_STATE_OFFSET_DEFAULT                                         0x00000000
#define mmCP_HQD_CTX_SAVE_SIZE_DEFAULT                                           0x00000000
#define mmCP_HQD_GDS_RESOURCE_STATE_DEFAULT                                      0x00000000
#define mmCP_HQD_ERROR_DEFAULT                                                   0x00000000
#define mmCP_HQD_EOP_WPTR_MEM_DEFAULT                                            0x00000000
#define mmCP_HQD_AQL_CONTROL_DEFAULT                                             0x00000000
#define mmCP_HQD_PQ_WPTR_LO_DEFAULT                                              0x00000000
#define mmCP_HQD_PQ_WPTR_HI_DEFAULT                                              0x00000000
#define mmCP_HQD_SUSPEND_CNTL_STACK_OFFSET_DEFAULT                               0x00000000
#define mmCP_HQD_SUSPEND_CNTL_STACK_DW_CNT_DEFAULT                               0x00000000
#define mmCP_HQD_SUSPEND_WG_STATE_OFFSET_DEFAULT                                 0x00000000
#define mmCP_HQD_DDID_RPTR_DEFAULT                                               0x00000000
#define mmCP_HQD_DDID_WPTR_DEFAULT                                               0x00000000
#define mmCP_HQD_DDID_INFLIGHT_COUNT_DEFAULT                                     0x00000000
#define mmCP_HQD_DDID_DELTA_RPT_COUNT_DEFAULT                                    0x00000000
#define mmCP_HQD_DEQUEUE_STATUS_DEFAULT                                          0x00000000


// addressBlock: gc_didtdec
#define mmDIDT_IND_INDEX_DEFAULT                                                 0x00000000
#define mmDIDT_IND_DATA_DEFAULT                                                  0x00000000
#define mmDIDT_INDEX_AUTO_INCR_EN_DEFAULT                                        0x00000000


// addressBlock: gc_gccacdec
#define mmGC_CAC_CTRL_1_DEFAULT                                                  0x01000100
#define mmGC_CAC_CTRL_2_DEFAULT                                                  0x00000000
#define mmGC_CAC_AGGR_LOWER_DEFAULT                                              0x00000000
#define mmGC_CAC_AGGR_UPPER_DEFAULT                                              0x00000000
#define mmGC_CAC_SOFT_CTRL_DEFAULT                                               0x00000000
#define mmGC_EDC_CTRL_DEFAULT                                                    0x00003c00
#define mmGC_EDC_THRESHOLD_DEFAULT                                               0x00000000
#define mmGC_EDC_STATUS_DEFAULT                                                  0x00000000
#define mmGC_EDC_OVERFLOW_DEFAULT                                                0x00000000
#define mmGC_EDC_ROLLING_POWER_DELTA_DEFAULT                                     0x00000000
#define mmGC_THROTTLE_CTRL_DEFAULT                                               0x00002040
#define mmGC_THROTTLE_CTRL1_DEFAULT                                              0x00cc0660
#define mmGC_THROTTLE_STATUS_DEFAULT                                             0x00000000
#define mmEDC_PERF_COUNTER_DEFAULT                                               0x00000000
#define mmPCC_PERF_COUNTER_DEFAULT                                               0x00000000
#define mmPWRBRK_PERF_COUNTER_DEFAULT                                            0x00000000
#define mmGC_EDC_STRETCH_CTRL_DEFAULT                                            0x00000000
#define mmGC_EDC_STRETCH_THRESHOLD_DEFAULT                                       0x00000000
#define mmEDC_HYSTERESIS_CNTL_DEFAULT                                            0x00000001
#define mmEDC_HYSTERESIS_STAT_DEFAULT                                            0x00000000
#define mmGC_CAC_IND_INDEX_DEFAULT                                               0x00000000
#define mmGC_CAC_IND_DATA_DEFAULT                                                0x00000000
#define mmSE_CAC_IND_INDEX_DEFAULT                                               0x00000000
#define mmSE_CAC_IND_DATA_DEFAULT                                                0x00000000


// addressBlock: gc_tcpdec
#define mmTCP_WATCH0_ADDR_H_DEFAULT                                              0x00000000
#define mmTCP_WATCH0_ADDR_L_DEFAULT                                              0x00000000
#define mmTCP_WATCH0_CNTL_DEFAULT                                                0x00000000
#define mmTCP_WATCH1_ADDR_H_DEFAULT                                              0x00000000
#define mmTCP_WATCH1_ADDR_L_DEFAULT                                              0x00000000
#define mmTCP_WATCH1_CNTL_DEFAULT                                                0x00000000
#define mmTCP_WATCH2_ADDR_H_DEFAULT                                              0x00000000
#define mmTCP_WATCH2_ADDR_L_DEFAULT                                              0x00000000
#define mmTCP_WATCH2_CNTL_DEFAULT                                                0x00000000
#define mmTCP_WATCH3_ADDR_H_DEFAULT                                              0x00000000
#define mmTCP_WATCH3_ADDR_L_DEFAULT                                              0x00000000
#define mmTCP_WATCH3_CNTL_DEFAULT                                                0x00000000
#define mmTCP_PERFCOUNTER_FILTER_DEFAULT                                         0x00000000
#define mmTCP_PERFCOUNTER_FILTER_EN_DEFAULT                                      0x00000000
#define mmTCP_PERFCOUNTER_FILTER2_DEFAULT                                        0x00000000


// addressBlock: gc_gdspdec
#define mmGDS_VMID0_BASE_DEFAULT                                                 0x00000000
#define mmGDS_VMID0_SIZE_DEFAULT                                                 0x00010000
#define mmGDS_VMID1_BASE_DEFAULT                                                 0x00000000
#define mmGDS_VMID1_SIZE_DEFAULT                                                 0x00010000
#define mmGDS_VMID2_BASE_DEFAULT                                                 0x00000000
#define mmGDS_VMID2_SIZE_DEFAULT                                                 0x00010000
#define mmGDS_VMID3_BASE_DEFAULT                                                 0x00000000
#define mmGDS_VMID3_SIZE_DEFAULT                                                 0x00010000
#define mmGDS_VMID4_BASE_DEFAULT                                                 0x00000000
#define mmGDS_VMID4_SIZE_DEFAULT                                                 0x00010000
#define mmGDS_VMID5_BASE_DEFAULT                                                 0x00000000
#define mmGDS_VMID5_SIZE_DEFAULT                                                 0x00010000
#define mmGDS_VMID6_BASE_DEFAULT                                                 0x00000000
#define mmGDS_VMID6_SIZE_DEFAULT                                                 0x00010000
#define mmGDS_VMID7_BASE_DEFAULT                                                 0x00000000
#define mmGDS_VMID7_SIZE_DEFAULT                                                 0x00010000
#define mmGDS_VMID8_BASE_DEFAULT                                                 0x00000000
#define mmGDS_VMID8_SIZE_DEFAULT                                                 0x00010000
#define mmGDS_VMID9_BASE_DEFAULT                                                 0x00000000
#define mmGDS_VMID9_SIZE_DEFAULT                                                 0x00010000
#define mmGDS_VMID10_BASE_DEFAULT                                                0x00000000
#define mmGDS_VMID10_SIZE_DEFAULT                                                0x00010000
#define mmGDS_VMID11_BASE_DEFAULT                                                0x00000000
#define mmGDS_VMID11_SIZE_DEFAULT                                                0x00010000
#define mmGDS_VMID12_BASE_DEFAULT                                                0x00000000
#define mmGDS_VMID12_SIZE_DEFAULT                                                0x00010000
#define mmGDS_VMID13_BASE_DEFAULT                                                0x00000000
#define mmGDS_VMID13_SIZE_DEFAULT                                                0x00010000
#define mmGDS_VMID14_BASE_DEFAULT                                                0x00000000
#define mmGDS_VMID14_SIZE_DEFAULT                                                0x00010000
#define mmGDS_VMID15_BASE_DEFAULT                                                0x00000000
#define mmGDS_VMID15_SIZE_DEFAULT                                                0x00010000
#define mmGDS_GWS_VMID0_DEFAULT                                                  0x00400000
#define mmGDS_GWS_VMID1_DEFAULT                                                  0x00400000
#define mmGDS_GWS_VMID2_DEFAULT                                                  0x00400000
#define mmGDS_GWS_VMID3_DEFAULT                                                  0x00400000
#define mmGDS_GWS_VMID4_DEFAULT                                                  0x00400000
#define mmGDS_GWS_VMID5_DEFAULT                                                  0x00400000
#define mmGDS_GWS_VMID6_DEFAULT                                                  0x00400000
#define mmGDS_GWS_VMID7_DEFAULT                                                  0x00400000
#define mmGDS_GWS_VMID8_DEFAULT                                                  0x00400000
#define mmGDS_GWS_VMID9_DEFAULT                                                  0x00400000
#define mmGDS_GWS_VMID10_DEFAULT                                                 0x00400000
#define mmGDS_GWS_VMID11_DEFAULT                                                 0x00400000
#define mmGDS_GWS_VMID12_DEFAULT                                                 0x00400000
#define mmGDS_GWS_VMID13_DEFAULT                                                 0x00400000
#define mmGDS_GWS_VMID14_DEFAULT                                                 0x00400000
#define mmGDS_GWS_VMID15_DEFAULT                                                 0x00400000
#define mmGDS_OA_VMID0_DEFAULT                                                   0x00000000
#define mmGDS_OA_VMID1_DEFAULT                                                   0x00000000
#define mmGDS_OA_VMID2_DEFAULT                                                   0x00000000
#define mmGDS_OA_VMID3_DEFAULT                                                   0x00000000
#define mmGDS_OA_VMID4_DEFAULT                                                   0x00000000
#define mmGDS_OA_VMID5_DEFAULT                                                   0x00000000
#define mmGDS_OA_VMID6_DEFAULT                                                   0x00000000
#define mmGDS_OA_VMID7_DEFAULT                                                   0x00000000
#define mmGDS_OA_VMID8_DEFAULT                                                   0x00000000
#define mmGDS_OA_VMID9_DEFAULT                                                   0x00000000
#define mmGDS_OA_VMID10_DEFAULT                                                  0x00000000
#define mmGDS_OA_VMID11_DEFAULT                                                  0x00000000
#define mmGDS_OA_VMID12_DEFAULT                                                  0x00000000
#define mmGDS_OA_VMID13_DEFAULT                                                  0x00000000
#define mmGDS_OA_VMID14_DEFAULT                                                  0x00000000
#define mmGDS_OA_VMID15_DEFAULT                                                  0x00000000
#define mmGDS_GWS_RESET0_DEFAULT                                                 0x00000000
#define mmGDS_GWS_RESET1_DEFAULT                                                 0x00000000
#define mmGDS_GWS_RESOURCE_RESET_DEFAULT                                         0x00000000
#define mmGDS_COMPUTE_MAX_WAVE_ID_DEFAULT                                        0x000009ff
#define mmGDS_OA_RESET_MASK_DEFAULT                                              0x00000000
#define mmGDS_OA_RESET_DEFAULT                                                   0x00000000
#define mmGDS_ENHANCE2_DEFAULT                                                   0x00000000
#define mmGDS_OA_CGPG_RESTORE_DEFAULT                                            0x00000000
#define mmGDS_CS_CTXSW_STATUS_DEFAULT                                            0x00000000
#define mmGDS_CS_CTXSW_CNT0_DEFAULT                                              0x00000000
#define mmGDS_CS_CTXSW_CNT1_DEFAULT                                              0x00000000
#define mmGDS_CS_CTXSW_CNT2_DEFAULT                                              0x00000000
#define mmGDS_CS_CTXSW_CNT3_DEFAULT                                              0x00000000
#define mmGDS_GFX_CTXSW_STATUS_DEFAULT                                           0x00000000
#define mmGDS_VS_CTXSW_CNT0_DEFAULT                                              0x00000000
#define mmGDS_VS_CTXSW_CNT1_DEFAULT                                              0x00000000
#define mmGDS_VS_CTXSW_CNT2_DEFAULT                                              0x00000000
#define mmGDS_VS_CTXSW_CNT3_DEFAULT                                              0x00000000
#define mmGDS_PS_CTXSW_CNT0_DEFAULT                                              0x00000000
#define mmGDS_PS_CTXSW_CNT1_DEFAULT                                              0x00000000
#define mmGDS_PS_CTXSW_CNT2_DEFAULT                                              0x00000000
#define mmGDS_PS_CTXSW_CNT3_DEFAULT                                              0x00000000
#define mmGDS_PS_CTXSW_IDX_DEFAULT                                               0x00000000
#define mmGDS_GS_CTXSW_CNT0_DEFAULT                                              0x00000000
#define mmGDS_GS_CTXSW_CNT1_DEFAULT                                              0x00000000
#define mmGDS_GS_CTXSW_CNT2_DEFAULT                                              0x00000000
#define mmGDS_GS_CTXSW_CNT3_DEFAULT                                              0x00000000
#define mmGDS_MEMORY_CLEAN_DEFAULT                                               0x00000000


// addressBlock: gc_gfxdec0
#define mmDB_RENDER_CONTROL_DEFAULT                                              0x00000000
#define mmDB_COUNT_CONTROL_DEFAULT                                               0x00000000
#define mmDB_DEPTH_VIEW_DEFAULT                                                  0x00000000
#define mmDB_RENDER_OVERRIDE_DEFAULT                                             0x00000000
#define mmDB_RENDER_OVERRIDE2_DEFAULT                                            0x00000000
#define mmDB_HTILE_DATA_BASE_DEFAULT                                             0x00000000
#define mmDB_DEPTH_SIZE_XY_DEFAULT                                               0x00000000
#define mmDB_DEPTH_BOUNDS_MIN_DEFAULT                                            0x00000000
#define mmDB_DEPTH_BOUNDS_MAX_DEFAULT                                            0x00000000
#define mmDB_STENCIL_CLEAR_DEFAULT                                               0x00000000
#define mmDB_DEPTH_CLEAR_DEFAULT                                                 0x00000000
#define mmPA_SC_SCREEN_SCISSOR_TL_DEFAULT                                        0x00000000
#define mmPA_SC_SCREEN_SCISSOR_BR_DEFAULT                                        0x00000000
#define mmDB_DFSM_CONTROL_DEFAULT                                                0x00000000
#define mmDB_RESERVED_REG_2_DEFAULT                                              0x00000000
#define mmDB_Z_INFO_DEFAULT                                                      0x00000000
#define mmDB_STENCIL_INFO_DEFAULT                                                0x00000000
#define mmDB_Z_READ_BASE_DEFAULT                                                 0x00000000
#define mmDB_STENCIL_READ_BASE_DEFAULT                                           0x00000000
#define mmDB_Z_WRITE_BASE_DEFAULT                                                0x00000000
#define mmDB_STENCIL_WRITE_BASE_DEFAULT                                          0x00000000
#define mmDB_RESERVED_REG_1_DEFAULT                                              0x00000000
#define mmDB_RESERVED_REG_3_DEFAULT                                              0x00000000
#define mmDB_VRS_OVERRIDE_CNTL_DEFAULT                                           0x00000000
#define mmDB_Z_READ_BASE_HI_DEFAULT                                              0x00000000
#define mmDB_STENCIL_READ_BASE_HI_DEFAULT                                        0x00000000
#define mmDB_Z_WRITE_BASE_HI_DEFAULT                                             0x00000000
#define mmDB_STENCIL_WRITE_BASE_HI_DEFAULT                                       0x00000000
#define mmDB_HTILE_DATA_BASE_HI_DEFAULT                                          0x00000000
#define mmDB_RMI_L2_CACHE_CONTROL_DEFAULT                                        0x00000000
#define mmTA_BC_BASE_ADDR_DEFAULT                                                0x00000000
#define mmTA_BC_BASE_ADDR_HI_DEFAULT                                             0x00000000
#define mmCOHER_DEST_BASE_HI_0_DEFAULT                                           0x00000000
#define mmCOHER_DEST_BASE_HI_1_DEFAULT                                           0x00000000
#define mmCOHER_DEST_BASE_HI_2_DEFAULT                                           0x00000000
#define mmCOHER_DEST_BASE_HI_3_DEFAULT                                           0x00000000
#define mmCOHER_DEST_BASE_2_DEFAULT                                              0x00000000
#define mmCOHER_DEST_BASE_3_DEFAULT                                              0x00000000
#define mmPA_SC_WINDOW_OFFSET_DEFAULT                                            0x00000000
#define mmPA_SC_WINDOW_SCISSOR_TL_DEFAULT                                        0x00000000
#define mmPA_SC_WINDOW_SCISSOR_BR_DEFAULT                                        0x00000000
#define mmPA_SC_CLIPRECT_RULE_DEFAULT                                            0x00000000
#define mmPA_SC_CLIPRECT_0_TL_DEFAULT                                            0x00000000
#define mmPA_SC_CLIPRECT_0_BR_DEFAULT                                            0x00000000
#define mmPA_SC_CLIPRECT_1_TL_DEFAULT                                            0x00000000
#define mmPA_SC_CLIPRECT_1_BR_DEFAULT                                            0x00000000
#define mmPA_SC_CLIPRECT_2_TL_DEFAULT                                            0x00000000
#define mmPA_SC_CLIPRECT_2_BR_DEFAULT                                            0x00000000
#define mmPA_SC_CLIPRECT_3_TL_DEFAULT                                            0x00000000
#define mmPA_SC_CLIPRECT_3_BR_DEFAULT                                            0x00000000
#define mmPA_SC_EDGERULE_DEFAULT                                                 0x00000000
#define mmPA_SU_HARDWARE_SCREEN_OFFSET_DEFAULT                                   0x00000000
#define mmCB_TARGET_MASK_DEFAULT                                                 0x00000000
#define mmCB_SHADER_MASK_DEFAULT                                                 0x00000000
#define mmPA_SC_GENERIC_SCISSOR_TL_DEFAULT                                       0x00000000
#define mmPA_SC_GENERIC_SCISSOR_BR_DEFAULT                                       0x00000000
#define mmCOHER_DEST_BASE_0_DEFAULT                                              0x00000000
#define mmCOHER_DEST_BASE_1_DEFAULT                                              0x00000000
#define mmPA_SC_VPORT_SCISSOR_0_TL_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_0_BR_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_1_TL_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_1_BR_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_2_TL_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_2_BR_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_3_TL_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_3_BR_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_4_TL_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_4_BR_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_5_TL_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_5_BR_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_6_TL_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_6_BR_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_7_TL_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_7_BR_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_8_TL_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_8_BR_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_9_TL_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_9_BR_DEFAULT                                       0x00000000
#define mmPA_SC_VPORT_SCISSOR_10_TL_DEFAULT                                      0x00000000
#define mmPA_SC_VPORT_SCISSOR_10_BR_DEFAULT                                      0x00000000
#define mmPA_SC_VPORT_SCISSOR_11_TL_DEFAULT                                      0x00000000
#define mmPA_SC_VPORT_SCISSOR_11_BR_DEFAULT                                      0x00000000
#define mmPA_SC_VPORT_SCISSOR_12_TL_DEFAULT                                      0x00000000
#define mmPA_SC_VPORT_SCISSOR_12_BR_DEFAULT                                      0x00000000
#define mmPA_SC_VPORT_SCISSOR_13_TL_DEFAULT                                      0x00000000
#define mmPA_SC_VPORT_SCISSOR_13_BR_DEFAULT                                      0x00000000
#define mmPA_SC_VPORT_SCISSOR_14_TL_DEFAULT                                      0x00000000
#define mmPA_SC_VPORT_SCISSOR_14_BR_DEFAULT                                      0x00000000
#define mmPA_SC_VPORT_SCISSOR_15_TL_DEFAULT                                      0x00000000
#define mmPA_SC_VPORT_SCISSOR_15_BR_DEFAULT                                      0x00000000
#define mmPA_SC_VPORT_ZMIN_0_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMAX_0_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMIN_1_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMAX_1_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMIN_2_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMAX_2_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMIN_3_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMAX_3_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMIN_4_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMAX_4_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMIN_5_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMAX_5_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMIN_6_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMAX_6_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMIN_7_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMAX_7_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMIN_8_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMAX_8_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMIN_9_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMAX_9_DEFAULT                                             0x00000000
#define mmPA_SC_VPORT_ZMIN_10_DEFAULT                                            0x00000000
#define mmPA_SC_VPORT_ZMAX_10_DEFAULT                                            0x00000000
#define mmPA_SC_VPORT_ZMIN_11_DEFAULT                                            0x00000000
#define mmPA_SC_VPORT_ZMAX_11_DEFAULT                                            0x00000000
#define mmPA_SC_VPORT_ZMIN_12_DEFAULT                                            0x00000000
#define mmPA_SC_VPORT_ZMAX_12_DEFAULT                                            0x00000000
#define mmPA_SC_VPORT_ZMIN_13_DEFAULT                                            0x00000000
#define mmPA_SC_VPORT_ZMAX_13_DEFAULT                                            0x00000000
#define mmPA_SC_VPORT_ZMIN_14_DEFAULT                                            0x00000000
#define mmPA_SC_VPORT_ZMAX_14_DEFAULT                                            0x00000000
#define mmPA_SC_VPORT_ZMIN_15_DEFAULT                                            0x00000000
#define mmPA_SC_VPORT_ZMAX_15_DEFAULT                                            0x00000000
#define mmPA_SC_RASTER_CONFIG_DEFAULT                                            0x2a00126a
#define mmPA_SC_RASTER_CONFIG_1_DEFAULT                                          0x00000000
#define mmPA_SC_SCREEN_EXTENT_CONTROL_DEFAULT                                    0x00000000
#define mmPA_SC_TILE_STEERING_OVERRIDE_DEFAULT                                   0x00000000
#define mmCP_PERFMON_CNTX_CNTL_DEFAULT                                           0x00000000
#define mmCP_PIPEID_DEFAULT                                                      0x00000000
#define mmCP_RINGID_DEFAULT                                                      0x00000000
#define mmCP_VMID_DEFAULT                                                        0x00000000
#define mmCONTEXT_RESERVED_REG0_DEFAULT                                          0x00000000
#define mmCONTEXT_RESERVED_REG1_DEFAULT                                          0x00000000
#define mmVGT_MAX_VTX_INDX_DEFAULT                                               0x00000000
#define mmVGT_MIN_VTX_INDX_DEFAULT                                               0x00000000
#define mmVGT_INDX_OFFSET_DEFAULT                                                0x00000000
#define mmVGT_MULTI_PRIM_IB_RESET_INDX_DEFAULT                                   0x00000000
#define mmCB_RMI_GL2_CACHE_CONTROL_DEFAULT                                       0x00000000
#define mmCB_BLEND_RED_DEFAULT                                                   0x00000000
#define mmCB_BLEND_GREEN_DEFAULT                                                 0x00000000
#define mmCB_BLEND_BLUE_DEFAULT                                                  0x00000000
#define mmCB_BLEND_ALPHA_DEFAULT                                                 0x00000000
#define mmCB_DCC_CONTROL_DEFAULT                                                 0x00000000
#define mmCB_COVERAGE_OUT_CONTROL_DEFAULT                                        0x00000000
#define mmDB_STENCIL_CONTROL_DEFAULT                                             0x00000000
#define mmDB_STENCILREFMASK_DEFAULT                                              0x00000000
#define mmDB_STENCILREFMASK_BF_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_XSCALE_DEFAULT                                             0x00000000
#define mmPA_CL_VPORT_XOFFSET_DEFAULT                                            0x00000000
#define mmPA_CL_VPORT_YSCALE_DEFAULT                                             0x00000000
#define mmPA_CL_VPORT_YOFFSET_DEFAULT                                            0x00000000
#define mmPA_CL_VPORT_ZSCALE_DEFAULT                                             0x00000000
#define mmPA_CL_VPORT_ZOFFSET_DEFAULT                                            0x00000000
#define mmPA_CL_VPORT_XSCALE_1_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_XOFFSET_1_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YSCALE_1_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_YOFFSET_1_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZSCALE_1_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_ZOFFSET_1_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XSCALE_2_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_XOFFSET_2_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YSCALE_2_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_YOFFSET_2_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZSCALE_2_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_ZOFFSET_2_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XSCALE_3_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_XOFFSET_3_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YSCALE_3_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_YOFFSET_3_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZSCALE_3_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_ZOFFSET_3_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XSCALE_4_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_XOFFSET_4_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YSCALE_4_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_YOFFSET_4_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZSCALE_4_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_ZOFFSET_4_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XSCALE_5_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_XOFFSET_5_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YSCALE_5_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_YOFFSET_5_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZSCALE_5_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_ZOFFSET_5_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XSCALE_6_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_XOFFSET_6_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YSCALE_6_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_YOFFSET_6_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZSCALE_6_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_ZOFFSET_6_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XSCALE_7_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_XOFFSET_7_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YSCALE_7_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_YOFFSET_7_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZSCALE_7_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_ZOFFSET_7_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XSCALE_8_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_XOFFSET_8_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YSCALE_8_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_YOFFSET_8_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZSCALE_8_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_ZOFFSET_8_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XSCALE_9_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_XOFFSET_9_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YSCALE_9_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_YOFFSET_9_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZSCALE_9_DEFAULT                                           0x00000000
#define mmPA_CL_VPORT_ZOFFSET_9_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XSCALE_10_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XOFFSET_10_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_YSCALE_10_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YOFFSET_10_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_ZSCALE_10_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZOFFSET_10_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_XSCALE_11_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XOFFSET_11_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_YSCALE_11_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YOFFSET_11_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_ZSCALE_11_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZOFFSET_11_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_XSCALE_12_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XOFFSET_12_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_YSCALE_12_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YOFFSET_12_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_ZSCALE_12_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZOFFSET_12_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_XSCALE_13_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XOFFSET_13_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_YSCALE_13_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YOFFSET_13_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_ZSCALE_13_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZOFFSET_13_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_XSCALE_14_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XOFFSET_14_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_YSCALE_14_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YOFFSET_14_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_ZSCALE_14_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZOFFSET_14_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_XSCALE_15_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_XOFFSET_15_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_YSCALE_15_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_YOFFSET_15_DEFAULT                                         0x00000000
#define mmPA_CL_VPORT_ZSCALE_15_DEFAULT                                          0x00000000
#define mmPA_CL_VPORT_ZOFFSET_15_DEFAULT                                         0x00000000
#define mmPA_CL_UCP_0_X_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_0_Y_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_0_Z_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_0_W_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_1_X_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_1_Y_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_1_Z_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_1_W_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_2_X_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_2_Y_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_2_Z_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_2_W_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_3_X_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_3_Y_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_3_Z_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_3_W_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_4_X_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_4_Y_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_4_Z_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_4_W_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_5_X_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_5_Y_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_5_Z_DEFAULT                                                  0x00000000
#define mmPA_CL_UCP_5_W_DEFAULT                                                  0x00000000
#define mmPA_CL_PROG_NEAR_CLIP_Z_DEFAULT                                         0x00000000
#define mmSPI_PS_INPUT_CNTL_0_DEFAULT                                            0x00000000
#define mmSPI_PS_INPUT_CNTL_1_DEFAULT                                            0x00000000
#define mmSPI_PS_INPUT_CNTL_2_DEFAULT                                            0x00000000
#define mmSPI_PS_INPUT_CNTL_3_DEFAULT                                            0x00000000
#define mmSPI_PS_INPUT_CNTL_4_DEFAULT                                            0x00000000
#define mmSPI_PS_INPUT_CNTL_5_DEFAULT                                            0x00000000
#define mmSPI_PS_INPUT_CNTL_6_DEFAULT                                            0x00000000
#define mmSPI_PS_INPUT_CNTL_7_DEFAULT                                            0x00000000
#define mmSPI_PS_INPUT_CNTL_8_DEFAULT                                            0x00000000
#define mmSPI_PS_INPUT_CNTL_9_DEFAULT                                            0x00000000
#define mmSPI_PS_INPUT_CNTL_10_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_11_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_12_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_13_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_14_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_15_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_16_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_17_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_18_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_19_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_20_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_21_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_22_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_23_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_24_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_25_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_26_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_27_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_28_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_29_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_30_DEFAULT                                           0x00000000
#define mmSPI_PS_INPUT_CNTL_31_DEFAULT                                           0x00000000
#define mmSPI_VS_OUT_CONFIG_DEFAULT                                              0x00000000
#define mmSPI_PS_INPUT_ENA_DEFAULT                                               0x00000000
#define mmSPI_PS_INPUT_ADDR_DEFAULT                                              0x00000000
#define mmSPI_INTERP_CONTROL_0_DEFAULT                                           0x00000000
#define mmSPI_PS_IN_CONTROL_DEFAULT                                              0x00000000
#define mmSPI_BARYC_CNTL_DEFAULT                                                 0x00000000
#define mmSPI_TMPRING_SIZE_DEFAULT                                               0x00000000
#define mmSPI_SHADER_IDX_FORMAT_DEFAULT                                          0x00000000
#define mmSPI_SHADER_POS_FORMAT_DEFAULT                                          0x00000000
#define mmSPI_SHADER_Z_FORMAT_DEFAULT                                            0x00000000
#define mmSPI_SHADER_COL_FORMAT_DEFAULT                                          0x00000000
#define mmSX_PS_DOWNCONVERT_CONTROL_DEFAULT                                      0x00000000
#define mmSX_PS_DOWNCONVERT_DEFAULT                                              0x00000000
#define mmSX_BLEND_OPT_EPSILON_DEFAULT                                           0x00000000
#define mmSX_BLEND_OPT_CONTROL_DEFAULT                                           0x00000000
#define mmSX_MRT0_BLEND_OPT_DEFAULT                                              0x00000000
#define mmSX_MRT1_BLEND_OPT_DEFAULT                                              0x00000000
#define mmSX_MRT2_BLEND_OPT_DEFAULT                                              0x00000000
#define mmSX_MRT3_BLEND_OPT_DEFAULT                                              0x00000000
#define mmSX_MRT4_BLEND_OPT_DEFAULT                                              0x00000000
#define mmSX_MRT5_BLEND_OPT_DEFAULT                                              0x00000000
#define mmSX_MRT6_BLEND_OPT_DEFAULT                                              0x00000000
#define mmSX_MRT7_BLEND_OPT_DEFAULT                                              0x00000000
#define mmCB_BLEND0_CONTROL_DEFAULT                                              0x00000000
#define mmCB_BLEND1_CONTROL_DEFAULT                                              0x00000000
#define mmCB_BLEND2_CONTROL_DEFAULT                                              0x00000000
#define mmCB_BLEND3_CONTROL_DEFAULT                                              0x00000000
#define mmCB_BLEND4_CONTROL_DEFAULT                                              0x00000000
#define mmCB_BLEND5_CONTROL_DEFAULT                                              0x00000000
#define mmCB_BLEND6_CONTROL_DEFAULT                                              0x00000000
#define mmCB_BLEND7_CONTROL_DEFAULT                                              0x00000000
#define mmCS_COPY_STATE_DEFAULT                                                  0x00000000
#define mmGFX_COPY_STATE_DEFAULT                                                 0x00000000
#define mmPA_CL_POINT_X_RAD_DEFAULT                                              0x00000000
#define mmPA_CL_POINT_Y_RAD_DEFAULT                                              0x00000000
#define mmPA_CL_POINT_SIZE_DEFAULT                                               0x00000000
#define mmPA_CL_POINT_CULL_RAD_DEFAULT                                           0x00000000
#define mmVGT_DMA_BASE_HI_DEFAULT                                                0x00000000
#define mmVGT_DMA_BASE_DEFAULT                                                   0x00000000
#define mmVGT_DRAW_INITIATOR_DEFAULT                                             0x00000000
#define mmVGT_IMMED_DATA_DEFAULT                                                 0x00000000
#define mmVGT_EVENT_ADDRESS_REG_DEFAULT                                          0x00000000
#define mmGE_MAX_OUTPUT_PER_SUBGROUP_DEFAULT                                     0x00000000
#define mmDB_DEPTH_CONTROL_DEFAULT                                               0x00000000
#define mmDB_EQAA_DEFAULT                                                        0x00000000
#define mmCB_COLOR_CONTROL_DEFAULT                                               0x00000000
#define mmDB_SHADER_CONTROL_DEFAULT                                              0x00000000
#define mmPA_CL_CLIP_CNTL_DEFAULT                                                0x00000000
#define mmPA_SU_SC_MODE_CNTL_DEFAULT                                             0x00000000
#define mmPA_CL_VTE_CNTL_DEFAULT                                                 0x00000000
#define mmPA_CL_VS_OUT_CNTL_DEFAULT                                              0x00000000
#define mmPA_CL_NANINF_CNTL_DEFAULT                                              0x00000000
#define mmPA_SU_LINE_STIPPLE_CNTL_DEFAULT                                        0x00000000
#define mmPA_SU_LINE_STIPPLE_SCALE_DEFAULT                                       0x00000000
#define mmPA_SU_PRIM_FILTER_CNTL_DEFAULT                                         0x00000000
#define mmPA_SU_SMALL_PRIM_FILTER_CNTL_DEFAULT                                   0x00000000
#define mmPA_CL_NGG_CNTL_DEFAULT                                                 0x00000000
#define mmPA_SU_OVER_RASTERIZATION_CNTL_DEFAULT                                  0x00000000
#define mmPA_STEREO_CNTL_DEFAULT                                                 0x00000000
#define mmPA_STATE_STEREO_X_DEFAULT                                              0x00000000
#define mmPA_CL_VRS_CNTL_DEFAULT                                                 0x00000000
#define mmPA_SU_POINT_SIZE_DEFAULT                                               0x00000000
#define mmPA_SU_POINT_MINMAX_DEFAULT                                             0x00000000
#define mmPA_SU_LINE_CNTL_DEFAULT                                                0x00000000
#define mmPA_SC_LINE_STIPPLE_DEFAULT                                             0x00000000
#define mmVGT_OUTPUT_PATH_CNTL_DEFAULT                                           0x00000000
#define mmVGT_HOS_CNTL_DEFAULT                                                   0x00000000
#define mmVGT_HOS_MAX_TESS_LEVEL_DEFAULT                                         0x00000000
#define mmVGT_HOS_MIN_TESS_LEVEL_DEFAULT                                         0x00000000
#define mmVGT_HOS_REUSE_DEPTH_DEFAULT                                            0x00000000
#define mmVGT_GROUP_PRIM_TYPE_DEFAULT                                            0x00000000
#define mmVGT_GROUP_FIRST_DECR_DEFAULT                                           0x00000000
#define mmVGT_GROUP_DECR_DEFAULT                                                 0x00000000
#define mmVGT_GROUP_VECT_0_CNTL_DEFAULT                                          0x00000000
#define mmVGT_GROUP_VECT_1_CNTL_DEFAULT                                          0x00000000
#define mmVGT_GROUP_VECT_0_FMT_CNTL_DEFAULT                                      0x00000000
#define mmVGT_GROUP_VECT_1_FMT_CNTL_DEFAULT                                      0x00000000
#define mmVGT_GS_MODE_DEFAULT                                                    0x00000000
#define mmVGT_GS_ONCHIP_CNTL_DEFAULT                                             0x00000000
#define mmPA_SC_MODE_CNTL_0_DEFAULT                                              0x00000000
#define mmPA_SC_MODE_CNTL_1_DEFAULT                                              0x06000000
#define mmVGT_ENHANCE_DEFAULT                                                    0x00000000
#define mmVGT_GS_PER_ES_DEFAULT                                                  0x00000000
#define mmVGT_ES_PER_GS_DEFAULT                                                  0x00000000
#define mmVGT_GS_PER_VS_DEFAULT                                                  0x00000000
#define mmVGT_GSVS_RING_OFFSET_1_DEFAULT                                         0x00000000
#define mmVGT_GSVS_RING_OFFSET_2_DEFAULT                                         0x00000000
#define mmVGT_GSVS_RING_OFFSET_3_DEFAULT                                         0x00000000
#define mmVGT_GS_OUT_PRIM_TYPE_DEFAULT                                           0x00000000
#define mmIA_ENHANCE_DEFAULT                                                     0x00000000
#define mmVGT_DMA_SIZE_DEFAULT                                                   0x00000000
#define mmVGT_DMA_MAX_SIZE_DEFAULT                                               0x00000000
#define mmVGT_DMA_INDEX_TYPE_DEFAULT                                             0x00000000
#define mmWD_ENHANCE_DEFAULT                                                     0x00000000
#define mmVGT_PRIMITIVEID_EN_DEFAULT                                             0x00000000
#define mmVGT_DMA_NUM_INSTANCES_DEFAULT                                          0x00000000
#define mmVGT_PRIMITIVEID_RESET_DEFAULT                                          0x00000000
#define mmVGT_EVENT_INITIATOR_DEFAULT                                            0x00000000
#define mmVGT_MULTI_PRIM_IB_RESET_EN_DEFAULT                                     0x00000000
#define mmVGT_DRAW_PAYLOAD_CNTL_DEFAULT                                          0x00000000
#define mmVGT_INSTANCE_STEP_RATE_0_DEFAULT                                       0x00000000
#define mmVGT_INSTANCE_STEP_RATE_1_DEFAULT                                       0x00000000
#define mmIA_MULTI_VGT_PARAM_DEFAULT                                             0x000000ff
#define mmVGT_ESGS_RING_ITEMSIZE_DEFAULT                                         0x00000000
#define mmVGT_GSVS_RING_ITEMSIZE_DEFAULT                                         0x00000000
#define mmVGT_REUSE_OFF_DEFAULT                                                  0x00000000
#define mmVGT_VTX_CNT_EN_DEFAULT                                                 0x00000000
#define mmDB_HTILE_SURFACE_DEFAULT                                               0x00000000
#define mmDB_SRESULTS_COMPARE_STATE0_DEFAULT                                     0x00000000
#define mmDB_SRESULTS_COMPARE_STATE1_DEFAULT                                     0x00000000
#define mmDB_PRELOAD_CONTROL_DEFAULT                                             0x00000000
#define mmVGT_STRMOUT_BUFFER_SIZE_0_DEFAULT                                      0x00000000
#define mmVGT_STRMOUT_VTX_STRIDE_0_DEFAULT                                       0x00000000
#define mmVGT_STRMOUT_BUFFER_OFFSET_0_DEFAULT                                    0x00000000
#define mmVGT_STRMOUT_BUFFER_SIZE_1_DEFAULT                                      0x00000000
#define mmVGT_STRMOUT_VTX_STRIDE_1_DEFAULT                                       0x00000000
#define mmVGT_STRMOUT_BUFFER_OFFSET_1_DEFAULT                                    0x00000000
#define mmVGT_STRMOUT_BUFFER_SIZE_2_DEFAULT                                      0x00000000
#define mmVGT_STRMOUT_VTX_STRIDE_2_DEFAULT                                       0x00000000
#define mmVGT_STRMOUT_BUFFER_OFFSET_2_DEFAULT                                    0x00000000
#define mmVGT_STRMOUT_BUFFER_SIZE_3_DEFAULT                                      0x00000000
#define mmVGT_STRMOUT_VTX_STRIDE_3_DEFAULT                                       0x00000000
#define mmVGT_STRMOUT_BUFFER_OFFSET_3_DEFAULT                                    0x00000000
#define mmVGT_STRMOUT_DRAW_OPAQUE_OFFSET_DEFAULT                                 0x00000000
#define mmVGT_STRMOUT_DRAW_OPAQUE_BUFFER_FILLED_SIZE_DEFAULT                     0x00000000
#define mmVGT_STRMOUT_DRAW_OPAQUE_VERTEX_STRIDE_DEFAULT                          0x00000000
#define mmVGT_GS_MAX_VERT_OUT_DEFAULT                                            0x00000000
#define mmGE_NGG_SUBGRP_CNTL_DEFAULT                                             0x00000000
#define mmVGT_TESS_DISTRIBUTION_DEFAULT                                          0x00000000
#define mmVGT_SHADER_STAGES_EN_DEFAULT                                           0x00000000
#define mmVGT_LS_HS_CONFIG_DEFAULT                                               0x00000000
#define mmVGT_GS_VERT_ITEMSIZE_DEFAULT                                           0x00000000
#define mmVGT_GS_VERT_ITEMSIZE_1_DEFAULT                                         0x00000000
#define mmVGT_GS_VERT_ITEMSIZE_2_DEFAULT                                         0x00000000
#define mmVGT_GS_VERT_ITEMSIZE_3_DEFAULT                                         0x00000000
#define mmVGT_TF_PARAM_DEFAULT                                                   0x00000000
#define mmDB_ALPHA_TO_MASK_DEFAULT                                               0x00000000
#define mmVGT_DISPATCH_DRAW_INDEX_DEFAULT                                        0x00000000
#define mmPA_SU_POLY_OFFSET_DB_FMT_CNTL_DEFAULT                                  0x00000000
#define mmPA_SU_POLY_OFFSET_CLAMP_DEFAULT                                        0x00000000
#define mmPA_SU_POLY_OFFSET_FRONT_SCALE_DEFAULT                                  0x00000000
#define mmPA_SU_POLY_OFFSET_FRONT_OFFSET_DEFAULT                                 0x00000000
#define mmPA_SU_POLY_OFFSET_BACK_SCALE_DEFAULT                                   0x00000000
#define mmPA_SU_POLY_OFFSET_BACK_OFFSET_DEFAULT                                  0x00000000
#define mmVGT_GS_INSTANCE_CNT_DEFAULT                                            0x00000000
#define mmVGT_STRMOUT_CONFIG_DEFAULT                                             0x00000000
#define mmVGT_STRMOUT_BUFFER_CONFIG_DEFAULT                                      0x00000000
#define mmVGT_DMA_EVENT_INITIATOR_DEFAULT                                        0x00000000
#define mmPA_SC_CENTROID_PRIORITY_0_DEFAULT                                      0x00000000
#define mmPA_SC_CENTROID_PRIORITY_1_DEFAULT                                      0x00000000
#define mmPA_SC_LINE_CNTL_DEFAULT                                                0x00000000
#define mmPA_SC_AA_CONFIG_DEFAULT                                                0x00000000
#define mmPA_SU_VTX_CNTL_DEFAULT                                                 0x00000000
#define mmPA_CL_GB_VERT_CLIP_ADJ_DEFAULT                                         0x00000000
#define mmPA_CL_GB_VERT_DISC_ADJ_DEFAULT                                         0x00000000
#define mmPA_CL_GB_HORZ_CLIP_ADJ_DEFAULT                                         0x00000000
#define mmPA_CL_GB_HORZ_DISC_ADJ_DEFAULT                                         0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y0_0_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y0_1_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y0_2_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y0_3_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y0_0_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y0_1_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y0_2_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y0_3_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y1_0_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y1_1_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y1_2_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y1_3_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y1_0_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y1_1_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y1_2_DEFAULT                              0x00000000
#define mmPA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y1_3_DEFAULT                              0x00000000
#define mmPA_SC_AA_MASK_X0Y0_X1Y0_DEFAULT                                        0x00000000
#define mmPA_SC_AA_MASK_X0Y1_X1Y1_DEFAULT                                        0x00000000
#define mmPA_SC_SHADER_CONTROL_DEFAULT                                           0x00000000
#define mmPA_SC_BINNER_CNTL_0_DEFAULT                                            0x00000000
#define mmPA_SC_BINNER_CNTL_1_DEFAULT                                            0x00000000
#define mmPA_SC_CONSERVATIVE_RASTERIZATION_CNTL_DEFAULT                          0x00000000
#define mmPA_SC_NGG_MODE_CNTL_DEFAULT                                            0x00000000
#define mmVGT_VERTEX_REUSE_BLOCK_CNTL_DEFAULT                                    0x00000000
#define mmVGT_OUT_DEALLOC_CNTL_DEFAULT                                           0x00000000
#define mmCB_COLOR0_BASE_DEFAULT                                                 0x00000000
#define mmCB_COLOR0_PITCH_DEFAULT                                                0x00000000
#define mmCB_COLOR0_SLICE_DEFAULT                                                0x00000000
#define mmCB_COLOR0_VIEW_DEFAULT                                                 0x00000000
#define mmCB_COLOR0_INFO_DEFAULT                                                 0x00000000
#define mmCB_COLOR0_ATTRIB_DEFAULT                                               0x00000000
#define mmCB_COLOR0_DCC_CONTROL_DEFAULT                                          0x00000000
#define mmCB_COLOR0_CMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR0_CMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR0_FMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR0_FMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR0_CLEAR_WORD0_DEFAULT                                          0x00000000
#define mmCB_COLOR0_CLEAR_WORD1_DEFAULT                                          0x00000000
#define mmCB_COLOR0_DCC_BASE_DEFAULT                                             0x00000000
#define mmCB_COLOR1_BASE_DEFAULT                                                 0x00000000
#define mmCB_COLOR1_PITCH_DEFAULT                                                0x00000000
#define mmCB_COLOR1_SLICE_DEFAULT                                                0x00000000
#define mmCB_COLOR1_VIEW_DEFAULT                                                 0x00000000
#define mmCB_COLOR1_INFO_DEFAULT                                                 0x00000000
#define mmCB_COLOR1_ATTRIB_DEFAULT                                               0x00000000
#define mmCB_COLOR1_DCC_CONTROL_DEFAULT                                          0x00000000
#define mmCB_COLOR1_CMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR1_CMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR1_FMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR1_FMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR1_CLEAR_WORD0_DEFAULT                                          0x00000000
#define mmCB_COLOR1_CLEAR_WORD1_DEFAULT                                          0x00000000
#define mmCB_COLOR1_DCC_BASE_DEFAULT                                             0x00000000
#define mmCB_COLOR2_BASE_DEFAULT                                                 0x00000000
#define mmCB_COLOR2_PITCH_DEFAULT                                                0x00000000
#define mmCB_COLOR2_SLICE_DEFAULT                                                0x00000000
#define mmCB_COLOR2_VIEW_DEFAULT                                                 0x00000000
#define mmCB_COLOR2_INFO_DEFAULT                                                 0x00000000
#define mmCB_COLOR2_ATTRIB_DEFAULT                                               0x00000000
#define mmCB_COLOR2_DCC_CONTROL_DEFAULT                                          0x00000000
#define mmCB_COLOR2_CMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR2_CMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR2_FMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR2_FMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR2_CLEAR_WORD0_DEFAULT                                          0x00000000
#define mmCB_COLOR2_CLEAR_WORD1_DEFAULT                                          0x00000000
#define mmCB_COLOR2_DCC_BASE_DEFAULT                                             0x00000000
#define mmCB_COLOR3_BASE_DEFAULT                                                 0x00000000
#define mmCB_COLOR3_PITCH_DEFAULT                                                0x00000000
#define mmCB_COLOR3_SLICE_DEFAULT                                                0x00000000
#define mmCB_COLOR3_VIEW_DEFAULT                                                 0x00000000
#define mmCB_COLOR3_INFO_DEFAULT                                                 0x00000000
#define mmCB_COLOR3_ATTRIB_DEFAULT                                               0x00000000
#define mmCB_COLOR3_DCC_CONTROL_DEFAULT                                          0x00000000
#define mmCB_COLOR3_CMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR3_CMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR3_FMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR3_FMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR3_CLEAR_WORD0_DEFAULT                                          0x00000000
#define mmCB_COLOR3_CLEAR_WORD1_DEFAULT                                          0x00000000
#define mmCB_COLOR3_DCC_BASE_DEFAULT                                             0x00000000
#define mmCB_COLOR4_BASE_DEFAULT                                                 0x00000000
#define mmCB_COLOR4_PITCH_DEFAULT                                                0x00000000
#define mmCB_COLOR4_SLICE_DEFAULT                                                0x00000000
#define mmCB_COLOR4_VIEW_DEFAULT                                                 0x00000000
#define mmCB_COLOR4_INFO_DEFAULT                                                 0x00000000
#define mmCB_COLOR4_ATTRIB_DEFAULT                                               0x00000000
#define mmCB_COLOR4_DCC_CONTROL_DEFAULT                                          0x00000000
#define mmCB_COLOR4_CMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR4_CMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR4_FMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR4_FMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR4_CLEAR_WORD0_DEFAULT                                          0x00000000
#define mmCB_COLOR4_CLEAR_WORD1_DEFAULT                                          0x00000000
#define mmCB_COLOR4_DCC_BASE_DEFAULT                                             0x00000000
#define mmCB_COLOR5_BASE_DEFAULT                                                 0x00000000
#define mmCB_COLOR5_PITCH_DEFAULT                                                0x00000000
#define mmCB_COLOR5_SLICE_DEFAULT                                                0x00000000
#define mmCB_COLOR5_VIEW_DEFAULT                                                 0x00000000
#define mmCB_COLOR5_INFO_DEFAULT                                                 0x00000000
#define mmCB_COLOR5_ATTRIB_DEFAULT                                               0x00000000
#define mmCB_COLOR5_DCC_CONTROL_DEFAULT                                          0x00000000
#define mmCB_COLOR5_CMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR5_CMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR5_FMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR5_FMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR5_CLEAR_WORD0_DEFAULT                                          0x00000000
#define mmCB_COLOR5_CLEAR_WORD1_DEFAULT                                          0x00000000
#define mmCB_COLOR5_DCC_BASE_DEFAULT                                             0x00000000
#define mmCB_COLOR6_BASE_DEFAULT                                                 0x00000000
#define mmCB_COLOR6_PITCH_DEFAULT                                                0x00000000
#define mmCB_COLOR6_SLICE_DEFAULT                                                0x00000000
#define mmCB_COLOR6_VIEW_DEFAULT                                                 0x00000000
#define mmCB_COLOR6_INFO_DEFAULT                                                 0x00000000
#define mmCB_COLOR6_ATTRIB_DEFAULT                                               0x00000000
#define mmCB_COLOR6_DCC_CONTROL_DEFAULT                                          0x00000000
#define mmCB_COLOR6_CMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR6_CMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR6_FMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR6_FMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR6_CLEAR_WORD0_DEFAULT                                          0x00000000
#define mmCB_COLOR6_CLEAR_WORD1_DEFAULT                                          0x00000000
#define mmCB_COLOR6_DCC_BASE_DEFAULT                                             0x00000000
#define mmCB_COLOR7_BASE_DEFAULT                                                 0x00000000
#define mmCB_COLOR7_PITCH_DEFAULT                                                0x00000000
#define mmCB_COLOR7_SLICE_DEFAULT                                                0x00000000
#define mmCB_COLOR7_VIEW_DEFAULT                                                 0x00000000
#define mmCB_COLOR7_INFO_DEFAULT                                                 0x00000000
#define mmCB_COLOR7_ATTRIB_DEFAULT                                               0x00000000
#define mmCB_COLOR7_DCC_CONTROL_DEFAULT                                          0x00000000
#define mmCB_COLOR7_CMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR7_CMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR7_FMASK_DEFAULT                                                0x00000000
#define mmCB_COLOR7_FMASK_SLICE_DEFAULT                                          0x00000000
#define mmCB_COLOR7_CLEAR_WORD0_DEFAULT                                          0x00000000
#define mmCB_COLOR7_CLEAR_WORD1_DEFAULT                                          0x00000000
#define mmCB_COLOR7_DCC_BASE_DEFAULT                                             0x00000000
#define mmCB_COLOR0_BASE_EXT_DEFAULT                                             0x00000000
#define mmCB_COLOR1_BASE_EXT_DEFAULT                                             0x00000000
#define mmCB_COLOR2_BASE_EXT_DEFAULT                                             0x00000000
#define mmCB_COLOR3_BASE_EXT_DEFAULT                                             0x00000000
#define mmCB_COLOR4_BASE_EXT_DEFAULT                                             0x00000000
#define mmCB_COLOR5_BASE_EXT_DEFAULT                                             0x00000000
#define mmCB_COLOR6_BASE_EXT_DEFAULT                                             0x00000000
#define mmCB_COLOR7_BASE_EXT_DEFAULT                                             0x00000000
#define mmCB_COLOR0_CMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR1_CMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR2_CMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR3_CMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR4_CMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR5_CMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR6_CMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR7_CMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR0_FMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR1_FMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR2_FMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR3_FMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR4_FMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR5_FMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR6_FMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR7_FMASK_BASE_EXT_DEFAULT                                       0x00000000
#define mmCB_COLOR0_DCC_BASE_EXT_DEFAULT                                         0x00000000
#define mmCB_COLOR1_DCC_BASE_EXT_DEFAULT                                         0x00000000
#define mmCB_COLOR2_DCC_BASE_EXT_DEFAULT                                         0x00000000
#define mmCB_COLOR3_DCC_BASE_EXT_DEFAULT                                         0x00000000
#define mmCB_COLOR4_DCC_BASE_EXT_DEFAULT                                         0x00000000
#define mmCB_COLOR5_DCC_BASE_EXT_DEFAULT                                         0x00000000
#define mmCB_COLOR6_DCC_BASE_EXT_DEFAULT                                         0x00000000
#define mmCB_COLOR7_DCC_BASE_EXT_DEFAULT                                         0x00000000
#define mmCB_COLOR0_ATTRIB2_DEFAULT                                              0x00000000
#define mmCB_COLOR1_ATTRIB2_DEFAULT                                              0x00000000
#define mmCB_COLOR2_ATTRIB2_DEFAULT                                              0x00000000
#define mmCB_COLOR3_ATTRIB2_DEFAULT                                              0x00000000
#define mmCB_COLOR4_ATTRIB2_DEFAULT                                              0x00000000
#define mmCB_COLOR5_ATTRIB2_DEFAULT                                              0x00000000
#define mmCB_COLOR6_ATTRIB2_DEFAULT                                              0x00000000
#define mmCB_COLOR7_ATTRIB2_DEFAULT                                              0x00000000
#define mmCB_COLOR0_ATTRIB3_DEFAULT                                              0x00000000
#define mmCB_COLOR1_ATTRIB3_DEFAULT                                              0x00000000
#define mmCB_COLOR2_ATTRIB3_DEFAULT                                              0x00000000
#define mmCB_COLOR3_ATTRIB3_DEFAULT                                              0x00000000
#define mmCB_COLOR4_ATTRIB3_DEFAULT                                              0x00000000
#define mmCB_COLOR5_ATTRIB3_DEFAULT                                              0x00000000
#define mmCB_COLOR6_ATTRIB3_DEFAULT                                              0x00000000
#define mmCB_COLOR7_ATTRIB3_DEFAULT                                              0x00000000


// addressBlock: gc_gfxudec
#define mmCP_EOP_DONE_ADDR_LO_DEFAULT                                            0x00000000
#define mmCP_EOP_DONE_ADDR_HI_DEFAULT                                            0x00000000
#define mmCP_EOP_DONE_DATA_LO_DEFAULT                                            0x00000000
#define mmCP_EOP_DONE_DATA_HI_DEFAULT                                            0x00000000
#define mmCP_EOP_LAST_FENCE_LO_DEFAULT                                           0x00000000
#define mmCP_EOP_LAST_FENCE_HI_DEFAULT                                           0x00000000
#define mmCP_STREAM_OUT_ADDR_LO_DEFAULT                                          0x00000000
#define mmCP_STREAM_OUT_ADDR_HI_DEFAULT                                          0x00000000
#define mmCP_NUM_PRIM_WRITTEN_COUNT0_LO_DEFAULT                                  0x00000000
#define mmCP_NUM_PRIM_WRITTEN_COUNT0_HI_DEFAULT                                  0x00000000
#define mmCP_NUM_PRIM_NEEDED_COUNT0_LO_DEFAULT                                   0x00000000
#define mmCP_NUM_PRIM_NEEDED_COUNT0_HI_DEFAULT                                   0x00000000
#define mmCP_NUM_PRIM_WRITTEN_COUNT1_LO_DEFAULT                                  0x00000000
#define mmCP_NUM_PRIM_WRITTEN_COUNT1_HI_DEFAULT                                  0x00000000
#define mmCP_NUM_PRIM_NEEDED_COUNT1_LO_DEFAULT                                   0x00000000
#define mmCP_NUM_PRIM_NEEDED_COUNT1_HI_DEFAULT                                   0x00000000
#define mmCP_NUM_PRIM_WRITTEN_COUNT2_LO_DEFAULT                                  0x00000000
#define mmCP_NUM_PRIM_WRITTEN_COUNT2_HI_DEFAULT                                  0x00000000
#define mmCP_NUM_PRIM_NEEDED_COUNT2_LO_DEFAULT                                   0x00000000
#define mmCP_NUM_PRIM_NEEDED_COUNT2_HI_DEFAULT                                   0x00000000
#define mmCP_NUM_PRIM_WRITTEN_COUNT3_LO_DEFAULT                                  0x00000000
#define mmCP_NUM_PRIM_WRITTEN_COUNT3_HI_DEFAULT                                  0x00000000
#define mmCP_NUM_PRIM_NEEDED_COUNT3_LO_DEFAULT                                   0x00000000
#define mmCP_NUM_PRIM_NEEDED_COUNT3_HI_DEFAULT                                   0x00000000
#define mmCP_PIPE_STATS_ADDR_LO_DEFAULT                                          0x00000000
#define mmCP_PIPE_STATS_ADDR_HI_DEFAULT                                          0x00000000
#define mmCP_VGT_IAVERT_COUNT_LO_DEFAULT                                         0x00000000
#define mmCP_VGT_IAVERT_COUNT_HI_DEFAULT                                         0x00000000
#define mmCP_VGT_IAPRIM_COUNT_LO_DEFAULT                                         0x00000000
#define mmCP_VGT_IAPRIM_COUNT_HI_DEFAULT                                         0x00000000
#define mmCP_VGT_GSPRIM_COUNT_LO_DEFAULT                                         0x00000000
#define mmCP_VGT_GSPRIM_COUNT_HI_DEFAULT                                         0x00000000
#define mmCP_VGT_VSINVOC_COUNT_LO_DEFAULT                                        0x00000000
#define mmCP_VGT_VSINVOC_COUNT_HI_DEFAULT                                        0x00000000
#define mmCP_VGT_GSINVOC_COUNT_LO_DEFAULT                                        0x00000000
#define mmCP_VGT_GSINVOC_COUNT_HI_DEFAULT                                        0x00000000
#define mmCP_VGT_HSINVOC_COUNT_LO_DEFAULT                                        0x00000000
#define mmCP_VGT_HSINVOC_COUNT_HI_DEFAULT                                        0x00000000
#define mmCP_VGT_DSINVOC_COUNT_LO_DEFAULT                                        0x00000000
#define mmCP_VGT_DSINVOC_COUNT_HI_DEFAULT                                        0x00000000
#define mmCP_PA_CINVOC_COUNT_LO_DEFAULT                                          0x00000000
#define mmCP_PA_CINVOC_COUNT_HI_DEFAULT                                          0x00000000
#define mmCP_PA_CPRIM_COUNT_LO_DEFAULT                                           0x00000000
#define mmCP_PA_CPRIM_COUNT_HI_DEFAULT                                           0x00000000
#define mmCP_SC_PSINVOC_COUNT0_LO_DEFAULT                                        0x00000000
#define mmCP_SC_PSINVOC_COUNT0_HI_DEFAULT                                        0x00000000
#define mmCP_SC_PSINVOC_COUNT1_LO_DEFAULT                                        0x00000000
#define mmCP_SC_PSINVOC_COUNT1_HI_DEFAULT                                        0x00000000
#define mmCP_VGT_CSINVOC_COUNT_LO_DEFAULT                                        0x00000000
#define mmCP_VGT_CSINVOC_COUNT_HI_DEFAULT                                        0x00000000
#define mmCP_PIPE_STATS_CONTROL_DEFAULT                                          0x00000000
#define mmCP_STREAM_OUT_CONTROL_DEFAULT                                          0x00000000
#define mmCP_STRMOUT_CNTL_DEFAULT                                                0x00000000
#define mmSCRATCH_REG0_DEFAULT                                                   0x00000000
#define mmSCRATCH_REG1_DEFAULT                                                   0x00000000
#define mmSCRATCH_REG2_DEFAULT                                                   0x00000000
#define mmSCRATCH_REG3_DEFAULT                                                   0x00000000
#define mmSCRATCH_REG4_DEFAULT                                                   0x00000000
#define mmSCRATCH_REG5_DEFAULT                                                   0x00000000
#define mmSCRATCH_REG6_DEFAULT                                                   0x00000000
#define mmSCRATCH_REG7_DEFAULT                                                   0x00000000
#define mmSCRATCH_REG_ATOMIC_DEFAULT                                             0x00000000
#define mmSCRATCH_REG_CMPSWAP_ATOMIC_DEFAULT                                     0x00000000
#define mmCP_APPEND_DDID_CNT_DEFAULT                                             0x00000000
#define mmCP_APPEND_DATA_HI_DEFAULT                                              0x00000000
#define mmCP_APPEND_LAST_CS_FENCE_HI_DEFAULT                                     0x00000000
#define mmCP_APPEND_LAST_PS_FENCE_HI_DEFAULT                                     0x00000000
#define mmSCRATCH_UMSK_DEFAULT                                                   0x00000000
#define mmSCRATCH_ADDR_DEFAULT                                                   0x00000000
#define mmCP_PFP_ATOMIC_PREOP_LO_DEFAULT                                         0x00000000
#define mmCP_PFP_ATOMIC_PREOP_HI_DEFAULT                                         0x00000000
#define mmCP_PFP_GDS_ATOMIC0_PREOP_LO_DEFAULT                                    0x00000000
#define mmCP_PFP_GDS_ATOMIC0_PREOP_HI_DEFAULT                                    0x00000000
#define mmCP_PFP_GDS_ATOMIC1_PREOP_LO_DEFAULT                                    0x00000000
#define mmCP_PFP_GDS_ATOMIC1_PREOP_HI_DEFAULT                                    0x00000000
#define mmCP_APPEND_ADDR_LO_DEFAULT                                              0x00000000
#define mmCP_APPEND_ADDR_HI_DEFAULT                                              0x00000000
#define mmCP_APPEND_DATA_DEFAULT                                                 0x00000000
#define mmCP_APPEND_DATA_LO_DEFAULT                                              0x00000000
#define mmCP_APPEND_LAST_CS_FENCE_DEFAULT                                        0x00000000
#define mmCP_APPEND_LAST_CS_FENCE_LO_DEFAULT                                     0x00000000
#define mmCP_APPEND_LAST_PS_FENCE_DEFAULT                                        0x00000000
#define mmCP_APPEND_LAST_PS_FENCE_LO_DEFAULT                                     0x00000000
#define mmCP_ATOMIC_PREOP_LO_DEFAULT                                             0x00000000
#define mmCP_ME_ATOMIC_PREOP_LO_DEFAULT                                          0x00000000
#define mmCP_ATOMIC_PREOP_HI_DEFAULT                                             0x00000000
#define mmCP_ME_ATOMIC_PREOP_HI_DEFAULT                                          0x00000000
#define mmCP_GDS_ATOMIC0_PREOP_LO_DEFAULT                                        0x00000000
#define mmCP_ME_GDS_ATOMIC0_PREOP_LO_DEFAULT                                     0x00000000
#define mmCP_GDS_ATOMIC0_PREOP_HI_DEFAULT                                        0x00000000
#define mmCP_ME_GDS_ATOMIC0_PREOP_HI_DEFAULT                                     0x00000000
#define mmCP_GDS_ATOMIC1_PREOP_LO_DEFAULT                                        0x00000000
#define mmCP_ME_GDS_ATOMIC1_PREOP_LO_DEFAULT                                     0x00000000
#define mmCP_GDS_ATOMIC1_PREOP_HI_DEFAULT                                        0x00000000
#define mmCP_ME_GDS_ATOMIC1_PREOP_HI_DEFAULT                                     0x00000000
#define mmCP_ME_MC_WADDR_LO_DEFAULT                                              0x00000000
#define mmCP_ME_MC_WADDR_HI_DEFAULT                                              0x00000000
#define mmCP_ME_MC_WDATA_LO_DEFAULT                                              0x00000000
#define mmCP_ME_MC_WDATA_HI_DEFAULT                                              0x00000000
#define mmCP_ME_MC_RADDR_LO_DEFAULT                                              0x00000000
#define mmCP_ME_MC_RADDR_HI_DEFAULT                                              0x00000000
#define mmCP_SEM_WAIT_TIMER_DEFAULT                                              0x00000000
#define mmCP_SIG_SEM_ADDR_LO_DEFAULT                                             0x00000000
#define mmCP_SIG_SEM_ADDR_HI_DEFAULT                                             0x00000000
#define mmCP_WAIT_REG_MEM_TIMEOUT_DEFAULT                                        0x00000000
#define mmCP_WAIT_SEM_ADDR_LO_DEFAULT                                            0x00000000
#define mmCP_WAIT_SEM_ADDR_HI_DEFAULT                                            0x00000000
#define mmCP_DMA_PFP_CONTROL_DEFAULT                                             0x00000000
#define mmCP_DMA_ME_CONTROL_DEFAULT                                              0x00000000
#define mmCP_COHER_BASE_HI_DEFAULT                                               0x00000000
#define mmCP_COHER_START_DELAY_DEFAULT                                           0x00000020
#define mmCP_COHER_CNTL_DEFAULT                                                  0x00000000
#define mmCP_COHER_SIZE_DEFAULT                                                  0x00000000
#define mmCP_COHER_BASE_DEFAULT                                                  0x00000000
#define mmCP_COHER_STATUS_DEFAULT                                                0x00000000
#define mmCP_DMA_ME_SRC_ADDR_DEFAULT                                             0x00000000
#define mmCP_DMA_ME_SRC_ADDR_HI_DEFAULT                                          0x00000000
#define mmCP_DMA_ME_DST_ADDR_DEFAULT                                             0x00000000
#define mmCP_DMA_ME_DST_ADDR_HI_DEFAULT                                          0x00000000
#define mmCP_DMA_ME_COMMAND_DEFAULT                                              0x00000000
#define mmCP_DMA_PFP_SRC_ADDR_DEFAULT                                            0x00000000
#define mmCP_DMA_PFP_SRC_ADDR_HI_DEFAULT                                         0x00000000
#define mmCP_DMA_PFP_DST_ADDR_DEFAULT                                            0x00000000
#define mmCP_DMA_PFP_DST_ADDR_HI_DEFAULT                                         0x00000000
#define mmCP_DMA_PFP_COMMAND_DEFAULT                                             0x00000000
#define mmCP_DMA_CNTL_DEFAULT                                                    0x00100020
#define mmCP_DMA_READ_TAGS_DEFAULT                                               0x00000000
#define mmCP_COHER_SIZE_HI_DEFAULT                                               0x00000000
#define mmCP_PFP_IB_CONTROL_DEFAULT                                              0x00000000
#define mmCP_PFP_LOAD_CONTROL_DEFAULT                                            0x00000000
#define mmCP_SCRATCH_INDEX_DEFAULT                                               0x00000000
#define mmCP_SCRATCH_DATA_DEFAULT                                                0x00000000
#define mmCP_RB_OFFSET_DEFAULT                                                   0x00000000
#define mmCP_IB2_OFFSET_DEFAULT                                                  0x00000000
#define mmCP_IB2_PREAMBLE_BEGIN_DEFAULT                                          0x00000000
#define mmCP_IB2_PREAMBLE_END_DEFAULT                                            0x00000000
#define mmCP_CE_IB1_OFFSET_DEFAULT                                               0x00000000
#define mmCP_CE_IB2_OFFSET_DEFAULT                                               0x00000000
#define mmCP_CE_COUNTER_DEFAULT                                                  0x00000000
#define mmCP_DMA_ME_CMD_ADDR_LO_DEFAULT                                          0x00000000
#define mmCP_DMA_ME_CMD_ADDR_HI_DEFAULT                                          0x00000000
#define mmCP_DMA_PFP_CMD_ADDR_LO_DEFAULT                                         0x00000000
#define mmCP_DMA_PFP_CMD_ADDR_HI_DEFAULT                                         0x00000000
#define mmCP_APPEND_CMD_ADDR_LO_DEFAULT                                          0x00000000
#define mmCP_APPEND_CMD_ADDR_HI_DEFAULT                                          0x00000000
#define mmUCONFIG_RESERVED_REG0_DEFAULT                                          0x00000000
#define mmUCONFIG_RESERVED_REG1_DEFAULT                                          0x00000000
#define mmCP_CE_ATOMIC_PREOP_LO_DEFAULT                                          0x00000000
#define mmCP_CE_ATOMIC_PREOP_HI_DEFAULT                                          0x00000000
#define mmCP_CE_GDS_ATOMIC0_PREOP_LO_DEFAULT                                     0x00000000
#define mmCP_CE_GDS_ATOMIC0_PREOP_HI_DEFAULT                                     0x00000000
#define mmCP_CE_GDS_ATOMIC1_PREOP_LO_DEFAULT                                     0x00000000
#define mmCP_CE_GDS_ATOMIC1_PREOP_HI_DEFAULT                                     0x00000000
#define mmCP_CE_INIT_CMD_BUFSZ_DEFAULT                                           0x00000000
#define mmCP_CE_IB1_CMD_BUFSZ_DEFAULT                                            0x00000000
#define mmCP_CE_IB2_CMD_BUFSZ_DEFAULT                                            0x00000000
#define mmCP_IB2_CMD_BUFSZ_DEFAULT                                               0x00000000
#define mmCP_ST_CMD_BUFSZ_DEFAULT                                                0x00000000
#define mmCP_CE_INIT_BASE_LO_DEFAULT                                             0x00000000
#define mmCP_CE_INIT_BASE_HI_DEFAULT                                             0x00000000
#define mmCP_CE_INIT_BUFSZ_DEFAULT                                               0x00000000
#define mmCP_CE_IB1_BASE_LO_DEFAULT                                              0x00000000
#define mmCP_CE_IB1_BASE_HI_DEFAULT                                              0x00000000
#define mmCP_CE_IB1_BUFSZ_DEFAULT                                                0x00000000
#define mmCP_CE_IB2_BASE_LO_DEFAULT                                              0x00000000
#define mmCP_CE_IB2_BASE_HI_DEFAULT                                              0x00000000
#define mmCP_CE_IB2_BUFSZ_DEFAULT                                                0x00000000
#define mmCP_IB2_BASE_LO_DEFAULT                                                 0x00000000
#define mmCP_IB2_BASE_HI_DEFAULT                                                 0x00000000
#define mmCP_IB2_BUFSZ_DEFAULT                                                   0x00000000
#define mmCP_ST_BASE_LO_DEFAULT                                                  0x00000000
#define mmCP_ST_BASE_HI_DEFAULT                                                  0x00000000
#define mmCP_ST_BUFSZ_DEFAULT                                                    0x00000000
#define mmCP_EOP_DONE_EVENT_CNTL_DEFAULT                                         0x00000000
#define mmCP_EOP_DONE_DATA_CNTL_DEFAULT                                          0x00000000
#define mmCP_EOP_DONE_CNTX_ID_DEFAULT                                            0x00000000
#define mmCP_DB_BASE_LO_DEFAULT                                                  0x00000000
#define mmCP_DB_BASE_HI_DEFAULT                                                  0x00000000
#define mmCP_DB_BUFSZ_DEFAULT                                                    0x00000000
#define mmCP_DB_CMD_BUFSZ_DEFAULT                                                0x00000000
#define mmCP_CE_DB_BASE_LO_DEFAULT                                               0x00000000
#define mmCP_CE_DB_BASE_HI_DEFAULT                                               0x00000000
#define mmCP_CE_DB_BUFSZ_DEFAULT                                                 0x00000000
#define mmCP_CE_DB_CMD_BUFSZ_DEFAULT                                             0x00000000
#define mmCP_PFP_COMPLETION_STATUS_DEFAULT                                       0x00000000
#define mmCP_CE_COMPLETION_STATUS_DEFAULT                                        0x00000000
#define mmCP_PRED_NOT_VISIBLE_DEFAULT                                            0x00000000
#define mmCP_PFP_METADATA_BASE_ADDR_DEFAULT                                      0x00000000
#define mmCP_PFP_METADATA_BASE_ADDR_HI_DEFAULT                                   0x00000000
#define mmCP_CE_METADATA_BASE_ADDR_DEFAULT                                       0x00000000
#define mmCP_CE_METADATA_BASE_ADDR_HI_DEFAULT                                    0x00000000
#define mmCP_DRAW_INDX_INDR_ADDR_DEFAULT                                         0x00000000
#define mmCP_DRAW_INDX_INDR_ADDR_HI_DEFAULT                                      0x00000000
#define mmCP_DISPATCH_INDR_ADDR_DEFAULT                                          0x00000000
#define mmCP_DISPATCH_INDR_ADDR_HI_DEFAULT                                       0x00000000
#define mmCP_INDEX_BASE_ADDR_DEFAULT                                             0x00000000
#define mmCP_INDEX_BASE_ADDR_HI_DEFAULT                                          0x00000000
#define mmCP_INDEX_TYPE_DEFAULT                                                  0x00000000
#define mmCP_GDS_BKUP_ADDR_DEFAULT                                               0x00000000
#define mmCP_GDS_BKUP_ADDR_HI_DEFAULT                                            0x00000000
#define mmCP_SAMPLE_STATUS_DEFAULT                                               0x00000000
#define mmCP_ME_COHER_CNTL_DEFAULT                                               0x00000000
#define mmCP_ME_COHER_SIZE_DEFAULT                                               0x00000000
#define mmCP_ME_COHER_SIZE_HI_DEFAULT                                            0x00000000
#define mmCP_ME_COHER_BASE_DEFAULT                                               0x00000000
#define mmCP_ME_COHER_BASE_HI_DEFAULT                                            0x00000000
#define mmCP_ME_COHER_STATUS_DEFAULT                                             0x00000000
#define mmRLC_GPM_PERF_COUNT_0_DEFAULT                                           0x00000000
#define mmRLC_GPM_PERF_COUNT_1_DEFAULT                                           0x00000000
#define mmGRBM_GFX_INDEX_DEFAULT                                                 0xe0000000
#define mmVGT_ESGS_RING_SIZE_UMD_DEFAULT                                         0x00000000
#define mmVGT_GSVS_RING_SIZE_UMD_DEFAULT                                         0x00000000
#define mmVGT_PRIMITIVE_TYPE_DEFAULT                                             0x00000000
#define mmVGT_INDEX_TYPE_DEFAULT                                                 0x00000000
#define mmVGT_STRMOUT_BUFFER_FILLED_SIZE_0_DEFAULT                               0x00000000
#define mmVGT_STRMOUT_BUFFER_FILLED_SIZE_1_DEFAULT                               0x00000000
#define mmVGT_STRMOUT_BUFFER_FILLED_SIZE_2_DEFAULT                               0x00000000
#define mmVGT_STRMOUT_BUFFER_FILLED_SIZE_3_DEFAULT                               0x00000000
#define mmGE_MIN_VTX_INDX_DEFAULT                                                0x00000000
#define mmGE_INDX_OFFSET_DEFAULT                                                 0x00000000
#define mmGE_MULTI_PRIM_IB_RESET_EN_DEFAULT                                      0x00000000
#define mmVGT_NUM_INDICES_DEFAULT                                                0x00000000
#define mmVGT_NUM_INSTANCES_DEFAULT                                              0x00000000
#define mmVGT_TF_RING_SIZE_UMD_DEFAULT                                           0x0000c000
#define mmVGT_HS_OFFCHIP_PARAM_UMD_DEFAULT                                       0x00000000
#define mmVGT_TF_MEMORY_BASE_UMD_DEFAULT                                         0x00000000
#define mmGE_DMA_FIRST_INDEX_DEFAULT                                             0x00000000
#define mmWD_POS_BUF_BASE_DEFAULT                                                0x00000000
#define mmWD_POS_BUF_BASE_HI_DEFAULT                                             0x00000000
#define mmWD_CNTL_SB_BUF_BASE_DEFAULT                                            0x00000000
#define mmWD_CNTL_SB_BUF_BASE_HI_DEFAULT                                         0x00000000
#define mmWD_INDEX_BUF_BASE_DEFAULT                                              0x00000000
#define mmWD_INDEX_BUF_BASE_HI_DEFAULT                                           0x00000000
#define mmIA_MULTI_VGT_PARAM_PIPED_DEFAULT                                       0x006000ff
#define mmGE_MAX_VTX_INDX_DEFAULT                                                0x00000000
#define mmVGT_INSTANCE_BASE_ID_DEFAULT                                           0x00000000
#define mmGE_CNTL_DEFAULT                                                        0x00000000
#define mmGE_USER_VGPR1_DEFAULT                                                  0x00000000
#define mmGE_USER_VGPR2_DEFAULT                                                  0x00000000
#define mmGE_USER_VGPR3_DEFAULT                                                  0x00000000
#define mmGE_STEREO_CNTL_DEFAULT                                                 0x00000000
#define mmGE_PC_ALLOC_DEFAULT                                                    0x00000000
#define mmVGT_TF_MEMORY_BASE_HI_UMD_DEFAULT                                      0x00000000
#define mmGE_USER_VGPR_EN_DEFAULT                                                0x00000000
#define mmPA_SU_LINE_STIPPLE_VALUE_DEFAULT                                       0x00000000
#define mmPA_SC_LINE_STIPPLE_STATE_DEFAULT                                       0x00000000
#define mmPA_SC_SCREEN_EXTENT_MIN_0_DEFAULT                                      0x7fff7fff
#define mmPA_SC_SCREEN_EXTENT_MAX_0_DEFAULT                                      0x80008000
#define mmPA_SC_SCREEN_EXTENT_MIN_1_DEFAULT                                      0x7fff7fff
#define mmPA_SC_SCREEN_EXTENT_MAX_1_DEFAULT                                      0x80008000
#define mmPA_SC_P3D_TRAP_SCREEN_HV_EN_DEFAULT                                    0x00000000
#define mmPA_SC_P3D_TRAP_SCREEN_H_DEFAULT                                        0x00000000
#define mmPA_SC_P3D_TRAP_SCREEN_V_DEFAULT                                        0x00000000
#define mmPA_SC_P3D_TRAP_SCREEN_OCCURRENCE_DEFAULT                               0x00000000
#define mmPA_SC_P3D_TRAP_SCREEN_COUNT_DEFAULT                                    0x00000000
#define mmPA_SC_HP3D_TRAP_SCREEN_HV_EN_DEFAULT                                   0x00000000
#define mmPA_SC_HP3D_TRAP_SCREEN_H_DEFAULT                                       0x00000000
#define mmPA_SC_HP3D_TRAP_SCREEN_V_DEFAULT                                       0x00000000
#define mmPA_SC_HP3D_TRAP_SCREEN_OCCURRENCE_DEFAULT                              0x00000000
#define mmPA_SC_HP3D_TRAP_SCREEN_COUNT_DEFAULT                                   0x00000000
#define mmPA_SC_TRAP_SCREEN_HV_EN_DEFAULT                                        0x00000000
#define mmPA_SC_TRAP_SCREEN_H_DEFAULT                                            0x00000000
#define mmPA_SC_TRAP_SCREEN_V_DEFAULT                                            0x00000000
#define mmPA_SC_TRAP_SCREEN_OCCURRENCE_DEFAULT                                   0x00000000
#define mmPA_SC_TRAP_SCREEN_COUNT_DEFAULT                                        0x00000000
#define mmSQ_THREAD_TRACE_USERDATA_0_DEFAULT                                     0x00000000
#define mmSQ_THREAD_TRACE_USERDATA_1_DEFAULT                                     0x00000000
#define mmSQ_THREAD_TRACE_USERDATA_2_DEFAULT                                     0x00000000
#define mmSQ_THREAD_TRACE_USERDATA_3_DEFAULT                                     0x00000000
#define mmSQ_THREAD_TRACE_USERDATA_4_DEFAULT                                     0x00000000
#define mmSQ_THREAD_TRACE_USERDATA_5_DEFAULT                                     0x00000000
#define mmSQ_THREAD_TRACE_USERDATA_6_DEFAULT                                     0x00000000
#define mmSQ_THREAD_TRACE_USERDATA_7_DEFAULT                                     0x00000000
#define mmSQC_CACHES_DEFAULT                                                     0x00000000
#define mmTA_CS_BC_BASE_ADDR_DEFAULT                                             0x00000000
#define mmTA_CS_BC_BASE_ADDR_HI_DEFAULT                                          0x00000000
#define mmDB_OCCLUSION_COUNT0_LOW_DEFAULT                                        0x00000000
#define mmDB_OCCLUSION_COUNT0_HI_DEFAULT                                         0x00000000
#define mmDB_OCCLUSION_COUNT1_LOW_DEFAULT                                        0x00000000
#define mmDB_OCCLUSION_COUNT1_HI_DEFAULT                                         0x00000000
#define mmDB_OCCLUSION_COUNT2_LOW_DEFAULT                                        0x00000000
#define mmDB_OCCLUSION_COUNT2_HI_DEFAULT                                         0x00000000
#define mmDB_OCCLUSION_COUNT3_LOW_DEFAULT                                        0x00000000
#define mmDB_OCCLUSION_COUNT3_HI_DEFAULT                                         0x00000000
#define mmDB_ZPASS_COUNT_LOW_DEFAULT                                             0x00000000
#define mmDB_ZPASS_COUNT_HI_DEFAULT                                              0x00000000
#define mmGDS_RD_ADDR_DEFAULT                                                    0x00000000
#define mmGDS_RD_DATA_DEFAULT                                                    0x00000000
#define mmGDS_RD_BURST_ADDR_DEFAULT                                              0x00000000
#define mmGDS_RD_BURST_COUNT_DEFAULT                                             0x00000000
#define mmGDS_RD_BURST_DATA_DEFAULT                                              0x00000000
#define mmGDS_WR_ADDR_DEFAULT                                                    0x00000000
#define mmGDS_WR_DATA_DEFAULT                                                    0x00000000
#define mmGDS_WR_BURST_ADDR_DEFAULT                                              0x00000000
#define mmGDS_WR_BURST_DATA_DEFAULT                                              0x00000000
#define mmGDS_WRITE_COMPLETE_DEFAULT                                             0x00000000
#define mmGDS_ATOM_CNTL_DEFAULT                                                  0x00000000
#define mmGDS_ATOM_COMPLETE_DEFAULT                                              0x00000001
#define mmGDS_ATOM_BASE_DEFAULT                                                  0x00000000
#define mmGDS_ATOM_SIZE_DEFAULT                                                  0x00000000
#define mmGDS_ATOM_OFFSET0_DEFAULT                                               0x00000000
#define mmGDS_ATOM_OFFSET1_DEFAULT                                               0x00000000
#define mmGDS_ATOM_DST_DEFAULT                                                   0x00000000
#define mmGDS_ATOM_OP_DEFAULT                                                    0x00000000
#define mmGDS_ATOM_SRC0_DEFAULT                                                  0x00000000
#define mmGDS_ATOM_SRC0_U_DEFAULT                                                0x00000000
#define mmGDS_ATOM_SRC1_DEFAULT                                                  0x00000000
#define mmGDS_ATOM_SRC1_U_DEFAULT                                                0x00000000
#define mmGDS_ATOM_READ0_DEFAULT                                                 0x00000000
#define mmGDS_ATOM_READ0_U_DEFAULT                                               0x00000000
#define mmGDS_ATOM_READ1_DEFAULT                                                 0x00000000
#define mmGDS_ATOM_READ1_U_DEFAULT                                               0x00000000
#define mmGDS_GWS_RESOURCE_CNTL_DEFAULT                                          0x00000000
#define mmGDS_GWS_RESOURCE_DEFAULT                                               0x00000000
#define mmGDS_GWS_RESOURCE_CNT_DEFAULT                                           0x00000000
#define mmGDS_OA_CNTL_DEFAULT                                                    0x00000000
#define mmGDS_OA_COUNTER_DEFAULT                                                 0x00000000
#define mmGDS_OA_ADDRESS_DEFAULT                                                 0x00000000
#define mmGDS_OA_INCDEC_DEFAULT                                                  0x00000000
#define mmGDS_OA_RING_SIZE_DEFAULT                                               0x00000000
#define mmSPI_CONFIG_CNTL_REMAP_DEFAULT                                          0x00000000
#define mmSPI_CONFIG_CNTL_1_REMAP_DEFAULT                                        0x00000000
#define mmSPI_CONFIG_CNTL_2_REMAP_DEFAULT                                        0x00000000
#define mmSPI_WAVE_LIMIT_CNTL_REMAP_DEFAULT                                      0x00000000


// addressBlock: gc_cprs64dec
#define mmCP_MES_PRGRM_CNTR_START_DEFAULT                                        0x00000800
#define mmCP_MES_INTR_ROUTINE_START_DEFAULT                                      0x00000000
#define mmCP_MES_MTVEC_LO_DEFAULT                                                0x00000000
#define mmCP_MES_MTVEC_HI_DEFAULT                                                0x00000000
#define mmCP_MES_CNTL_DEFAULT                                                    0x40000000
#define mmCP_MES_PIPE_PRIORITY_CNTS_DEFAULT                                      0x08081020
#define mmCP_MES_PIPE0_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_MES_PIPE1_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_MES_PIPE2_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_MES_PIPE3_PRIORITY_DEFAULT                                          0x00000002
#define mmCP_MES_HEADER_DUMP_DEFAULT                                             0x00000000
#define mmCP_MES_MIE_LO_DEFAULT                                                  0x00000000
#define mmCP_MES_MIE_HI_DEFAULT                                                  0x00000000
#define mmCP_MES_INTERRUPT_DEFAULT                                               0x00000000
#define mmCP_MES_SCRATCH_INDEX_DEFAULT                                           0x00000000
#define mmCP_MES_SCRATCH_DATA_DEFAULT                                            0x00000000
#define mmCP_MES_INSTR_PNTR_DEFAULT                                              0x00000000
#define mmCP_MES_MSCRATCH_HI_DEFAULT                                             0x00000000
#define mmCP_MES_MSCRATCH_LO_DEFAULT                                             0x00000000
#define mmCP_MES_MSTATUS_LO_DEFAULT                                              0x00000000
#define mmCP_MES_MSTATUS_HI_DEFAULT                                              0x00000000
#define mmCP_MES_MEPC_LO_DEFAULT                                                 0x00000000
#define mmCP_MES_MEPC_HI_DEFAULT                                                 0x00000000
#define mmCP_MES_MCAUSE_LO_DEFAULT                                               0x00000000
#define mmCP_MES_MCAUSE_HI_DEFAULT                                               0x00000000
#define mmCP_MES_MBADADDR_LO_DEFAULT                                             0x00000000
#define mmCP_MES_MBADADDR_HI_DEFAULT                                             0x00000000
#define mmCP_MES_MIP_LO_DEFAULT                                                  0x00000000
#define mmCP_MES_MIP_HI_DEFAULT                                                  0x00000000
#define mmCP_MES_IC_OP_CNTL_DEFAULT                                              0x00000000
#define mmCP_MES_MCYCLE_LO_DEFAULT                                               0x00000000
#define mmCP_MES_MCYCLE_HI_DEFAULT                                               0x00000000
#define mmCP_MES_MTIME_LO_DEFAULT                                                0x00000000
#define mmCP_MES_MTIME_HI_DEFAULT                                                0x00000000
#define mmCP_MES_MINSTRET_LO_DEFAULT                                             0x00000000
#define mmCP_MES_MINSTRET_HI_DEFAULT                                             0x00000000
#define mmCP_MES_MISA_LO_DEFAULT                                                 0x00000000
#define mmCP_MES_MISA_HI_DEFAULT                                                 0x00000000
#define mmCP_MES_MVENDORID_LO_DEFAULT                                            0x00000000
#define mmCP_MES_MVENDORID_HI_DEFAULT                                            0x00000000
#define mmCP_MES_MARCHID_LO_DEFAULT                                              0x00000000
#define mmCP_MES_MARCHID_HI_DEFAULT                                              0x00000000
#define mmCP_MES_MIMPID_LO_DEFAULT                                               0x00000000
#define mmCP_MES_MIMPID_HI_DEFAULT                                               0x00000000
#define mmCP_MES_MHARTID_LO_DEFAULT                                              0x00000000
#define mmCP_MES_MHARTID_HI_DEFAULT                                              0x00000000
#define mmCP_MES_DC_BASE_CNTL_DEFAULT                                            0x00000000
#define mmCP_MES_DC_OP_CNTL_DEFAULT                                              0x00000000
#define mmCP_MES_MTIMECMP_LO_DEFAULT                                             0x00000000
#define mmCP_MES_MTIMECMP_HI_DEFAULT                                             0x00000000
#define mmCP_MES_PROCESS_QUANTUM_PIPE0_DEFAULT                                   0x00000008
#define mmCP_MES_PROCESS_QUANTUM_PIPE1_DEFAULT                                   0x00000008
#define mmCP_MES_DOORBELL_CONTROL1_DEFAULT                                       0x00000000
#define mmCP_MES_DOORBELL_CONTROL2_DEFAULT                                       0x00000000
#define mmCP_MES_DOORBELL_CONTROL3_DEFAULT                                       0x00000000
#define mmCP_MES_DOORBELL_CONTROL4_DEFAULT                                       0x00000000
#define mmCP_MES_DOORBELL_CONTROL5_DEFAULT                                       0x00000000
#define mmCP_MES_DOORBELL_CONTROL6_DEFAULT                                       0x00000000
#define mmCP_MES_GP0_LO_DEFAULT                                                  0x00000000
#define mmCP_MES_GP0_HI_DEFAULT                                                  0x00000000
#define mmCP_MES_GP1_LO_DEFAULT                                                  0x00002001
#define mmCP_MES_GP1_HI_DEFAULT                                                  0x00000000
#define mmCP_MES_GP2_LO_DEFAULT                                                  0x00000000
#define mmCP_MES_GP2_HI_DEFAULT                                                  0x00000000
#define mmCP_MES_GP3_LO_DEFAULT                                                  0x00000000
#define mmCP_MES_GP3_HI_DEFAULT                                                  0x00000000
#define mmCP_MES_GP4_LO_DEFAULT                                                  0x00000000
#define mmCP_MES_GP4_HI_DEFAULT                                                  0x00000000
#define mmCP_MES_GP5_LO_DEFAULT                                                  0x00000000
#define mmCP_MES_GP5_HI_DEFAULT                                                  0x00000000
#define mmCP_MES_GP6_LO_DEFAULT                                                  0x00000000
#define mmCP_MES_GP6_HI_DEFAULT                                                  0x00000000
#define mmCP_MES_GP7_LO_DEFAULT                                                  0x00000000
#define mmCP_MES_GP7_HI_DEFAULT                                                  0x00000000
#define mmCP_MES_GP8_LO_DEFAULT                                                  0x00000000
#define mmCP_MES_GP8_HI_DEFAULT                                                  0x00000000
#define mmCP_MES_GP9_LO_DEFAULT                                                  0x40000000
#define mmCP_MES_GP9_HI_DEFAULT                                                  0x40000000
#define mmCP_MES_DM_INDEX_ADDR_DEFAULT                                           0x00000000
#define mmCP_MES_DM_INDEX_DATA_DEFAULT                                           0x00000000
#define mmCP_MES_PERFCOUNT_CNTL_DEFAULT                                          0x00000000
#define mmCP_MES_PENDING_INTERRUPT_DEFAULT                                       0x00000000


// addressBlock: gc_gusdec
#define mmGUS_IO_RD_COMBINE_FLUSH_DEFAULT                                        0x00000000
#define mmGUS_IO_WR_COMBINE_FLUSH_DEFAULT                                        0x00000000
#define mmGUS_IO_RD_PRI_AGE_RATE_DEFAULT                                         0x00000000
#define mmGUS_IO_WR_PRI_AGE_RATE_DEFAULT                                         0x00000000
#define mmGUS_IO_RD_PRI_AGE_COEFF_DEFAULT                                        0x0003ffff
#define mmGUS_IO_WR_PRI_AGE_COEFF_DEFAULT                                        0x0003ffff
#define mmGUS_IO_RD_PRI_QUEUING_DEFAULT                                          0x0003ffff
#define mmGUS_IO_WR_PRI_QUEUING_DEFAULT                                          0x0003ffff
#define mmGUS_IO_RD_PRI_FIXED_DEFAULT                                            0x00000000
#define mmGUS_IO_WR_PRI_FIXED_DEFAULT                                            0x00000000
#define mmGUS_IO_RD_PRI_URGENCY_COEFF_DEFAULT                                    0x00000000
#define mmGUS_IO_WR_PRI_URGENCY_COEFF_DEFAULT                                    0x00000000
#define mmGUS_IO_RD_PRI_URGENCY_MODE_DEFAULT                                     0x00000000
#define mmGUS_IO_WR_PRI_URGENCY_MODE_DEFAULT                                     0x00000000
#define mmGUS_IO_RD_PRI_QUANT_PRI1_DEFAULT                                       0x1f1f1f1f
#define mmGUS_IO_RD_PRI_QUANT_PRI2_DEFAULT                                       0x3f3f3f3f
#define mmGUS_IO_RD_PRI_QUANT_PRI3_DEFAULT                                       0x7f7f7f7f
#define mmGUS_IO_RD_PRI_QUANT_PRI4_DEFAULT                                       0xffffffff
#define mmGUS_IO_WR_PRI_QUANT_PRI1_DEFAULT                                       0x1f1f1f1f
#define mmGUS_IO_WR_PRI_QUANT_PRI2_DEFAULT                                       0x3f3f3f3f
#define mmGUS_IO_WR_PRI_QUANT_PRI3_DEFAULT                                       0x7f7f7f7f
#define mmGUS_IO_WR_PRI_QUANT_PRI4_DEFAULT                                       0xffffffff
#define mmGUS_IO_RD_PRI_QUANT1_PRI1_DEFAULT                                      0x00001f1f
#define mmGUS_IO_RD_PRI_QUANT1_PRI2_DEFAULT                                      0x00003f3f
#define mmGUS_IO_RD_PRI_QUANT1_PRI3_DEFAULT                                      0x00007f7f
#define mmGUS_IO_RD_PRI_QUANT1_PRI4_DEFAULT                                      0x0000ffff
#define mmGUS_IO_WR_PRI_QUANT1_PRI1_DEFAULT                                      0x00001f1f
#define mmGUS_IO_WR_PRI_QUANT1_PRI2_DEFAULT                                      0x00003f3f
#define mmGUS_IO_WR_PRI_QUANT1_PRI3_DEFAULT                                      0x00007f7f
#define mmGUS_IO_WR_PRI_QUANT1_PRI4_DEFAULT                                      0x0000ffff
#define mmGUS_DRAM_COMBINE_FLUSH_DEFAULT                                         0x00000000
#define mmGUS_DRAM_COMBINE_RD_WR_EN_DEFAULT                                      0x00000fff
#define mmGUS_DRAM_PRI_AGE_RATE_DEFAULT                                          0x00001249
#define mmGUS_DRAM_PRI_AGE_COEFF_DEFAULT                                         0x0003ffff
#define mmGUS_DRAM_PRI_QUEUING_DEFAULT                                           0x0003edb6
#define mmGUS_DRAM_PRI_FIXED_DEFAULT                                             0x00000000
#define mmGUS_DRAM_PRI_URGENCY_COEFF_DEFAULT                                     0x00000000
#define mmGUS_DRAM_PRI_URGENCY_MODE_DEFAULT                                      0x00000000
#define mmGUS_DRAM_PRI_QUANT_PRI1_DEFAULT                                        0x0f0f0f0f
#define mmGUS_DRAM_PRI_QUANT_PRI2_DEFAULT                                        0x1f1f1f1f
#define mmGUS_DRAM_PRI_QUANT_PRI3_DEFAULT                                        0x3f3f3f3f
#define mmGUS_DRAM_PRI_QUANT_PRI4_DEFAULT                                        0x7f7f7f7f
#define mmGUS_DRAM_PRI_QUANT_PRI5_DEFAULT                                        0xffffffff
#define mmGUS_DRAM_PRI_QUANT1_PRI1_DEFAULT                                       0x00000f0f
#define mmGUS_DRAM_PRI_QUANT1_PRI2_DEFAULT                                       0x00001f1f
#define mmGUS_DRAM_PRI_QUANT1_PRI3_DEFAULT                                       0x00003f3f
#define mmGUS_DRAM_PRI_QUANT1_PRI4_DEFAULT                                       0x00007f7f
#define mmGUS_DRAM_PRI_QUANT1_PRI5_DEFAULT                                       0x0000ffff
#define mmGUS_IO_GROUP_BURST_DEFAULT                                             0x05040504
#define mmGUS_DRAM_GROUP_BURST_DEFAULT                                           0x00000504
#define mmGUS_SDP_ARB_FINAL_DEFAULT                                              0x00007fff
#define mmGUS_SDP_QOS_VC_PRIORITY_DEFAULT                                        0x0000a000
#define mmGUS_SDP_CREDITS_DEFAULT                                                0x000100ff
#define mmGUS_SDP_TAG_RESERVE0_DEFAULT                                           0x07070000
#define mmGUS_SDP_TAG_RESERVE1_DEFAULT                                           0x00000707
#define mmGUS_SDP_VCC_RESERVE0_DEFAULT                                           0x02041000
#define mmGUS_SDP_VCC_RESERVE1_DEFAULT                                           0x00000002
#define mmGUS_SDP_VCD_RESERVE0_DEFAULT                                           0x02040000
#define mmGUS_SDP_VCD_RESERVE1_DEFAULT                                           0x00000002
#define mmGUS_SDP_REQ_CNTL_DEFAULT                                               0x0000001f
#define mmGUS_MISC_DEFAULT                                                       0x00003c07
#define mmGUS_LATENCY_SAMPLING_DEFAULT                                           0x00000000
#define mmGUS_ERR_STATUS_DEFAULT                                                 0x00000300
#define mmGUS_MISC2_DEFAULT                                                      0x000017fe
#define mmGUS_SDP_ENABLE_DEFAULT                                                 0x00000000
#define mmGUS_L1_CH0_CMD_IN_DEFAULT                                              0x00000000
#define mmGUS_L1_CH0_CMD_OUT_DEFAULT                                             0x00000000
#define mmGUS_L1_CH0_DATA_IN_DEFAULT                                             0x00000000
#define mmGUS_L1_CH0_DATA_OUT_DEFAULT                                            0x00000000
#define mmGUS_L1_CH0_DATA_U_IN_DEFAULT                                           0x00000000
#define mmGUS_L1_CH0_DATA_U_OUT_DEFAULT                                          0x00000000
#define mmGUS_L1_CH1_CMD_IN_DEFAULT                                              0x00000000
#define mmGUS_L1_CH1_CMD_OUT_DEFAULT                                             0x00000000
#define mmGUS_L1_CH1_DATA_IN_DEFAULT                                             0x00000000
#define mmGUS_L1_CH1_DATA_OUT_DEFAULT                                            0x00000000
#define mmGUS_L1_CH1_DATA_U_IN_DEFAULT                                           0x00000000
#define mmGUS_L1_CH1_DATA_U_OUT_DEFAULT                                          0x00000000
#define mmGUS_L1_SA0_CMD_IN_DEFAULT                                              0x00000000
#define mmGUS_L1_SA0_CMD_OUT_DEFAULT                                             0x00000000
#define mmGUS_L1_SA0_DATA_IN_DEFAULT                                             0x00000000
#define mmGUS_L1_SA0_DATA_OUT_DEFAULT                                            0x00000000
#define mmGUS_L1_SA0_DATA_U_IN_DEFAULT                                           0x00000000
#define mmGUS_L1_SA0_DATA_U_OUT_DEFAULT                                          0x00000000
#define mmGUS_L1_SA1_CMD_IN_DEFAULT                                              0x00000000
#define mmGUS_L1_SA1_CMD_OUT_DEFAULT                                             0x00000000
#define mmGUS_L1_SA1_DATA_IN_DEFAULT                                             0x00000000
#define mmGUS_L1_SA1_DATA_OUT_DEFAULT                                            0x00000000
#define mmGUS_L1_SA1_DATA_U_IN_DEFAULT                                           0x00000000
#define mmGUS_L1_SA1_DATA_U_OUT_DEFAULT                                          0x00000000
#define mmGUS_L1_SA2_CMD_IN_DEFAULT                                              0x00000000
#define mmGUS_L1_SA2_CMD_OUT_DEFAULT                                             0x00000000
#define mmGUS_L1_SA2_DATA_IN_DEFAULT                                             0x00000000
#define mmGUS_L1_SA2_DATA_OUT_DEFAULT                                            0x00000000
#define mmGUS_L1_SA2_DATA_U_IN_DEFAULT                                           0x00000000
#define mmGUS_L1_SA2_DATA_U_OUT_DEFAULT                                          0x00000000
#define mmGUS_L1_SA3_CMD_IN_DEFAULT                                              0x00000000
#define mmGUS_L1_SA3_CMD_OUT_DEFAULT                                             0x00000000
#define mmGUS_L1_SA3_DATA_IN_DEFAULT                                             0x00000000
#define mmGUS_L1_SA3_DATA_OUT_DEFAULT                                            0x00000000
#define mmGUS_L1_SA3_DATA_U_IN_DEFAULT                                           0x00000000
#define mmGUS_L1_SA3_DATA_U_OUT_DEFAULT                                          0x00000000
#define mmGUS_MISC3_DEFAULT                                                      0x00000000
#define mmGUS_WRRSP_FIFO_CNTL_DEFAULT                                            0x0000000a


// addressBlock: gc_gl1dec
#define mmGL1_DRAM_BURST_MASK_DEFAULT                                            0x000000cf
#define mmGL1_ARB_STATUS_DEFAULT                                                 0x00000000
#define mmGL1_PIPE_STEER_DEFAULT                                                 0x0000008d
#define mmGL1C_STATUS_DEFAULT                                                    0x80000000
#define mmGL1C_UTCL0_CNTL2_DEFAULT                                               0x00000000
#define mmGL1C_UTCL0_STATUS_DEFAULT                                              0x00000000
#define mmGL1C_UTCL0_RETRY_DEFAULT                                               0x00000040


// addressBlock: gc_chdec
#define mmCH_ARB_CTRL_DEFAULT                                                    0x00000002
#define mmCH_DRAM_BURST_MASK_DEFAULT                                             0x000000cf
#define mmCH_ARB_STATUS_DEFAULT                                                  0x00000000
#define mmCH_DRAM_BURST_CTRL_DEFAULT                                             0x000001f7
#define mmCHA_CHC_CREDITS_DEFAULT                                                0x00000000
#define mmCHA_CLIENT_FREE_DELAY_DEFAULT                                          0x00000000
#define mmCH_PIPE_STEER_DEFAULT                                                  0x0000008d
#define mmCH_VC5_ENABLE_DEFAULT                                                  0x00000000
#define mmCHC_CTRL_DEFAULT                                                       0x0001428f
#define mmCHC_STATUS_DEFAULT                                                     0x00000000
#define mmCHCG_CTRL_DEFAULT                                                      0x001830ff
#define mmCHCG_STATUS_DEFAULT                                                    0x00000000


// addressBlock: gc_gl2dec
#define mmGL2C_CTRL_DEFAULT                                                      0xf37fff7f
#define mmGL2C_CTRL2_DEFAULT                                                     0x1402002f
#define mmGL2C_ADDR_MATCH_MASK_DEFAULT                                           0xffffffff
#define mmGL2C_ADDR_MATCH_SIZE_DEFAULT                                           0x00000007
#define mmGL2C_WBINVL2_DEFAULT                                                   0x00000010
#define mmGL2C_SOFT_RESET_DEFAULT                                                0x00000000
#define mmGL2C_CM_CTRL0_DEFAULT                                                  0x42108421
#define mmGL2C_CM_CTRL1_DEFAULT                                                  0x180f1008
#define mmGL2C_CM_STALL_DEFAULT                                                  0x00000000
#define mmGL2C_MDC_PF_FLAG_CTRL_DEFAULT                                          0x00010000
#define mmGL2C_LB_CTR_CTRL_DEFAULT                                               0x00000000
#define mmGL2C_LB_DATA0_DEFAULT                                                  0x00000000
#define mmGL2C_LB_DATA1_DEFAULT                                                  0x00000000
#define mmGL2C_LB_DATA2_DEFAULT                                                  0x00000000
#define mmGL2C_LB_DATA3_DEFAULT                                                  0x00000000
#define mmGL2C_LB_CTR_SEL0_DEFAULT                                               0x00000000
#define mmGL2C_LB_CTR_SEL1_DEFAULT                                               0x00000000
#define mmGL2A_ADDR_MATCH_CTRL_DEFAULT                                           0x00000000
#define mmGL2A_ADDR_MATCH_MASK_DEFAULT                                           0xffffffff
#define mmGL2A_ADDR_MATCH_SIZE_DEFAULT                                           0x00000007
#define mmGL2A_PRIORITY_CTRL_DEFAULT                                             0x00000000
#define mmGL2_PIPE_STEER_0_DEFAULT                                               0x32103210
#define mmGL2_PIPE_STEER_1_DEFAULT                                               0x32103210


// addressBlock: gc_perfddec
#define mmCPG_PERFCOUNTER1_LO_DEFAULT                                            0x00000000
#define mmCPG_PERFCOUNTER1_HI_DEFAULT                                            0x00000000
#define mmCPG_PERFCOUNTER0_LO_DEFAULT                                            0x00000000
#define mmCPG_PERFCOUNTER0_HI_DEFAULT                                            0x00000000
#define mmCPC_PERFCOUNTER1_LO_DEFAULT                                            0x00000000
#define mmCPC_PERFCOUNTER1_HI_DEFAULT                                            0x00000000
#define mmCPC_PERFCOUNTER0_LO_DEFAULT                                            0x00000000
#define mmCPC_PERFCOUNTER0_HI_DEFAULT                                            0x00000000
#define mmCPF_PERFCOUNTER1_LO_DEFAULT                                            0x00000000
#define mmCPF_PERFCOUNTER1_HI_DEFAULT                                            0x00000000
#define mmCPF_PERFCOUNTER0_LO_DEFAULT                                            0x00000000
#define mmCPF_PERFCOUNTER0_HI_DEFAULT                                            0x00000000
#define mmCPF_LATENCY_STATS_DATA_DEFAULT                                         0x00000000
#define mmCPG_LATENCY_STATS_DATA_DEFAULT                                         0x00000000
#define mmCPC_LATENCY_STATS_DATA_DEFAULT                                         0x00000000
#define mmGRBM_PERFCOUNTER0_LO_DEFAULT                                           0x00000000
#define mmGRBM_PERFCOUNTER0_HI_DEFAULT                                           0x00000000
#define mmGRBM_PERFCOUNTER1_LO_DEFAULT                                           0x00000000
#define mmGRBM_PERFCOUNTER1_HI_DEFAULT                                           0x00000000
#define mmGRBM_SE0_PERFCOUNTER_LO_DEFAULT                                        0x00000000
#define mmGRBM_SE0_PERFCOUNTER_HI_DEFAULT                                        0x00000000
#define mmGRBM_SE1_PERFCOUNTER_LO_DEFAULT                                        0x00000000
#define mmGRBM_SE1_PERFCOUNTER_HI_DEFAULT                                        0x00000000
#define mmGRBM_SE2_PERFCOUNTER_LO_DEFAULT                                        0x00000000
#define mmGRBM_SE2_PERFCOUNTER_HI_DEFAULT                                        0x00000000
#define mmGRBM_SE3_PERFCOUNTER_LO_DEFAULT                                        0x00000000
#define mmGRBM_SE3_PERFCOUNTER_HI_DEFAULT                                        0x00000000
#define mmGE1_PERFCOUNTER0_LO_DEFAULT                                            0x00000000
#define mmGE1_PERFCOUNTER0_HI_DEFAULT                                            0x00000000
#define mmGE1_PERFCOUNTER1_LO_DEFAULT                                            0x00000000
#define mmGE1_PERFCOUNTER1_HI_DEFAULT                                            0x00000000
#define mmGE1_PERFCOUNTER2_LO_DEFAULT                                            0x00000000
#define mmGE1_PERFCOUNTER2_HI_DEFAULT                                            0x00000000
#define mmGE1_PERFCOUNTER3_LO_DEFAULT                                            0x00000000
#define mmGE1_PERFCOUNTER3_HI_DEFAULT                                            0x00000000
#define mmGE2_DIST_PERFCOUNTER0_LO_DEFAULT                                       0x00000000
#define mmGE2_DIST_PERFCOUNTER0_HI_DEFAULT                                       0x00000000
#define mmGE2_DIST_PERFCOUNTER1_LO_DEFAULT                                       0x00000000
#define mmGE2_DIST_PERFCOUNTER1_HI_DEFAULT                                       0x00000000
#define mmGE2_DIST_PERFCOUNTER2_LO_DEFAULT                                       0x00000000
#define mmGE2_DIST_PERFCOUNTER2_HI_DEFAULT                                       0x00000000
#define mmGE2_DIST_PERFCOUNTER3_LO_DEFAULT                                       0x00000000
#define mmGE2_DIST_PERFCOUNTER3_HI_DEFAULT                                       0x00000000
#define mmGE2_SE_PERFCOUNTER0_LO_DEFAULT                                         0x00000000
#define mmGE2_SE_PERFCOUNTER0_HI_DEFAULT                                         0x00000000
#define mmGE2_SE_PERFCOUNTER1_LO_DEFAULT                                         0x00000000
#define mmGE2_SE_PERFCOUNTER1_HI_DEFAULT                                         0x00000000
#define mmGE2_SE_PERFCOUNTER2_LO_DEFAULT                                         0x00000000
#define mmGE2_SE_PERFCOUNTER2_HI_DEFAULT                                         0x00000000
#define mmGE2_SE_PERFCOUNTER3_LO_DEFAULT                                         0x00000000
#define mmGE2_SE_PERFCOUNTER3_HI_DEFAULT                                         0x00000000
#define mmPA_SU_PERFCOUNTER0_LO_DEFAULT                                          0x00000000
#define mmPA_SU_PERFCOUNTER0_HI_DEFAULT                                          0x00000000
#define mmPA_SU_PERFCOUNTER1_LO_DEFAULT                                          0x00000000
#define mmPA_SU_PERFCOUNTER1_HI_DEFAULT                                          0x00000000
#define mmPA_SU_PERFCOUNTER2_LO_DEFAULT                                          0x00000000
#define mmPA_SU_PERFCOUNTER2_HI_DEFAULT                                          0x00000000
#define mmPA_SU_PERFCOUNTER3_LO_DEFAULT                                          0x00000000
#define mmPA_SU_PERFCOUNTER3_HI_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER0_LO_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER0_HI_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER1_LO_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER1_HI_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER2_LO_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER2_HI_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER3_LO_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER3_HI_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER4_LO_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER4_HI_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER5_LO_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER5_HI_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER6_LO_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER6_HI_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER7_LO_DEFAULT                                          0x00000000
#define mmPA_SC_PERFCOUNTER7_HI_DEFAULT                                          0x00000000
#define mmSPI_PERFCOUNTER0_HI_DEFAULT                                            0x00000000
#define mmSPI_PERFCOUNTER0_LO_DEFAULT                                            0x00000000
#define mmSPI_PERFCOUNTER1_HI_DEFAULT                                            0x00000000
#define mmSPI_PERFCOUNTER1_LO_DEFAULT                                            0x00000000
#define mmSPI_PERFCOUNTER2_HI_DEFAULT                                            0x00000000
#define mmSPI_PERFCOUNTER2_LO_DEFAULT                                            0x00000000
#define mmSPI_PERFCOUNTER3_HI_DEFAULT                                            0x00000000
#define mmSPI_PERFCOUNTER3_LO_DEFAULT                                            0x00000000
#define mmSPI_PERFCOUNTER4_HI_DEFAULT                                            0x00000000
#define mmSPI_PERFCOUNTER4_LO_DEFAULT                                            0x00000000
#define mmSPI_PERFCOUNTER5_HI_DEFAULT                                            0x00000000
#define mmSPI_PERFCOUNTER5_LO_DEFAULT                                            0x00000000
#define mmSQ_PERFCOUNTER0_LO_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER0_HI_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER1_LO_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER1_HI_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER2_LO_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER2_HI_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER3_LO_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER3_HI_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER4_LO_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER4_HI_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER5_LO_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER5_HI_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER6_LO_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER6_HI_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER7_LO_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER7_HI_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER8_LO_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER8_HI_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER9_LO_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER9_HI_DEFAULT                                             0x00000000
#define mmSQ_PERFCOUNTER10_LO_DEFAULT                                            0x00000000
#define mmSQ_PERFCOUNTER10_HI_DEFAULT                                            0x00000000
#define mmSQ_PERFCOUNTER11_LO_DEFAULT                                            0x00000000
#define mmSQ_PERFCOUNTER11_HI_DEFAULT                                            0x00000000
#define mmSQ_PERFCOUNTER12_LO_DEFAULT                                            0x00000000
#define mmSQ_PERFCOUNTER12_HI_DEFAULT                                            0x00000000
#define mmSQ_PERFCOUNTER13_LO_DEFAULT                                            0x00000000
#define mmSQ_PERFCOUNTER13_HI_DEFAULT                                            0x00000000
#define mmSQ_PERFCOUNTER14_LO_DEFAULT                                            0x00000000
#define mmSQ_PERFCOUNTER14_HI_DEFAULT                                            0x00000000
#define mmSQ_PERFCOUNTER15_LO_DEFAULT                                            0x00000000
#define mmSQ_PERFCOUNTER15_HI_DEFAULT                                            0x00000000
#define mmSX_PERFCOUNTER0_LO_DEFAULT                                             0x00000000
#define mmSX_PERFCOUNTER0_HI_DEFAULT                                             0x00000000
#define mmSX_PERFCOUNTER1_LO_DEFAULT                                             0x00000000
#define mmSX_PERFCOUNTER1_HI_DEFAULT                                             0x00000000
#define mmSX_PERFCOUNTER2_LO_DEFAULT                                             0x00000000
#define mmSX_PERFCOUNTER2_HI_DEFAULT                                             0x00000000
#define mmSX_PERFCOUNTER3_LO_DEFAULT                                             0x00000000
#define mmSX_PERFCOUNTER3_HI_DEFAULT                                             0x00000000
#define mmGCEA_PERFCOUNTER2_LO_DEFAULT                                           0x00000000
#define mmGCEA_PERFCOUNTER2_HI_DEFAULT                                           0x00000000
#define mmGCEA_PERFCOUNTER_LO_DEFAULT                                            0x00000000
#define mmGCEA_PERFCOUNTER_HI_DEFAULT                                            0x00000000
#define mmGDS_PERFCOUNTER0_LO_DEFAULT                                            0x00000000
#define mmGDS_PERFCOUNTER0_HI_DEFAULT                                            0x00000000
#define mmGDS_PERFCOUNTER1_LO_DEFAULT                                            0x00000000
#define mmGDS_PERFCOUNTER1_HI_DEFAULT                                            0x00000000
#define mmGDS_PERFCOUNTER2_LO_DEFAULT                                            0x00000000
#define mmGDS_PERFCOUNTER2_HI_DEFAULT                                            0x00000000
#define mmGDS_PERFCOUNTER3_LO_DEFAULT                                            0x00000000
#define mmGDS_PERFCOUNTER3_HI_DEFAULT                                            0x00000000
#define mmTA_PERFCOUNTER0_LO_DEFAULT                                             0x00000000
#define mmTA_PERFCOUNTER0_HI_DEFAULT                                             0x00000000
#define mmTA_PERFCOUNTER1_LO_DEFAULT                                             0x00000000
#define mmTA_PERFCOUNTER1_HI_DEFAULT                                             0x00000000
#define mmTD_PERFCOUNTER0_LO_DEFAULT                                             0x00000000
#define mmTD_PERFCOUNTER0_HI_DEFAULT                                             0x00000000
#define mmTD_PERFCOUNTER1_LO_DEFAULT                                             0x00000000
#define mmTD_PERFCOUNTER1_HI_DEFAULT                                             0x00000000
#define mmTCP_PERFCOUNTER0_LO_DEFAULT                                            0x00000000
#define mmTCP_PERFCOUNTER0_HI_DEFAULT                                            0x00000000
#define mmTCP_PERFCOUNTER1_LO_DEFAULT                                            0x00000000
#define mmTCP_PERFCOUNTER1_HI_DEFAULT                                            0x00000000
#define mmTCP_PERFCOUNTER2_LO_DEFAULT                                            0x00000000
#define mmTCP_PERFCOUNTER2_HI_DEFAULT                                            0x00000000
#define mmTCP_PERFCOUNTER3_LO_DEFAULT                                            0x00000000
#define mmTCP_PERFCOUNTER3_HI_DEFAULT                                            0x00000000
#define mmGL2C_PERFCOUNTER0_LO_DEFAULT                                           0x00000000
#define mmGL2C_PERFCOUNTER0_HI_DEFAULT                                           0x00000000
#define mmGL2C_PERFCOUNTER1_LO_DEFAULT                                           0x00000000
#define mmGL2C_PERFCOUNTER1_HI_DEFAULT                                           0x00000000
#define mmGL2C_PERFCOUNTER2_LO_DEFAULT                                           0x00000000
#define mmGL2C_PERFCOUNTER2_HI_DEFAULT                                           0x00000000
#define mmGL2C_PERFCOUNTER3_LO_DEFAULT                                           0x00000000
#define mmGL2C_PERFCOUNTER3_HI_DEFAULT                                           0x00000000
#define mmGL2A_PERFCOUNTER0_LO_DEFAULT                                           0x00000000
#define mmGL2A_PERFCOUNTER0_HI_DEFAULT                                           0x00000000
#define mmGL2A_PERFCOUNTER1_LO_DEFAULT                                           0x00000000
#define mmGL2A_PERFCOUNTER1_HI_DEFAULT                                           0x00000000
#define mmGL2A_PERFCOUNTER2_LO_DEFAULT                                           0x00000000
#define mmGL2A_PERFCOUNTER2_HI_DEFAULT                                           0x00000000
#define mmGL2A_PERFCOUNTER3_LO_DEFAULT                                           0x00000000
#define mmGL2A_PERFCOUNTER3_HI_DEFAULT                                           0x00000000
#define mmGL1C_PERFCOUNTER0_LO_DEFAULT                                           0x00000000
#define mmGL1C_PERFCOUNTER0_HI_DEFAULT                                           0x00000000
#define mmGL1C_PERFCOUNTER1_LO_DEFAULT                                           0x00000000
#define mmGL1C_PERFCOUNTER1_HI_DEFAULT                                           0x00000000
#define mmGL1C_PERFCOUNTER2_LO_DEFAULT                                           0x00000000
#define mmGL1C_PERFCOUNTER2_HI_DEFAULT                                           0x00000000
#define mmGL1C_PERFCOUNTER3_LO_DEFAULT                                           0x00000000
#define mmGL1C_PERFCOUNTER3_HI_DEFAULT                                           0x00000000
#define mmCHC_PERFCOUNTER0_LO_DEFAULT                                            0x00000000
#define mmCHC_PERFCOUNTER0_HI_DEFAULT                                            0x00000000
#define mmCHC_PERFCOUNTER1_LO_DEFAULT                                            0x00000000
#define mmCHC_PERFCOUNTER1_HI_DEFAULT                                            0x00000000
#define mmCHC_PERFCOUNTER2_LO_DEFAULT                                            0x00000000
#define mmCHC_PERFCOUNTER2_HI_DEFAULT                                            0x00000000
#define mmCHC_PERFCOUNTER3_LO_DEFAULT                                            0x00000000
#define mmCHC_PERFCOUNTER3_HI_DEFAULT                                            0x00000000
#define mmCHCG_PERFCOUNTER0_LO_DEFAULT                                           0x00000000
#define mmCHCG_PERFCOUNTER0_HI_DEFAULT                                           0x00000000
#define mmCHCG_PERFCOUNTER1_LO_DEFAULT                                           0x00000000
#define mmCHCG_PERFCOUNTER1_HI_DEFAULT                                           0x00000000
#define mmCHCG_PERFCOUNTER2_LO_DEFAULT                                           0x00000000
#define mmCHCG_PERFCOUNTER2_HI_DEFAULT                                           0x00000000
#define mmCHCG_PERFCOUNTER3_LO_DEFAULT                                           0x00000000
#define mmCHCG_PERFCOUNTER3_HI_DEFAULT                                           0x00000000
#define mmCB_PERFCOUNTER0_LO_DEFAULT                                             0x00000000
#define mmCB_PERFCOUNTER0_HI_DEFAULT                                             0x00000000
#define mmCB_PERFCOUNTER1_LO_DEFAULT                                             0x00000000
#define mmCB_PERFCOUNTER1_HI_DEFAULT                                             0x00000000
#define mmCB_PERFCOUNTER2_LO_DEFAULT                                             0x00000000
#define mmCB_PERFCOUNTER2_HI_DEFAULT                                             0x00000000
#define mmCB_PERFCOUNTER3_LO_DEFAULT                                             0x00000000
#define mmCB_PERFCOUNTER3_HI_DEFAULT                                             0x00000000
#define mmDB_PERFCOUNTER0_LO_DEFAULT                                             0x00000000
#define mmDB_PERFCOUNTER0_HI_DEFAULT                                             0x00000000
#define mmDB_PERFCOUNTER1_LO_DEFAULT                                             0x00000000
#define mmDB_PERFCOUNTER1_HI_DEFAULT                                             0x00000000
#define mmDB_PERFCOUNTER2_LO_DEFAULT                                             0x00000000
#define mmDB_PERFCOUNTER2_HI_DEFAULT                                             0x00000000
#define mmDB_PERFCOUNTER3_LO_DEFAULT                                             0x00000000
#define mmDB_PERFCOUNTER3_HI_DEFAULT                                             0x00000000
#define mmRLC_PERFCOUNTER0_LO_DEFAULT                                            0x00000000
#define mmRLC_PERFCOUNTER0_HI_DEFAULT                                            0x00000000
#define mmRLC_PERFCOUNTER1_LO_DEFAULT                                            0x00000000
#define mmRLC_PERFCOUNTER1_HI_DEFAULT                                            0x00000000
#define mmRMI_PERFCOUNTER0_LO_DEFAULT                                            0x00000000
#define mmRMI_PERFCOUNTER0_HI_DEFAULT                                            0x00000000
#define mmRMI_PERFCOUNTER1_LO_DEFAULT                                            0x00000000
#define mmRMI_PERFCOUNTER1_HI_DEFAULT                                            0x00000000
#define mmRMI_PERFCOUNTER2_LO_DEFAULT                                            0x00000000
#define mmRMI_PERFCOUNTER2_HI_DEFAULT                                            0x00000000
#define mmRMI_PERFCOUNTER3_LO_DEFAULT                                            0x00000000
#define mmRMI_PERFCOUNTER3_HI_DEFAULT                                            0x00000000
#define mmUTCL1_PERFCOUNTER0_LO_DEFAULT                                          0x00000000
#define mmUTCL1_PERFCOUNTER0_HI_DEFAULT                                          0x00000000
#define mmUTCL1_PERFCOUNTER1_LO_DEFAULT                                          0x00000000
#define mmUTCL1_PERFCOUNTER1_HI_DEFAULT                                          0x00000000
#define mmGCR_PERFCOUNTER0_LO_DEFAULT                                            0x00000000
#define mmGCR_PERFCOUNTER0_HI_DEFAULT                                            0x00000000
#define mmGCR_PERFCOUNTER1_LO_DEFAULT                                            0x00000000
#define mmGCR_PERFCOUNTER1_HI_DEFAULT                                            0x00000000
#define mmPA_PH_PERFCOUNTER0_LO_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER0_HI_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER1_LO_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER1_HI_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER2_LO_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER2_HI_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER3_LO_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER3_HI_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER4_LO_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER4_HI_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER5_LO_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER5_HI_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER6_LO_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER6_HI_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER7_LO_DEFAULT                                          0x00000000
#define mmPA_PH_PERFCOUNTER7_HI_DEFAULT                                          0x00000000
#define mmGL1A_PERFCOUNTER0_LO_DEFAULT                                           0x00000000
#define mmGL1A_PERFCOUNTER0_HI_DEFAULT                                           0x00000000
#define mmGL1A_PERFCOUNTER1_LO_DEFAULT                                           0x00000000
#define mmGL1A_PERFCOUNTER1_HI_DEFAULT                                           0x00000000
#define mmGL1A_PERFCOUNTER2_LO_DEFAULT                                           0x00000000
#define mmGL1A_PERFCOUNTER2_HI_DEFAULT                                           0x00000000
#define mmGL1A_PERFCOUNTER3_LO_DEFAULT                                           0x00000000
#define mmGL1A_PERFCOUNTER3_HI_DEFAULT                                           0x00000000
#define mmCHA_PERFCOUNTER0_LO_DEFAULT                                            0x00000000
#define mmCHA_PERFCOUNTER0_HI_DEFAULT                                            0x00000000
#define mmCHA_PERFCOUNTER1_LO_DEFAULT                                            0x00000000
#define mmCHA_PERFCOUNTER1_HI_DEFAULT                                            0x00000000
#define mmCHA_PERFCOUNTER2_LO_DEFAULT                                            0x00000000
#define mmCHA_PERFCOUNTER2_HI_DEFAULT                                            0x00000000
#define mmCHA_PERFCOUNTER3_LO_DEFAULT                                            0x00000000
#define mmCHA_PERFCOUNTER3_HI_DEFAULT                                            0x00000000
#define mmGUS_PERFCOUNTER2_LO_DEFAULT                                            0x00000000
#define mmGUS_PERFCOUNTER2_HI_DEFAULT                                            0x00000000
#define mmGUS_PERFCOUNTER_LO_DEFAULT                                             0x00000000
#define mmGUS_PERFCOUNTER_HI_DEFAULT                                             0x00000000


// addressBlock: gc_gcvml2prdec
#define mmGCMC_VM_L2_PERFCOUNTER_LO_DEFAULT                                      0x00000000
#define mmGCMC_VM_L2_PERFCOUNTER_HI_DEFAULT                                      0x00000000
#define mmGCUTCL2_PERFCOUNTER_LO_DEFAULT                                         0x00000000
#define mmGCUTCL2_PERFCOUNTER_HI_DEFAULT                                         0x00000000


// addressBlock: gc_gcvml2perfddec
#define mmGCVML2_PERFCOUNTER2_0_LO_DEFAULT                                       0x00000000
#define mmGCVML2_PERFCOUNTER2_1_LO_DEFAULT                                       0x00000000
#define mmGCVML2_PERFCOUNTER2_0_HI_DEFAULT                                       0x00000000
#define mmGCVML2_PERFCOUNTER2_1_HI_DEFAULT                                       0x00000000


// addressBlock: gc_sdma0_sdma0perfddec
#define mmSDMA0_PERFCNT_PERFCOUNTER_LO_DEFAULT                                   0x00000000
#define mmSDMA0_PERFCNT_PERFCOUNTER_HI_DEFAULT                                   0x00000000
#define mmSDMA0_PERFCOUNTER0_LO_DEFAULT                                          0x00000000
#define mmSDMA0_PERFCOUNTER0_HI_DEFAULT                                          0x00000000
#define mmSDMA0_PERFCOUNTER1_LO_DEFAULT                                          0x00000000
#define mmSDMA0_PERFCOUNTER1_HI_DEFAULT                                          0x00000000


// addressBlock: gc_sdma1_sdma1perfddec
#define mmSDMA1_PERFCNT_PERFCOUNTER_LO_DEFAULT                                   0x00000000
#define mmSDMA1_PERFCNT_PERFCOUNTER_HI_DEFAULT                                   0x00000000
#define mmSDMA1_PERFCOUNTER0_LO_DEFAULT                                          0x00000000
#define mmSDMA1_PERFCOUNTER0_HI_DEFAULT                                          0x00000000
#define mmSDMA1_PERFCOUNTER1_LO_DEFAULT                                          0x00000000
#define mmSDMA1_PERFCOUNTER1_HI_DEFAULT                                          0x00000000


// addressBlock: gc_sdma2_sdma2perfddec
#define mmSDMA2_PERFCNT_PERFCOUNTER_LO_DEFAULT                                   0x00000000
#define mmSDMA2_PERFCNT_PERFCOUNTER_HI_DEFAULT                                   0x00000000
#define mmSDMA2_PERFCOUNTER0_LO_DEFAULT                                          0x00000000
#define mmSDMA2_PERFCOUNTER0_HI_DEFAULT                                          0x00000000
#define mmSDMA2_PERFCOUNTER1_LO_DEFAULT                                          0x00000000
#define mmSDMA2_PERFCOUNTER1_HI_DEFAULT                                          0x00000000


// addressBlock: gc_sdma3_sdma3perfddec
#define mmSDMA3_PERFCNT_PERFCOUNTER_LO_DEFAULT                                   0x00000000
#define mmSDMA3_PERFCNT_PERFCOUNTER_HI_DEFAULT                                   0x00000000
#define mmSDMA3_PERFCOUNTER0_LO_DEFAULT                                          0x00000000
#define mmSDMA3_PERFCOUNTER0_HI_DEFAULT                                          0x00000000
#define mmSDMA3_PERFCOUNTER1_LO_DEFAULT                                          0x00000000
#define mmSDMA3_PERFCOUNTER1_HI_DEFAULT                                          0x00000000


// addressBlock: gc_perfsdec
#define mmCPG_PERFCOUNTER1_SELECT_DEFAULT                                        0x000fffff
#define mmCPG_PERFCOUNTER0_SELECT1_DEFAULT                                       0x000fffff
#define mmCPG_PERFCOUNTER0_SELECT_DEFAULT                                        0x000fffff
#define mmCPC_PERFCOUNTER1_SELECT_DEFAULT                                        0x000fffff
#define mmCPC_PERFCOUNTER0_SELECT1_DEFAULT                                       0x000fffff
#define mmCPF_PERFCOUNTER1_SELECT_DEFAULT                                        0x000fffff
#define mmCPF_PERFCOUNTER0_SELECT1_DEFAULT                                       0x000fffff
#define mmCPF_PERFCOUNTER0_SELECT_DEFAULT                                        0x000fffff
#define mmCP_PERFMON_CNTL_DEFAULT                                                0x00000000
#define mmCPC_PERFCOUNTER0_SELECT_DEFAULT                                        0x000fffff
#define mmCPF_TC_PERF_COUNTER_WINDOW_SELECT_DEFAULT                              0x00000000
#define mmCPG_TC_PERF_COUNTER_WINDOW_SELECT_DEFAULT                              0x00000000
#define mmCPF_LATENCY_STATS_SELECT_DEFAULT                                       0x00000000
#define mmCPG_LATENCY_STATS_SELECT_DEFAULT                                       0x00000000
#define mmCPC_LATENCY_STATS_SELECT_DEFAULT                                       0x00000000
#define mmCP_DRAW_OBJECT_DEFAULT                                                 0x00000000
#define mmCP_DRAW_OBJECT_COUNTER_DEFAULT                                         0x00000000
#define mmCP_DRAW_WINDOW_MASK_HI_DEFAULT                                         0x00000000
#define mmCP_DRAW_WINDOW_HI_DEFAULT                                              0x00000000
#define mmCP_DRAW_WINDOW_LO_DEFAULT                                              0x00000000
#define mmCP_DRAW_WINDOW_CNTL_DEFAULT                                            0x00000007
#define mmGRBM_PERFCOUNTER0_SELECT_DEFAULT                                       0x00000000
#define mmGRBM_PERFCOUNTER1_SELECT_DEFAULT                                       0x00000000
#define mmGRBM_SE0_PERFCOUNTER_SELECT_DEFAULT                                    0x00000000
#define mmGRBM_SE1_PERFCOUNTER_SELECT_DEFAULT                                    0x00000000
#define mmGRBM_SE2_PERFCOUNTER_SELECT_DEFAULT                                    0x00000000
#define mmGRBM_SE3_PERFCOUNTER_SELECT_DEFAULT                                    0x00000000
#define mmGRBM_PERFCOUNTER0_SELECT_HI_DEFAULT                                    0x00000000
#define mmGRBM_PERFCOUNTER1_SELECT_HI_DEFAULT                                    0x00000000
#define mmGE1_PERFCOUNTER0_SELECT_DEFAULT                                        0x00000000
#define mmGE1_PERFCOUNTER0_SELECT1_DEFAULT                                       0x00000000
#define mmGE1_PERFCOUNTER1_SELECT_DEFAULT                                        0x00000000
#define mmGE1_PERFCOUNTER1_SELECT1_DEFAULT                                       0x00000000
#define mmGE1_PERFCOUNTER2_SELECT_DEFAULT                                        0x00000000
#define mmGE1_PERFCOUNTER2_SELECT1_DEFAULT                                       0x00000000
#define mmGE1_PERFCOUNTER3_SELECT_DEFAULT                                        0x00000000
#define mmGE1_PERFCOUNTER3_SELECT1_DEFAULT                                       0x00000000
#define mmGE2_DIST_PERFCOUNTER0_SELECT_DEFAULT                                   0x00000000
#define mmGE2_DIST_PERFCOUNTER0_SELECT1_DEFAULT                                  0x00000000
#define mmGE2_DIST_PERFCOUNTER1_SELECT_DEFAULT                                   0x00000000
#define mmGE2_DIST_PERFCOUNTER1_SELECT1_DEFAULT                                  0x00000000
#define mmGE2_DIST_PERFCOUNTER2_SELECT_DEFAULT                                   0x00000000
#define mmGE2_DIST_PERFCOUNTER2_SELECT1_DEFAULT                                  0x00000000
#define mmGE2_DIST_PERFCOUNTER3_SELECT_DEFAULT                                   0x00000000
#define mmGE2_DIST_PERFCOUNTER3_SELECT1_DEFAULT                                  0x00000000
#define mmGE2_SE_PERFCOUNTER0_SELECT_DEFAULT                                     0x00000000
#define mmGE2_SE_PERFCOUNTER0_SELECT1_DEFAULT                                    0x00000000
#define mmGE2_SE_PERFCOUNTER1_SELECT_DEFAULT                                     0x00000000
#define mmGE2_SE_PERFCOUNTER1_SELECT1_DEFAULT                                    0x00000000
#define mmGE2_SE_PERFCOUNTER2_SELECT_DEFAULT                                     0x00000000
#define mmGE2_SE_PERFCOUNTER2_SELECT1_DEFAULT                                    0x00000000
#define mmGE2_SE_PERFCOUNTER3_SELECT_DEFAULT                                     0x00000000
#define mmGE2_SE_PERFCOUNTER3_SELECT1_DEFAULT                                    0x00000000
#define mmPA_SU_PERFCOUNTER0_SELECT_DEFAULT                                      0x00000000
#define mmPA_SU_PERFCOUNTER0_SELECT1_DEFAULT                                     0x00000000
#define mmPA_SU_PERFCOUNTER1_SELECT_DEFAULT                                      0x00000000
#define mmPA_SU_PERFCOUNTER1_SELECT1_DEFAULT                                     0x00000000
#define mmPA_SU_PERFCOUNTER2_SELECT_DEFAULT                                      0x00000000
#define mmPA_SU_PERFCOUNTER2_SELECT1_DEFAULT                                     0x00000000
#define mmPA_SU_PERFCOUNTER3_SELECT_DEFAULT                                      0x00000000
#define mmPA_SU_PERFCOUNTER3_SELECT1_DEFAULT                                     0x00000000
#define mmPA_SC_PERFCOUNTER0_SELECT_DEFAULT                                      0x00000000
#define mmPA_SC_PERFCOUNTER0_SELECT1_DEFAULT                                     0x00000000
#define mmPA_SC_PERFCOUNTER1_SELECT_DEFAULT                                      0x00000000
#define mmPA_SC_PERFCOUNTER2_SELECT_DEFAULT                                      0x00000000
#define mmPA_SC_PERFCOUNTER3_SELECT_DEFAULT                                      0x00000000
#define mmPA_SC_PERFCOUNTER4_SELECT_DEFAULT                                      0x00000000
#define mmPA_SC_PERFCOUNTER5_SELECT_DEFAULT                                      0x00000000
#define mmPA_SC_PERFCOUNTER6_SELECT_DEFAULT                                      0x00000000
#define mmPA_SC_PERFCOUNTER7_SELECT_DEFAULT                                      0x00000000
#define mmSPI_PERFCOUNTER0_SELECT_DEFAULT                                        0x000fffff
#define mmSPI_PERFCOUNTER1_SELECT_DEFAULT                                        0x000fffff
#define mmSPI_PERFCOUNTER2_SELECT_DEFAULT                                        0x000fffff
#define mmSPI_PERFCOUNTER3_SELECT_DEFAULT                                        0x000fffff
#define mmSPI_PERFCOUNTER0_SELECT1_DEFAULT                                       0x000fffff
#define mmSPI_PERFCOUNTER1_SELECT1_DEFAULT                                       0x000fffff
#define mmSPI_PERFCOUNTER2_SELECT1_DEFAULT                                       0x000fffff
#define mmSPI_PERFCOUNTER3_SELECT1_DEFAULT                                       0x000fffff
#define mmSPI_PERFCOUNTER4_SELECT_DEFAULT                                        0x000003ff
#define mmSPI_PERFCOUNTER5_SELECT_DEFAULT                                        0x000003ff
#define mmSPI_PERFCOUNTER_BINS_DEFAULT                                           0xfcb87430
#define mmSQ_PERFCOUNTER0_SELECT_DEFAULT                                         0x00000000
#define mmSQ_PERFCOUNTER1_SELECT_DEFAULT                                         0x00000000
#define mmSQ_PERFCOUNTER2_SELECT_DEFAULT                                         0x00000000
#define mmSQ_PERFCOUNTER3_SELECT_DEFAULT                                         0x00000000
#define mmSQ_PERFCOUNTER4_SELECT_DEFAULT                                         0x00000000
#define mmSQ_PERFCOUNTER5_SELECT_DEFAULT                                         0x00000000
#define mmSQ_PERFCOUNTER6_SELECT_DEFAULT                                         0x00000000
#define mmSQ_PERFCOUNTER7_SELECT_DEFAULT                                         0x00000000
#define mmSQ_PERFCOUNTER8_SELECT_DEFAULT                                         0x00000000
#define mmSQ_PERFCOUNTER9_SELECT_DEFAULT                                         0x00000000
#define mmSQ_PERFCOUNTER10_SELECT_DEFAULT                                        0x00000000
#define mmSQ_PERFCOUNTER11_SELECT_DEFAULT                                        0x00000000
#define mmSQ_PERFCOUNTER12_SELECT_DEFAULT                                        0x00000000
#define mmSQ_PERFCOUNTER13_SELECT_DEFAULT                                        0x00000000
#define mmSQ_PERFCOUNTER14_SELECT_DEFAULT                                        0x00000000
#define mmSQ_PERFCOUNTER15_SELECT_DEFAULT                                        0x00000000
#define mmSQ_PERFCOUNTER_CTRL_DEFAULT                                            0x00000200
#define mmSQ_PERFCOUNTER_CTRL2_DEFAULT                                           0x00000000
#define mmGCEA_PERFCOUNTER2_SELECT_DEFAULT                                       0x000fffff
#define mmGCEA_PERFCOUNTER2_SELECT1_DEFAULT                                      0x000fffff
#define mmGCEA_PERFCOUNTER2_MODE_DEFAULT                                         0x00000000
#define mmGCEA_PERFCOUNTER0_CFG_DEFAULT                                          0x00000000
#define mmGCEA_PERFCOUNTER1_CFG_DEFAULT                                          0x00000000
#define mmGCEA_PERFCOUNTER_RSLT_CNTL_DEFAULT                                     0x04000000
#define mmSX_PERFCOUNTER0_SELECT_DEFAULT                                         0x000fffff
#define mmSX_PERFCOUNTER1_SELECT_DEFAULT                                         0x000fffff
#define mmSX_PERFCOUNTER2_SELECT_DEFAULT                                         0x000003ff
#define mmSX_PERFCOUNTER3_SELECT_DEFAULT                                         0x000003ff
#define mmSX_PERFCOUNTER0_SELECT1_DEFAULT                                        0x000fffff
#define mmSX_PERFCOUNTER1_SELECT1_DEFAULT                                        0x000fffff
#define mmGDS_PERFCOUNTER0_SELECT_DEFAULT                                        0x00000000
#define mmGDS_PERFCOUNTER1_SELECT_DEFAULT                                        0x00000000
#define mmGDS_PERFCOUNTER2_SELECT_DEFAULT                                        0x00000000
#define mmGDS_PERFCOUNTER3_SELECT_DEFAULT                                        0x00000000
#define mmGDS_PERFCOUNTER0_SELECT1_DEFAULT                                       0x00000000
#define mmGDS_PERFCOUNTER1_SELECT1_DEFAULT                                       0x00000000
#define mmGDS_PERFCOUNTER2_SELECT1_DEFAULT                                       0x00000000
#define mmGDS_PERFCOUNTER3_SELECT1_DEFAULT                                       0x00000000
#define mmTA_PERFCOUNTER0_SELECT_DEFAULT                                         0x00000000
#define mmTA_PERFCOUNTER0_SELECT1_DEFAULT                                        0x00000000
#define mmTA_PERFCOUNTER1_SELECT_DEFAULT                                         0x00000000
#define mmTD_PERFCOUNTER0_SELECT_DEFAULT                                         0x00000000
#define mmTD_PERFCOUNTER0_SELECT1_DEFAULT                                        0x00000000
#define mmTD_PERFCOUNTER1_SELECT_DEFAULT                                         0x00000000
#define mmTCP_PERFCOUNTER0_SELECT_DEFAULT                                        0x000fffff
#define mmTCP_PERFCOUNTER0_SELECT1_DEFAULT                                       0x000fffff
#define mmTCP_PERFCOUNTER1_SELECT_DEFAULT                                        0x000fffff
#define mmTCP_PERFCOUNTER1_SELECT1_DEFAULT                                       0x000fffff
#define mmTCP_PERFCOUNTER2_SELECT_DEFAULT                                        0x000003ff
#define mmTCP_PERFCOUNTER3_SELECT_DEFAULT                                        0x000003ff
#define mmGL2C_PERFCOUNTER0_SELECT_DEFAULT                                       0x000fffff
#define mmGL2C_PERFCOUNTER0_SELECT1_DEFAULT                                      0x000fffff
#define mmGL2C_PERFCOUNTER1_SELECT_DEFAULT                                       0x000fffff
#define mmGL2C_PERFCOUNTER1_SELECT1_DEFAULT                                      0x000fffff
#define mmGL2C_PERFCOUNTER2_SELECT_DEFAULT                                       0x000003ff
#define mmGL2C_PERFCOUNTER3_SELECT_DEFAULT                                       0x000003ff
#define mmGL2A_PERFCOUNTER0_SELECT_DEFAULT                                       0x000fffff
#define mmGL2A_PERFCOUNTER0_SELECT1_DEFAULT                                      0x000fffff
#define mmGL2A_PERFCOUNTER1_SELECT_DEFAULT                                       0x000fffff
#define mmGL2A_PERFCOUNTER1_SELECT1_DEFAULT                                      0x000fffff
#define mmGL2A_PERFCOUNTER2_SELECT_DEFAULT                                       0x000003ff
#define mmGL2A_PERFCOUNTER3_SELECT_DEFAULT                                       0x000003ff
#define mmGL1C_PERFCOUNTER0_SELECT_DEFAULT                                       0x000fffff
#define mmGL1C_PERFCOUNTER0_SELECT1_DEFAULT                                      0x000fffff
#define mmGL1C_PERFCOUNTER1_SELECT_DEFAULT                                       0x000003ff
#define mmGL1C_PERFCOUNTER2_SELECT_DEFAULT                                       0x000003ff
#define mmGL1C_PERFCOUNTER3_SELECT_DEFAULT                                       0x000003ff
#define mmCHC_PERFCOUNTER0_SELECT_DEFAULT                                        0x000fffff
#define mmCHC_PERFCOUNTER0_SELECT1_DEFAULT                                       0x000fffff
#define mmCHC_PERFCOUNTER1_SELECT_DEFAULT                                        0x000003ff
#define mmCHC_PERFCOUNTER2_SELECT_DEFAULT                                        0x000003ff
#define mmCHC_PERFCOUNTER3_SELECT_DEFAULT                                        0x000003ff
#define mmCHCG_PERFCOUNTER0_SELECT_DEFAULT                                       0x000fffff
#define mmCHCG_PERFCOUNTER0_SELECT1_DEFAULT                                      0x000fffff
#define mmCHCG_PERFCOUNTER1_SELECT_DEFAULT                                       0x000003ff
#define mmCHCG_PERFCOUNTER2_SELECT_DEFAULT                                       0x000003ff
#define mmCHCG_PERFCOUNTER3_SELECT_DEFAULT                                       0x000003ff
#define mmCB_PERFCOUNTER_FILTER_DEFAULT                                          0x00000000
#define mmCB_PERFCOUNTER0_SELECT_DEFAULT                                         0x00000000
#define mmCB_PERFCOUNTER0_SELECT1_DEFAULT                                        0x00000000
#define mmCB_PERFCOUNTER1_SELECT_DEFAULT                                         0x00000000
#define mmCB_PERFCOUNTER2_SELECT_DEFAULT                                         0x00000000
#define mmCB_PERFCOUNTER3_SELECT_DEFAULT                                         0x00000000
#define mmDB_PERFCOUNTER0_SELECT_DEFAULT                                         0x00000000
#define mmDB_PERFCOUNTER0_SELECT1_DEFAULT                                        0x00000000
#define mmDB_PERFCOUNTER1_SELECT_DEFAULT                                         0x00000000
#define mmDB_PERFCOUNTER1_SELECT1_DEFAULT                                        0x00000000
#define mmDB_PERFCOUNTER2_SELECT_DEFAULT                                         0x00000000
#define mmDB_PERFCOUNTER3_SELECT_DEFAULT                                         0x00000000
#define mmRLC_SPM_PERFMON_CNTL_DEFAULT                                           0x00000000
#define mmRLC_SPM_PERFMON_RING_BASE_LO_DEFAULT                                   0x00000000
#define mmRLC_SPM_PERFMON_RING_BASE_HI_DEFAULT                                   0x00000000
#define mmRLC_SPM_PERFMON_RING_SIZE_DEFAULT                                      0x00000000
#define mmRLC_SPM_PERFMON_SEGMENT_SIZE_DEFAULT                                   0x00000000
#define mmRLC_SPM_RING_RDPTR_DEFAULT                                             0x00000000
#define mmRLC_SPM_SEGMENT_THRESHOLD_DEFAULT                                      0x00000000
#define mmRLC_SPM_SE_MUXSEL_ADDR_DEFAULT                                         0x00000000
#define mmRLC_SPM_SE_MUXSEL_DATA_DEFAULT                                         0x00000000
#define mmRLC_SPM_GLOBAL_MUXSEL_ADDR_DEFAULT                                     0x00000000
#define mmRLC_SPM_GLOBAL_MUXSEL_DATA_DEFAULT                                     0x00000000
#define mmRLC_SPM_DESER_START_SKEW_DEFAULT                                       0x00000000
#define mmRLC_SPM_GLOBALS_SAMPLE_SKEW_DEFAULT                                    0x00000000
#define mmRLC_SPM_GLOBALS_MUXSEL_SKEW_DEFAULT                                    0x00000000
#define mmRLC_SPM_SE_SAMPLE_SKEW_DEFAULT                                         0x00000000
#define mmRLC_SPM_SE_MUXSEL_SKEW_DEFAULT                                         0x00000000
#define mmRLC_SPM_GLB_SAMPLEDELAY_IND_ADDR_DEFAULT                               0x00000000
#define mmRLC_SPM_GLB_SAMPLEDELAY_IND_DATA_DEFAULT                               0x00000000
#define mmRLC_SPM_SE_SAMPLEDELAY_IND_ADDR_DEFAULT                                0x00000000
#define mmRLC_SPM_SE_SAMPLEDELAY_IND_DATA_DEFAULT                                0x00000000
#define mmRLC_SPM_RING_WRPTR_DEFAULT                                             0x00000000
#define mmRLC_SPM_ACCUM_DATARAM_ADDR_DEFAULT                                     0x00000000
#define mmRLC_SPM_ACCUM_DATARAM_DATA_DEFAULT                                     0x00000000
#define mmRLC_SPM_ACCUM_CTRLRAM_ADDR_DEFAULT                                     0x00000000
#define mmRLC_SPM_ACCUM_CTRLRAM_DATA_DEFAULT                                     0x00000000
#define mmRLC_SPM_ACCUM_STATUS_DEFAULT                                           0x00000000
#define mmRLC_SPM_ACCUM_CTRL_DEFAULT                                             0x00000000
#define mmRLC_SPM_ACCUM_MODE_DEFAULT                                             0x0007e004
#define mmRLC_SPM_ACCUM_THRESHOLD_DEFAULT                                        0x00000001
#define mmRLC_SPM_ACCUM_SAMPLES_REQUESTED_DEFAULT                                0x00000001
#define mmRLC_SPM_ACCUM_DATARAM_WRCOUNT_DEFAULT                                  0x00000000
#define mmRLC_SPM_PERFMON_SE3TO0_SEGMENT_SIZE_DEFAULT                            0x00000000
#define mmRLC_SPM_PERFMON_GLB_SEGMENT_SIZE_DEFAULT                               0x00000000
#define mmRLC_SPM_VIRT_CTRL_DEFAULT                                              0x00000000
#define mmRLC_SPM_PERFMON_SWA_SEGMENT_SIZE_DEFAULT                               0x00000000
#define mmRLC_SPM_VIRT_STATUS_DEFAULT                                            0x00000000
#define mmRLC_SPM_GFXCLOCK_HIGHCOUNT_DEFAULT                                     0x00000000
#define mmRLC_SPM_GFXCLOCK_LOWCOUNT_DEFAULT                                      0x00000000
#define mmRLC_SPM_PERFMON_SWA_SE3TO0_SEGMENT_SIZE_DEFAULT                        0x00000000
#define mmRLC_SPM_GLOBAL_MUXSEL_ADDR_OFFSET_DEFAULT                              0x00000000
#define mmRLC_SPM_SE_MUXSEL_ADDR_OFFSET_DEFAULT                                  0x00000000
#define mmRLC_SPM_ACCUM_SWA_DATARAM_ADDR_DEFAULT                                 0x00000000
#define mmRLC_SPM_ACCUM_SWA_DATARAM_DATA_DEFAULT                                 0x00000000
#define mmRLC_SPM_ACCUM_CTRLRAM_ADDR_OFFSET_DEFAULT                              0x00000008
#define mmRLC_SPM_PERFMON_SWA_GLB_SEGMENT_SIZE_DEFAULT                           0x00000000
#define mmRLC_SPM_ACCUM_DATARAM_32BITCNTRS_REGIONS_DEFAULT                       0x0000ffff
#define mmRLC_PERFMON_CNTL_DEFAULT                                               0x00000000
#define mmRLC_PERFCOUNTER0_SELECT_DEFAULT                                        0x00000000
#define mmRLC_PERFCOUNTER1_SELECT_DEFAULT                                        0x00000000
#define mmRLC_GPU_IOV_PERF_CNT_CNTL_DEFAULT                                      0x00000000
#define mmRLC_GPU_IOV_PERF_CNT_WR_ADDR_DEFAULT                                   0x00000000
#define mmRLC_GPU_IOV_PERF_CNT_WR_DATA_DEFAULT                                   0x00000000
#define mmRLC_GPU_IOV_PERF_CNT_RD_ADDR_DEFAULT                                   0x00000000
#define mmRLC_GPU_IOV_PERF_CNT_RD_DATA_DEFAULT                                   0x00000000
#define mmRLC_PERFMON_CLK_CNTL_DEFAULT                                           0x00000001
#define mmRMI_PERFCOUNTER0_SELECT_DEFAULT                                        0x00000000
#define mmRMI_PERFCOUNTER0_SELECT1_DEFAULT                                       0x00000000
#define mmRMI_PERFCOUNTER1_SELECT_DEFAULT                                        0x00000000
#define mmRMI_PERFCOUNTER2_SELECT_DEFAULT                                        0x00000000
#define mmRMI_PERFCOUNTER2_SELECT1_DEFAULT                                       0x00000000
#define mmRMI_PERFCOUNTER3_SELECT_DEFAULT                                        0x00000000
#define mmRMI_PERF_COUNTER_CNTL_DEFAULT                                          0x00080240
#define mmGCR_PERFCOUNTER0_SELECT_DEFAULT                                        0x00000000
#define mmGCR_PERFCOUNTER0_SELECT1_DEFAULT                                       0x00000000
#define mmGCR_PERFCOUNTER1_SELECT_DEFAULT                                        0x00000000
#define mmUTCL1_PERFCOUNTER0_SELECT_DEFAULT                                      0x00000000
#define mmUTCL1_PERFCOUNTER1_SELECT_DEFAULT                                      0x00000000
#define mmPA_PH_PERFCOUNTER0_SELECT_DEFAULT                                      0x00000000
#define mmPA_PH_PERFCOUNTER0_SELECT1_DEFAULT                                     0x00000000
#define mmPA_PH_PERFCOUNTER1_SELECT_DEFAULT                                      0x00000000
#define mmPA_PH_PERFCOUNTER2_SELECT_DEFAULT                                      0x00000000
#define mmPA_PH_PERFCOUNTER3_SELECT_DEFAULT                                      0x00000000
#define mmPA_PH_PERFCOUNTER4_SELECT_DEFAULT                                      0x00000000
#define mmPA_PH_PERFCOUNTER5_SELECT_DEFAULT                                      0x00000000
#define mmPA_PH_PERFCOUNTER6_SELECT_DEFAULT                                      0x00000000
#define mmPA_PH_PERFCOUNTER7_SELECT_DEFAULT                                      0x00000000
#define mmPA_PH_PERFCOUNTER1_SELECT1_DEFAULT                                     0x00000000
#define mmPA_PH_PERFCOUNTER2_SELECT1_DEFAULT                                     0x00000000
#define mmPA_PH_PERFCOUNTER3_SELECT1_DEFAULT                                     0x00000000
#define mmGL1A_PERFCOUNTER0_SELECT_DEFAULT                                       0x000fffff
#define mmGL1A_PERFCOUNTER0_SELECT1_DEFAULT                                      0x000fffff
#define mmGL1A_PERFCOUNTER1_SELECT_DEFAULT                                       0x000003ff
#define mmGL1A_PERFCOUNTER2_SELECT_DEFAULT                                       0x000003ff
#define mmGL1A_PERFCOUNTER3_SELECT_DEFAULT                                       0x000003ff
#define mmCHA_PERFCOUNTER0_SELECT_DEFAULT                                        0x000fffff
#define mmCHA_PERFCOUNTER0_SELECT1_DEFAULT                                       0x000fffff
#define mmCHA_PERFCOUNTER1_SELECT_DEFAULT                                        0x000003ff
#define mmCHA_PERFCOUNTER2_SELECT_DEFAULT                                        0x000003ff
#define mmCHA_PERFCOUNTER3_SELECT_DEFAULT                                        0x000003ff
#define mmGUS_PERFCOUNTER2_SELECT_DEFAULT                                        0x000fffff
#define mmGUS_PERFCOUNTER2_SELECT1_DEFAULT                                       0x000fffff
#define mmGUS_PERFCOUNTER2_MODE_DEFAULT                                          0x00000000
#define mmGUS_PERFCOUNTER0_CFG_DEFAULT                                           0x00000000
#define mmGUS_PERFCOUNTER1_CFG_DEFAULT                                           0x00000000
#define mmGUS_PERFCOUNTER_RSLT_CNTL_DEFAULT                                      0x04000000


// addressBlock: gc_gcvml2pldec
#define mmGCMC_VM_L2_PERFCOUNTER0_CFG_DEFAULT                                    0x00000000
#define mmGCMC_VM_L2_PERFCOUNTER1_CFG_DEFAULT                                    0x00000000
#define mmGCMC_VM_L2_PERFCOUNTER2_CFG_DEFAULT                                    0x00000000
#define mmGCMC_VM_L2_PERFCOUNTER3_CFG_DEFAULT                                    0x00000000
#define mmGCMC_VM_L2_PERFCOUNTER4_CFG_DEFAULT                                    0x00000000
#define mmGCMC_VM_L2_PERFCOUNTER5_CFG_DEFAULT                                    0x00000000
#define mmGCMC_VM_L2_PERFCOUNTER6_CFG_DEFAULT                                    0x00000000
#define mmGCMC_VM_L2_PERFCOUNTER7_CFG_DEFAULT                                    0x00000000
#define mmGCMC_VM_L2_PERFCOUNTER_RSLT_CNTL_DEFAULT                               0x04000000
#define mmGCUTCL2_PERFCOUNTER0_CFG_DEFAULT                                       0x00000000
#define mmGCUTCL2_PERFCOUNTER1_CFG_DEFAULT                                       0x00000000
#define mmGCUTCL2_PERFCOUNTER2_CFG_DEFAULT                                       0x00000000
#define mmGCUTCL2_PERFCOUNTER3_CFG_DEFAULT                                       0x00000000
#define mmGCUTCL2_PERFCOUNTER_RSLT_CNTL_DEFAULT                                  0x04000000


// addressBlock: gc_gcvml2perfsdec
#define mmGCVML2_PERFCOUNTER2_0_SELECT_DEFAULT                                   0x000fffff
#define mmGCVML2_PERFCOUNTER2_1_SELECT_DEFAULT                                   0x000fffff
#define mmGCVML2_PERFCOUNTER2_0_SELECT1_DEFAULT                                  0x000fffff
#define mmGCVML2_PERFCOUNTER2_1_SELECT1_DEFAULT                                  0x000fffff
#define mmGCVML2_PERFCOUNTER2_0_MODE_DEFAULT                                     0x00000000
#define mmGCVML2_PERFCOUNTER2_1_MODE_DEFAULT                                     0x00000000


// addressBlock: gc_sdma0_sdma0perfsdec
#define mmSDMA0_PERFCNT_PERFCOUNTER0_CFG_DEFAULT                                 0x0000ffff
#define mmSDMA0_PERFCNT_PERFCOUNTER1_CFG_DEFAULT                                 0x0000ffff
#define mmSDMA0_PERFCNT_PERFCOUNTER_RSLT_CNTL_DEFAULT                            0x04000000
#define mmSDMA0_PERFCNT_MISC_CNTL_DEFAULT                                        0x00000000
#define mmSDMA0_PERFCOUNTER0_SELECT_DEFAULT                                      0x000fffff
#define mmSDMA0_PERFCOUNTER0_SELECT1_DEFAULT                                     0x000fffff
#define mmSDMA0_PERFCOUNTER1_SELECT_DEFAULT                                      0x000fffff
#define mmSDMA0_PERFCOUNTER1_SELECT1_DEFAULT                                     0x000fffff


// addressBlock: gc_sdma1_sdma1perfsdec
#define mmSDMA1_PERFCNT_PERFCOUNTER0_CFG_DEFAULT                                 0x0000ffff
#define mmSDMA1_PERFCNT_PERFCOUNTER1_CFG_DEFAULT                                 0x0000ffff
#define mmSDMA1_PERFCNT_PERFCOUNTER_RSLT_CNTL_DEFAULT                            0x04000000
#define mmSDMA1_PERFCNT_MISC_CNTL_DEFAULT                                        0x00000000
#define mmSDMA1_PERFCOUNTER0_SELECT_DEFAULT                                      0x000fffff
#define mmSDMA1_PERFCOUNTER0_SELECT1_DEFAULT                                     0x000fffff
#define mmSDMA1_PERFCOUNTER1_SELECT_DEFAULT                                      0x000fffff
#define mmSDMA1_PERFCOUNTER1_SELECT1_DEFAULT                                     0x000fffff


// addressBlock: gc_sdma2_sdma2perfsdec
#define mmSDMA2_PERFCNT_PERFCOUNTER0_CFG_DEFAULT                                 0x0000ffff
#define mmSDMA2_PERFCNT_PERFCOUNTER1_CFG_DEFAULT                                 0x0000ffff
#define mmSDMA2_PERFCNT_PERFCOUNTER_RSLT_CNTL_DEFAULT                            0x04000000
#define mmSDMA2_PERFCNT_MISC_CNTL_DEFAULT                                        0x00000000
#define mmSDMA2_PERFCOUNTER0_SELECT_DEFAULT                                      0x000fffff
#define mmSDMA2_PERFCOUNTER0_SELECT1_DEFAULT                                     0x000fffff
#define mmSDMA2_PERFCOUNTER1_SELECT_DEFAULT                                      0x000fffff
#define mmSDMA2_PERFCOUNTER1_SELECT1_DEFAULT                                     0x000fffff


// addressBlock: gc_sdma3_sdma3perfsdec
#define mmSDMA3_PERFCNT_PERFCOUNTER0_CFG_DEFAULT                                 0x0000ffff
#define mmSDMA3_PERFCNT_PERFCOUNTER1_CFG_DEFAULT                                 0x0000ffff
#define mmSDMA3_PERFCNT_PERFCOUNTER_RSLT_CNTL_DEFAULT                            0x04000000
#define mmSDMA3_PERFCNT_MISC_CNTL_DEFAULT                                        0x00000000
#define mmSDMA3_PERFCOUNTER0_SELECT_DEFAULT                                      0x000fffff
#define mmSDMA3_PERFCOUNTER0_SELECT1_DEFAULT                                     0x000fffff
#define mmSDMA3_PERFCOUNTER1_SELECT_DEFAULT                                      0x000fffff
#define mmSDMA3_PERFCOUNTER1_SELECT1_DEFAULT                                     0x000fffff




// addressBlock: gc_grtavfsdec
#define mmGRTAVFS_RTAVFS_REG_ADDR_DEFAULT                                        0x00000000
#define mmRTAVFS_RTAVFS_REG_ADDR_DEFAULT                                         0x00000000
#define mmGRTAVFS_RTAVFS_WR_DATA_DEFAULT                                         0x00000000
#define mmRTAVFS_RTAVFS_WR_DATA_DEFAULT                                          0x00000000
#define mmGRTAVFS_GENERAL_0_DEFAULT                                              0x00000000
#define mmGRTAVFS_RTAVFS_RD_DATA_DEFAULT                                         0x00000000
#define mmGRTAVFS_RTAVFS_REG_CTRL_DEFAULT                                        0x00000000
#define mmGRTAVFS_RTAVFS_REG_STATUS_DEFAULT                                      0x00000000
#define mmGRTAVFS_TARG_FREQ_DEFAULT                                              0x00000000
#define mmGRTAVFS_TARG_VOLT_DEFAULT                                              0x00000000
#define mmGRTAVFS_SOFT_RESET_DEFAULT                                             0x00000001
#define mmGRTAVFS_PSM_CNTL_DEFAULT                                               0x00000001
#define mmGRTAVFS_CLK_CNTL_DEFAULT                                               0x00000003


// addressBlock: gc_rlcdec
#define mmRLC_CNTL_DEFAULT                                                       0x00000001
#define mmRLC_F32_UCODE_VERSION_DEFAULT                                          0x00000000
#define mmRLC_STAT_DEFAULT                                                       0x00000000
#define mmRLC_MEM_SLP_CNTL_DEFAULT                                               0x00020200
#define mmSMU_RLC_RESPONSE_DEFAULT                                               0x00000000
#define mmRLC_RLCV_SAFE_MODE_DEFAULT                                             0x00000000
#define mmRLC_SMU_SAFE_MODE_DEFAULT                                              0x00000000
#define mmRLC_RLCV_COMMAND_DEFAULT                                               0x00000000
#define mmRLC_REFCLOCK_TIMESTAMP_LSB_DEFAULT                                     0x00000000
#define mmRLC_REFCLOCK_TIMESTAMP_MSB_DEFAULT                                     0x00000000
#define mmRLC_GPM_TIMER_INT_0_DEFAULT                                            0x00000063
#define mmRLC_GPM_TIMER_INT_1_DEFAULT                                            0x00000063
#define mmRLC_GPM_TIMER_INT_2_DEFAULT                                            0x00000063
#define mmRLC_GPM_TIMER_CTRL_DEFAULT                                             0x00000000
#define mmRLC_LB_CNTR_MAX_1_DEFAULT                                              0xffffffff
#define mmRLC_GPM_TIMER_STAT_DEFAULT                                             0x00000000
#define mmRLC_GPM_TIMER_INT_3_DEFAULT                                            0x00000063
#define mmRLC_GPM_LEGACY_INT_STAT_DEFAULT                                        0x00000000
#define mmRLC_GPM_LEGACY_INT_CLEAR_DEFAULT                                       0x00000000
#define mmRLC_INT_STAT_DEFAULT                                                   0x00000000
#define mmRLC_LB_CNTL_DEFAULT                                                    0x00000000
#define mmRLC_MGCG_CTRL_DEFAULT                                                  0x00018800
#define mmRLC_LB_CNTR_INIT_1_DEFAULT                                             0x00000000
#define mmRLC_LB_CNTR_1_DEFAULT                                                  0x00000000
#define mmRLC_JUMP_TABLE_RESTORE_DEFAULT                                         0x00000000
#define mmRLC_PG_DELAY_2_DEFAULT                                                 0x00000004
#define mmRLC_GPU_CLOCK_COUNT_LSB_DEFAULT                                        0x00000000
#define mmRLC_GPU_CLOCK_COUNT_MSB_DEFAULT                                        0x00000000
#define mmRLC_CAPTURE_GPU_CLOCK_COUNT_DEFAULT                                    0x00000000
#define mmRLC_UCODE_CNTL_DEFAULT                                                 0x00000000
#define mmRLC_GPM_THREAD_RESET_DEFAULT                                           0x00000004
#define mmRLC_GPM_CP_DMA_COMPLETE_T0_DEFAULT                                     0x00000000
#define mmRLC_GPM_CP_DMA_COMPLETE_T1_DEFAULT                                     0x00000000
#define mmRLC_LB_CNTR_INIT_2_DEFAULT                                             0x00000000
#define mmRLC_LB_CNTR_MAX_2_DEFAULT                                              0xffffffff
#define mmRLC_LB_CONFIG_5_DEFAULT                                                0x00000000
#define mmRLC_GPM_TIMER_INT_4_DEFAULT                                            0x00000063
#define mmRLC_CLK_COUNT_GFXCLK_LSB_DEFAULT                                       0x00000000
#define mmRLC_CLK_COUNT_GFXCLK_MSB_DEFAULT                                       0x00000000
#define mmRLC_CLK_COUNT_REFCLK_LSB_DEFAULT                                       0x00000000
#define mmRLC_CLK_COUNT_REFCLK_MSB_DEFAULT                                       0x00000000
#define mmRLC_CLK_COUNT_CTRL_DEFAULT                                             0x00000000
#define mmRLC_CLK_COUNT_STAT_DEFAULT                                             0x00000000
#define mmRLC_RLCG_DOORBELL_CNTL_DEFAULT                                         0x00260000
#define mmRLC_RLCG_DOORBELL_STAT_DEFAULT                                         0x00000000
#define mmRLC_RLCG_DOORBELL_0_DATA_LO_DEFAULT                                    0x00000000
#define mmRLC_RLCG_DOORBELL_0_DATA_HI_DEFAULT                                    0x00000000
#define mmRLC_RLCG_DOORBELL_1_DATA_LO_DEFAULT                                    0x00000000
#define mmRLC_RLCG_DOORBELL_1_DATA_HI_DEFAULT                                    0x00000000
#define mmRLC_RLCG_DOORBELL_2_DATA_LO_DEFAULT                                    0x00000000
#define mmRLC_RLCG_DOORBELL_2_DATA_HI_DEFAULT                                    0x00000000
#define mmRLC_RLCG_DOORBELL_3_DATA_LO_DEFAULT                                    0x00000000
#define mmRLC_RLCG_DOORBELL_3_DATA_HI_DEFAULT                                    0x00000000
#define mmRLC_GPU_CLOCK_32_RES_SEL_DEFAULT                                       0x00000000
#define mmRLC_GPU_CLOCK_32_DEFAULT                                               0x00000000
#define mmRLC_PG_CNTL_DEFAULT                                                    0x00000000
#define mmRLC_GPM_THREAD_PRIORITY_DEFAULT                                        0x08080808
#define mmRLC_GPM_THREAD_ENABLE_DEFAULT                                          0x00000001
#define mmRLC_RLCG_DOORBELL_RANGE_DEFAULT                                        0x00000000
#define mmRLC_CGCG_CGLS_CTRL_DEFAULT                                             0x0001003c
#define mmRLC_CGCG_RAMP_CTRL_DEFAULT                                             0x00021711
#define mmRLC_DYN_PG_STATUS_DEFAULT                                              0xffffffff
#define mmRLC_DYN_PG_REQUEST_DEFAULT                                             0xffffffff
#define mmRLC_PG_DELAY_DEFAULT                                                   0x00101010
#define mmRLC_WGP_STATUS_DEFAULT                                                 0x00000000
#define mmRLC_LB_INIT_WGP_MASK_DEFAULT                                           0xffffffff
#define mmRLC_LB_ALWAYS_ACTIVE_WGP_MASK_DEFAULT                                  0x00000001
#define mmRLC_LB_PARAMS_DEFAULT                                                  0x00601008
#define mmRLC_LB_DELAY_DEFAULT                                                   0x00400401
#define mmRLC_PG_ALWAYS_ON_WGP_MASK_DEFAULT                                      0x00000003
#define mmRLC_MAX_PG_WGP_DEFAULT                                                 0x0000000a
#define mmRLC_AUTO_PG_CTRL_DEFAULT                                               0x00000000
#define mmRLC_SMU_GRBM_REG_SAVE_CTRL_DEFAULT                                     0x00000000
#define mmRLC_SERDES_RD_INDEX_DEFAULT                                            0x00000000
#define mmRLC_SERDES_RD_DATA_0_DEFAULT                                           0x00000000
#define mmRLC_SERDES_RD_DATA_1_DEFAULT                                           0x00000000
#define mmRLC_SERDES_RD_DATA_2_DEFAULT                                           0x00000000
#define mmRLC_SERDES_RD_DATA_3_DEFAULT                                           0x00000000
#define mmRLC_SERDES_MASK_DEFAULT                                                0x00000000
#define mmRLC_SERDES_CTRL_DEFAULT                                                0x00000000
#define mmRLC_SERDES_DATA_DEFAULT                                                0x00000000
#define mmRLC_SERDES_BUSY_DEFAULT                                                0x00000000
#define mmRLC_GPM_GENERAL_0_DEFAULT                                              0x00000000
#define mmRLC_GPM_GENERAL_1_DEFAULT                                              0x00000000
#define mmRLC_GPM_GENERAL_2_DEFAULT                                              0x00000000
#define mmRLC_GPM_GENERAL_3_DEFAULT                                              0x00000000
#define mmRLC_GPM_GENERAL_4_DEFAULT                                              0x00000000
#define mmRLC_GPM_GENERAL_5_DEFAULT                                              0x00000000
#define mmRLC_GPM_GENERAL_6_DEFAULT                                              0x00000000
#define mmRLC_GPM_GENERAL_7_DEFAULT                                              0x00000000
#define mmRLC_STATIC_PG_STATUS_DEFAULT                                           0xffffffff
#define mmRLC_SPM_INT_INFO_1_DEFAULT                                             0x00000000
#define mmRLC_SPM_INT_INFO_2_DEFAULT                                             0x00ca0000
#define mmRLC_SPM_MC_CNTL_DEFAULT                                                0x00000000
#define mmRLC_SPM_INT_CNTL_DEFAULT                                               0x00000000
#define mmRLC_SPM_INT_STATUS_DEFAULT                                             0x00000000
#define mmRLC_SMU_MESSAGE_DEFAULT                                                0x00000000
#define mmRLC_GPM_LOG_SIZE_DEFAULT                                               0x00000000
#define mmRLC_PG_DELAY_3_DEFAULT                                                 0x00000000
#define mmRLC_GPR_REG1_DEFAULT                                                   0x00000000
#define mmRLC_GPR_REG2_DEFAULT                                                   0x00000000
#define mmRLC_GPM_LOG_CONT_DEFAULT                                               0x00000000
#define mmRLC_GPM_INT_DISABLE_TH0_DEFAULT                                        0xffffffff
#define mmRLC_GPM_LEGACY_INT_DISABLE_DEFAULT                                     0x00000007
#define mmRLC_GPM_INT_FORCE_TH0_DEFAULT                                          0x00000000
#define mmRLC_SRM_CNTL_DEFAULT                                                   0x00000002
#define mmRLC_SRM_GPM_COMMAND_DEFAULT                                            0x00000000
#define mmRLC_SRM_GPM_COMMAND_STATUS_DEFAULT                                     0x00000000
#define mmRLC_SRM_RLCV_COMMAND_DEFAULT                                           0x00000000
#define mmRLC_SRM_RLCV_COMMAND_STATUS_DEFAULT                                    0x00000000
#define mmRLC_SRM_INDEX_CNTL_ADDR_0_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_ADDR_1_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_ADDR_2_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_ADDR_3_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_ADDR_4_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_ADDR_5_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_ADDR_6_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_ADDR_7_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_DATA_0_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_DATA_1_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_DATA_2_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_DATA_3_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_DATA_4_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_DATA_5_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_DATA_6_DEFAULT                                      0x00000000
#define mmRLC_SRM_INDEX_CNTL_DATA_7_DEFAULT                                      0x00000000
#define mmRLC_SRM_STAT_DEFAULT                                                   0x00000000
#define mmRLC_SRM_GPM_ABORT_DEFAULT                                              0x00000000
#define mmRLC_SPARE_INT_2_DEFAULT                                                0x00000000
#define mmRLC_RLCV_SPARE_INT_1_DEFAULT                                           0x00000000
#define mmRLC_PACE_SPARE_INT_1_DEFAULT                                           0x00000000
#define mmRLC_SAFE_MODE_DEFAULT                                                  0x00000000
#define mmRLC_CP_SCHEDULERS_DEFAULT                                              0x58504840
#define mmRLC_CSIB_ADDR_LO_DEFAULT                                               0x00000000
#define mmRLC_CSIB_ADDR_HI_DEFAULT                                               0x00000000
#define mmRLC_CSIB_LENGTH_DEFAULT                                                0x00000000
#define mmRLC_SPARE_INT_0_DEFAULT                                                0x00000000
#define mmRLC_CP_EOF_INT_CNT_DEFAULT                                             0x00000000
#define mmRLC_CP_EOF_INT_DEFAULT                                                 0x00000000
#define mmRLC_SMU_COMMAND_DEFAULT                                                0x00000000
#define mmRLC_SMU_ARGUMENT_1_DEFAULT                                             0x00000000
#define mmRLC_SMU_ARGUMENT_2_DEFAULT                                             0x00000000
#define mmRLC_GPM_GENERAL_8_DEFAULT                                              0x00000000
#define mmRLC_GPM_GENERAL_9_DEFAULT                                              0x00000000
#define mmRLC_GPM_GENERAL_10_DEFAULT                                             0x00000000
#define mmRLC_GPM_GENERAL_11_DEFAULT                                             0x00000000
#define mmRLC_GPM_GENERAL_12_DEFAULT                                             0x00000000
#define mmRLC_GPM_UTCL1_CNTL_0_DEFAULT                                           0x00000080
#define mmRLC_GPM_UTCL1_CNTL_1_DEFAULT                                           0x00000080
#define mmRLC_GPM_UTCL1_CNTL_2_DEFAULT                                           0x00000080
#define mmRLC_SPM_UTCL1_CNTL_DEFAULT                                             0x00000080
#define mmRLC_UTCL1_STATUS_2_DEFAULT                                             0x00000000
#define mmRLC_LB_CONFIG_2_DEFAULT                                                0x00000000
#define mmRLC_LB_CONFIG_3_DEFAULT                                                0x00000000
#define mmRLC_LB_CONFIG_4_DEFAULT                                                0x00000000
#define mmRLC_SPM_UTCL1_ERROR_1_DEFAULT                                          0x00000000
#define mmRLC_SPM_UTCL1_ERROR_2_DEFAULT                                          0x00000000
#define mmRLC_GPM_UTCL1_TH0_ERROR_1_DEFAULT                                      0x00000000
#define mmRLC_LB_CONFIG_1_DEFAULT                                                0x00000000
#define mmRLC_GPM_UTCL1_TH0_ERROR_2_DEFAULT                                      0x00000000
#define mmRLC_GPM_UTCL1_TH1_ERROR_1_DEFAULT                                      0x00000000
#define mmRLC_GPM_UTCL1_TH1_ERROR_2_DEFAULT                                      0x00000000
#define mmRLC_GPM_UTCL1_TH2_ERROR_1_DEFAULT                                      0x00000000
#define mmRLC_GPM_UTCL1_TH2_ERROR_2_DEFAULT                                      0x00000000
#define mmRLC_CGCG_CGLS_CTRL_3D_DEFAULT                                          0x0001003c
#define mmRLC_CGCG_RAMP_CTRL_3D_DEFAULT                                          0x00021711
#define mmRLC_SEMAPHORE_0_DEFAULT                                                0x00000000
#define mmRLC_SEMAPHORE_1_DEFAULT                                                0x00000000
#define mmRLC_PACE_INT_STAT_DEFAULT                                              0x00000000
#define mmRLC_PREWALKER_UTCL1_CNTL_DEFAULT                                       0x00000080
#define mmRLC_PREWALKER_UTCL1_TRIG_DEFAULT                                       0x00000000
#define mmRLC_PREWALKER_UTCL1_ADDR_LSB_DEFAULT                                   0x00000000
#define mmRLC_PREWALKER_UTCL1_ADDR_MSB_DEFAULT                                   0x00000000
#define mmRLC_PREWALKER_UTCL1_SIZE_LSB_DEFAULT                                   0x00000000
#define mmRLC_PREWALKER_UTCL1_SIZE_MSB_DEFAULT                                   0x00000000
#define mmRLC_UTCL1_STATUS_DEFAULT                                               0x00000000
#define mmRLC_R2I_CNTL_0_DEFAULT                                                 0x00000000
#define mmRLC_R2I_CNTL_1_DEFAULT                                                 0x00000000
#define mmRLC_R2I_CNTL_2_DEFAULT                                                 0x00000000
#define mmRLC_R2I_CNTL_3_DEFAULT                                                 0x00000000
#define mmRLC_LB_WGP_STAT_DEFAULT                                                0x00000000
#define mmRLC_GPM_INT_STAT_TH0_DEFAULT                                           0x00000000
#define mmRLC_GPM_GENERAL_13_DEFAULT                                             0x00000000
#define mmRLC_GPM_GENERAL_14_DEFAULT                                             0x00000000
#define mmRLC_GPM_GENERAL_15_DEFAULT                                             0x00000000
#define mmRLC_SPARE_INT_1_DEFAULT                                                0x00000000
#define mmRLC_SEMAPHORE_2_DEFAULT                                                0x00000000
#define mmRLC_SEMAPHORE_3_DEFAULT                                                0x00000000
#define mmRLC_SMU_ARGUMENT_3_DEFAULT                                             0x00000000
#define mmRLC_SMU_ARGUMENT_4_DEFAULT                                             0x00000000
#define mmRLC_GPU_CLOCK_COUNT_LSB_1_DEFAULT                                      0x00000000
#define mmRLC_GPU_CLOCK_COUNT_MSB_1_DEFAULT                                      0x00000000
#define mmRLC_CAPTURE_GPU_CLOCK_COUNT_1_DEFAULT                                  0x00000000
#define mmRLC_GPU_CLOCK_COUNT_LSB_2_DEFAULT                                      0x00000000
#define mmRLC_GPU_CLOCK_COUNT_MSB_2_DEFAULT                                      0x00000000
#define mmRLC_PACE_INT_DISABLE_DEFAULT                                           0xffffffff
#define mmRLC_CAPTURE_GPU_CLOCK_COUNT_2_DEFAULT                                  0x00000000
#define mmRLC_RLCV_DOORBELL_RANGE_DEFAULT                                        0x00000000
#define mmRLC_RLCV_DOORBELL_CNTL_DEFAULT                                         0x00260000
#define mmRLC_RLCV_DOORBELL_STAT_DEFAULT                                         0x00000000
#define mmRLC_RLCV_DOORBELL_0_DATA_LO_DEFAULT                                    0x00000000
#define mmRLC_RLCV_DOORBELL_0_DATA_HI_DEFAULT                                    0x00000000
#define mmRLC_RLCV_DOORBELL_1_DATA_LO_DEFAULT                                    0x00000000
#define mmRLC_RLCV_DOORBELL_1_DATA_HI_DEFAULT                                    0x00000000
#define mmRLC_RLCV_DOORBELL_2_DATA_LO_DEFAULT                                    0x00000000
#define mmRLC_RLCV_DOORBELL_2_DATA_HI_DEFAULT                                    0x00000000
#define mmRLC_RLCV_DOORBELL_3_DATA_LO_DEFAULT                                    0x00000000
#define mmRLC_RLCV_DOORBELL_3_DATA_HI_DEFAULT                                    0x00000000
#define mmRLC_RLCV_SPARE_INT_DEFAULT                                             0x00000000
#define mmRLC_PACE_TIMER_INT_0_DEFAULT                                           0x00000063
#define mmRLC_PACE_TIMER_CTRL_DEFAULT                                            0x00000000
#define mmRLC_PACE_TIMER_INT_1_DEFAULT                                           0x00000063
#define mmRLC_PACE_SPARE_INT_DEFAULT                                             0x00000000
#define mmRLC_SMU_CLK_REQ_DEFAULT                                                0x00000000
#define mmRLC_CP_STAT_INVAL_STAT_DEFAULT                                         0x00000000
#define mmRLC_CP_STAT_INVAL_CTRL_DEFAULT                                         0x00000007
#define mmRLC_CLK_STATUS_DEFAULT                                                 0x00000000
#define mmRLC_SPP_CTRL_DEFAULT                                                   0x00000000
#define mmRLC_SPP_SHADER_PROFILE_EN_DEFAULT                                      0x00000000
#define mmRLC_SPP_SSF_CAPTURE_EN_DEFAULT                                         0x00000000
#define mmRLC_SPP_SSF_THRESHOLD_0_DEFAULT                                        0x009f009f
#define mmRLC_SPP_SSF_THRESHOLD_1_DEFAULT                                        0x009f009f
#define mmRLC_SPP_SSF_THRESHOLD_2_DEFAULT                                        0x009f009f
#define mmRLC_SPP_INFLIGHT_RD_ADDR_DEFAULT                                       0x00000000
#define mmRLC_SPP_INFLIGHT_RD_DATA_DEFAULT                                       0x00000000
#define mmRLC_GPM_GENERAL_16_DEFAULT                                             0x00000000
#define mmRLC_SPP_PROF_INFO_1_DEFAULT                                            0x00000000
#define mmRLC_SPP_PROF_INFO_2_DEFAULT                                            0x00000000
#define mmRLC_SPP_GLOBAL_SH_ID_DEFAULT                                           0x00000000
#define mmRLC_SPP_GLOBAL_SH_ID_VALID_DEFAULT                                     0x00000000
#define mmRLC_SPP_STATUS_DEFAULT                                                 0x00000000
#define mmRLC_SPP_PVT_STAT_0_DEFAULT                                             0x00000000
#define mmRLC_SPP_PVT_STAT_1_DEFAULT                                             0x00000000
#define mmRLC_SPP_PVT_STAT_2_DEFAULT                                             0x00000000
#define mmRLC_SPP_PVT_STAT_3_DEFAULT                                             0x00000000
#define mmRLC_SPP_PVT_LEVEL_MAX_DEFAULT                                          0x00000000
#define mmRLC_SPP_STALL_STATE_UPDATE_DEFAULT                                     0x00000000
#define mmRLC_SPP_PBB_INFO_DEFAULT                                               0x00000000
#define mmRLC_SPP_RESET_DEFAULT                                                  0x00000000
#define mmRLC_SPM_SAMPLE_CNT_DEFAULT                                             0x00000000
#define mmRLC_RLCP_DOORBELL_RANGE_DEFAULT                                        0x00000000
#define mmRLC_RLCP_DOORBELL_CNTL_DEFAULT                                         0x00260000
#define mmRLC_RLCP_DOORBELL_STAT_DEFAULT                                         0x00000000
#define mmRLC_RLCP_DOORBELL_0_DATA_LO_DEFAULT                                    0x00000000
#define mmRLC_RLCP_DOORBELL_0_DATA_HI_DEFAULT                                    0x00000000
#define mmRLC_RLCP_DOORBELL_1_DATA_LO_DEFAULT                                    0x00000000
#define mmRLC_RLCP_DOORBELL_1_DATA_HI_DEFAULT                                    0x00000000
#define mmRLC_RLCP_DOORBELL_2_DATA_LO_DEFAULT                                    0x00000000
#define mmRLC_RLCP_DOORBELL_2_DATA_HI_DEFAULT                                    0x00000000
#define mmRLC_RLCP_DOORBELL_3_DATA_LO_DEFAULT                                    0x00000000
#define mmRLC_RLCP_DOORBELL_3_DATA_HI_DEFAULT                                    0x00000000
#define mmRLC_PCC_STRETCH_HYSTERESIS_CNTL_DEFAULT                                0x00000001
#define mmRLC_CAC_MASK_CNTL_DEFAULT                                              0x000000bf
#define mmRLC_GPU_CLOCK_COUNT_SPM_LSB_DEFAULT                                    0x00000000
#define mmRLC_GPU_CLOCK_COUNT_SPM_MSB_DEFAULT                                    0x00000000
#define mmRLC_SPM_THREAD_TRACE_CTRL_DEFAULT                                      0x00000000
#define mmRLC_LB_CNTR_2_DEFAULT                                                  0x00000000
#define mmRLC_CPAXI_DOORBELL_MON_CTRL_DEFAULT                                    0x00000000
#define mmRLC_CPAXI_DOORBELL_MON_STAT_DEFAULT                                    0x00000000
#define mmRLC_CPAXI_DOORBELL_MON_DATA_LSB_DEFAULT                                0x00000000
#define mmRLC_CPAXI_DOORBELL_MON_DATA_MSB_DEFAULT                                0x00000000
#define mmRLC_XT_DOORBELL_RANGE_DEFAULT                                          0x00000000
#define mmRLC_XT_DOORBELL_CNTL_DEFAULT                                           0x00260000
#define mmRLC_XT_DOORBELL_STAT_DEFAULT                                           0x00000000
#define mmRLC_XT_DOORBELL_0_DATA_LO_DEFAULT                                      0x00000000
#define mmRLC_XT_DOORBELL_0_DATA_HI_DEFAULT                                      0x00000000
#define mmRLC_XT_DOORBELL_1_DATA_LO_DEFAULT                                      0x00000000
#define mmRLC_XT_DOORBELL_1_DATA_HI_DEFAULT                                      0x00000000
#define mmRLC_XT_DOORBELL_2_DATA_LO_DEFAULT                                      0x00000000
#define mmRLC_XT_DOORBELL_2_DATA_HI_DEFAULT                                      0x00000000
#define mmRLC_XT_DOORBELL_3_DATA_LO_DEFAULT                                      0x00000000
#define mmRLC_XT_DOORBELL_3_DATA_HI_DEFAULT                                      0x00000000


// addressBlock: gc_rlcrdec
#define mmRLC_SPP_CAM_ADDR_DEFAULT                                               0x00000000
#define mmRLC_SPP_CAM_DATA_DEFAULT                                               0x00000000
#define mmRLC_SPP_CAM_EXT_ADDR_DEFAULT                                           0x00000000
#define mmRLC_SPP_CAM_EXT_DATA_DEFAULT                                           0x00000000
#define mmRLC_PACE_SCRATCH_ADDR_DEFAULT                                          0x00000000
#define mmRLC_PACE_SCRATCH_DATA_DEFAULT                                          0x00000000


// addressBlock: gc_rlcsdec
#define mmRLC_RLCS_DEC_START_DEFAULT                                             0x00000000
#define mmRLC_RLCS_DEC_DUMP_ADDR_DEFAULT                                         0x00000000
#define mmRLC_RLCS_EXCEPTION_REG_1_DEFAULT                                       0x0003b984
#define mmRLC_RLCS_EXCEPTION_REG_2_DEFAULT                                       0x0003b984
#define mmRLC_RLCS_EXCEPTION_REG_3_DEFAULT                                       0x0003b984
#define mmRLC_RLCS_EXCEPTION_REG_4_DEFAULT                                       0x0003b984
#define mmRLC_RLCS_GENERAL_6_DEFAULT                                             0x00000000
#define mmRLC_RLCS_GENERAL_7_DEFAULT                                             0x00000000
#define mmRLC_RLCS_CGCG_REQUEST_DEFAULT                                          0x00000003
#define mmRLC_RLCS_CGCG_STATUS_DEFAULT                                           0x00000000
#define mmRLC_RLCS_SMU_GFXCLK_STATUS_DEFAULT                                     0x00000000
#define mmRLC_RLCS_SMU_GFXCLK_CONTROL_DEFAULT                                    0x00000000
#define mmRLC_RLCS_SOC_DS_CNTL_DEFAULT                                           0x00ff00c6
#define mmRLC_RLCS_GFX_DS_CNTL_DEFAULT                                           0x00ff00c6
#define mmRLC_GPM_STAT_DEFAULT                                                   0x00a40012
#define mmRLC_RLCS_GPM_STAT_DEFAULT                                              0x00a40012
#define mmRLC_RLCS_ABORTED_PD_SEQUENCE_DEFAULT                                   0x00000000
#define mmRLC_RLCS_DIDT_FORCE_STALL_DEFAULT                                      0x00000000
#define mmRLC_RLCS_IOV_CMD_STATUS_DEFAULT                                        0x00000000
#define mmRLC_RLCS_IOV_CNTX_LOC_SIZE_DEFAULT                                     0x00000000
#define mmRLC_RLCS_IOV_SCH_BLOCK_DEFAULT                                         0x00000000
#define mmRLC_RLCS_IOV_VM_BUSY_STATUS_DEFAULT                                    0x00000000
#define mmRLC_RLCS_GPM_STAT_2_DEFAULT                                            0x00000000
#define mmRLC_RLCS_GRBM_SOFT_RESET_DEFAULT                                       0x00000001
#define mmRLC_RLCS_PG_CHANGE_STATUS_DEFAULT                                      0x00000000
#define mmRLC_RLCS_PG_CHANGE_READ_DEFAULT                                        0x00000000
#define mmRLC_RLCS_LB_STATUS_DEFAULT                                             0x00000000
#define mmRLC_RLCS_LB_READ_DEFAULT                                               0x00000000
#define mmRLC_RLCS_LB_CONTROL_DEFAULT                                            0x00000000
#define mmRLC_RLCS_IH_SEMAPHORE_DEFAULT                                          0x00000000
#define mmRLC_RLCS_IH_COOKIE_SEMAPHORE_DEFAULT                                   0x00000000
#define mmRLC_RLCS_IH_CTRL_1_DEFAULT                                             0x00000000
#define mmRLC_RLCS_IH_CTRL_2_DEFAULT                                             0x00000000
#define mmRLC_RLCS_IH_CTRL_3_DEFAULT                                             0x00000000
#define mmRLC_RLCS_IH_STATUS_DEFAULT                                             0x00000040
#define mmRLC_RLCS_WGP_STATUS_DEFAULT                                            0x00000000
#define mmRLC_RLCS_WGP_READ_DEFAULT                                              0x00000000
#define mmRLC_RLCS_CP_INT_CTRL_1_DEFAULT                                         0x00000000
#define mmRLC_RLCS_CP_INT_CTRL_2_DEFAULT                                         0x00000000
#define mmRLC_RLCS_CP_INT_INFO_1_DEFAULT                                         0x00000000
#define mmRLC_RLCS_CP_INT_INFO_2_DEFAULT                                         0x00000000
#define mmRLC_RLCS_SPM_INT_CTRL_DEFAULT                                          0x00000000
#define mmRLC_RLCS_SPM_INT_INFO_1_DEFAULT                                        0x00000000
#define mmRLC_RLCS_SPM_INT_INFO_2_DEFAULT                                        0x00000000
#define mmRLC_RLCS_DSM_TRIG_DEFAULT                                              0x00000000
#define mmRLC_RLCS_BOOTLOAD_STATUS_DEFAULT                                       0x00000000
#define mmRLC_RLCS_POWER_BRAKE_CNTL_DEFAULT                                      0x00000004
#define mmRLC_RLCS_GENERAL_0_DEFAULT                                             0x00000000
#define mmRLC_RLCS_GENERAL_1_DEFAULT                                             0x00000000
#define mmRLC_RLCS_GENERAL_2_DEFAULT                                             0x00000000
#define mmRLC_RLCS_GENERAL_3_DEFAULT                                             0x00000000
#define mmRLC_RLCS_GENERAL_4_DEFAULT                                             0x00000000
#define mmRLC_RLCS_GENERAL_5_DEFAULT                                             0x00000000
#define mmRLC_RLCS_GRBM_IDLE_BUSY_STAT_DEFAULT                                   0x00000000
#define mmRLC_RLCS_GRBM_IDLE_BUSY_INT_CNTL_DEFAULT                               0x00000000
#define mmRLC_RLCS_CMP_IDLE_CNTL_DEFAULT                                         0x00000100
#define mmRLC_RLCS_POWER_BRAKE_CNTL_TH1_DEFAULT                                  0x00000004
#define mmRLC_RLCS_AUXILIARY_REG_1_DEFAULT                                       0x0003b984
#define mmRLC_RLCS_AUXILIARY_REG_2_DEFAULT                                       0x0003b984
#define mmRLC_RLCS_AUXILIARY_REG_3_DEFAULT                                       0x0003b984
#define mmRLC_RLCS_AUXILIARY_REG_4_DEFAULT                                       0x0003b984
#define mmRLC_RLCS_SPM_SQTT_MODE_DEFAULT                                         0x00000000
#define mmRLC_RLCS_CP_DMA_SRCID_OVER_DEFAULT                                     0x00000000
#define mmRLC_RLCS_UTCL2_CNTL_DEFAULT                                            0x00000018
#define mmRLC_RLCS_MP1_RLC_DOORBELL_CTRL_DEFAULT                                 0x00000000
#define mmRLC_RLCS_BOOTLOAD_ID_STATUS1_DEFAULT                                   0x00000000
#define mmRLC_RLCS_BOOTLOAD_ID_STATUS2_DEFAULT                                   0x00000000
#define mmRLC_RLCS_SMUIO_VIDCHG_CTRL_DEFAULT                                     0x00000000
#define mmRLC_RLCS_EDC_INT_CNTL_DEFAULT                                          0x00000000
#define mmRLC_RLCS_KMD_LOG_CNTL1_DEFAULT                                         0x00000000
#define mmRLC_RLCS_KMD_LOG_CNTL2_DEFAULT                                         0x00000000
#define mmRLC_RLCS_GPM_LEGACY_INT_STAT_DEFAULT                                   0x00000000
#define mmRLC_RLCS_GPM_LEGACY_INT_DISABLE_DEFAULT                                0x00000003
#define mmRLC_RLCS_SRM_SRCID_CNTL_DEFAULT                                        0x00000006
#define mmRLC_RLCS_PERFMON_CLK_CNTL_UCODE_DEFAULT                                0x00000001
#define mmRLC_RLCS_DEC_END_DEFAULT                                               0x00000000


// addressBlock: gc_pwrdec
#define mmSQ_ALU_CLK_CTRL_DEFAULT                                                0x00000000
#define mmSQ_TEX_CLK_CTRL_DEFAULT                                                0x00000000
#define mmSQ_LDS_CLK_CTRL_DEFAULT                                                0x00000000
#define mmRLC_GFX_RM_CNTL_DEFAULT                                                0x00000000


// addressBlock: gc_hypdec
#define mmCP_HYP_PFP_UCODE_ADDR_DEFAULT                                          0x00000000
#define mmCP_PFP_UCODE_ADDR_DEFAULT                                              0x00000000
#define mmCP_HYP_PFP_UCODE_DATA_DEFAULT                                          0x00000000
#define mmCP_PFP_UCODE_DATA_DEFAULT                                              0x00000000
#define mmCP_HYP_ME_UCODE_ADDR_DEFAULT                                           0x00000000
#define mmCP_ME_RAM_RADDR_DEFAULT                                                0x00000000
#define mmCP_ME_RAM_WADDR_DEFAULT                                                0x00000000
#define mmCP_HYP_ME_UCODE_DATA_DEFAULT                                           0x00000000
#define mmCP_ME_RAM_DATA_DEFAULT                                                 0x00000000
#define mmCP_CE_UCODE_ADDR_DEFAULT                                               0x00000000
#define mmCP_HYP_CE_UCODE_ADDR_DEFAULT                                           0x00000000
#define mmCP_CE_UCODE_DATA_DEFAULT                                               0x00000000
#define mmCP_HYP_CE_UCODE_DATA_DEFAULT                                           0x00000000
#define mmCP_HYP_MEC1_UCODE_ADDR_DEFAULT                                         0x00000000
#define mmCP_MEC_ME1_UCODE_ADDR_DEFAULT                                          0x00000000
#define mmCP_HYP_MEC1_UCODE_DATA_DEFAULT                                         0x00000000
#define mmCP_MEC_ME1_UCODE_DATA_DEFAULT                                          0x00000000
#define mmCP_HYP_MEC2_UCODE_ADDR_DEFAULT                                         0x00000000
#define mmCP_MEC_ME2_UCODE_ADDR_DEFAULT                                          0x00000000
#define mmCP_HYP_MEC2_UCODE_DATA_DEFAULT                                         0x00000000
#define mmCP_MEC_ME2_UCODE_DATA_DEFAULT                                          0x00000000
#define mmCP_PFP_IC_BASE_LO_DEFAULT                                              0x00000000
#define mmCP_PFP_IC_BASE_HI_DEFAULT                                              0x00000000
#define mmCP_PFP_IC_BASE_CNTL_DEFAULT                                            0x00000010
#define mmCP_PFP_IC_OP_CNTL_DEFAULT                                              0x00000000
#define mmCP_ME_IC_BASE_LO_DEFAULT                                               0x00000000
#define mmCP_ME_IC_BASE_HI_DEFAULT                                               0x00000000
#define mmCP_ME_IC_BASE_CNTL_DEFAULT                                             0x00000010
#define mmCP_ME_IC_OP_CNTL_DEFAULT                                               0x00000000
#define mmCP_CE_IC_BASE_LO_DEFAULT                                               0x00000000
#define mmCP_CE_IC_BASE_HI_DEFAULT                                               0x00000000
#define mmCP_CE_IC_BASE_CNTL_DEFAULT                                             0x00000010
#define mmCP_CE_IC_OP_CNTL_DEFAULT                                               0x00000000
#define mmCP_CPC_IC_BASE_LO_DEFAULT                                              0x00000000
#define mmCP_CPC_IC_BASE_HI_DEFAULT                                              0x00000000
#define mmCP_CPC_IC_BASE_CNTL_DEFAULT                                            0x00000010
#define mmCP_CPC_IC_OP_CNTL_DEFAULT                                              0x00000000
#define mmCP_MES_IC_BASE_LO_DEFAULT                                              0x00000000
#define mmCP_MES_MIBASE_LO_DEFAULT                                               0x00000000
#define mmCP_MES_IC_BASE_HI_DEFAULT                                              0x00000000
#define mmCP_MES_MIBASE_HI_DEFAULT                                               0x00000000
#define mmCP_MES_IC_BASE_CNTL_DEFAULT                                            0x00000000
#define mmCP_MES_DC_BASE_LO_DEFAULT                                              0x00000000
#define mmCP_MES_MDBASE_LO_DEFAULT                                               0x00000000
#define mmCP_MES_DC_BASE_HI_DEFAULT                                              0x00000000
#define mmCP_MES_MDBASE_HI_DEFAULT                                               0x00000000
#define mmCP_MES_LOCAL_BASE0_LO_DEFAULT                                          0x00000000
#define mmCP_MES_LOCAL_BASE0_HI_DEFAULT                                          0x00000000
#define mmCP_MES_LOCAL_MASK0_LO_DEFAULT                                          0xffff0000
#define mmCP_MES_LOCAL_MASK0_HI_DEFAULT                                          0x0000ffff
#define mmCP_MES_LOCAL_APERTURE_DEFAULT                                          0x00000003
#define mmCP_MES_MIBOUND_LO_DEFAULT                                              0x0000ffff
#define mmCP_MES_MIBOUND_HI_DEFAULT                                              0x00000000
#define mmCP_MES_MDBOUND_LO_DEFAULT                                              0x0000ffff
#define mmCP_MES_MDBOUND_HI_DEFAULT                                              0x0000ffff
#define mmGFX_PIPE_PRIORITY_DEFAULT                                              0x00000001
#define mmGRBM_GFX_INDEX_SR_SELECT_DEFAULT                                       0x00000000
#define mmGRBM_GFX_INDEX_SR_DATA_DEFAULT                                         0xe0000000
#define mmGRBM_GFX_CNTL_SR_SELECT_DEFAULT                                        0x00000000
#define mmGRBM_GFX_CNTL_SR_DATA_DEFAULT                                          0x00000000
#define mmGRBM_CAM_INDEX_DEFAULT                                                 0x00000000
#define mmGRBM_HYP_CAM_INDEX_DEFAULT                                             0x00000000
#define mmGRBM_CAM_DATA_DEFAULT                                                  0x00000000
#define mmGRBM_HYP_CAM_DATA_DEFAULT                                              0x00000000
#define mmGRBM_CAM_DATA_UPPER_DEFAULT                                            0x00000000
#define mmGRBM_HYP_CAM_DATA_UPPER_DEFAULT                                        0x00000000
#define mmGC_IH_COOKIE_0_PTR_DEFAULT                                             0x00004300
#define mmGRBM_SE_REMAP_CNTL_DEFAULT                                             0xeca86420
#define mmRLC_GPU_IOV_VF_ENABLE_DEFAULT                                          0x00000000
#define mmRLC_GPU_IOV_CFG_REG6_DEFAULT                                           0x00000000
#define mmRLC_SDMA0_STATUS_DEFAULT                                               0x00000000
#define mmRLC_SDMA1_STATUS_DEFAULT                                               0x00000000
#define mmRLC_SDMA2_STATUS_DEFAULT                                               0x00000000
#define mmRLC_SDMA3_STATUS_DEFAULT                                               0x00000000
#define mmRLC_SDMA0_BUSY_STATUS_DEFAULT                                          0x00000000
#define mmRLC_SDMA1_BUSY_STATUS_DEFAULT                                          0x00000000
#define mmRLC_SDMA2_BUSY_STATUS_DEFAULT                                          0x00000000
#define mmRLC_SDMA3_BUSY_STATUS_DEFAULT                                          0x00000000
#define mmRLC_GPU_IOV_CFG_REG8_DEFAULT                                           0x00000000
#define mmRLC_RLCV_TIMER_INT_0_DEFAULT                                           0x00000063
#define mmRLC_RLCV_TIMER_CTRL_DEFAULT                                            0x00000000
#define mmRLC_RLCV_TIMER_STAT_DEFAULT                                            0x00000000
#define mmRLC_GPU_IOV_VF_DOORBELL_STATUS_DEFAULT                                 0x7fffffff
#define mmRLC_GPU_IOV_VF_DOORBELL_STATUS_SET_DEFAULT                             0x00000000
#define mmRLC_GPU_IOV_VF_DOORBELL_STATUS_CLR_DEFAULT                             0x00000000
#define mmRLC_GPU_IOV_VF_MASK_DEFAULT                                            0x7fffffff
#define mmRLC_HYP_SEMAPHORE_0_DEFAULT                                            0x00000000
#define mmRLC_HYP_SEMAPHORE_1_DEFAULT                                            0x00000000
#define mmRLC_BUSY_CLK_CNTL_DEFAULT                                              0x00000010
#define mmRLC_CLK_CNTL_DEFAULT                                                   0x00030c0f
#define mmRLC_PACE_TIMER_STAT_DEFAULT                                            0x00000000
#define mmRLC_GPU_IOV_SCH_BLOCK_DEFAULT                                          0x00000000
#define mmRLC_GPU_IOV_CFG_REG1_DEFAULT                                           0x00000000
#define mmRLC_GPU_IOV_CFG_REG2_DEFAULT                                           0x00000000
#define mmRLC_GPU_IOV_VM_BUSY_STATUS_DEFAULT                                     0x00000000
#define mmRLC_GPU_IOV_SCH_0_DEFAULT                                              0x00000000
#define mmRLC_GPU_IOV_ACTIVE_FCN_ID_DEFAULT                                      0x00000000
#define mmRLC_GPU_IOV_SCH_3_DEFAULT                                              0x00000000
#define mmRLC_GPU_IOV_SCH_1_DEFAULT                                              0x00000000
#define mmRLC_GPU_IOV_SCH_2_DEFAULT                                              0x00000000
#define mmRLC_PACE_INT_FORCE_DEFAULT                                             0x00000000
#define mmRLC_PACE_INT_CLEAR_DEFAULT                                             0x00000000
#define mmRLC_GPU_IOV_INT_STAT_DEFAULT                                           0x00000000
#define mmRLC_RLCV_TIMER_INT_1_DEFAULT                                           0x00000063
#define mmRLC_IH_COOKIE_DEFAULT                                                  0x00000000
#define mmRLC_IH_COOKIE_CNTL_DEFAULT                                             0x00000002
#define mmRLC_HYP_RLCG_UCODE_CHKSUM_DEFAULT                                      0x00000000
#define mmRLC_HYP_RLCP_UCODE_CHKSUM_DEFAULT                                      0x00000000
#define mmRLC_HYP_RLCV_UCODE_CHKSUM_DEFAULT                                      0x00000000
#define mmRLC_GPU_IOV_F32_CNTL_DEFAULT                                           0x00000000
#define mmRLC_GPU_IOV_F32_RESET_DEFAULT                                          0x00000000
#define mmRLC_GPU_IOV_SMU_RESPONSE_DEFAULT                                       0x00000000
#define mmRLC_GPU_IOV_VIRT_RESET_REQ_DEFAULT                                     0x00000000
#define mmRLC_GPU_IOV_RLC_RESPONSE_DEFAULT                                       0x00000000
#define mmRLC_GPU_IOV_INT_DISABLE_DEFAULT                                        0xffffffff
#define mmRLC_GPU_IOV_INT_FORCE_DEFAULT                                          0x00000000
#define mmRLC_HYP_SEMAPHORE_2_DEFAULT                                            0x00000000
#define mmRLC_HYP_SEMAPHORE_3_DEFAULT                                            0x00000000
#define mmRLC_HYP_RESET_VECTOR_DEFAULT                                           0x00000000
#define mmRLC_HYP_BOOTLOAD_SIZE_DEFAULT                                          0x00000000
#define mmRLC_HYP_BOOTLOAD_ADDR_LO_DEFAULT                                       0x00000000
#define mmRLC_HYP_BOOTLOAD_ADDR_HI_DEFAULT                                       0x00000000
#define mmRLC_GPM_IRAM_ADDR_DEFAULT                                              0x00000000
#define mmRLC_GPM_IRAM_DATA_DEFAULT                                              0x00000000
#define mmRLC_GPM_UCODE_ADDR_DEFAULT                                             0x00000000
#define mmRLC_GPM_UCODE_DATA_DEFAULT                                             0x00000000
#define mmRLC_PACE_UCODE_ADDR_DEFAULT                                            0x00000000
#define mmRLC_PACE_UCODE_DATA_DEFAULT                                            0x00000000
#define mmRLC_GPU_IOV_UCODE_ADDR_DEFAULT                                         0x00000000
#define mmRLC_GPU_IOV_UCODE_DATA_DEFAULT                                         0x00000000
#define mmRLC_GPU_IOV_SCRATCH_ADDR_DEFAULT                                       0x00000000
#define mmRLC_GPU_IOV_SCRATCH_DATA_DEFAULT                                       0x00000000
#define mmRLC_RLCV_IRAM_ADDR_DEFAULT                                             0x00000000
#define mmRLC_RLCV_IRAM_DATA_DEFAULT                                             0x00000000
#define mmRLC_RLCP_IRAM_ADDR_DEFAULT                                             0x00000000
#define mmRLC_RLCP_IRAM_DATA_DEFAULT                                             0x00000000
#define mmRLC_SRM_DRAM_ADDR_DEFAULT                                              0x00000000
#define mmRLC_SRM_DRAM_DATA_DEFAULT                                              0x00000000
#define mmRLC_SRM_ARAM_ADDR_DEFAULT                                              0x00000000
#define mmRLC_SRM_ARAM_DATA_DEFAULT                                              0x00000000
#define mmRLC_GPM_SCRATCH_ADDR_DEFAULT                                           0x00000000
#define mmRLC_GPM_SCRATCH_DATA_DEFAULT                                           0x00000000
#define mmRLC_GTS_OFFSET_LSB_DEFAULT                                             0x00000000
#define mmRLC_GTS_OFFSET_MSB_DEFAULT                                             0x00000000
#define mmRLC_GPU_IOV_SDMA0_STATUS_DEFAULT                                       0x0000000f
#define mmRLC_GPU_IOV_SDMA1_STATUS_DEFAULT                                       0x0000000f
#define mmRLC_GPU_IOV_SDMA2_STATUS_DEFAULT                                       0x0000000f
#define mmRLC_GPU_IOV_SDMA3_STATUS_DEFAULT                                       0x0000000f
#define mmRLC_GPU_IOV_SDMA4_STATUS_DEFAULT                                       0x0000000f
#define mmRLC_GPU_IOV_SDMA5_STATUS_DEFAULT                                       0x0000000f
#define mmRLC_GPU_IOV_SDMA6_STATUS_DEFAULT                                       0x0000000f
#define mmRLC_GPU_IOV_SDMA7_STATUS_DEFAULT                                       0x0000000f
#define mmRLC_GPU_IOV_SDMA0_BUSY_STATUS_DEFAULT                                  0x00000000
#define mmRLC_GPU_IOV_SDMA1_BUSY_STATUS_DEFAULT                                  0x00000000
#define mmRLC_GPU_IOV_SDMA2_BUSY_STATUS_DEFAULT                                  0x00000000
#define mmRLC_GPU_IOV_SDMA3_BUSY_STATUS_DEFAULT                                  0x00000000
#define mmRLC_GPU_IOV_SDMA4_BUSY_STATUS_DEFAULT                                  0x00000000
#define mmRLC_GPU_IOV_SDMA5_BUSY_STATUS_DEFAULT                                  0x00000000
#define mmRLC_GPU_IOV_SDMA6_BUSY_STATUS_DEFAULT                                  0x00000000
#define mmRLC_GPU_IOV_SDMA7_BUSY_STATUS_DEFAULT                                  0x00000000


// addressBlock: gc_sdma0_sdma0hypdec
#define mmSDMA0_UCODE_ADDR_DEFAULT                                               0x00000000
#define mmSDMA0_UCODE_DATA_DEFAULT                                               0x00000000
#define mmSDMA0_VM_CTX_LO_DEFAULT                                                0x00000000
#define mmSDMA0_VM_CTX_HI_DEFAULT                                                0x00000000
#define mmSDMA0_ACTIVE_FCN_ID_DEFAULT                                            0x00000000
#define mmSDMA0_VM_CTX_CNTL_DEFAULT                                              0x00000001
#define mmSDMA0_VIRT_RESET_REQ_DEFAULT                                           0x00000000
#define mmSDMA0_VF_ENABLE_DEFAULT                                                0x00000000
#define mmSDMA0_CONTEXT_REG_TYPE0_DEFAULT                                        0xfffdf79f
#define mmSDMA0_CONTEXT_REG_TYPE1_DEFAULT                                        0x003fbcff
#define mmSDMA0_CONTEXT_REG_TYPE2_DEFAULT                                        0x00000fff
#define mmSDMA0_CONTEXT_REG_TYPE3_DEFAULT                                        0x00000000
#define mmSDMA0_PUB_REG_TYPE0_DEFAULT                                            0xf4000000
#define mmSDMA0_PUB_REG_TYPE1_DEFAULT                                            0x30003882
#define mmSDMA0_PUB_REG_TYPE2_DEFAULT                                            0x5c46e880
#define mmSDMA0_PUB_REG_TYPE3_DEFAULT                                            0x20000004
#define mmSDMA0_VM_CNTL_DEFAULT                                                  0x00000000
#define mmSDMA0_BROADCAST_UCODE_ADDR_DEFAULT                                     0x00000000
#define mmSDMA0_BROADCAST_UCODE_DATA_DEFAULT                                     0x00000000


// addressBlock: gc_sdma1_sdma1hypdec
#define mmSDMA1_UCODE_ADDR_DEFAULT                                               0x00000000
#define mmSDMA1_UCODE_DATA_DEFAULT                                               0x00000000
#define mmSDMA1_VM_CTX_LO_DEFAULT                                                0x00000000
#define mmSDMA1_VM_CTX_HI_DEFAULT                                                0x00000000
#define mmSDMA1_ACTIVE_FCN_ID_DEFAULT                                            0x00000000
#define mmSDMA1_VM_CTX_CNTL_DEFAULT                                              0x00000001
#define mmSDMA1_VIRT_RESET_REQ_DEFAULT                                           0x00000000
#define mmSDMA1_VF_ENABLE_DEFAULT                                                0x00000000
#define mmSDMA1_CONTEXT_REG_TYPE0_DEFAULT                                        0xfffdf79f
#define mmSDMA1_CONTEXT_REG_TYPE1_DEFAULT                                        0x003fbcff
#define mmSDMA1_CONTEXT_REG_TYPE2_DEFAULT                                        0x00000fff
#define mmSDMA1_CONTEXT_REG_TYPE3_DEFAULT                                        0x00000000
#define mmSDMA1_PUB_REG_TYPE0_DEFAULT                                            0xf4000000
#define mmSDMA1_PUB_REG_TYPE1_DEFAULT                                            0x30003882
#define mmSDMA1_PUB_REG_TYPE2_DEFAULT                                            0x5c46e880
#define mmSDMA1_PUB_REG_TYPE3_DEFAULT                                            0x20000004
#define mmSDMA1_VM_CNTL_DEFAULT                                                  0x00000000


// addressBlock: gc_sdma2_sdma2hypdec
#define mmSDMA2_UCODE_ADDR_DEFAULT                                               0x00000000
#define mmSDMA2_UCODE_DATA_DEFAULT                                               0x00000000
#define mmSDMA2_VM_CTX_LO_DEFAULT                                                0x00000000
#define mmSDMA2_VM_CTX_HI_DEFAULT                                                0x00000000
#define mmSDMA2_ACTIVE_FCN_ID_DEFAULT                                            0x00000000
#define mmSDMA2_VM_CTX_CNTL_DEFAULT                                              0x00000001
#define mmSDMA2_VIRT_RESET_REQ_DEFAULT                                           0x00000000
#define mmSDMA2_VF_ENABLE_DEFAULT                                                0x00000000
#define mmSDMA2_CONTEXT_REG_TYPE0_DEFAULT                                        0xfffdf79f
#define mmSDMA2_CONTEXT_REG_TYPE1_DEFAULT                                        0x003fbcff
#define mmSDMA2_CONTEXT_REG_TYPE2_DEFAULT                                        0x00000fff
#define mmSDMA2_CONTEXT_REG_TYPE3_DEFAULT                                        0x00000000
#define mmSDMA2_PUB_REG_TYPE0_DEFAULT                                            0xf4000000
#define mmSDMA2_PUB_REG_TYPE1_DEFAULT                                            0x30003882
#define mmSDMA2_PUB_REG_TYPE2_DEFAULT                                            0x5c46e880
#define mmSDMA2_PUB_REG_TYPE3_DEFAULT                                            0x20000004
#define mmSDMA2_VM_CNTL_DEFAULT                                                  0x00000000


// addressBlock: gc_sdma3_sdma3hypdec
#define mmSDMA3_UCODE_ADDR_DEFAULT                                               0x00000000
#define mmSDMA3_UCODE_DATA_DEFAULT                                               0x00000000
#define mmSDMA3_VM_CTX_LO_DEFAULT                                                0x00000000
#define mmSDMA3_VM_CTX_HI_DEFAULT                                                0x00000000
#define mmSDMA3_ACTIVE_FCN_ID_DEFAULT                                            0x00000000
#define mmSDMA3_VM_CTX_CNTL_DEFAULT                                              0x00000001
#define mmSDMA3_VIRT_RESET_REQ_DEFAULT                                           0x00000000
#define mmSDMA3_VF_ENABLE_DEFAULT                                                0x00000000
#define mmSDMA3_CONTEXT_REG_TYPE0_DEFAULT                                        0xfffdf79f
#define mmSDMA3_CONTEXT_REG_TYPE1_DEFAULT                                        0x003fbcff
#define mmSDMA3_CONTEXT_REG_TYPE2_DEFAULT                                        0x00000fff
#define mmSDMA3_CONTEXT_REG_TYPE3_DEFAULT                                        0x00000000
#define mmSDMA3_PUB_REG_TYPE0_DEFAULT                                            0xf4000000
#define mmSDMA3_PUB_REG_TYPE1_DEFAULT                                            0x30003882
#define mmSDMA3_PUB_REG_TYPE2_DEFAULT                                            0x5c46e880
#define mmSDMA3_PUB_REG_TYPE3_DEFAULT                                            0x20000004
#define mmSDMA3_VM_CNTL_DEFAULT                                                  0x00000000


// addressBlock: gc_gcvmsharedhvdec
#define mmGCMC_VM_FB_SIZE_OFFSET_VF0_DEFAULT                                     0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF1_DEFAULT                                     0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF2_DEFAULT                                     0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF3_DEFAULT                                     0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF4_DEFAULT                                     0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF5_DEFAULT                                     0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF6_DEFAULT                                     0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF7_DEFAULT                                     0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF8_DEFAULT                                     0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF9_DEFAULT                                     0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF10_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF11_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF12_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF13_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF14_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF15_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF16_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF17_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF18_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF19_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF20_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF21_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF22_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF23_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF24_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF25_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF26_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF27_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF28_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF29_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF30_DEFAULT                                    0x00000000
#define mmGCMC_VM_FB_SIZE_OFFSET_VF31_DEFAULT                                    0x00000000
#define mmGCVM_IOMMU_MMIO_CNTRL_1_DEFAULT                                        0x00000100
#define mmGCMC_VM_MARC_BASE_LO_0_DEFAULT                                         0x00000000
#define mmGCMC_VM_MARC_BASE_LO_1_DEFAULT                                         0x00000000
#define mmGCMC_VM_MARC_BASE_LO_2_DEFAULT                                         0x00000000
#define mmGCMC_VM_MARC_BASE_LO_3_DEFAULT                                         0x00000000
#define mmGCMC_VM_MARC_BASE_HI_0_DEFAULT                                         0x00000000
#define mmGCMC_VM_MARC_BASE_HI_1_DEFAULT                                         0x00000000
#define mmGCMC_VM_MARC_BASE_HI_2_DEFAULT                                         0x00000000
#define mmGCMC_VM_MARC_BASE_HI_3_DEFAULT                                         0x00000000
#define mmGCMC_VM_MARC_RELOC_LO_0_DEFAULT                                        0x00000000
#define mmGCMC_VM_MARC_RELOC_LO_1_DEFAULT                                        0x00000000
#define mmGCMC_VM_MARC_RELOC_LO_2_DEFAULT                                        0x00000000
#define mmGCMC_VM_MARC_RELOC_LO_3_DEFAULT                                        0x00000000
#define mmGCMC_VM_MARC_RELOC_HI_0_DEFAULT                                        0x00000000
#define mmGCMC_VM_MARC_RELOC_HI_1_DEFAULT                                        0x00000000
#define mmGCMC_VM_MARC_RELOC_HI_2_DEFAULT                                        0x00000000
#define mmGCMC_VM_MARC_RELOC_HI_3_DEFAULT                                        0x00000000
#define mmGCMC_VM_MARC_LEN_LO_0_DEFAULT                                          0x00000000
#define mmGCMC_VM_MARC_LEN_LO_1_DEFAULT                                          0x00000000
#define mmGCMC_VM_MARC_LEN_LO_2_DEFAULT                                          0x00000000
#define mmGCMC_VM_MARC_LEN_LO_3_DEFAULT                                          0x00000000
#define mmGCMC_VM_MARC_LEN_HI_0_DEFAULT                                          0x00000000
#define mmGCMC_VM_MARC_LEN_HI_1_DEFAULT                                          0x00000000
#define mmGCMC_VM_MARC_LEN_HI_2_DEFAULT                                          0x00000000
#define mmGCMC_VM_MARC_LEN_HI_3_DEFAULT                                          0x00000000
#define mmGCVM_IOMMU_CONTROL_REGISTER_DEFAULT                                    0x00000000
#define mmGCVM_IOMMU_PERFORMANCE_OPTIMIZATION_CONTROL_REGISTER_DEFAULT           0x00000000
#define mmGCMC_VM_XGMI_GPUIOV_ENABLE_DEFAULT                                     0x00000000


// addressBlock: gc_pspdec
#define mmCPG_PSP_DEBUG_DEFAULT                                                  0x00000000
#define mmCPC_PSP_DEBUG_DEFAULT                                                  0x00000000
#define mmGRBM_SEC_CNTL_DEFAULT                                                  0x00000000
#define mmRLC_FWL_FIRST_VIOL_ADDR_DEFAULT                                        0x00000000
#define mmRLC_SRM_FWL_FIRST_VIOL_ADDR_DEFAULT                                    0x00000000


// addressBlock: gc_gcvml2pspdec
#define mmGCVM_L2_ID_CTRL0_DEFAULT                                               0xffffffff
#define mmGCVM_L2_ID_CTRL1_DEFAULT                                               0xffffffff
#define mmGCVM_L2_ID_CTRL2_DEFAULT                                               0xffffffff
#define mmGCVM_L2_ID_CTRL3_DEFAULT                                               0xffffffff
#define mmGCVM_L2_ID_CTRL4_DEFAULT                                               0xffffffff
#define mmGCVM_L2_ID_CTRL5_DEFAULT                                               0xffffffff
#define mmGCVM_L2_ID_CTRL6_DEFAULT                                               0xffffffff
#define mmGCVM_L2_ID_CTRL7_DEFAULT                                               0xffffffff
#define mmGCVM_L2_ID_CTRL_HI_DEFAULT                                             0x0000ffff
#define mmGCVM_L2_ID_STATUS_DEFAULT                                              0x00000000
#define mmGCUTCL2_TRANSLATION_BYPASS_BY_VMID_DEFAULT                             0x00000000
#define mmGCVM_IOMMU_GPU_HOST_TRANSLATION_ENABLE_DEFAULT                         0x00000000
#define mmGCUTC_GPUVA_VMID_TRANSLATION_ASSIST_REQUEST_LO_DEFAULT                 0x00000000
#define mmGCUTC_GPUVA_VMID_TRANSLATION_ASSIST_REQUEST_HI_DEFAULT                 0x00000000
#define mmGCUTC_GPUVA_VMID_TRANSLATION_ASSIST_RESPONSE_LO_DEFAULT                0x00000000
#define mmGCUTC_GPUVA_VMID_TRANSLATION_ASSIST_RESPONSE_HI_DEFAULT                0x00000000


// addressBlock: gc_sdma2_sdma2dec
#define mmSDMA2_DEC_START_DEFAULT                                                0x00000000
#define mmSDMA2_GLOBAL_TIMESTAMP_LO_DEFAULT                                      0x00000000
#define mmSDMA2_GLOBAL_TIMESTAMP_HI_DEFAULT                                      0x00000000
#define mmSDMA2_PG_CNTL_DEFAULT                                                  0x00000000
#define mmSDMA2_PG_CTX_LO_DEFAULT                                                0x00000000
#define mmSDMA2_PG_CTX_HI_DEFAULT                                                0x00000000
#define mmSDMA2_PG_CTX_CNTL_DEFAULT                                              0x00000000
#define mmSDMA2_POWER_CNTL_DEFAULT                                               0x40000050
#define mmSDMA2_CLK_CTRL_DEFAULT                                                 0x00000100
#define mmSDMA2_CNTL_DEFAULT                                                     0x000000c2
#define mmSDMA2_CHICKEN_BITS_DEFAULT                                             0x03ef0107
#define mmSDMA2_GB_ADDR_CONFIG_DEFAULT                                           0x00000444
#define mmSDMA2_GB_ADDR_CONFIG_READ_DEFAULT                                      0x00000444
#define mmSDMA2_RB_RPTR_FETCH_HI_DEFAULT                                         0x00000000
#define mmSDMA2_SEM_WAIT_FAIL_TIMER_CNTL_DEFAULT                                 0x00000000
#define mmSDMA2_RB_RPTR_FETCH_DEFAULT                                            0x00000000
#define mmSDMA2_IB_OFFSET_FETCH_DEFAULT                                          0x00000000
#define mmSDMA2_PROGRAM_DEFAULT                                                  0x00000000
#define mmSDMA2_STATUS_REG_DEFAULT                                               0x46dee557
#define mmSDMA2_STATUS1_REG_DEFAULT                                              0x000003ff
#define mmSDMA2_RD_BURST_CNTL_DEFAULT                                            0x00000002
#define mmSDMA2_HBM_PAGE_CONFIG_DEFAULT                                          0x00000000
#define mmSDMA2_UCODE_CHECKSUM_DEFAULT                                           0x00000000
#define mmSDMA2_F32_CNTL_DEFAULT                                                 0x00000001
#define mmSDMA2_FREEZE_DEFAULT                                                   0x00000000
#define mmSDMA2_PHASE0_QUANTUM_DEFAULT                                           0x00010002
#define mmSDMA2_PHASE1_QUANTUM_DEFAULT                                           0x00010002
#define mmSDMA2_EDC_CONFIG_DEFAULT                                               0x00000002
#define mmSDMA2_BA_THRESHOLD_DEFAULT                                             0x03ff03ff
#define mmSDMA2_ID_DEFAULT                                                       0x00000001
#define mmSDMA2_VERSION_DEFAULT                                                  0x00000500
#define mmSDMA2_EDC_COUNTER_DEFAULT                                              0x00000000
#define mmSDMA2_EDC_COUNTER_CLEAR_DEFAULT                                        0x00000000
#define mmSDMA2_STATUS2_REG_DEFAULT                                              0x00000001
#define mmSDMA2_ATOMIC_CNTL_DEFAULT                                              0x00000200
#define mmSDMA2_ATOMIC_PREOP_LO_DEFAULT                                          0x00000000
#define mmSDMA2_ATOMIC_PREOP_HI_DEFAULT                                          0x00000000
#define mmSDMA2_UTCL1_CNTL_DEFAULT                                               0xd0000191
#define mmSDMA2_UTCL1_WATERMK_DEFAULT                                            0xfffbd9fb
#define mmSDMA2_UTCL1_RD_STATUS_DEFAULT                                          0x01011555
#define mmSDMA2_UTCL1_WR_STATUS_DEFAULT                                          0x51011555
#define mmSDMA2_UTCL1_INV0_DEFAULT                                               0x00000800
#define mmSDMA2_UTCL1_INV1_DEFAULT                                               0x00000000
#define mmSDMA2_UTCL1_INV2_DEFAULT                                               0x00000000
#define mmSDMA2_UTCL1_RD_XNACK0_DEFAULT                                          0x00000000
#define mmSDMA2_UTCL1_RD_XNACK1_DEFAULT                                          0x00000000
#define mmSDMA2_UTCL1_WR_XNACK0_DEFAULT                                          0x00000000
#define mmSDMA2_UTCL1_WR_XNACK1_DEFAULT                                          0x00000000
#define mmSDMA2_UTCL1_TIMEOUT_DEFAULT                                            0x00000000
#define mmSDMA2_UTCL1_PAGE_DEFAULT                                               0x010cec00
#define mmSDMA2_RELAX_ORDERING_LUT_DEFAULT                                       0xc0000006
#define mmSDMA2_CHICKEN_BITS_2_DEFAULT                                           0x00100007
#define mmSDMA2_STATUS3_REG_DEFAULT                                              0x03f00000
#define mmSDMA2_PHYSICAL_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA2_PHYSICAL_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA2_PHASE2_QUANTUM_DEFAULT                                           0x00010002
#define mmSDMA2_ERROR_LOG_DEFAULT                                                0x0000000f
#define mmSDMA2_PUB_DUMMY_REG0_DEFAULT                                           0x00000000
#define mmSDMA2_PUB_DUMMY_REG1_DEFAULT                                           0x00000000
#define mmSDMA2_PUB_DUMMY_REG2_DEFAULT                                           0x00000000
#define mmSDMA2_PUB_DUMMY_REG3_DEFAULT                                           0x00000000
#define mmSDMA2_F32_COUNTER_DEFAULT                                              0x00000000
#define mmSDMA2_CRD_CNTL_DEFAULT                                                 0x1850c640
#define mmSDMA2_AQL_STATUS_DEFAULT                                               0x00000003
#define mmSDMA2_EA_DBIT_ADDR_DATA_DEFAULT                                        0x00000000
#define mmSDMA2_EA_DBIT_ADDR_INDEX_DEFAULT                                       0x00000000
#define mmSDMA2_TLBI_GCR_CNTL_DEFAULT                                            0x40180454
#define mmSDMA2_TILING_CONFIG_DEFAULT                                            0x00000000
#define mmSDMA2_INT_STATUS_DEFAULT                                               0x00000000
#define mmSDMA2_HOLE_ADDR_LO_DEFAULT                                             0x00000000
#define mmSDMA2_HOLE_ADDR_HI_DEFAULT                                             0x00000000
#define mmSDMA2_CLOCK_GATING_REG_DEFAULT                                         0x00000000
#define mmSDMA2_STATUS4_REG_DEFAULT                                              0x00000001
#define mmSDMA2_SCRATCH_RAM_DATA_DEFAULT                                         0x00000000
#define mmSDMA2_SCRATCH_RAM_ADDR_DEFAULT                                         0x00000000
#define mmSDMA2_TIMESTAMP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA2_STATUS5_REG_DEFAULT                                              0x00000000
#define mmSDMA2_QUEUE_RESET_REQ_DEFAULT                                          0x00000000
#define mmSDMA2_GFX_RB_CNTL_DEFAULT                                              0x80840000
#define mmSDMA2_GFX_RB_BASE_DEFAULT                                              0x00000000
#define mmSDMA2_GFX_RB_BASE_HI_DEFAULT                                           0x00000000
#define mmSDMA2_GFX_RB_RPTR_DEFAULT                                              0x00000000
#define mmSDMA2_GFX_RB_RPTR_HI_DEFAULT                                           0x00000000
#define mmSDMA2_GFX_RB_WPTR_DEFAULT                                              0x00000000
#define mmSDMA2_GFX_RB_WPTR_HI_DEFAULT                                           0x00000000
#define mmSDMA2_GFX_RB_WPTR_POLL_CNTL_DEFAULT                                    0x00403000
#define mmSDMA2_GFX_RB_RPTR_ADDR_HI_DEFAULT                                      0x00000000
#define mmSDMA2_GFX_RB_RPTR_ADDR_LO_DEFAULT                                      0x00000000
#define mmSDMA2_GFX_IB_CNTL_DEFAULT                                              0x00000100
#define mmSDMA2_GFX_IB_RPTR_DEFAULT                                              0x00000000
#define mmSDMA2_GFX_IB_OFFSET_DEFAULT                                            0x00000000
#define mmSDMA2_GFX_IB_BASE_LO_DEFAULT                                           0x00000000
#define mmSDMA2_GFX_IB_BASE_HI_DEFAULT                                           0x00000000
#define mmSDMA2_GFX_IB_SIZE_DEFAULT                                              0x00000000
#define mmSDMA2_GFX_SKIP_CNTL_DEFAULT                                            0x00000000
#define mmSDMA2_GFX_CONTEXT_STATUS_DEFAULT                                       0x00000005
#define mmSDMA2_GFX_DOORBELL_DEFAULT                                             0x00000000
#define mmSDMA2_GFX_CONTEXT_CNTL_DEFAULT                                         0x00000000
#define mmSDMA2_GFX_STATUS_DEFAULT                                               0x00000000
#define mmSDMA2_GFX_DOORBELL_LOG_DEFAULT                                         0x00000000
#define mmSDMA2_GFX_WATERMARK_DEFAULT                                            0x00000000
#define mmSDMA2_GFX_DOORBELL_OFFSET_DEFAULT                                      0x00000000
#define mmSDMA2_GFX_CSA_ADDR_LO_DEFAULT                                          0x00000000
#define mmSDMA2_GFX_CSA_ADDR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_GFX_IB_SUB_REMAIN_DEFAULT                                        0x00000000
#define mmSDMA2_GFX_PREEMPT_DEFAULT                                              0x00000000
#define mmSDMA2_GFX_DUMMY_REG_DEFAULT                                            0x0000000f
#define mmSDMA2_GFX_RB_WPTR_POLL_ADDR_HI_DEFAULT                                 0x00000000
#define mmSDMA2_GFX_RB_WPTR_POLL_ADDR_LO_DEFAULT                                 0x00000000
#define mmSDMA2_GFX_RB_AQL_CNTL_DEFAULT                                          0x00004000
#define mmSDMA2_GFX_MINOR_PTR_UPDATE_DEFAULT                                     0x00000000
#define mmSDMA2_GFX_MIDCMD_DATA0_DEFAULT                                         0x00000000
#define mmSDMA2_GFX_MIDCMD_DATA1_DEFAULT                                         0x00000000
#define mmSDMA2_GFX_MIDCMD_DATA2_DEFAULT                                         0x00000000
#define mmSDMA2_GFX_MIDCMD_DATA3_DEFAULT                                         0x00000000
#define mmSDMA2_GFX_MIDCMD_DATA4_DEFAULT                                         0x00000000
#define mmSDMA2_GFX_MIDCMD_DATA5_DEFAULT                                         0x00000000
#define mmSDMA2_GFX_MIDCMD_DATA6_DEFAULT                                         0x00000000
#define mmSDMA2_GFX_MIDCMD_DATA7_DEFAULT                                         0x00000000
#define mmSDMA2_GFX_MIDCMD_DATA8_DEFAULT                                         0x00000000
#define mmSDMA2_GFX_MIDCMD_DATA9_DEFAULT                                         0x00000000
#define mmSDMA2_GFX_MIDCMD_DATA10_DEFAULT                                        0x00000000
#define mmSDMA2_GFX_MIDCMD_CNTL_DEFAULT                                          0x00000000
#define mmSDMA2_PAGE_RB_CNTL_DEFAULT                                             0x80840000
#define mmSDMA2_PAGE_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA2_PAGE_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_PAGE_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_PAGE_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_PAGE_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA2_PAGE_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_PAGE_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA2_PAGE_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA2_PAGE_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA2_PAGE_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA2_PAGE_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_PAGE_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA2_PAGE_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA2_PAGE_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_PAGE_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA2_PAGE_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA2_PAGE_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA2_PAGE_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA2_PAGE_STATUS_DEFAULT                                              0x00000000
#define mmSDMA2_PAGE_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA2_PAGE_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA2_PAGE_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA2_PAGE_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA2_PAGE_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA2_PAGE_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA2_PAGE_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA2_PAGE_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA2_PAGE_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA2_PAGE_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA2_PAGE_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA2_PAGE_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA2_PAGE_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA2_PAGE_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA2_PAGE_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA2_PAGE_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA2_PAGE_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA2_PAGE_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA2_PAGE_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA2_PAGE_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA2_PAGE_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA2_PAGE_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA2_PAGE_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA2_PAGE_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA2_RLC0_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA2_RLC0_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC0_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC0_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC0_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC0_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC0_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC0_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA2_RLC0_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA2_RLC0_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA2_RLC0_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA2_RLC0_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC0_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA2_RLC0_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA2_RLC0_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC0_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC0_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA2_RLC0_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA2_RLC0_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA2_RLC0_STATUS_DEFAULT                                              0x00000000
#define mmSDMA2_RLC0_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA2_RLC0_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA2_RLC0_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA2_RLC0_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA2_RLC0_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA2_RLC0_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA2_RLC0_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA2_RLC0_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA2_RLC0_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA2_RLC0_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA2_RLC0_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA2_RLC0_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA2_RLC0_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA2_RLC0_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA2_RLC0_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA2_RLC0_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA2_RLC0_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA2_RLC0_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA2_RLC0_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA2_RLC0_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA2_RLC0_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA2_RLC0_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA2_RLC0_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA2_RLC0_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA2_RLC1_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA2_RLC1_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC1_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC1_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC1_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC1_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC1_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC1_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA2_RLC1_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA2_RLC1_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA2_RLC1_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA2_RLC1_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC1_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA2_RLC1_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA2_RLC1_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC1_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC1_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA2_RLC1_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA2_RLC1_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA2_RLC1_STATUS_DEFAULT                                              0x00000000
#define mmSDMA2_RLC1_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA2_RLC1_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA2_RLC1_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA2_RLC1_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA2_RLC1_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA2_RLC1_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA2_RLC1_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA2_RLC1_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA2_RLC1_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA2_RLC1_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA2_RLC1_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA2_RLC1_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA2_RLC1_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA2_RLC1_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA2_RLC1_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA2_RLC1_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA2_RLC1_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA2_RLC1_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA2_RLC1_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA2_RLC1_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA2_RLC1_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA2_RLC1_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA2_RLC1_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA2_RLC1_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA2_RLC2_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA2_RLC2_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC2_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC2_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC2_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC2_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC2_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC2_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA2_RLC2_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA2_RLC2_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA2_RLC2_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA2_RLC2_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC2_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA2_RLC2_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA2_RLC2_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC2_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC2_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA2_RLC2_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA2_RLC2_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA2_RLC2_STATUS_DEFAULT                                              0x00000000
#define mmSDMA2_RLC2_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA2_RLC2_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA2_RLC2_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA2_RLC2_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA2_RLC2_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA2_RLC2_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA2_RLC2_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA2_RLC2_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA2_RLC2_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA2_RLC2_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA2_RLC2_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA2_RLC2_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA2_RLC2_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA2_RLC2_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA2_RLC2_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA2_RLC2_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA2_RLC2_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA2_RLC2_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA2_RLC2_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA2_RLC2_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA2_RLC2_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA2_RLC2_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA2_RLC2_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA2_RLC2_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA2_RLC3_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA2_RLC3_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC3_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC3_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC3_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC3_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC3_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC3_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA2_RLC3_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA2_RLC3_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA2_RLC3_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA2_RLC3_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC3_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA2_RLC3_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA2_RLC3_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC3_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC3_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA2_RLC3_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA2_RLC3_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA2_RLC3_STATUS_DEFAULT                                              0x00000000
#define mmSDMA2_RLC3_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA2_RLC3_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA2_RLC3_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA2_RLC3_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA2_RLC3_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA2_RLC3_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA2_RLC3_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA2_RLC3_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA2_RLC3_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA2_RLC3_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA2_RLC3_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA2_RLC3_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA2_RLC3_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA2_RLC3_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA2_RLC3_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA2_RLC3_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA2_RLC3_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA2_RLC3_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA2_RLC3_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA2_RLC3_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA2_RLC3_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA2_RLC3_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA2_RLC3_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA2_RLC3_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA2_RLC4_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA2_RLC4_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC4_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC4_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC4_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC4_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC4_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC4_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA2_RLC4_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA2_RLC4_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA2_RLC4_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA2_RLC4_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC4_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA2_RLC4_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA2_RLC4_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC4_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC4_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA2_RLC4_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA2_RLC4_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA2_RLC4_STATUS_DEFAULT                                              0x00000000
#define mmSDMA2_RLC4_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA2_RLC4_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA2_RLC4_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA2_RLC4_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA2_RLC4_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA2_RLC4_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA2_RLC4_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA2_RLC4_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA2_RLC4_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA2_RLC4_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA2_RLC4_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA2_RLC4_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA2_RLC4_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA2_RLC4_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA2_RLC4_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA2_RLC4_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA2_RLC4_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA2_RLC4_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA2_RLC4_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA2_RLC4_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA2_RLC4_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA2_RLC4_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA2_RLC4_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA2_RLC4_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA2_RLC5_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA2_RLC5_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC5_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC5_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC5_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC5_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC5_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC5_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA2_RLC5_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA2_RLC5_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA2_RLC5_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA2_RLC5_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC5_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA2_RLC5_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA2_RLC5_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC5_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC5_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA2_RLC5_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA2_RLC5_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA2_RLC5_STATUS_DEFAULT                                              0x00000000
#define mmSDMA2_RLC5_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA2_RLC5_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA2_RLC5_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA2_RLC5_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA2_RLC5_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA2_RLC5_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA2_RLC5_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA2_RLC5_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA2_RLC5_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA2_RLC5_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA2_RLC5_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA2_RLC5_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA2_RLC5_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA2_RLC5_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA2_RLC5_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA2_RLC5_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA2_RLC5_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA2_RLC5_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA2_RLC5_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA2_RLC5_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA2_RLC5_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA2_RLC5_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA2_RLC5_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA2_RLC5_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA2_RLC6_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA2_RLC6_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC6_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC6_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC6_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC6_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC6_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC6_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA2_RLC6_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA2_RLC6_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA2_RLC6_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA2_RLC6_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC6_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA2_RLC6_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA2_RLC6_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC6_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC6_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA2_RLC6_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA2_RLC6_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA2_RLC6_STATUS_DEFAULT                                              0x00000000
#define mmSDMA2_RLC6_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA2_RLC6_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA2_RLC6_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA2_RLC6_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA2_RLC6_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA2_RLC6_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA2_RLC6_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA2_RLC6_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA2_RLC6_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA2_RLC6_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA2_RLC6_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA2_RLC6_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA2_RLC6_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA2_RLC6_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA2_RLC6_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA2_RLC6_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA2_RLC6_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA2_RLC6_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA2_RLC6_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA2_RLC6_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA2_RLC6_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA2_RLC6_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA2_RLC6_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA2_RLC6_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA2_RLC7_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA2_RLC7_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC7_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC7_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC7_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC7_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC7_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC7_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA2_RLC7_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA2_RLC7_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA2_RLC7_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA2_RLC7_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA2_RLC7_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA2_RLC7_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA2_RLC7_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA2_RLC7_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA2_RLC7_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA2_RLC7_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA2_RLC7_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA2_RLC7_STATUS_DEFAULT                                              0x00000000
#define mmSDMA2_RLC7_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA2_RLC7_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA2_RLC7_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA2_RLC7_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA2_RLC7_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA2_RLC7_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA2_RLC7_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA2_RLC7_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA2_RLC7_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA2_RLC7_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA2_RLC7_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA2_RLC7_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA2_RLC7_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA2_RLC7_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA2_RLC7_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA2_RLC7_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA2_RLC7_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA2_RLC7_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA2_RLC7_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA2_RLC7_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA2_RLC7_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA2_RLC7_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA2_RLC7_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA2_RLC7_MIDCMD_CNTL_DEFAULT                                         0x00000000


// addressBlock: gc_sdma3_sdma3dec
#define mmSDMA3_DEC_START_DEFAULT                                                0x00000000
#define mmSDMA3_GLOBAL_TIMESTAMP_LO_DEFAULT                                      0x00000000
#define mmSDMA3_GLOBAL_TIMESTAMP_HI_DEFAULT                                      0x00000000
#define mmSDMA3_PG_CNTL_DEFAULT                                                  0x00000000
#define mmSDMA3_PG_CTX_LO_DEFAULT                                                0x00000000
#define mmSDMA3_PG_CTX_HI_DEFAULT                                                0x00000000
#define mmSDMA3_PG_CTX_CNTL_DEFAULT                                              0x00000000
#define mmSDMA3_POWER_CNTL_DEFAULT                                               0x40000050
#define mmSDMA3_CLK_CTRL_DEFAULT                                                 0x00000100
#define mmSDMA3_CNTL_DEFAULT                                                     0x000000c2
#define mmSDMA3_CHICKEN_BITS_DEFAULT                                             0x03ef0107
#define mmSDMA3_GB_ADDR_CONFIG_DEFAULT                                           0x00000444
#define mmSDMA3_GB_ADDR_CONFIG_READ_DEFAULT                                      0x00000444
#define mmSDMA3_RB_RPTR_FETCH_HI_DEFAULT                                         0x00000000
#define mmSDMA3_SEM_WAIT_FAIL_TIMER_CNTL_DEFAULT                                 0x00000000
#define mmSDMA3_RB_RPTR_FETCH_DEFAULT                                            0x00000000
#define mmSDMA3_IB_OFFSET_FETCH_DEFAULT                                          0x00000000
#define mmSDMA3_PROGRAM_DEFAULT                                                  0x00000000
#define mmSDMA3_STATUS_REG_DEFAULT                                               0x46dee557
#define mmSDMA3_STATUS1_REG_DEFAULT                                              0x000003ff
#define mmSDMA3_RD_BURST_CNTL_DEFAULT                                            0x00000002
#define mmSDMA3_HBM_PAGE_CONFIG_DEFAULT                                          0x00000000
#define mmSDMA3_UCODE_CHECKSUM_DEFAULT                                           0x00000000
#define mmSDMA3_F32_CNTL_DEFAULT                                                 0x00000001
#define mmSDMA3_FREEZE_DEFAULT                                                   0x00000000
#define mmSDMA3_PHASE0_QUANTUM_DEFAULT                                           0x00010002
#define mmSDMA3_PHASE1_QUANTUM_DEFAULT                                           0x00010002
#define mmSDMA3_EDC_CONFIG_DEFAULT                                               0x00000002
#define mmSDMA3_BA_THRESHOLD_DEFAULT                                             0x03ff03ff
#define mmSDMA3_ID_DEFAULT                                                       0x00000001
#define mmSDMA3_VERSION_DEFAULT                                                  0x00000500
#define mmSDMA3_EDC_COUNTER_DEFAULT                                              0x00000000
#define mmSDMA3_EDC_COUNTER_CLEAR_DEFAULT                                        0x00000000
#define mmSDMA3_STATUS2_REG_DEFAULT                                              0x00000001
#define mmSDMA3_ATOMIC_CNTL_DEFAULT                                              0x00000200
#define mmSDMA3_ATOMIC_PREOP_LO_DEFAULT                                          0x00000000
#define mmSDMA3_ATOMIC_PREOP_HI_DEFAULT                                          0x00000000
#define mmSDMA3_UTCL1_CNTL_DEFAULT                                               0xd0000191
#define mmSDMA3_UTCL1_WATERMK_DEFAULT                                            0xfffbd9fb
#define mmSDMA3_UTCL1_RD_STATUS_DEFAULT                                          0x01011555
#define mmSDMA3_UTCL1_WR_STATUS_DEFAULT                                          0x51011555
#define mmSDMA3_UTCL1_INV0_DEFAULT                                               0x00000800
#define mmSDMA3_UTCL1_INV1_DEFAULT                                               0x00000000
#define mmSDMA3_UTCL1_INV2_DEFAULT                                               0x00000000
#define mmSDMA3_UTCL1_RD_XNACK0_DEFAULT                                          0x00000000
#define mmSDMA3_UTCL1_RD_XNACK1_DEFAULT                                          0x00000000
#define mmSDMA3_UTCL1_WR_XNACK0_DEFAULT                                          0x00000000
#define mmSDMA3_UTCL1_WR_XNACK1_DEFAULT                                          0x00000000
#define mmSDMA3_UTCL1_TIMEOUT_DEFAULT                                            0x00000000
#define mmSDMA3_UTCL1_PAGE_DEFAULT                                               0x010cec00
#define mmSDMA3_RELAX_ORDERING_LUT_DEFAULT                                       0xc0000006
#define mmSDMA3_CHICKEN_BITS_2_DEFAULT                                           0x00100007
#define mmSDMA3_STATUS3_REG_DEFAULT                                              0x03f00000
#define mmSDMA3_PHYSICAL_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA3_PHYSICAL_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA3_PHASE2_QUANTUM_DEFAULT                                           0x00010002
#define mmSDMA3_ERROR_LOG_DEFAULT                                                0x0000000f
#define mmSDMA3_PUB_DUMMY_REG0_DEFAULT                                           0x00000000
#define mmSDMA3_PUB_DUMMY_REG1_DEFAULT                                           0x00000000
#define mmSDMA3_PUB_DUMMY_REG2_DEFAULT                                           0x00000000
#define mmSDMA3_PUB_DUMMY_REG3_DEFAULT                                           0x00000000
#define mmSDMA3_F32_COUNTER_DEFAULT                                              0x00000000
#define mmSDMA3_CRD_CNTL_DEFAULT                                                 0x1850c640
#define mmSDMA3_AQL_STATUS_DEFAULT                                               0x00000003
#define mmSDMA3_EA_DBIT_ADDR_DATA_DEFAULT                                        0x00000000
#define mmSDMA3_EA_DBIT_ADDR_INDEX_DEFAULT                                       0x00000000
#define mmSDMA3_TLBI_GCR_CNTL_DEFAULT                                            0x40180454
#define mmSDMA3_TILING_CONFIG_DEFAULT                                            0x00000000
#define mmSDMA3_INT_STATUS_DEFAULT                                               0x00000000
#define mmSDMA3_HOLE_ADDR_LO_DEFAULT                                             0x00000000
#define mmSDMA3_HOLE_ADDR_HI_DEFAULT                                             0x00000000
#define mmSDMA3_CLOCK_GATING_REG_DEFAULT                                         0x00000000
#define mmSDMA3_STATUS4_REG_DEFAULT                                              0x00000001
#define mmSDMA3_SCRATCH_RAM_DATA_DEFAULT                                         0x00000000
#define mmSDMA3_SCRATCH_RAM_ADDR_DEFAULT                                         0x00000000
#define mmSDMA3_TIMESTAMP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA3_STATUS5_REG_DEFAULT                                              0x00000000
#define mmSDMA3_QUEUE_RESET_REQ_DEFAULT                                          0x00000000
#define mmSDMA3_GFX_RB_CNTL_DEFAULT                                              0x80840000
#define mmSDMA3_GFX_RB_BASE_DEFAULT                                              0x00000000
#define mmSDMA3_GFX_RB_BASE_HI_DEFAULT                                           0x00000000
#define mmSDMA3_GFX_RB_RPTR_DEFAULT                                              0x00000000
#define mmSDMA3_GFX_RB_RPTR_HI_DEFAULT                                           0x00000000
#define mmSDMA3_GFX_RB_WPTR_DEFAULT                                              0x00000000
#define mmSDMA3_GFX_RB_WPTR_HI_DEFAULT                                           0x00000000
#define mmSDMA3_GFX_RB_WPTR_POLL_CNTL_DEFAULT                                    0x00403000
#define mmSDMA3_GFX_RB_RPTR_ADDR_HI_DEFAULT                                      0x00000000
#define mmSDMA3_GFX_RB_RPTR_ADDR_LO_DEFAULT                                      0x00000000
#define mmSDMA3_GFX_IB_CNTL_DEFAULT                                              0x00000100
#define mmSDMA3_GFX_IB_RPTR_DEFAULT                                              0x00000000
#define mmSDMA3_GFX_IB_OFFSET_DEFAULT                                            0x00000000
#define mmSDMA3_GFX_IB_BASE_LO_DEFAULT                                           0x00000000
#define mmSDMA3_GFX_IB_BASE_HI_DEFAULT                                           0x00000000
#define mmSDMA3_GFX_IB_SIZE_DEFAULT                                              0x00000000
#define mmSDMA3_GFX_SKIP_CNTL_DEFAULT                                            0x00000000
#define mmSDMA3_GFX_CONTEXT_STATUS_DEFAULT                                       0x00000005
#define mmSDMA3_GFX_DOORBELL_DEFAULT                                             0x00000000
#define mmSDMA3_GFX_CONTEXT_CNTL_DEFAULT                                         0x00000000
#define mmSDMA3_GFX_STATUS_DEFAULT                                               0x00000000
#define mmSDMA3_GFX_DOORBELL_LOG_DEFAULT                                         0x00000000
#define mmSDMA3_GFX_WATERMARK_DEFAULT                                            0x00000000
#define mmSDMA3_GFX_DOORBELL_OFFSET_DEFAULT                                      0x00000000
#define mmSDMA3_GFX_CSA_ADDR_LO_DEFAULT                                          0x00000000
#define mmSDMA3_GFX_CSA_ADDR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_GFX_IB_SUB_REMAIN_DEFAULT                                        0x00000000
#define mmSDMA3_GFX_PREEMPT_DEFAULT                                              0x00000000
#define mmSDMA3_GFX_DUMMY_REG_DEFAULT                                            0x0000000f
#define mmSDMA3_GFX_RB_WPTR_POLL_ADDR_HI_DEFAULT                                 0x00000000
#define mmSDMA3_GFX_RB_WPTR_POLL_ADDR_LO_DEFAULT                                 0x00000000
#define mmSDMA3_GFX_RB_AQL_CNTL_DEFAULT                                          0x00004000
#define mmSDMA3_GFX_MINOR_PTR_UPDATE_DEFAULT                                     0x00000000
#define mmSDMA3_GFX_MIDCMD_DATA0_DEFAULT                                         0x00000000
#define mmSDMA3_GFX_MIDCMD_DATA1_DEFAULT                                         0x00000000
#define mmSDMA3_GFX_MIDCMD_DATA2_DEFAULT                                         0x00000000
#define mmSDMA3_GFX_MIDCMD_DATA3_DEFAULT                                         0x00000000
#define mmSDMA3_GFX_MIDCMD_DATA4_DEFAULT                                         0x00000000
#define mmSDMA3_GFX_MIDCMD_DATA5_DEFAULT                                         0x00000000
#define mmSDMA3_GFX_MIDCMD_DATA6_DEFAULT                                         0x00000000
#define mmSDMA3_GFX_MIDCMD_DATA7_DEFAULT                                         0x00000000
#define mmSDMA3_GFX_MIDCMD_DATA8_DEFAULT                                         0x00000000
#define mmSDMA3_GFX_MIDCMD_DATA9_DEFAULT                                         0x00000000
#define mmSDMA3_GFX_MIDCMD_DATA10_DEFAULT                                        0x00000000
#define mmSDMA3_GFX_MIDCMD_CNTL_DEFAULT                                          0x00000000
#define mmSDMA3_PAGE_RB_CNTL_DEFAULT                                             0x80840000
#define mmSDMA3_PAGE_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA3_PAGE_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_PAGE_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_PAGE_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_PAGE_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA3_PAGE_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_PAGE_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA3_PAGE_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA3_PAGE_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA3_PAGE_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA3_PAGE_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_PAGE_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA3_PAGE_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA3_PAGE_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_PAGE_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA3_PAGE_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA3_PAGE_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA3_PAGE_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA3_PAGE_STATUS_DEFAULT                                              0x00000000
#define mmSDMA3_PAGE_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA3_PAGE_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA3_PAGE_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA3_PAGE_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA3_PAGE_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA3_PAGE_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA3_PAGE_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA3_PAGE_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA3_PAGE_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA3_PAGE_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA3_PAGE_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA3_PAGE_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA3_PAGE_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA3_PAGE_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA3_PAGE_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA3_PAGE_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA3_PAGE_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA3_PAGE_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA3_PAGE_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA3_PAGE_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA3_PAGE_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA3_PAGE_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA3_PAGE_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA3_PAGE_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA3_RLC0_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA3_RLC0_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC0_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC0_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC0_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC0_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC0_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC0_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA3_RLC0_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA3_RLC0_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA3_RLC0_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA3_RLC0_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC0_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA3_RLC0_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA3_RLC0_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC0_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC0_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA3_RLC0_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA3_RLC0_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA3_RLC0_STATUS_DEFAULT                                              0x00000000
#define mmSDMA3_RLC0_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA3_RLC0_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA3_RLC0_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA3_RLC0_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA3_RLC0_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA3_RLC0_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA3_RLC0_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA3_RLC0_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA3_RLC0_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA3_RLC0_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA3_RLC0_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA3_RLC0_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA3_RLC0_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA3_RLC0_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA3_RLC0_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA3_RLC0_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA3_RLC0_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA3_RLC0_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA3_RLC0_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA3_RLC0_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA3_RLC0_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA3_RLC0_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA3_RLC0_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA3_RLC0_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA3_RLC1_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA3_RLC1_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC1_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC1_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC1_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC1_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC1_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC1_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA3_RLC1_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA3_RLC1_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA3_RLC1_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA3_RLC1_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC1_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA3_RLC1_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA3_RLC1_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC1_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC1_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA3_RLC1_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA3_RLC1_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA3_RLC1_STATUS_DEFAULT                                              0x00000000
#define mmSDMA3_RLC1_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA3_RLC1_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA3_RLC1_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA3_RLC1_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA3_RLC1_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA3_RLC1_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA3_RLC1_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA3_RLC1_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA3_RLC1_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA3_RLC1_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA3_RLC1_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA3_RLC1_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA3_RLC1_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA3_RLC1_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA3_RLC1_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA3_RLC1_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA3_RLC1_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA3_RLC1_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA3_RLC1_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA3_RLC1_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA3_RLC1_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA3_RLC1_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA3_RLC1_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA3_RLC1_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA3_RLC2_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA3_RLC2_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC2_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC2_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC2_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC2_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC2_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC2_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA3_RLC2_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA3_RLC2_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA3_RLC2_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA3_RLC2_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC2_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA3_RLC2_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA3_RLC2_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC2_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC2_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA3_RLC2_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA3_RLC2_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA3_RLC2_STATUS_DEFAULT                                              0x00000000
#define mmSDMA3_RLC2_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA3_RLC2_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA3_RLC2_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA3_RLC2_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA3_RLC2_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA3_RLC2_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA3_RLC2_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA3_RLC2_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA3_RLC2_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA3_RLC2_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA3_RLC2_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA3_RLC2_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA3_RLC2_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA3_RLC2_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA3_RLC2_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA3_RLC2_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA3_RLC2_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA3_RLC2_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA3_RLC2_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA3_RLC2_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA3_RLC2_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA3_RLC2_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA3_RLC2_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA3_RLC2_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA3_RLC3_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA3_RLC3_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC3_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC3_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC3_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC3_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC3_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC3_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA3_RLC3_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA3_RLC3_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA3_RLC3_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA3_RLC3_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC3_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA3_RLC3_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA3_RLC3_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC3_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC3_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA3_RLC3_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA3_RLC3_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA3_RLC3_STATUS_DEFAULT                                              0x00000000
#define mmSDMA3_RLC3_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA3_RLC3_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA3_RLC3_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA3_RLC3_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA3_RLC3_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA3_RLC3_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA3_RLC3_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA3_RLC3_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA3_RLC3_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA3_RLC3_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA3_RLC3_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA3_RLC3_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA3_RLC3_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA3_RLC3_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA3_RLC3_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA3_RLC3_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA3_RLC3_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA3_RLC3_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA3_RLC3_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA3_RLC3_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA3_RLC3_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA3_RLC3_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA3_RLC3_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA3_RLC3_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA3_RLC4_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA3_RLC4_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC4_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC4_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC4_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC4_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC4_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC4_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA3_RLC4_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA3_RLC4_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA3_RLC4_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA3_RLC4_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC4_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA3_RLC4_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA3_RLC4_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC4_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC4_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA3_RLC4_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA3_RLC4_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA3_RLC4_STATUS_DEFAULT                                              0x00000000
#define mmSDMA3_RLC4_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA3_RLC4_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA3_RLC4_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA3_RLC4_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA3_RLC4_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA3_RLC4_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA3_RLC4_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA3_RLC4_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA3_RLC4_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA3_RLC4_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA3_RLC4_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA3_RLC4_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA3_RLC4_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA3_RLC4_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA3_RLC4_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA3_RLC4_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA3_RLC4_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA3_RLC4_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA3_RLC4_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA3_RLC4_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA3_RLC4_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA3_RLC4_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA3_RLC4_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA3_RLC4_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA3_RLC5_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA3_RLC5_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC5_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC5_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC5_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC5_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC5_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC5_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA3_RLC5_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA3_RLC5_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA3_RLC5_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA3_RLC5_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC5_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA3_RLC5_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA3_RLC5_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC5_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC5_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA3_RLC5_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA3_RLC5_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA3_RLC5_STATUS_DEFAULT                                              0x00000000
#define mmSDMA3_RLC5_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA3_RLC5_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA3_RLC5_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA3_RLC5_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA3_RLC5_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA3_RLC5_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA3_RLC5_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA3_RLC5_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA3_RLC5_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA3_RLC5_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA3_RLC5_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA3_RLC5_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA3_RLC5_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA3_RLC5_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA3_RLC5_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA3_RLC5_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA3_RLC5_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA3_RLC5_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA3_RLC5_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA3_RLC5_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA3_RLC5_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA3_RLC5_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA3_RLC5_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA3_RLC5_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA3_RLC6_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA3_RLC6_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC6_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC6_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC6_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC6_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC6_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC6_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA3_RLC6_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA3_RLC6_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA3_RLC6_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA3_RLC6_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC6_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA3_RLC6_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA3_RLC6_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC6_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC6_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA3_RLC6_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA3_RLC6_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA3_RLC6_STATUS_DEFAULT                                              0x00000000
#define mmSDMA3_RLC6_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA3_RLC6_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA3_RLC6_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA3_RLC6_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA3_RLC6_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA3_RLC6_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA3_RLC6_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA3_RLC6_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA3_RLC6_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA3_RLC6_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA3_RLC6_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA3_RLC6_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA3_RLC6_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA3_RLC6_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA3_RLC6_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA3_RLC6_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA3_RLC6_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA3_RLC6_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA3_RLC6_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA3_RLC6_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA3_RLC6_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA3_RLC6_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA3_RLC6_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA3_RLC6_MIDCMD_CNTL_DEFAULT                                         0x00000000
#define mmSDMA3_RLC7_RB_CNTL_DEFAULT                                             0x80040000
#define mmSDMA3_RLC7_RB_BASE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC7_RB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC7_RB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC7_RB_RPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC7_RB_WPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC7_RB_WPTR_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC7_RB_WPTR_POLL_CNTL_DEFAULT                                   0x00403000
#define mmSDMA3_RLC7_RB_RPTR_ADDR_HI_DEFAULT                                     0x00000000
#define mmSDMA3_RLC7_RB_RPTR_ADDR_LO_DEFAULT                                     0x00000000
#define mmSDMA3_RLC7_IB_CNTL_DEFAULT                                             0x00000100
#define mmSDMA3_RLC7_IB_RPTR_DEFAULT                                             0x00000000
#define mmSDMA3_RLC7_IB_OFFSET_DEFAULT                                           0x00000000
#define mmSDMA3_RLC7_IB_BASE_LO_DEFAULT                                          0x00000000
#define mmSDMA3_RLC7_IB_BASE_HI_DEFAULT                                          0x00000000
#define mmSDMA3_RLC7_IB_SIZE_DEFAULT                                             0x00000000
#define mmSDMA3_RLC7_SKIP_CNTL_DEFAULT                                           0x00000000
#define mmSDMA3_RLC7_CONTEXT_STATUS_DEFAULT                                      0x00000004
#define mmSDMA3_RLC7_DOORBELL_DEFAULT                                            0x00000000
#define mmSDMA3_RLC7_STATUS_DEFAULT                                              0x00000000
#define mmSDMA3_RLC7_DOORBELL_LOG_DEFAULT                                        0x00000000
#define mmSDMA3_RLC7_WATERMARK_DEFAULT                                           0x00000000
#define mmSDMA3_RLC7_DOORBELL_OFFSET_DEFAULT                                     0x00000000
#define mmSDMA3_RLC7_CSA_ADDR_LO_DEFAULT                                         0x00000000
#define mmSDMA3_RLC7_CSA_ADDR_HI_DEFAULT                                         0x00000000
#define mmSDMA3_RLC7_IB_SUB_REMAIN_DEFAULT                                       0x00000000
#define mmSDMA3_RLC7_PREEMPT_DEFAULT                                             0x00000000
#define mmSDMA3_RLC7_DUMMY_REG_DEFAULT                                           0x0000000f
#define mmSDMA3_RLC7_RB_WPTR_POLL_ADDR_HI_DEFAULT                                0x00000000
#define mmSDMA3_RLC7_RB_WPTR_POLL_ADDR_LO_DEFAULT                                0x00000000
#define mmSDMA3_RLC7_RB_AQL_CNTL_DEFAULT                                         0x00004000
#define mmSDMA3_RLC7_MINOR_PTR_UPDATE_DEFAULT                                    0x00000000
#define mmSDMA3_RLC7_MIDCMD_DATA0_DEFAULT                                        0x00000000
#define mmSDMA3_RLC7_MIDCMD_DATA1_DEFAULT                                        0x00000000
#define mmSDMA3_RLC7_MIDCMD_DATA2_DEFAULT                                        0x00000000
#define mmSDMA3_RLC7_MIDCMD_DATA3_DEFAULT                                        0x00000000
#define mmSDMA3_RLC7_MIDCMD_DATA4_DEFAULT                                        0x00000000
#define mmSDMA3_RLC7_MIDCMD_DATA5_DEFAULT                                        0x00000000
#define mmSDMA3_RLC7_MIDCMD_DATA6_DEFAULT                                        0x00000000
#define mmSDMA3_RLC7_MIDCMD_DATA7_DEFAULT                                        0x00000000
#define mmSDMA3_RLC7_MIDCMD_DATA8_DEFAULT                                        0x00000000
#define mmSDMA3_RLC7_MIDCMD_DATA9_DEFAULT                                        0x00000000
#define mmSDMA3_RLC7_MIDCMD_DATA10_DEFAULT                                       0x00000000
#define mmSDMA3_RLC7_MIDCMD_CNTL_DEFAULT                                         0x00000000


// addressBlock: gccacind
#define ixPCC_STALL_PATTERN_CTRL_DEFAULT                                         0x07fa0401
#define ixPWRBRK_STALL_PATTERN_CTRL_DEFAULT                                      0x00fa0401
#define ixPCC_STALL_PATTERN_1_2_DEFAULT                                          0x00000000
#define ixPCC_STALL_PATTERN_3_4_DEFAULT                                          0x00000000
#define ixPCC_STALL_PATTERN_5_6_DEFAULT                                          0x00000000
#define ixPCC_STALL_PATTERN_7_DEFAULT                                            0x00000000
#define ixPWRBRK_STALL_PATTERN_1_2_DEFAULT                                       0x00000000
#define ixPWRBRK_STALL_PATTERN_3_4_DEFAULT                                       0x00000000
#define ixPWRBRK_STALL_PATTERN_5_6_DEFAULT                                       0x00000000
#define ixPWRBRK_STALL_PATTERN_7_DEFAULT                                         0x00000000
#define ixPCC_PWRBRK_HYSTERESIS_CTRL_DEFAULT                                     0x00000000
#define ixEDC_STRETCH_PERF_COUNTER_DEFAULT                                       0x00000000
#define ixEDC_UNSTRETCH_PERF_COUNTER_DEFAULT                                     0x00000000
#define ixEDC_STRETCH_NUM_PERF_COUNTER_DEFAULT                                   0x00000000
#define ixGC_CAC_ID_DEFAULT                                                      0x00000000
#define ixGC_CAC_CNTL_DEFAULT                                                    0x000001fe
#define ixGC_CAC_OVR_SEL_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVR_VAL_DEFAULT                                                 0x00000000
#define ixGC_CAC_WEIGHT_BCI_0_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_CB_0_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_CB_1_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_CB_2_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_CB_3_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_CB_4_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_CP_0_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_CP_1_DEFAULT                                             0x00000001
#define ixGC_CAC_WEIGHT_DB_0_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_DB_1_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_DB_2_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_DB_3_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_DB_4_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_GDS_0_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_GDS_1_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_GDS_2_DEFAULT                                            0x00000001
#define ixGC_CAC_WEIGHT_LDS_0_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_LDS_1_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_LDS_2_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_LDS_3_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_LDS_4_DEFAULT                                            0x00000001
#define ixGC_CAC_WEIGHT_PA_0_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_PA_1_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_PA_2_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_PA_3_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_PC_0_DEFAULT                                             0x00000001
#define ixGC_CAC_WEIGHT_SC_0_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_SC_1_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_SC_2_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_SC_3_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_SPI_0_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_SPI_1_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_SPI_2_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_SQ_0_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_SQ_1_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_SQ_2_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_SQ_3_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_SX_0_DEFAULT                                             0x00000001
#define ixGC_CAC_WEIGHT_SXRB_0_DEFAULT                                           0x00010001
#define ixGC_CAC_WEIGHT_TA_0_DEFAULT                                             0x00000001
#define ixGC_CAC_WEIGHT_TCP_0_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_TCP_1_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_TCP_2_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_TCP_3_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_TD_0_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_TD_1_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_TD_2_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_TD_3_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_TD_4_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_TD_5_DEFAULT                                             0x00000001
#define ixGC_CAC_WEIGHT_RMI_0_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_RMI_1_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_EA_0_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_EA_1_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_EA_2_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_ATCL2_0_DEFAULT                                    0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_ATCL2_1_DEFAULT                                    0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_ATCL2_2_DEFAULT                                    0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_ROUTER_0_DEFAULT                                   0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_ROUTER_1_DEFAULT                                   0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_ROUTER_2_DEFAULT                                   0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_ROUTER_3_DEFAULT                                   0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_ROUTER_4_DEFAULT                                   0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_VML2_0_DEFAULT                                     0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_VML2_1_DEFAULT                                     0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_VML2_2_DEFAULT                                     0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_WALKER_0_DEFAULT                                   0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_WALKER_1_DEFAULT                                   0x00010001
#define ixGC_CAC_WEIGHT_UTCL2_WALKER_2_DEFAULT                                   0x00010001
#define ixGC_CAC_WEIGHT_CU_0_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_UTCL1_0_DEFAULT                                          0x00000001
#define ixGC_CAC_WEIGHT_GE_0_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_GE_1_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_GE_2_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_GE_3_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_GE_4_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_GE_5_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_GE_6_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_GE_7_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_GE_8_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_GE_9_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_GE_10_DEFAULT                                            0x00000001
#define ixGC_CAC_WEIGHT_PMM_0_DEFAULT                                            0x00000001
#define ixGC_CAC_WEIGHT_GL2C_0_DEFAULT                                           0x00010001
#define ixGC_CAC_WEIGHT_GL2C_1_DEFAULT                                           0x00010001
#define ixGC_CAC_WEIGHT_GL2C_2_DEFAULT                                           0x00000001
#define ixGC_CAC_WEIGHT_GUS_0_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_GUS_1_DEFAULT                                            0x00000001
#define ixGC_CAC_WEIGHT_PH_0_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_PH_1_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_PH_2_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_PH_3_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_SDMA_0_DEFAULT                                           0x00010001
#define ixGC_CAC_WEIGHT_SDMA_1_DEFAULT                                           0x00010001
#define ixGC_CAC_WEIGHT_SDMA_2_DEFAULT                                           0x00010001
#define ixGC_CAC_WEIGHT_SDMA_3_DEFAULT                                           0x00010001
#define ixGC_CAC_WEIGHT_SDMA_4_DEFAULT                                           0x00010001
#define ixGC_CAC_WEIGHT_SDMA_5_DEFAULT                                           0x00010001
#define ixGC_CAC_WEIGHT_SP_0_DEFAULT                                             0x00010001
#define ixGC_CAC_WEIGHT_SP_1_DEFAULT                                             0x00000001
#define ixGC_CAC_WEIGHT_GL1C_0_DEFAULT                                           0x00010001
#define ixGC_CAC_WEIGHT_GL1C_1_DEFAULT                                           0x00010001
#define ixGC_CAC_WEIGHT_GL1C_2_DEFAULT                                           0x00000001
#define ixGC_CAC_WEIGHT_CHC_0_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_CHC_1_DEFAULT                                            0x00000001
#define ixGC_CAC_WEIGHT_SQC_0_DEFAULT                                            0x00010001
#define ixGC_CAC_WEIGHT_SQC_1_DEFAULT                                            0x00000001
#define ixGC_CAC_WEIGHT_RLC_0_DEFAULT                                            0x00000001
#define ixGC_CAC_ACC_LDS0_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_LDS1_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_LDS2_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_LDS3_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_LDS4_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_LDS5_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_LDS6_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_LDS7_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_LDS8_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_BCI0_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_BCI1_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_CB0_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_CB1_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_CB2_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_CB3_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_CB4_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_CB5_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_CB6_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_CB7_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_CB8_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_CB9_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_CP0_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_CP1_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_CP2_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_DB0_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_DB1_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_DB2_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_DB3_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_DB4_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_DB5_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_DB6_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_DB7_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_DB8_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_DB9_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_GDS0_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GDS1_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GDS2_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GDS3_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GDS4_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GDS5_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GDS6_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_PA0_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PA1_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PA2_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PA3_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PA4_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PA5_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PA6_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PA7_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PC0_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_SC0_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_SC1_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_SC2_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_SC3_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_SC4_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_SC5_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_SC6_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_SC7_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_SPI0_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_SPI1_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_SPI2_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_SPI3_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_SPI4_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_SPI5_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_SQ0_LOWER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ0_UPPER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ1_LOWER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ1_UPPER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ2_LOWER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ2_UPPER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ3_LOWER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ3_UPPER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ4_LOWER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ4_UPPER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ5_LOWER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ5_UPPER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ6_LOWER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ6_UPPER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ7_LOWER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ7_UPPER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ8_LOWER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SQ8_UPPER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SX0_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_SXRB0_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_TA0_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_TCP0_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_TCP1_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_TCP2_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_TCP3_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_TCP4_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_TCP5_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_TCP6_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_TCP7_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_TD0_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_TD1_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_TD2_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_TD3_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_TD4_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_TD5_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_TD6_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_TD7_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_TD8_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_TD9_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_TD10_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_RMI0_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_RMI1_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_RMI2_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_RMI3_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_EA0_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_EA1_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_EA2_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_EA3_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_EA4_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_EA5_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_UTCL2_ATCL20_DEFAULT                                        0x00000000
#define ixGC_CAC_ACC_UTCL2_ATCL21_DEFAULT                                        0x00000000
#define ixGC_CAC_ACC_UTCL2_ATCL22_DEFAULT                                        0x00000000
#define ixGC_CAC_ACC_UTCL2_ATCL23_DEFAULT                                        0x00000000
#define ixGC_CAC_ACC_UTCL2_ATCL24_DEFAULT                                        0x00000000
#define ixGC_CAC_ACC_UTCL2_ROUTER0_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_ROUTER1_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_ROUTER2_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_ROUTER3_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_ROUTER4_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_ROUTER5_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_ROUTER6_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_ROUTER7_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_ROUTER8_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_ROUTER9_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_VML20_DEFAULT                                         0x00000000
#define ixGC_CAC_ACC_UTCL2_VML21_DEFAULT                                         0x00000000
#define ixGC_CAC_ACC_UTCL2_VML22_DEFAULT                                         0x00000000
#define ixGC_CAC_ACC_UTCL2_VML23_DEFAULT                                         0x00000000
#define ixGC_CAC_ACC_UTCL2_VML24_DEFAULT                                         0x00000000
#define ixGC_CAC_ACC_UTCL2_WALKER0_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_WALKER1_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_WALKER2_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_WALKER3_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_UTCL2_WALKER4_DEFAULT                                       0x00000000
#define ixGC_CAC_ACC_CU0_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_UTCL10_DEFAULT                                              0x00000000
#define ixGC_CAC_ACC_CHC0_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_CHC1_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_CHC2_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GE0_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_GE1_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_GE2_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_GE3_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_GE4_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_GE5_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_GE6_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_GE7_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_GE8_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_GE9_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_GE10_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GE11_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GE12_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GE13_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GE14_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GE15_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GE16_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GE17_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GE18_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GE19_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GE20_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_PMM0_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GL2C0_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_GL2C1_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_GL2C2_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_GL2C3_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_GL2C4_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_GUS0_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GUS1_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_GUS2_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_PH0_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PH1_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PH2_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PH3_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PH4_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PH5_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PH6_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_PH7_DEFAULT                                                 0x00000000
#define ixGC_CAC_ACC_SDMA0_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_SDMA1_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_SDMA2_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_SDMA3_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_SDMA4_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_SDMA5_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_SDMA6_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_SDMA7_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_SDMA8_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_SDMA9_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_SDMA10_DEFAULT                                              0x00000000
#define ixGC_CAC_ACC_SDMA11_DEFAULT                                              0x00000000
#define ixGC_CAC_ACC_SP0_LOWER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SP0_UPPER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SP1_LOWER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SP1_UPPER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SP2_LOWER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_SP2_UPPER_DEFAULT                                           0x00000000
#define ixGC_CAC_ACC_GL1C0_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_GL1C1_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_GL1C2_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_GL1C3_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_GL1C4_DEFAULT                                               0x00000000
#define ixGC_CAC_ACC_SQC0_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_SQC1_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_SQC2_DEFAULT                                                0x00000000
#define ixGC_CAC_ACC_RLC0_DEFAULT                                                0x00000000
#define ixGC_CAC_OVRD_BCI_DEFAULT                                                0x00000000
#define ixGC_CAC_OVRD_CB_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_CP_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_DB_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_GDS_DEFAULT                                                0x00000000
#define ixGC_CAC_OVRD_LDS_DEFAULT                                                0x00000000
#define ixGC_CAC_OVRD_PA_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_PC_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_SC_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_SPI_DEFAULT                                                0x00000000
#define ixGC_CAC_OVRD_CU_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_SQ_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_SX_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_SXRB_DEFAULT                                               0x00000000
#define ixGC_CAC_OVRD_TA_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_TCP_DEFAULT                                                0x00000000
#define ixGC_CAC_OVRD_TD_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_RMI_DEFAULT                                                0x00000000
#define ixGC_CAC_OVRD_EA_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_UTCL2_ATCL2_DEFAULT                                        0x00000000
#define ixGC_CAC_OVRD_UTCL2_ROUTER_DEFAULT                                       0x00000000
#define ixGC_CAC_OVRD_UTCL2_VML2_DEFAULT                                         0x00000000
#define ixGC_CAC_OVRD_UTCL2_WALKER_DEFAULT                                       0x00000000
#define ixGC_CAC_OVRD_SP_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_UTCL1_DEFAULT                                              0x00000000
#define ixGC_CAC_OVRD_CHC_DEFAULT                                                0x00000000
#define ixGC_CAC_OVRD_GE_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_PMM_DEFAULT                                                0x00000000
#define ixGC_CAC_OVRD_GL2C_DEFAULT                                               0x00000000
#define ixGC_CAC_OVRD_GUS_DEFAULT                                                0x00000000
#define ixGC_CAC_OVRD_PH_DEFAULT                                                 0x00000000
#define ixGC_CAC_OVRD_SDMA_DEFAULT                                               0x00000000
#define ixGC_CAC_OVRD_GL1C_DEFAULT                                               0x00000000
#define ixGC_CAC_OVRD_SQC_DEFAULT                                                0x00000000
#define ixGC_CAC_OVRD_RLC_DEFAULT                                                0x00000000
#define ixGC_CAC_OVRD_GE_HI_DEFAULT                                              0x00000000
#define ixRELEASE_TO_STALL_LUT_1_8_DEFAULT                                       0x00000000
#define ixRELEASE_TO_STALL_LUT_9_16_DEFAULT                                      0x00000000
#define ixRELEASE_TO_STALL_LUT_17_20_DEFAULT                                     0x00000000
#define ixSTALL_TO_RELEASE_LUT_1_4_DEFAULT                                       0x00000000
#define ixSTALL_TO_RELEASE_LUT_5_7_DEFAULT                                       0x00000000
#define ixSTALL_TO_PWRBRK_LUT_1_4_DEFAULT                                        0x00000000
#define ixSTALL_TO_PWRBRK_LUT_5_7_DEFAULT                                        0x00000000
#define ixPWRBRK_STALL_TO_RELEASE_LUT_1_4_DEFAULT                                0x00000000
#define ixPWRBRK_STALL_TO_RELEASE_LUT_5_7_DEFAULT                                0x00000000
#define ixPWRBRK_RELEASE_TO_STALL_LUT_1_8_DEFAULT                                0x00000000
#define ixPWRBRK_RELEASE_TO_STALL_LUT_9_16_DEFAULT                               0x00000000
#define ixPWRBRK_RELEASE_TO_STALL_LUT_17_20_DEFAULT                              0x00000000
#define ixFIXED_PATTERN_PERF_COUNTER_1_DEFAULT                                   0x00000000
#define ixFIXED_PATTERN_PERF_COUNTER_2_DEFAULT                                   0x00000000
#define ixFIXED_PATTERN_PERF_COUNTER_3_DEFAULT                                   0x00000000
#define ixFIXED_PATTERN_PERF_COUNTER_4_DEFAULT                                   0x00000000
#define ixFIXED_PATTERN_PERF_COUNTER_5_DEFAULT                                   0x00000000
#define ixFIXED_PATTERN_PERF_COUNTER_6_DEFAULT                                   0x00000000
#define ixFIXED_PATTERN_PERF_COUNTER_7_DEFAULT                                   0x00000000
#define ixFIXED_PATTERN_PERF_COUNTER_8_DEFAULT                                   0x00000000
#define ixFIXED_PATTERN_PERF_COUNTER_9_DEFAULT                                   0x00000000
#define ixFIXED_PATTERN_PERF_COUNTER_10_DEFAULT                                  0x00000000
#define ixHW_LUT_UPDATE_STATUS_DEFAULT                                           0x00000000


// addressBlock: secacind
#define ixSE_CAC_ID_DEFAULT                                                      0x00000000
#define ixSE_CAC_CNTL_DEFAULT                                                    0x000001fe
#define ixSE_CAC_OVR_SEL_DEFAULT                                                 0x00000000
#define ixSE_CAC_OVR_VAL_DEFAULT                                                 0x00000000


// addressBlock: spmglbind
#define ixGLB_CPG_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_CPC_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_CPF_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_GDS_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_GCR_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_PH_SAMPLEDELAY_DEFAULT                                             0x00000000
#define ixGLB_GE1_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_GE2DIST_SAMPLEDELAY_DEFAULT                                        0x00000000
#define ixGLB_GUS_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_CHA_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_CHCG_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixGLB_ATCL2_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_VML2_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixGLB_SDMA0_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_SDMA1_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_SDMA2_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_SDMA3_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2A0_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2A1_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2A2_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2A3_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2C0_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2C1_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2C2_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2C3_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2C4_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2C5_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2C6_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2C7_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2C8_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2C9_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixGLB_GL2C10_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixGLB_GL2C11_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixGLB_GL2C12_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixGLB_GL2C13_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixGLB_GL2C14_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixGLB_GL2C15_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixGLB_EA0_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_EA1_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_EA2_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_EA3_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_EA4_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_EA5_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_EA6_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_EA7_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_EA8_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_EA9_SAMPLEDELAY_DEFAULT                                            0x00000000
#define ixGLB_EA10_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixGLB_EA11_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixGLB_EA12_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixGLB_EA13_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixGLB_EA14_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixGLB_EA15_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixGLB_CHC0_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixGLB_CHC1_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixGLB_CHC2_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixGLB_CHC3_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixGLB_GE2SE0_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixGLB_GE2SE1_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixGLB_GE2SE2_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixGLB_GE2SE3_SAMPLEDELAY_DEFAULT                                         0x00000000


// addressBlock: spmind
#define ixSE_SPI_SAMPLEDELAY_DEFAULT                                             0x00000000
#define ixSE_SQG_SAMPLEDELAY_DEFAULT                                             0x00000000
#define ixSE_CBR_SAMPLEDELAY_DEFAULT                                             0x00000000
#define ixSE_DBR_SAMPLEDELAY_DEFAULT                                             0x00000000
#define ixSE_PA_SAMPLEDELAY_DEFAULT                                              0x00000000
#define ixSE_SA0SX_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixSE_SA0GL1A_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixSE_SA0GL1CG_SAMPLEDELAY_DEFAULT                                        0x00000000
#define ixSE_SA0CB0_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixSE_SA0CB1_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixSE_SA0DB0_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixSE_SA0DB1_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixSE_SA0SC0_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixSE_SA0SC1_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixSE_SA0RMI0_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixSE_SA0RMI1_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixSE_SA0GL1C0_SAMPLEDELAY_DEFAULT                                        0x00000000
#define ixSE_SA0GL1C1_SAMPLEDELAY_DEFAULT                                        0x00000000
#define ixSE_SA0GL1C2_SAMPLEDELAY_DEFAULT                                        0x00000000
#define ixSE_SA0GL1C3_SAMPLEDELAY_DEFAULT                                        0x00000000
#define ixSE_SA0WGP00TA0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP00TA1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP00TD0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP00TD1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP00TCP0_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA0WGP00TCP1_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA0WGP01TA0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP01TA1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP01TD0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP01TD1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP01TCP0_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA0WGP01TCP1_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA0WGP02TA0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP02TA1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP02TD0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP02TD1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP02TCP0_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA0WGP02TCP1_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA0WGP03TA0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP03TA1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP03TD0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP03TD1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP03TCP0_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA0WGP03TCP1_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA0WGP04TA0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP04TA1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP04TD0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP04TD1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA0WGP04TCP0_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA0WGP04TCP1_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA1SX_SAMPLEDELAY_DEFAULT                                           0x00000000
#define ixSE_SA1GL1A_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixSE_SA1GL1CG_SAMPLEDELAY_DEFAULT                                        0x00000000
#define ixSE_SA1CB0_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixSE_SA1CB1_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixSE_SA1DB0_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixSE_SA1DB1_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixSE_SA1SC0_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixSE_SA1SC1_SAMPLEDELAY_DEFAULT                                          0x00000000
#define ixSE_SA1RMI0_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixSE_SA1RMI1_SAMPLEDELAY_DEFAULT                                         0x00000000
#define ixSE_SA1GL1C0_SAMPLEDELAY_DEFAULT                                        0x00000000
#define ixSE_SA1GL1C1_SAMPLEDELAY_DEFAULT                                        0x00000000
#define ixSE_SA1GL1C2_SAMPLEDELAY_DEFAULT                                        0x00000000
#define ixSE_SA1GL1C3_SAMPLEDELAY_DEFAULT                                        0x00000000
#define ixSE_SA1WGP00TA0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP00TA1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP00TD0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP00TD1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP00TCP0_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA1WGP00TCP1_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA1WGP01TA0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP01TA1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP01TD0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP01TD1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP01TCP0_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA1WGP01TCP1_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA1WGP02TA0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP02TA1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP02TD0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP02TD1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP02TCP0_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA1WGP02TCP1_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA1WGP03TA0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP03TA1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP03TD0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP03TD1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP03TCP0_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA1WGP03TCP1_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA1WGP04TA0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP04TA1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP04TD0_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP04TD1_SAMPLEDELAY_DEFAULT                                     0x00000000
#define ixSE_SA1WGP04TCP0_SAMPLEDELAY_DEFAULT                                    0x00000000
#define ixSE_SA1WGP04TCP1_SAMPLEDELAY_DEFAULT                                    0x00000000




// addressBlock: grtavfsind
#define ixRTAVFS_REG0_DEFAULT                                                    0x01000000
#define ixRTAVFS_REG1_DEFAULT                                                    0x00000000
#define ixRTAVFS_REG2_DEFAULT                                                    0x01000000
#define ixRTAVFS_REG3_DEFAULT                                                    0x00000000
#define ixRTAVFS_REG4_DEFAULT                                                    0x01000000
#define ixRTAVFS_REG5_DEFAULT                                                    0x00000000
#define ixRTAVFS_REG6_DEFAULT                                                    0x01000000
#define ixRTAVFS_REG7_DEFAULT                                                    0x00000000
#define ixRTAVFS_REG8_DEFAULT                                                    0x01000000
#define ixRTAVFS_REG9_DEFAULT                                                    0x00000000
#define ixRTAVFS_REG10_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG11_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG12_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG13_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG14_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG15_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG16_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG17_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG18_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG19_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG20_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG21_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG22_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG23_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG24_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG25_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG26_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG27_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG28_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG29_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG30_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG31_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG32_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG33_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG34_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG35_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG36_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG37_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG38_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG39_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG40_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG41_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG42_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG43_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG44_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG45_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG46_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG47_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG48_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG49_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG50_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG51_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG52_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG53_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG54_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG55_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG56_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG57_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG58_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG59_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG60_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG61_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG62_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG63_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG64_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG65_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG66_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG67_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG68_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG69_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG70_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG71_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG72_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG73_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG74_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG75_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG76_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG77_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG78_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG79_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG80_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG81_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG82_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG83_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG84_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG85_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG86_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG87_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG88_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG89_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG90_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG91_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG92_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG93_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG94_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG95_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG96_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG97_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG98_DEFAULT                                                   0x01000000
#define ixRTAVFS_REG99_DEFAULT                                                   0x00000000
#define ixRTAVFS_REG100_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG101_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG102_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG103_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG104_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG105_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG106_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG107_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG108_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG109_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG110_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG111_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG112_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG113_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG114_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG115_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG116_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG117_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG118_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG119_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG120_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG121_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG122_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG123_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG124_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG125_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG126_DEFAULT                                                  0x01000000
#define ixRTAVFS_REG127_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG128_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG129_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG130_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG131_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG132_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG133_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG134_DEFAULT                                                  0x000211cd
#define ixRTAVFS_REG135_DEFAULT                                                  0x000af12c
#define ixRTAVFS_REG136_DEFAULT                                                  0x00000010
#define ixRTAVFS_REG137_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG138_DEFAULT                                                  0x00000008
#define ixRTAVFS_REG139_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG140_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG141_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG142_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG143_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG144_DEFAULT                                                  0x0015c040
#define ixRTAVFS_REG145_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG146_DEFAULT                                                  0x83c00260
#define ixRTAVFS_REG147_DEFAULT                                                  0x00000800
#define ixRTAVFS_REG148_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG149_DEFAULT                                                  0x000000ff
#define ixRTAVFS_REG150_DEFAULT                                                  0x000000ff
#define ixRTAVFS_REG151_DEFAULT                                                  0x000000ff
#define ixRTAVFS_REG152_DEFAULT                                                  0x000000ff
#define ixRTAVFS_REG153_DEFAULT                                                  0x000000ff
#define ixRTAVFS_REG154_DEFAULT                                                  0x000000ff
#define ixRTAVFS_REG155_DEFAULT                                                  0x000000ff
#define ixRTAVFS_REG156_DEFAULT                                                  0x000000ff
#define ixRTAVFS_REG157_DEFAULT                                                  0x000000ff
#define ixRTAVFS_REG158_DEFAULT                                                  0x000000ff
#define ixRTAVFS_REG159_DEFAULT                                                  0x000000ff
#define ixRTAVFS_REG160_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG161_DEFAULT                                                  0xcccdbcdd
#define ixRTAVFS_REG162_DEFAULT                                                  0x2587d190
#define ixRTAVFS_REG163_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG164_DEFAULT                                                  0x00000000
#define ixRTAVFS_REG165_DEFAULT                                                  0x00000000


// addressBlock: spiind
#define ixSA_WGP_BLK_ID_DEFAULT                                                  0x00000000


// addressBlock: sqind
#define ixSQ_WAVE_ACTIVE_DEFAULT                                                 0x00000000
#define ixSQ_WAVE_VALID_AND_IDLE_DEFAULT                                         0x00000000
#define ixSQ_WAVE_MODE_DEFAULT                                                   0x00000000
#define ixSQ_WAVE_STATUS_DEFAULT                                                 0x00000000
#define ixSQ_WAVE_TRAPSTS_DEFAULT                                                0x00000000
#define ixSQ_WAVE_HW_ID_LEGACY_DEFAULT                                           0x00000000
#define ixSQ_WAVE_GPR_ALLOC_DEFAULT                                              0x00000000
#define ixSQ_WAVE_LDS_ALLOC_DEFAULT                                              0x00000000
#define ixSQ_WAVE_IB_STS_DEFAULT                                                 0x00000000
#define ixSQ_WAVE_PC_LO_DEFAULT                                                  0x00000000
#define ixSQ_WAVE_PC_HI_DEFAULT                                                  0x00000000
#define ixSQ_WAVE_INST_DW0_DEFAULT                                               0x00000000
#define ixSQ_WAVE_IB_DBG1_DEFAULT                                                0x00000000
#define ixSQ_WAVE_FLUSH_IB_DEFAULT                                               0x00000000
#define ixSQ_WAVE_FLAT_SCRATCH_LO_DEFAULT                                        0x00000000
#define ixSQ_WAVE_FLAT_SCRATCH_HI_DEFAULT                                        0x00000000
#define ixSQ_WAVE_HW_ID1_DEFAULT                                                 0x00000000
#define ixSQ_WAVE_HW_ID2_DEFAULT                                                 0x00000000
#define ixSQ_WAVE_POPS_PACKER_DEFAULT                                            0x00000000
#define ixSQ_WAVE_SCHED_MODE_DEFAULT                                             0x00000000
#define ixSQ_WAVE_VGPR_OFFSET_DEFAULT                                            0x00000000
#define ixSQ_WAVE_IB_STS2_DEFAULT                                                0x00000000
#define ixSQ_WAVE_SHADER_CYCLES_DEFAULT                                          0x00000000
#define ixSQ_WAVE_TTMP0_DEFAULT                                                  0x00000000
#define ixSQ_WAVE_TTMP1_DEFAULT                                                  0x00000000
#define ixSQ_WAVE_TTMP2_DEFAULT                                                  0x00000000
#define ixSQ_WAVE_TTMP3_DEFAULT                                                  0x00000000
#define ixSQ_WAVE_TTMP4_DEFAULT                                                  0x00000000
#define ixSQ_WAVE_TTMP5_DEFAULT                                                  0x00000000
#define ixSQ_WAVE_TTMP6_DEFAULT                                                  0x00000000
#define ixSQ_WAVE_TTMP7_DEFAULT                                                  0x00000000
#define ixSQ_WAVE_TTMP8_DEFAULT                                                  0x00000000
#define ixSQ_WAVE_TTMP9_DEFAULT                                                  0x00000000
#define ixSQ_WAVE_TTMP10_DEFAULT                                                 0x00000000
#define ixSQ_WAVE_TTMP11_DEFAULT                                                 0x00000000
#define ixSQ_WAVE_TTMP12_DEFAULT                                                 0x00000000
#define ixSQ_WAVE_TTMP13_DEFAULT                                                 0x00000000
#define ixSQ_WAVE_TTMP14_DEFAULT                                                 0x00000000
#define ixSQ_WAVE_TTMP15_DEFAULT                                                 0x00000000
#define ixSQ_WAVE_M0_DEFAULT                                                     0x00000000
#define ixSQ_WAVE_EXEC_LO_DEFAULT                                                0x00000000
#define ixSQ_WAVE_EXEC_HI_DEFAULT                                                0x00000000
#define ixSQ_INTERRUPT_WORD_AUTO_DEFAULT                                         0x00000000
#define ixSQ_INTERRUPT_WORD_ERROR_DEFAULT                                        0x00000000
#define ixSQ_INTERRUPT_WORD_WAVE_DEFAULT                                         0x00000000


// addressBlock: didtind
#define ixDIDT_SQ_CTRL0_DEFAULT                                                  0x0000ff00
#define ixDIDT_SQ_CTRL1_DEFAULT                                                  0x00ff00ff
#define ixDIDT_SQ_CTRL2_DEFAULT                                                  0x18800004
#define ixDIDT_SQ_CTRL_OCP_DEFAULT                                               0x000000ff
#define ixDIDT_SQ_STALL_CTRL_DEFAULT                                             0x00fff000
#define ixDIDT_SQ_TUNING_CTRL_DEFAULT                                            0x00010004
#define ixDIDT_SQ_STALL_AUTO_RELEASE_CTRL_DEFAULT                                0x00ffffff
#define ixDIDT_SQ_CTRL3_DEFAULT                                                  0x00038000
#define ixDIDT_SQ_STALL_PATTERN_1_2_DEFAULT                                      0x01010001
#define ixDIDT_SQ_STALL_PATTERN_3_4_DEFAULT                                      0x11110421
#define ixDIDT_SQ_STALL_PATTERN_5_6_DEFAULT                                      0x25291249
#define ixDIDT_SQ_STALL_PATTERN_7_DEFAULT                                        0x00002aaa
#define ixDIDT_SQ_MPD_SCALE_FACTOR_DEFAULT                                       0x00000000
#define ixDIDT_SQ_STALL_RELEASE_CNTL0_DEFAULT                                    0x00000000
#define ixDIDT_SQ_STALL_RELEASE_CNTL1_DEFAULT                                    0x00000000
#define ixDIDT_SQ_STALL_RELEASE_CNTL_STATUS_DEFAULT                              0x00000000
#define ixDIDT_SQ_WEIGHT0_3_DEFAULT                                              0x00000000
#define ixDIDT_SQ_WEIGHT4_7_DEFAULT                                              0x00000000
#define ixDIDT_SQ_WEIGHT8_11_DEFAULT                                             0x00000000
#define ixDIDT_SQ_EDC_CTRL_DEFAULT                                               0x00001c00
#define ixDIDT_SQ_EDC_THRESHOLD_DEFAULT                                          0x00000000
#define ixDIDT_SQ_EDC_STALL_PATTERN_1_2_DEFAULT                                  0x01010001
#define ixDIDT_SQ_EDC_STALL_PATTERN_3_4_DEFAULT                                  0x11110421
#define ixDIDT_SQ_EDC_STALL_PATTERN_5_6_DEFAULT                                  0x25291249
#define ixDIDT_SQ_EDC_STALL_PATTERN_7_DEFAULT                                    0x00002aaa
#define ixDIDT_SQ_EDC_TIMER_PERIOD_DEFAULT                                       0x00003fff
#define ixDIDT_SQ_THROTTLE_CTRL_DEFAULT                                          0x00000000
#define ixDIDT_SQ_EDC_STALL_DELAY_1_DEFAULT                                      0x00000000
#define ixDIDT_SQ_EDC_STALL_DELAY_2_DEFAULT                                      0x00000000
#define ixDIDT_SQ_EDC_STALL_DELAY_3_DEFAULT                                      0x00000000
#define ixDIDT_SQ_EDC_STATUS_DEFAULT                                             0x00000000
#define ixDIDT_SQ_EDC_OVERFLOW_DEFAULT                                           0x00000000
#define ixDIDT_SQ_EDC_ROLLING_POWER_DELTA_DEFAULT                                0x00000000
#define ixDIDT_SQ_EDC_PCC_PERF_COUNTER_DEFAULT                                   0x00000000
#define ixDIDT_DB_CTRL0_DEFAULT                                                  0x0000ff00
#define ixDIDT_DB_CTRL1_DEFAULT                                                  0x00ff00ff
#define ixDIDT_DB_CTRL2_DEFAULT                                                  0x18800004
#define ixDIDT_DB_CTRL_OCP_DEFAULT                                               0x000000ff
#define ixDIDT_DB_STALL_CTRL_DEFAULT                                             0x00fff000
#define ixDIDT_DB_TUNING_CTRL_DEFAULT                                            0x00010004
#define ixDIDT_DB_STALL_AUTO_RELEASE_CTRL_DEFAULT                                0x00ffffff
#define ixDIDT_DB_CTRL3_DEFAULT                                                  0x00038000
#define ixDIDT_DB_STALL_PATTERN_1_2_DEFAULT                                      0x01010001
#define ixDIDT_DB_STALL_PATTERN_3_4_DEFAULT                                      0x11110421
#define ixDIDT_DB_STALL_PATTERN_5_6_DEFAULT                                      0x25291249
#define ixDIDT_DB_STALL_PATTERN_7_DEFAULT                                        0x00002aaa
#define ixDIDT_DB_MPD_SCALE_FACTOR_DEFAULT                                       0x00000000
#define ixDIDT_DB_STALL_RELEASE_CNTL0_DEFAULT                                    0x00000000
#define ixDIDT_DB_STALL_RELEASE_CNTL1_DEFAULT                                    0x00000000
#define ixDIDT_DB_STALL_RELEASE_CNTL_STATUS_DEFAULT                              0x00000000
#define ixDIDT_DB_WEIGHT0_3_DEFAULT                                              0x00000000
#define ixDIDT_DB_WEIGHT4_7_DEFAULT                                              0x00000000
#define ixDIDT_DB_WEIGHT8_11_DEFAULT                                             0x00000000
#define ixDIDT_DB_EDC_CTRL_DEFAULT                                               0x00001c00
#define ixDIDT_DB_EDC_THRESHOLD_DEFAULT                                          0x00000000
#define ixDIDT_DB_EDC_STALL_PATTERN_1_2_DEFAULT                                  0x01010001
#define ixDIDT_DB_EDC_STALL_PATTERN_3_4_DEFAULT                                  0x11110421
#define ixDIDT_DB_EDC_STALL_PATTERN_5_6_DEFAULT                                  0x25291249
#define ixDIDT_DB_EDC_STALL_PATTERN_7_DEFAULT                                    0x00002aaa
#define ixDIDT_DB_EDC_TIMER_PERIOD_DEFAULT                                       0x00003fff
#define ixDIDT_DB_THROTTLE_CTRL_DEFAULT                                          0x00000000
#define ixDIDT_DB_EDC_STALL_DELAY_1_DEFAULT                                      0x00000000
#define ixDIDT_DB_EDC_STATUS_DEFAULT                                             0x00000000
#define ixDIDT_DB_EDC_OVERFLOW_DEFAULT                                           0x00000000
#define ixDIDT_DB_EDC_ROLLING_POWER_DELTA_DEFAULT                                0x00000000
#define ixDIDT_DB_EDC_PCC_PERF_COUNTER_DEFAULT                                   0x00000000
#define ixDIDT_TD_CTRL0_DEFAULT                                                  0x0000ff00
#define ixDIDT_TD_CTRL1_DEFAULT                                                  0x00ff00ff
#define ixDIDT_TD_CTRL2_DEFAULT                                                  0x18800004
#define ixDIDT_TD_CTRL_OCP_DEFAULT                                               0x000000ff
#define ixDIDT_TD_STALL_CTRL_DEFAULT                                             0x00fff000
#define ixDIDT_TD_TUNING_CTRL_DEFAULT                                            0x00010004
#define ixDIDT_TD_STALL_AUTO_RELEASE_CTRL_DEFAULT                                0x00ffffff
#define ixDIDT_TD_CTRL3_DEFAULT                                                  0x00038000
#define ixDIDT_TD_STALL_PATTERN_1_2_DEFAULT                                      0x01010001
#define ixDIDT_TD_STALL_PATTERN_3_4_DEFAULT                                      0x11110421
#define ixDIDT_TD_STALL_PATTERN_5_6_DEFAULT                                      0x25291249
#define ixDIDT_TD_STALL_PATTERN_7_DEFAULT                                        0x00002aaa
#define ixDIDT_TD_MPD_SCALE_FACTOR_DEFAULT                                       0x00000000
#define ixDIDT_TD_STALL_RELEASE_CNTL0_DEFAULT                                    0x00000000
#define ixDIDT_TD_STALL_RELEASE_CNTL1_DEFAULT                                    0x00000000
#define ixDIDT_TD_STALL_RELEASE_CNTL_STATUS_DEFAULT                              0x00000000
#define ixDIDT_TD_WEIGHT0_3_DEFAULT                                              0x00000000
#define ixDIDT_TD_WEIGHT4_7_DEFAULT                                              0x00000000
#define ixDIDT_TD_WEIGHT8_11_DEFAULT                                             0x00000000
#define ixDIDT_TD_EDC_CTRL_DEFAULT                                               0x00001c00
#define ixDIDT_TD_EDC_THRESHOLD_DEFAULT                                          0x00000000
#define ixDIDT_TD_EDC_STALL_PATTERN_1_2_DEFAULT                                  0x01010001
#define ixDIDT_TD_EDC_STALL_PATTERN_3_4_DEFAULT                                  0x11110421
#define ixDIDT_TD_EDC_STALL_PATTERN_5_6_DEFAULT                                  0x25291249
#define ixDIDT_TD_EDC_STALL_PATTERN_7_DEFAULT                                    0x00002aaa
#define ixDIDT_TD_EDC_TIMER_PERIOD_DEFAULT                                       0x00003fff
#define ixDIDT_TD_THROTTLE_CTRL_DEFAULT                                          0x00000000
#define ixDIDT_TD_EDC_STALL_DELAY_1_DEFAULT                                      0x00000000
#define ixDIDT_TD_EDC_STALL_DELAY_2_DEFAULT                                      0x00000000
#define ixDIDT_TD_EDC_STALL_DELAY_3_DEFAULT                                      0x00000000
#define ixDIDT_TD_EDC_STATUS_DEFAULT                                             0x00000000
#define ixDIDT_TD_EDC_OVERFLOW_DEFAULT                                           0x00000000
#define ixDIDT_TD_EDC_ROLLING_POWER_DELTA_DEFAULT                                0x00000000
#define ixDIDT_TD_EDC_PCC_PERF_COUNTER_DEFAULT                                   0x00000000
#define ixDIDT_TCP_CTRL0_DEFAULT                                                 0x0000ff00
#define ixDIDT_TCP_CTRL1_DEFAULT                                                 0x00ff00ff
#define ixDIDT_TCP_CTRL2_DEFAULT                                                 0x18800004
#define ixDIDT_TCP_CTRL_OCP_DEFAULT                                              0x0000ffff
#define ixDIDT_TCP_STALL_CTRL_DEFAULT                                            0x00fff000
#define ixDIDT_TCP_TUNING_CTRL_DEFAULT                                           0x00010004
#define ixDIDT_TCP_STALL_AUTO_RELEASE_CTRL_DEFAULT                               0x00ffffff
#define ixDIDT_TCP_CTRL3_DEFAULT                                                 0x00038000
#define ixDIDT_TCP_STALL_PATTERN_1_2_DEFAULT                                     0x01010001
#define ixDIDT_TCP_STALL_PATTERN_3_4_DEFAULT                                     0x11110421
#define ixDIDT_TCP_STALL_PATTERN_5_6_DEFAULT                                     0x25291249
#define ixDIDT_TCP_STALL_PATTERN_7_DEFAULT                                       0x00002aaa
#define ixDIDT_TCP_MPD_SCALE_FACTOR_DEFAULT                                      0x00000000
#define ixDIDT_TCP_STALL_RELEASE_CNTL0_DEFAULT                                   0x00000000
#define ixDIDT_TCP_STALL_RELEASE_CNTL1_DEFAULT                                   0x00000000
#define ixDIDT_TCP_STALL_RELEASE_CNTL_STATUS_DEFAULT                             0x00000000
#define ixDIDT_TCP_WEIGHT0_3_DEFAULT                                             0x00000000
#define ixDIDT_TCP_WEIGHT4_7_DEFAULT                                             0x00000000
#define ixDIDT_TCP_WEIGHT8_11_DEFAULT                                            0x00000000
#define ixDIDT_TCP_EDC_CTRL_DEFAULT                                              0x00001c00
#define ixDIDT_TCP_EDC_THRESHOLD_DEFAULT                                         0x00000000
#define ixDIDT_TCP_EDC_STALL_PATTERN_1_2_DEFAULT                                 0x01010001
#define ixDIDT_TCP_EDC_STALL_PATTERN_3_4_DEFAULT                                 0x11110421
#define ixDIDT_TCP_EDC_STALL_PATTERN_5_6_DEFAULT                                 0x25291249
#define ixDIDT_TCP_EDC_STALL_PATTERN_7_DEFAULT                                   0x00002aaa
#define ixDIDT_TCP_EDC_TIMER_PERIOD_DEFAULT                                      0x00003fff
#define ixDIDT_TCP_THROTTLE_CTRL_DEFAULT                                         0x00000000
#define ixDIDT_TCP_EDC_STALL_DELAY_1_DEFAULT                                     0x00000000
#define ixDIDT_TCP_EDC_STALL_DELAY_2_DEFAULT                                     0x00000000
#define ixDIDT_TCP_EDC_STALL_DELAY_3_DEFAULT                                     0x00000000
#define ixDIDT_TCP_EDC_STATUS_DEFAULT                                            0x00000000
#define ixDIDT_TCP_EDC_OVERFLOW_DEFAULT                                          0x00000000
#define ixDIDT_TCP_EDC_ROLLING_POWER_DELTA_DEFAULT                               0x00000000
#define ixDIDT_TCP_EDC_PCC_PERF_COUNTER_DEFAULT                                  0x00000000
#define ixDIDT_SQ_STALL_EVENT_COUNTER_DEFAULT                                    0x00000000
#define ixDIDT_DB_STALL_EVENT_COUNTER_DEFAULT                                    0x00000000
#define ixDIDT_TD_STALL_EVENT_COUNTER_DEFAULT                                    0x00000000
#define ixDIDT_TCP_STALL_EVENT_COUNTER_DEFAULT                                   0x00000000


#endif
